#include <math.h>

#include "Base.h"
#include "UtilFunctions.h"
#include "Group.h"
#include "MapObject.h"

Base::Base(const std::string& name,unsigned int playerID) : MapObject(name,playerID) { // bases always have a speed of 0

}

Base::~Base(){

}

Base* Base::clone() const{
	return (new Base(*this));
		
}

inline bool Base::withinRange(Group* otherObject,const std::string& unitType){
	Vector3D test = otherObject->getPosition();
	
	if(test.x < (position.x - extents/2.0)) test.x = position.x - extents/2.0;
		
	if(test.x > (position.x + extents/2.0)) test.x = position.x + extents/2.0;
		
	if(test.y < (position.y - extents/2.0)) test.y = position.y - extents/2.0;
		
	if(test.y > (position.y + extents/2.0)) test.y = position.y + extents/2.0;

	double range =  otherObject->getAttackRangeForType(unitType);
	if((otherObject->getPosition() - test).length() < range){
		return true;
	}else{
		return false;
	}
}
bool Base::withinRange(Group* otherObject){
	Vector3D test = otherObject->getPosition();
	
	if(test.x < (position.x - extents/2.0)) test.x = position.x - extents/2.0;
	
	if(test.x > (position.x + extents/2.0)) test.x = position.x + extents/2.0;
	
	if(test.y < (position.y - extents/2.0)) test.y = position.y - extents/2.0;
	
	if(test.y > (position.y + extents/2.0)) test.y = position.y + extents/2.0;
	
	double range =  otherObject->getMaxAttackRange();
	if((otherObject->getPosition() - test).length() < range){
		return true;
	}else{
		return false;
	}
}

inline bool Base::takeDamage(const double& damage,const std::string&){
	wasHit = true;
	if(hitPoints > 0){
		hitPoints-=damage;
		recalculateExtents();
		return true;
	}else{
		return false;
	}
}
inline bool Base::eliminateDead(){
	if(!wasHit) return false; // wasn't hit, no need to do anything
	wasHit = false;
	if(hitPoints <= 0.0){
		return true;
	}
	return false;
}

inline void Base::draw(){
	
	drawSquare(position,extents,color);
	// TODO this is kind of a hack, but hard to work with fonts.... only for esthetics anyways, can be fixed later if need be
	if(displayObjectInfo){
		if(draw3DTextInfo){
			draw3DString(position.x-extents/2.0,position.y-extents-1.5,"VP: " +convertToString(victoryValue),color,1.0/(-cameraPos.z/4.0));
			draw3DString(position.x-extents/2.0,position.y-extents-4.5,"HP: " + convertToString(victoryValue),color,1.0/(-cameraPos.z/4.0));
		}else{
			draw2DString(position.x-extents/2.0,position.y-extents-1.5,"VP: "+convertToString(victoryValue),color,true);
			draw2DString(position.x-extents/2.0,position.y-extents-4.5,"HP: "+ convertToString(hitPoints),color,true);
		}
	}
}

inline void Base::drawInSelectionMode(){
	glPushName(BASE_TYPE_ID);
	
	MapObject::drawInSelectionMode();
	
	glPopName();
}

inline void Base::recalculateExtents(){
	extents = (sqrt(hitPoints)/BASE_SIZE_DIVISOR);
}

double Base::getHitPoints() const{
	return hitPoints;
}
unsigned int Base::getVictoryValue() const{
	return victoryValue;
}

void Base::setHitPoints(double hitPoints){
	this->hitPoints = hitPoints;
	recalculateExtents();
}
void Base::setVictoryValue(unsigned int victoryValue){
	this->victoryValue = victoryValue;
}
double Base::getExtents() const{
	return extents;
}

inline std::string Base::toString(){
	std::ostringstream baseStream;
	
	baseStream << "base: "  << MapObject::toString();
	baseStream << "hp " << hitPoints << "\n";
	baseStream << "victory_value " << victoryValue << "\n";
	baseStream << "end" << "\n";

	return baseStream.str();
}


