#ifndef DEFAULTS_H
#define DEFAULTS_H

#include <string>
#include <vector>
#include <sstream>
#include <fstream>
#include <map>
#include <set>

#define COMMENT_STYLE "//"

const double DOUBLE_MAX = 1e20;

class Player;
class Vector3D;
class Color;
class MapObject;
class Unit;
class Action;
class Strategy;

typedef std::map<const unsigned int,Player*> PlayerMap;
typedef std::map<const std::string,Unit*> UnitMap;
typedef std::map<const std::string,Action*> ActionMap;
typedef std::map<const std::string,Strategy*> StrategyMap;

typedef std::set<MapObject*, bool (*)(const MapObject*,const MapObject*)> MapObjectSet;

extern PlayerMap playerMap;
extern UnitMap unitMap;
extern ActionMap actionMap;
extern StrategyMap strategyMap;

extern MapObjectSet selectedObjects;

extern bool unitSemaphore;

extern std::vector<std::string> statusBuffer;

extern char fpsString[10];

extern Color red;
extern Color green;
extern Color blue;
extern Color yellow;
extern Color white;
extern Color transparentBlue;

// default config values
extern double viewDistance;

extern bool debug;
extern bool renderText;
extern bool showAttackRanges;
extern bool displayObjectInfo;
extern bool draw3DTextInfo;
extern bool paused;

// for window
extern int windowX;
extern int windowY;
extern int windowWidth;
extern int windowHeight;

// for camera
extern double yaw;
extern double pitch;
extern double roll;

extern Vector3D cameraPos;
extern double cameraSpeed;

extern unsigned int statusBufferSize;
extern bool smallFont;

extern double fps;
extern std::string groupSaveFile;

extern Vector3D statusMessagePos;

extern std::string configFile;
extern std::string groupFile;
extern std::string unitFile;
extern std::string actionFile;
extern std::string strategyFile;

#endif
