#ifndef PLAYER_H
#define PLAYER_H

#include <iostream>
#include <string>
#include <sstream>
#include <vector>
#include <set>
#include <map>
#include <math.h>

#include "Defaults.h"

class MapObject;
class Group;
class Base;

typedef std::map<const unsigned int,MapObject*> PlayerObjectMap;

const double positionCompareRange = 0.05;

class Player{
	
public:
	Player(unsigned int playerID);
	~Player();
	Player(const Player& p);
	
	Player* clone() const;
	
	unsigned int getID() const;
	
	void addObject(MapObject* newObject);
	void drawObjects();
	void drawObjectsInSelectionMode();
	
	void executeOrders(double timeElapsed);

	
	void addContainedObjectsToList(const Vector3D& selectionRectangleStart,const Vector3D& selectionRectangleEnd,MapObjectSet& selectedObjects);
	
	void printObjects(std::ofstream& stream);
	std::string Player::printObjects();
		
	std::string toString();
	
	PlayerObjectMap* getObjects();
	MapObject* getObject(unsigned int objectID);
	
	void checkForDestroyedObjects(std::vector<MapObject*>& deletedObjects); // return all the destroyed groups... so we can get rid of them from selectedObjects for example...
	

	void getGroupsAtLocation(const Group* otherGroup,std::vector<Group*>& groupsAtLocation);
	Base* getNearestEnemyBase(const Vector3D& position);
		
private:
	Player& operator=(const Player& b) {}; // no = operator for now
	
	PlayerObjectMap objects;

	unsigned int playerID;
};

#endif

