
#include "UtilFunctions.h"
#include "Defaults.h"

#include "Player.h"
#include "Group.h"
#include "Unit.h"
#include "Base.h"


Unit* getUnit(const std::string& name){
	UnitMap::iterator it;
	it = unitMap.find(name);
	if(it != unitMap.end()){
		return it->second;
	}else{
		return NULL;
	}	
}

bool unitExists(const std::string& name){
	if(getUnit(name) != NULL){
		return true;
	}else{
		return false;
	}
}

Player* getPlayer(const unsigned int& playerID){
	
	PlayerMap::iterator it;
	it = playerMap.find(playerID);
	if(it != playerMap.end()){
		return it->second;
	}else{
		return NULL;
	}
}

bool playerExists(const unsigned int& playerID){
	if(getPlayer(playerID) != NULL){
		return true;
	}else{
		return false;
	}
}

void createNewPlayer(const unsigned int& playerID){
	if(!playerExists(playerID)){
		//playerMap.insert(PlayerMap::value_type(playerID,new Player(playerID)));
		playerMap[playerID] = new Player(playerID);
	print("New player with id: " + convertToString(playerID) + " created");
	}else{
	print("New player with id: " + convertToString(playerID) + " could not be created, because it already exists");
	}
}
void addGroup(std::ifstream& in,const std::string& groupName,const unsigned int& playerID,unsigned int& lineNumber){
	std::string newLine;
	Group* newGroup = NULL;
	if(playerExists(playerID)){
		newGroup = new Group(groupName,playerID);
		print("Creating new group named "+groupName);
	}else{
		printToScreen("Cannot add group for player:" + convertToString(playerID) + " because that player does not exist.");
	}
	
	do{
		getline(in,newLine);
		lineNumber++;
		if(newLine.length() == 0) continue;
		
		std::stringstream currentLine(newLine);
		std::string groupCommand;
		currentLine >> groupCommand;
		transform(groupCommand.begin(),groupCommand.end(),groupCommand.begin(),toupper);
		
		if(groupCommand.substr(0,2) == COMMENT_STYLE){
			// do nothing
		}else if(unitExists(groupCommand)){
			int numUnits;
			currentLine >> numUnits;
			if(newGroup != NULL){
				Unit* unitTemplate = getUnit(groupCommand);
				for(int i=0;i<numUnits;i++){
					Unit* newUnit = new Unit(*unitTemplate);
					
					newGroup->addUnit(groupCommand,newUnit);
				}
			}
		}else if(groupCommand == "POSITION"){
			Vector3D position;
			currentLine >> position;
			if(newGroup != NULL) newGroup->setPosition(position);
			
		}else if(groupCommand == "END"){
			break;
		}else{
			print("Unit command" + groupCommand + " at line number " + convertToString(lineNumber) + " has not been defined! Ignoring...");
		}
		
	}while(!(in.eof()) && in);
	
	if(newGroup != NULL && newGroup->getNumUnits() > 0){
		getPlayer(playerID)->addObject(newGroup);
  		print("Added group with name=" + groupName + " and playerID=" + convertToString(playerID) + " at (" + convertToString(newGroup->getPosition().x) + "," + convertToString(newGroup->getPosition().y) + ")");
	}
	//newGroup->printUnits();
}

void addBase(std::ifstream& in,const std::string& baseName,const unsigned int& playerID, unsigned int& lineNumber){
	std::string newLine;
	Base* newBase = NULL;
	if(playerExists(playerID)){
		newBase = new Base(baseName,playerID);
		print("Creating new base named " + baseName);
	}else{
		printToScreen("Cannot add base for player:" + convertToString(playerID) + " because that player does not exist.");
	}
	do{
		getline(in,newLine);
		lineNumber++;
		if(newLine.length() == 0) continue;
		
		std::stringstream currentLine(newLine);
		std::string baseCommand;
		currentLine >> baseCommand;
		transform(baseCommand.begin(),baseCommand.end(),baseCommand.begin(),toupper);
		
		if(baseCommand.substr(0,2) == COMMENT_STYLE){
			// do nothing
		}else if(baseCommand == "POSITION"){
			Vector3D position;
			currentLine >> position;
			position.z = 0.1; // just to make bases go underneath groups
			if(newBase != NULL) newBase->setPosition(position);
			
		}else if(baseCommand == "HP"){
			double hitpoints;
			currentLine >> hitpoints;
			if(newBase != NULL) newBase->setHitPoints(hitpoints);
			
		}else if(baseCommand == "VICTORY_VALUE"){
			unsigned int victoryValue;
			currentLine >> victoryValue;
			if(newBase != NULL) newBase->setVictoryValue(victoryValue);
			
		}else if(baseCommand == "END"){
			break;
		}else{
			print("Base command" + baseCommand + " at line number " + convertToString(lineNumber) + " has not been defined! Ignoring...");
		}
		
	}while(!(in.eof()) && in);
	
	if(newBase != NULL){
		getPlayer(playerID)->addObject(newBase);
		print("Added base with name=" + baseName + " and playerID=" + convertToString(playerID) + " at (" + convertToString(newBase->getPosition().x) + "," + convertToString(newBase->getPosition().y) + ")");
	}
}

void readUnitFile(const std::string& unitFile){
	std::ifstream in; // input file stream
	unsigned int lineNumber = 0;
	
	in.open(unitFile.c_str());
	
	if(!in.is_open()){
		printToScreen("Failed to load " + unitFile);
		return;
	}else{
		print("Loading " + unitFile);
	}
	
	std::string newLine;
	
	do{
		getline(in,newLine);
		lineNumber++;
		// if we have a blank line, ignore it 
		if(newLine.length() == 0) continue;
		
		std::stringstream currentLine(newLine);
		std::string currentCommand;
		currentLine >> currentCommand; // get the first token of the line
		//make everything uppercase for the first token
		transform(currentCommand.begin(),currentCommand.end(),currentCommand.begin(),toupper);
		
		if(currentCommand.substr(0,2) == COMMENT_STYLE){
			// do nothing
		}else if(currentCommand == "UNIT:"){
			std::string unitName;
			currentLine >> unitName;
			
			transform(unitName.begin(),unitName.end(),unitName.begin(),toupper);
			
			bool newUnit = false;
			std::string unitCommand;
			if(unitExists(unitName)){
				print("Unit " + unitName + " already exists... ignoring redefinition");
				newUnit = false;
			}else{
				print("Defining unit " + unitName);
				
				unitMap[unitName] = new Unit();
				unitMap[unitName]->setName(unitName);
				newUnit = true;
			}
			do{
				getline(in,newLine);
				++lineNumber;
				if(newLine.length() == 0) continue;
				
				std::stringstream currentLine(newLine);
				currentLine >> unitCommand;
				transform(unitCommand.begin(),unitCommand.end(),unitCommand.begin(),toupper);
				
				if(unitCommand.substr(0,2) == COMMENT_STYLE){
					// do nothing
				}else if(unitCommand == "MOVE_SPEED"){
					double moveSpeed;
					currentLine >> moveSpeed;
					if(newUnit) unitMap[unitName]->setMoveSpeed(moveSpeed);
				}else if(unitCommand == "ATTACK_RATE"){
					double attackRate;
					currentLine >> attackRate;
					if(newUnit) unitMap[unitName]->setAttackRate(attackRate);
				}else if(unitCommand == "ATTACK_RANGE"){
					double attackRange;
					currentLine >> attackRange;
					if(newUnit) unitMap[unitName]->setAttackRange(attackRange);
					
				}else if(unitCommand == "ATTACK_POWER"){
					double attackPower;
					currentLine >> attackPower;
					if(newUnit) unitMap[unitName]->setAttackPower(attackPower);
				}else if(unitCommand == "HP"){
					double hitpoints;
					currentLine >> hitpoints;
					if(newUnit) unitMap[unitName]->setHitPoints(hitpoints);
				}else if(unitCommand == "ARMOR"){
					double armor;
					currentLine >> armor;
					if(newUnit) unitMap[unitName]->setArmor(armor);
				}else if(unitCommand == "END"){
					break;
				}else{
					print("Unknown unit definition command " + unitCommand + " at line number " + convertToString(lineNumber));
				}
			}while(!in.eof() && in);
			
		}else{
			print("Unknown unit command: " + currentCommand + " at line number " + convertToString(lineNumber));
		}
		
	}while(!in.eof() && in); // end of do-while loop
	
	in.close();
	
	printToScreen("Loaded " + unitFile);
	
}

void readGroupFile(const std::string& groupFile){
	std::ifstream in; // input file stream
	unsigned int lineNumber = 0;
	
	in.open(groupFile.c_str());
	
	unsigned int currentPlayerID = 0;
	
	if(!in.is_open()){
		printToScreen("Failed to load " + groupFile);
		return;
	}else{
		print("Loading " + groupFile);
	}
	
	unitSemaphore = true; // make sure noone is using these until they are all loaded
	
	std::string newLine;
	
	do{
		getline(in,newLine);
		lineNumber++;
		// if we have a blank line, ignore it 
		if(newLine.length() == 0) continue;
		
		std::stringstream currentLine(newLine);
		
		std::string currentCommand;
		currentLine >> currentCommand; // get the first token of the line
		
		//make everything uppercase for the first token
		transform(currentCommand.begin(),currentCommand.end(),currentCommand.begin(),toupper);
		
		if(currentCommand.substr(0,2) == COMMENT_STYLE){
			// do nothing to this line if the first token is a comment marker
			
		}else if(currentCommand == "PLAYER:"){
			currentLine >> currentPlayerID;
			createNewPlayer(currentPlayerID);
			print("Active player changed to: " + convertToString(currentPlayerID));
		}else if(currentCommand == "GROUP:"){
			std::string groupName;
			currentLine >> groupName;
			addGroup(in,groupName,currentPlayerID,lineNumber);
			
		}else if(currentCommand == "BASE:"){
			std::string baseName;
			currentLine >> baseName;
			addBase(in,baseName,currentPlayerID,lineNumber);
			
		}else{
			print("Unknown group command: " + currentCommand + " at line number " + convertToString(lineNumber));
		}
		
	}while(!in.eof() && in); // end of do-while loop
	
	in.close();
	
	unitSemaphore = false;
	
	printToScreen("Loaded " + groupFile);
}

void saveGroupFile(const std::string& saveFile){
	
	std::ofstream out; // output file stream
	out.open(saveFile.c_str());
	
	if(!out.is_open()){
		printToScreen("Your file " + saveFile + " could not be saved.\n");
		return;
	}else{
		print("Saving to " + saveFile);
	}
	
	unitSemaphore = true;
	
	out << "// This file is a saved state from the application\n\n";
	
	PlayerMap::iterator it;
	for(it= playerMap.begin(); it != playerMap.end(); ++it){
		out << (it->second)->toString();
		
		(it->second)->printObjects(out);
	}
	
	out.close();
	unitSemaphore = false;
	
	printToScreen("File " + saveFile + " saved");
}

void readConfigFile(const std::string& configFile){
	std::ifstream in; // input file stream
	unsigned int lineNumber = 0;
	
	in.open(configFile.c_str());
	
	if(!in.is_open()){
		printToScreen("Failed to load " + configFile + ", using default configuration");
		return;
	}else{
		print("Loading " + configFile);
	}
	
	std::string newLine;
	
	do{
		getline(in,newLine); // read in a line of the config file
		lineNumber++;
		// if we have a blank line, ignore it and get another one
		if(newLine.length() == 0) continue;
		
		std::stringstream currentLine(newLine);
		
		std::string currentCommand;
		
		currentLine >> currentCommand; // get the first token of the line
		
		//make everything uppercase for the first token
		transform(currentCommand.begin(),currentCommand.end(),currentCommand.begin(),toupper);
		
		if(currentCommand.substr(0,2) == COMMENT_STYLE){
			// do nothing to this line if the first token is a comment marker
			
		}else if(currentCommand == "DEBUG:"){
			currentLine >> debug;
			if(debug){
				print("Setting debug mode to ON");
			}else{
				print("Setting debug mode to OFF");
			}
		}else if(currentCommand == "RENDERTEXT:"){
			currentLine >> renderText;
			if(renderText){
				print("Setting rendertext mode to ON");
			}else{
				print("Setting rendertext mode to OFF");
			}
		}else if(currentCommand == "SHOWATTACKRANGES:"){
			currentLine >> showAttackRanges;
			if(showAttackRanges){
				print("Setting the display of attack ranges to ON");
			}else{
				print("Setting the display of attack ranges to OFF");
			}
		}else if(currentCommand == "DISPLAYOBJECTINFO:"){
			currentLine >> displayObjectInfo;
			if(displayObjectInfo){
				print("Setting displaying of object information to ON");
			}else{
				print("Setting displaying of object information ot OFF");	
			}
		}else if(currentCommand == "DRAW3DTEXTINFO:"){
			currentLine >> draw3DTextInfo;
			if(draw3DTextInfo){
				print("Setting 3D displaying of object information to ON");
			}else{
				print("Setting 3D displaying of object information ot OFF");	
			}
		}else if(currentCommand == "FPS:"){
			currentLine >> fps;
			print("FPS set to: " + convertToString(fps));
			
		}else if(currentCommand == "WIN_X:"){
			currentLine >> windowX;
			print("Window x position set to: " + convertToString(windowX));
			
		}else if(currentCommand == "WIN_Y:"){
			currentLine >> windowY;
			print("Window y position set to: " + convertToString(windowY));
			
		}else if(currentCommand == "WIN_WIDTH:"){
			currentLine >> windowWidth;
			print("Window width set to: " + convertToString(windowWidth));
			
		}else if(currentCommand == "WIN_HEIGHT:"){
			currentLine >> windowHeight;
			print("Window height set to: " + convertToString(windowHeight));
			
		}else if(currentCommand == "CAMERA_X:"){
			currentLine >> cameraPos.x;
			print("Camera x position set to: " + convertToString(cameraPos.x));
			
		}else if(currentCommand == "CAMERA_Y:"){
			currentLine >> cameraPos.y;
			print("Camera y position set to: " + convertToString(cameraPos.y));
			
		}else if(currentCommand == "CAMERA_Z:"){
			currentLine >> cameraPos.z;
			print("Camera z position set to: " + convertToString(cameraPos.z));
			
		}else if(currentCommand == "CAMERA_SPEED:"){
			currentLine >> cameraSpeed;
			print("Camera speed set to: " + convertToString(cameraSpeed));
			
		}else if(currentCommand =="GROUP_SAVEFILE:"){
			currentLine >> groupSaveFile;
			print("Savefile set to: " + groupSaveFile);
			
			
		}else if(currentCommand =="SMALL_FONT:"){
			currentLine >> smallFont;
			if(smallFont){
				print("Font set to SMALL(12pt)");
			}else{
				print("Font set to LARGE(18pt)");
			}
		}else if(currentCommand == "STATUS_MESSAGE_BUFFER:"){
			currentLine >> statusBufferSize;
			print("Status buffer size set to:" + convertToString(statusBufferSize));
			
		}else{
			print("Unknown config command: " + currentCommand + " at line number "+convertToString(lineNumber));
		}
		
	}while(!in.eof() && in); // end of do-while loop
	
	in.close();
	
	printToScreen("Loaded " + configFile);
}


