// $Id:commandertest.C v 1.0 Aaron Young, Ryan Neighbour
  
// This is an ORTS file (c) Michael Buro, licensed under the GPL

// this is the main file of the sample ORTS AI client
// options are specified by the opt.put lines below
// to test it start the ORTS server (bin/orts) and then run bin/sampleai [options]


 
#include "GfxGlobal.H"
#include "Options.H"
#include "GfxModule.H"
#include "GameStateModule.H"
#include "ShutDown.H"
#include "SDLinit.H"
#include "Game.H"
#include "splitAI.cpp"
#include "SimpleTerrain.H"
#include "ST_ForceField.H"
#include "Watcher.H"
static char *VERSION = "sampleai v0.1";

using namespace std;
     
     
void add_options()
{
  Options opt("SampleClient");
  opt.put("-usegfx", "display world in 3d");
  opt.put("-cinfo", "display game state changes");
  opt.put("-tinfo", "display tile info");
  opt.put("-oinfo", "display object info");
  opt.put("-minfo", "display map info");  
  opt.put("-nopush", "forcefield on");
  opt.put("-wpattract","no frackign idea");
  opt.put("-nostoptoplan", "no fracking idea");
  opt.put("-highres", "no fracking idea");
}  

 

struct NoGfxEventHandler : public EventHandler
{
  GameStateModule &gsm;

  NoGfxEventHandler(GameStateModule &state_) : gsm(state_) { }
 
  bool handle_event(const Event& e)
  {
    if (e.get_who() == GameStateModule::FROM) {

      //cout << "Event: " << e.who() << " " << e.what() << endl;

      if (e.get_what() == GameStateModule::STOP_MSG) exit(0);

      if (e.get_what() == GameStateModule::READ_ERROR_MSG) exit(10);

      if (e.get_what() == GameStateModule::VIEW_MSG) {

            
        ////////////////
        Vector<string> temp;
        Game &game =gsm.get_game();
        FORS (i, game.get_player_num()) temp.push_back(""); // dummy list of commands
        game.exec_actions(temp); // evaluate client actions and send actions to the server
        ////////////////

        gsm.send_actions();

        sint4 vf = game.get_view_frame();
        sint4 ai = game.get_action_frame();
        sint4 sk = game.get_skipped_actions();
        sint4 md = game.get_merged_actions();

        if (vf != ai || sk || md > 1) {

          cout << "frame " << vf;
          if (ai < 0) cout << " [no action]";
          else if (vf != ai) cout << " [behind by " << (vf - ai) << " frame(s)]";
          if (sk) cout << " [skipped " << sk << "]";
          if (md > 1) cout << " [merged " << md << "]";
          cout << endl;
        }
        return true;
      }

    }

    return false;
  }
};



// loop is called

class Looper : public LoopFunctor
{
public :

  Looper(GameStateModule& gsm_) : gsm(&gsm_) {}
  
  void loop() const {
    /*
    if (state->gfxm) {
 
      if (state->gfxm->get_quit()) { state->quit = true; exit(0); } // should return
    
      bool refresh;
      Options::get("-refresh", refresh);
      
      if (refresh) {

	// maximal gfx refresh rate
	
	if (!state->just_drew) 
	  state->gfxm->draw();
	else
	  state->just_drew = false;
      }
    }
    */
    // looks for server messages
    // if one or more arrived, calls handlers
    // if not, sleeps for one millisecond
    if (!gsm->recv_view()) SDL_Delay(1);
  }

private:
  GameStateModule* gsm;
};
 

int main(int argc, char **argv)
{
  signals_shut_down(true);  
  
  // populate opt with options from various modules

  add_options();
  GameStateModule::Options::add();
  GfxModule::Options::add();
  Game::Options::add();
 
  // process command line arguments
  if (Options::process(argc, argv, cerr, VERSION)) exit(20);

  // initialize SDL
  SDLinit::video_init();
  SDLinit::network_init();
  
  // populate option objects with command line values
  // and create modules
  
  GameStateModule::Options gsmo;
  GameStateModule gsm(gsmo);
 
  // ortsg event handler
  NoGfxEventHandler ngev(gsm);
  // gsm events also handled in ngev
  gsm.add_handler(&ngev);

  GfxModule::Options gfxmo;
  GfxModule *gfxm = 0;
  
  // change seed value to time(0) if 0

  sint4 seed;
  Options::get("-seed", seed);

  if (!seed) seed = time(0);

  Options::set("-seed", seed);
  cout << "seed=" << seed << endl;

  bool use_gfx;
  Options::get("-usegfx", use_gfx);

  if (use_gfx) gfxm = new GfxModule;

  // gives event handler access to the state
  
  SimpleTerrain::ST_ForceField timp;
  
  TerrainModule tm(gsm, timp, 110);
  
  Vector<GameObj*> units;
  Commander * c = 0;
  Watcher::WatcherModule* w=new Watcher::WatcherModule(gsm);
  
  splitAI* splitter=new splitAI(c, units, *w, 25, gsm, tm);
  cerr <<"Commander created." <<endl;
  // connect game state module to server
  if (!gsm.connect()) ERR("connection problems");
  
  Looper l(gsm);

  if (use_gfx) {
 
    // gfx mode

    glutInit(&argc, argv);
    gfxm->init(gsm.get_game(), gsm.get_changes(), gsm.get_action_changes(), gfxmo);

    // game loop (could become a thread)
    gfxm->start_loop(&l);

  } else {

    // no gfx
    
    while (1) l.loop();
  }
  
  delete gfxm;
  return 0;
}
