/* Test Class for Commander Framework 
 * Aaron Young
 * Ryan Neighbour
*/

#include "Commander.h"
#include "Global.H"
#include "ClientModule.H"
#include "TerrainModule.H"
#include "GameStateModule.H"
#include "Game.H"
#include "GameChanges.H"
#include <ctime>
#include "Watcher.H"

class splitAI : private Commander, ClientModule {
private:
  Vector<GameObj*> firstVec;
  Vector<GameObj*> secVec;
  int state;
  GameStateModule& gsm;
  TerrainModule* tm;
  bool doneLoading;
  int unitCountForCriteria;
  int pathSuccess;
 
 
public:
  //bool (Commander::*critFunc)(GameObj*);

  splitAI(Commander* const mySuperior, Vector<GameObj*>& units, Watcher::WatcherModule& w, int update, GameStateModule& gsm_, TerrainModule& tm_):
    Commander(mySuperior, units, w, update), 
    gsm(gsm_),
    tm(&tm_)
  {
    pathSuccess=0;
    state=0;
    gsm.add_handler(this);       // gsm -> ai
    cerr <<"Adding Commander as gsm handler" <<endl;
    tm->add_handler(this);        // tm -> ai
    cerr <<"Adding Commander as tm handler" <<endl;
    add_handler(tm);
    cerr <<"Adding TerrainModule as event handler for Commander" <<endl;
    unitCountForCriteria = 0;
  }
  
  /* split units based on index */
  Vector<GameObj*> splitOnIndex(sint4 index)
  {
    Vector<GameObj*> unitsMeetingCriteria;
    Vector<GameObj*> otherUnits;
    FORALL(unitVector, unit)
    {
      if(unitCountForCriteria<index)
      {
	unitsMeetingCriteria.push_back(*unit);
	unitCountForCriteria++;
      }
      else
      {
	otherUnits.push_back(*unit);
	unitCountForCriteria++;
      }
    }
    
    unitVector = otherUnits;
    //cerr << "unitvector " << unitVector.size() << "            units meeting crit " << unitsMeetingCriteria.size() << endl;
    return unitsMeetingCriteria;
  }


  
  /* Brain */
  void exec() 
  {
    //cerr <<"Running exec in state:" <<state <<endl;
  
 
    if (state == 0) {
      int sz = unitVector.size();
      sint4 x = 0;// = rand()%(32*16);
      sint4 y = 0;// = rand()%(32*16);
      sint4 dx = 0;
      sint4 dy = 0;
      
      if(sz == 8)
      {
	FORALL_DYN(unitVector, it)
	{
	
	  //dx = rand()%sz + 1;
	  dy = rand()%sz + 1;
	  //cerr << 16*16 + dx*8 - sz*4 << " " << 16*16 + dy*8 - sz*4 << endl;
	  dx = -16;
	  y = 16;
	  cerr << 16*16 + dx*16 << " " << 16*16  << endl;
	  TerrainBase::Loc goal_loc = TerrainBase::Loc(16*16 + dx*16, 16*16);
	  call_handlers( TerrainModule::PathEvent(commanderID, TerrainModule::FIND_PATH_MSG,
						  TerrainBase::Obj2LocTask(*it, goal_loc)) );
	  dx+=2;
	
	}
      }
      else
      {
	FORALL_DYN(unitVector, it)
	{
	  x = rand()%(32)+1;
	  y = rand()%(32)+1;
	  
	  dx = rand()%sz + 1;
	  dy = rand()%sz + 1;
	  cerr << x*16 + dx*4 - sz*2 << " " <<  y*16  + dy*4 - sz*2 << endl;


	  TerrainBase::Loc goal_loc = TerrainBase::Loc(x*16 + dx*4 - sz*2 , y*16  + dy*4 - sz*2);
	  call_handlers( TerrainModule::PathEvent(commanderID, TerrainModule::FIND_PATH_MSG,
						  TerrainBase::Obj2LocTask(*it, goal_loc)) );
	}
      }
      state=1;
    }
    
    //state 1 is limbo, waiting on units to arrive at destination
    
    if (state==2) {
      if(unitVector.size() > 1){

	firstVec = splitOnIndex((sint4)unitVector.size()/2);
	unitCountForCriteria = 0;
	secVec = splitOnIndex((sint4)unitVector.size());
	unitCountForCriteria = 0;

	cerr << "firstVec " << firstVec.size() << "   secVec " << secVec.size() << endl;
	remove_handler(tm);
	tm->remove_handler(this);

	splitAI* split1 = new splitAI(this, firstVec, myWatcher, maxUpdateCount, gsm, *tm);
	//unitCountForCriteria = 0;
	splitAI* split2 = new splitAI(this, secVec, myWatcher, maxUpdateCount, gsm, *tm);

	commVector.push_back(split1);
	commVector.push_back(split2); 
	
      }
      state=3;
    }
    //state 3 is limbo, waiting on sub-commanders to transfer back
    if (state==4) {
      //eventually merge...
      }
  }


  /**************************************************************************/
  
  bool handle_event(const Event& event) {
    
    if(event.get_what() == TerrainModule::PATH_SUCCESS) {
      pathSuccess++;
      //cerr << pathSuccess << " units arrived" << endl;
      if (pathSuccess == ((sint4)unitVector.size() - 1 )) {
      cerr <<"Path successfully completed" <<endl;
	state = 2;
      }
    }
    else if (event.get_what() == TerrainModule::PATH_FAILURE ) { 
      cerr <<"PATH FAILURE" <<endl;
    }
    else  if(event.get_what() == GameStateModule::VIEW_MSG) {
      insertUnits(gsm.get_changes().new_objs);
      this->update();
    }
    else return false;
    return true;
  }

    

  void insertUnits(const Vector<ScriptObj::WPtr>& objects)
  {
    cerr <<"Add units to Commander control" <<endl;
    const sint4 me = gsm.get_game().get_client_player();
    
    FORALL_DYN(objects, it) {
      GameObj* gob = (*it)->get_GameObj();
      if( !gob )
	continue;
      
      if( gob->bp_name() == "marine" && *(gob->sod.owner) == me ) {
	unitVector.push_back(gob);
      }
    }
  }

  ~splitAI() {
    tm->remove_handler(this);
    gsm.remove_handler(this);
    remove_handler(tm);
  }

  void giveUnitNotification(){}

};
