// $Id: mapeval_main.cpp 2487 2006-12-31 04:09:43Z orts_furtak $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "GfxGlobal.H"
#include "Options.H"
#include "GfxModule.H"
#include "ShutDown.H"
#include "SDLinit.H"
#include "GameStateModule.H"

// Frano includes
#include "HeuristicMapWidget.h"
#include "MapEvalHandler.h"
#include "PathProcessor.H"

using namespace std;

const char *ORTSG_VERSION = "GfxClient version 0.8 (2004-Oct-5)";

#define USE_EFENCE 0

extern int EF_PROTECT_FREE, EF_FILL, EF_PROTECT_BELOW, EF_ALLOW_MALLOC_0;

/*
struct ClientState {

  GameStateModule *gsm;
  GfxModule *gfxm;
  bool just_drew;

  ClientState(GameStateModule &gsm_, GfxModule &gfxm_) :
    gsm(&gsm_), gfxm(&gfxm_)
  {
    just_drew = false;
  }

};
*/

static bool just_drew = false;

struct GfxEventHandler : public EventHandler
{
  ClientState &state;

  GfxEventHandler(ClientState &state_) : state(state_) { }

  bool handle_event(const Event& e)
  {
    if (e.get_who() == GameStateModule::FROM) {

      cout << "Event: get_who: " << e.get_who() << " what: " << e.get_what() << endl;

      if (e.get_what() == GameStateModule::STOP_MSG) exit(0);

      if (e.get_what() == GameStateModule::READ_ERROR_MSG) exit(10);

      if (e.get_what() == GameStateModule::VIEW_MSG) {

#if 0
	if (state.gui) {
	  state.gui->event();
	  state.gui->display();
	  if (state.gui->quit) exit(0);
	}
#endif

        state.gfxm->process_changes();

        ////////////////
        Vector<string> temp;
	Game &game = state.gsm->get_game();
        FORS (i, game.get_player_num()) temp.push_back(""); // dummy list of commands
        game.exec_actions(temp); // evaluate client actions and send actions to the server
        ////////////////

        state.gsm->send_actions();

        sint4 vf = game.get_view_frame();
        sint4 ai = game.get_action_frame();
        sint4 sk = game.get_skipped_actions();
        sint4 md = game.get_merged_actions();

        if (vf != ai || sk || md > 1) {

          cout << "frame " << vf;
          if (ai < 0) cout << " [no action]";
          else if (vf != ai) cout << " [behind by " << (vf - ai) << " frame(s)]";
          if (sk) cout << " [skipped " << sk << "]";
          if (md > 1) cout << " [merged " << md << "]";
          cout << endl;
        }

        // don't draw if we are far behind
        if (abs(vf-ai) < 10){
          state.gfxm->draw();
          just_drew = true;
        }
      }

      return 0;
    }

    return 0;
  }
};


class ortsg_looper : public LoopFunctor{
public :
  ortsg_looper(ClientState* state_, Options* opt_):opt(opt_),state(state_){}

  void loop() const {

    if (state->gfxm->get_quit()) exit(0);

    bool refresh;
    opt->get("-refresh", refresh);

    if (refresh) {

      if (!just_drew) // allow some waiting time after a message
        state->gfxm->draw();
      else
        just_drew = false;
    }

    // server events
    if (!state->gsm->recv_view()) SDL_Delay(1);
  }

private:
  Options* opt;
  ClientState* state;
};

int main(int argc, char **argv)
{
  Options opt;

  glutInit(&argc, argv);
#if USE_EFENCE
  // aggressive efence checks
  EF_PROTECT_FREE = 1;
  EF_FILL = 0xee;
  EF_PROTECT_BELOW = 1;
  EF_ALLOW_MALLOC_0 = 1;
#endif

  signals_shut_down(true);

  // FRANO, add options

  // GameStateModule::Options::add(opt);
  //GfxModule::Options::add(opt);


  //widget options
  opt.put("-Moverlay",false,"Embed the evaluation map over the minimap");
  opt.put("-Mx",sint4(10),"x position of the evaluation map");
  opt.put("-My",sint4(250),"y position of the evaluation map");
  opt.put("-Mscale",real8(3.0),"amount we should scale the evaluation map");
  opt.put("-Malpha",real8(0.5),"amount of alpha transparency the evaluation map has");
  opt.put("-Mwidth",real8(40.0),"the desired width of each abstract square(in terms of tile points)");
  opt.put("-Mheight",real8(40.0),"the desired height of each abstract square(in terms of tile points)");

  opt.put("-Eimmobile",sint4(3),"the range of influence of immobile objects(in terms of tiles)");
  opt.put("-Erange",real8(1.5),"amount that the range of influence of objects is scaled by");
  opt.put("-Edecay",real8(0.5),"how much the influence decays per grid square");

  // process command line arguments
  if (opt.process(argc, argv, cerr, ORTSG_VERSION)) exit(20);

  // populate opt with options from various modules

  //path_add_options(opt);

  // process command line arguments
  //if (opt.process(argc, argv, cerr, ORTSG_VERSION)) exit(20);

  SDLinit::video_init();
  SDLinit::network_init();


  // populate option objects with command line values
  // and create modules
  GameStateModule::Options::add();
  //GameStateModule gsm(gsmo);

  GfxModule::Options::add();
  //GfxModule gfxm;

  GameStateModule::Options gsmo;
  GameStateModule gsm(gsmo);

  GfxModule::Options gfxmo;
  GfxModule gfxm;

  // gives event handler access to the state
  ClientState state(gsm, gfxm);

  // ortsg event handler
  GfxEventHandler gev(state);

  // frano stuff
  bool overlay;
  sint4 mapx,mapy;
  real8 scale,alpha,width,height;

  opt.get("-Moverlay",overlay);
  opt.get("-Mx",mapx);
  opt.get("-My",mapy);
  opt.get("-Mscale",scale);
  opt.get("-Malpha",alpha);
  opt.get("-Mwidth",width);
  opt.get("-Mheight",height);

  HeuristicMapWidget widget(Point2i(mapx,mapy),scale,width,height,alpha,overlay);

  // not sure how to add these from the command line...or if I even want to :P
  widget.addAttribute("hp",750.0);
  widget.addAttribute("max_speed",10.0);

  widget.addAttribute("armor.AC",10.0);
  widget.addAttribute("armor.slashing",10.0);
  widget.addAttribute("armor.piercing",10.0);
  widget.addAttribute("armor.explosive",10.0);
  widget.addAttribute("armor.blunt",10.0);

  widget.addAttribute("weapon.min_damage",10.0);
  widget.addAttribute("weapon.max_damage",10.0);
  widget.addAttribute("weapon.bonus",20.0);
  widget.addAttribute("weapon.max_air_range",200.0);
  widget.addAttribute("weapon.max_ground_range",200.0);

  //widget.initializeMap();
  sint4 immobileRange;
  real8 rangeMod, decay;

  opt.get("-Eimmobile",immobileRange);
  opt.get("-Erange",rangeMod);
  opt.get("-Edecay",decay);

  MapEvalHandler handler(state, &widget, immobileRange, rangeMod, decay);

  gsm.add_handler(&gev);

  gfxm.add_handler(&handler);
  gsm.add_handler(&handler);

  // end Frano stuff

  gsm.connect();
  gfxm.init(gsm.get_game(), gsm.get_changes(), gsm.get_action_changes(), gfxmo);

  ortsg_looper l(&state, &opt);
  gfxm.start_loop(&l);

  return 0;
}
