// $Id: Net_DB.C 5279 2007-06-23 02:49:08Z mburo $

// (c) Michael Buro 2003, mburo@cs.ualberta.ca, licensed under the GPL

#include "Net_DB.H"
using namespace std;

Net_DB::Net_DB(const string& file)
{
//  db_file = (char*)malloc((file.length()+1) * sizeof(char));  // fixme: is malloc really necessary?
//  strcpy(db_file, file.c_str());
  db_file = file;

  servicedb  = gdbm_open((char *)db_file.c_str(), 0, GDBM_WRCREAT, 0666, 0);

  if (!servicedb) { ERR2("File could not be opened / created : \n", file); }
  gdbm_close(servicedb);
}

Net_DB::~Net_DB() {

}

/**
   adds the given player to the data base
   overwrites it if it already exists
   @return -1 of operation fails
 */
sint4 Net_DB::put(DB_Type& p)
{

  servicedb  = gdbm_open((char *)db_file.c_str(), 0, GDBM_WRITER, 0666, 0);
  if(servicedb < 0){
    gdbm_close(servicedb);
    ERR("Cound not open DB file 1 ");
  }

  datum key, data;

  char keyString  [p.getKey().length()+1 ];
  char dataString [p.getData().length()+1];

  key.dptr = keyString;
  data.dptr = dataString;

  cout << "should add  : " << p.getKey() << " (" << p.getData() << ")" << endl;

  if (getData(p.getKey(), key) < 0) {
    gdbm_close(servicedb);
    return -1;
  }

  if (getData(p.getData(), data) < 0) {
    gdbm_close(servicedb);
    return -1;
  }
  printf("adding      : %s (%s)\n\n", key.dptr, data.dptr);

  // store
  if (gdbm_store (servicedb, key, data, GDBM_REPLACE)!= 0) {
    gdbm_close(servicedb);
    return -1;
  }

  printf("added       : %s (%s)\n\n", key.dptr, data.dptr);

  gdbm_close(servicedb);
  return 1;
}

/**
   gets the given player from the data base
   and puts the information in p
   @return -1 if error occures or player is not found
 */
sint4 Net_DB::get(DB_Type& p)
{
  servicedb  = gdbm_open((char *)db_file.c_str(), 0, GDBM_READER, 0666, 0);

  if(servicedb < 0){
    gdbm_close(servicedb);
    ERR("Cound not open DB file 2");
  }

  datum key, data;
  char keyString  [p.getKey().length()+1 ];
  char dataString [p.getData().length()+1];

  key.dptr = keyString;
  data.dptr = dataString;

  if(getData(p.getKey(), key) < 0){
    gdbm_close(servicedb);
    if(data.dptr){
      free(data.dptr);
    }
    return -1;
  }

  data = gdbm_fetch (servicedb, key);

  if (data.dptr == NULL){
    gdbm_close(servicedb);
    return -1;
  }

  string datas(data.dptr);
  free(data.dptr);

  p.setData(datas);

  gdbm_close(servicedb);
  return 1;
}

/**
   deletes the given player
   puts the deleted information in p
   @returns -1 of error occures or entry does not exist
 */
sint4 Net_DB::rem(DB_Type& p)
{
  servicedb  = gdbm_open((char *)db_file.c_str(), 0, GDBM_WRITER, 0666, 0);

  if(servicedb < 0){
    ERR("Cound not open DB file");
  }

  datum key;
  char keyString  [p.getKey().length()+1 ];
  key.dptr = keyString;


  if (getData(p.getKey(), key) < 0){
    gdbm_close(servicedb);
    return -1;
  }

  if (gdbm_delete (servicedb, key) != 0) {
    gdbm_close(servicedb);
    return -1;
  }

  gdbm_close(servicedb);
  return 1;
}



bool Net_DB::exists(DB_Type& p)
{
  servicedb = gdbm_open((char *)db_file.c_str(), 0, GDBM_READER, 0666, 0);

  if(servicedb < 0){
    gdbm_close(servicedb);
    ERR("Cound not open DB file");
  }

  datum key;
  char keyString  [p.getKey().length()+1 ];
  key.dptr = keyString;

  if(getData(p.getKey(), key) < 0){
    gdbm_close(servicedb);
    return false;
  }

  bool result = gdbm_exists(servicedb, key);

  gdbm_close(servicedb);
  return result;
}



sint4 Net_DB::getData(const string& datas, datum& data)
{
  data.dsize = datas.length() + 1;
//  data.dptr = (char*)malloc((data.dsize)*sizeof(char*));  // fixme: is malloc really necessary?
//  data.dptr = (char*)datas.c_str(); // causes trouble
//  if (!data.dptr) {
//    REM("OUT OF MEMORY");
//    return -1;
//  }
  (datas).copy(data.dptr,data.dsize-1, 0);
  data.dptr[data.dsize-1] = '\0';
  return 1;
}

void Net_DB::print()
{
  servicedb  = gdbm_open((char *)db_file.c_str(), 0, GDBM_READER, 0666, 0);

  if(servicedb < 0){
    ERR("Cound not open DB file");
  }

  datum key, data;

  key = gdbm_firstkey(servicedb);

  cout << "----------Player DB --------------" << endl;

  while (key.dptr != NULL) {
    data = gdbm_fetch (servicedb, key);

    if (data.dsize > 0) {
      printf ("\t %s (%s)\n", key.dptr, data.dptr);
      free (data.dptr);
    } else {
      printf ("Key %s not found.\n", key.dptr);
    }

    key = gdbm_nextkey(servicedb, key);
  }
  cout << "---------END Player DB -----------" << endl;

  gdbm_close(servicedb);
}

void Net_DB::close(){
  //   gdbm_close(servicedb); fixme
}
