#ifndef NET_DBTYPES_H
#define NET_DBTYPES_H

// $Id: Net_DBTypes.H 5279 2007-06-23 02:49:08Z mburo $

// (c) Michael Buro 2003, mburo@cs.ualberta.ca, licensed under the GPL

/**
 * \class DB_Player DB_Worker
 * \author Sami Wagiaalla
 * \brief
 */

#include "Global.H"
#include "Net_Protocol.H"

static const std::string SupersKey = "Supers"; /** the key used to store and retreve the list of roots form the D.B. */

class DB_Type
{
public:
  virtual std::string getData() const = 0;
  virtual std::string getKey() const = 0;
  virtual void setData(const std::string& data) = 0;
  bool good(){
    return loadGood;
  }
  virtual ~ DB_Type() {
  }
protected:
  bool loadGood;
};

class DB_Player : public DB_Type
{
public:
  /**
     the levels users can log in as
   */

  DB_Player(std::string name_, std::string pass_, Level level_);
  DB_Player(std::string name_);

  virtual ~ DB_Player();

  std::string getKey() const;
  std::string getData() const;

  /**
     parses the given std::string and sets the apropriet paramteters
   */
  void setData(const std::string& data);

  void addBuddy (const std::string& name);
  void remBuddy (const std::string& name);

  void addNotify(const std::string& name);
  void remNotify(const std::string& name);

  Vector<std::string>* getBuddies() {
    return &buddyList;
  }
  Vector<std::string>* getNotifys() {
    return &notifyList;
  }

  sint4 getLevel()  const {
    return level;
  }
  sint4 getRating() const {
    return rating;
  }
  std::string getName()  const {
    return name;
  }
  std::string getPass()  const {
    return pass;
  }

  void setLevel (Level level_)  {
    level  = level_;
  }
  void setRating(int rating_)   {
    rating = rating_;
  }
  void setName  (const std::string& name_)   {
    name   = name_;
  }
  void setPass  (const std::string& pass_)   {
    pass   = pass_;
  }

protected:
  std::string name;
  std::string pass;
  Level level;
  sint4 rating;
  Vector<std::string>  buddyList; /** this players buddies */
  Vector<std::string> notifyList; /** people that will be notified when this player comes online*/
};


class DB_Worker : public DB_Type
{

public:

  DB_Worker(const std::string& name_, const std::string& pass_);
  virtual ~ DB_Worker();
  std::string getKey() const;
  std::string getData() const;

  /**
     parses the given std::string and sets the apropriet paramteters
   */
  void setData(const std::string& data);

protected:
  std::string name;
  std::string pass;

  sint4 rating;
};


#endif
