// $Id: Net_Invitation.C 5693 2007-08-16 22:59:15Z krysta $

// (c) Michael Buro 2003, mburo@cs.ualberta.ca, licensed under the GPL

#include  "Net_Invitation.H"
using namespace std;

Net_Invitation::Net_Invitation(Invite invite)
{

  iid           = 0;
  owner         = invite.get_sender();
  name          = invite.get_game_name();
  maxPlayers    = invite.get_nplayers();

  invisible     = invite.get_invisible();
  anony_ids     = invite.get_anony_ids();
  alias         = invite.get_alias();
  passprotected = invite.get_passprotected();
  password      = invite.get_password();
  
  gameType      = invite.get_gameType();


  cout << "===========================================" << endl;
  cout << password << endl;
  cout << "===========================================" << endl;
  stringstream ss;

//   ss << "-nplayers "; // KM07 problematic, need the ACTUAL number of players
//   ss << maxPlayers;
  ss << " ";
  
  ss << "-game" << gameType << " "; 
  ss << "-bp tournament-2007/game" << gameType << ".bp" << " ";

  options     = ss.str();
  options     += invite.get_options();

  playerCount = 0;
  readyCount  = 0;

  started = false;

  updateInfo();
}

Net_Invitation::Net_Invitation(const Net_Invitation& n)
{

  owner       = n.owner;
  name        = n.name;
  maxPlayers  = n.maxPlayers;
  invisible   = n.invisible;
  anony_ids   = n.anony_ids;
  password    = n.password;
  gameType    = n.gameType;

  options     = n.options;

  playerCount = n.playerCount;
  readyCount  = n.readyCount;

  info = n.info;

  players = n.players;
  present = n.present;
  ready   = n.ready;

  started = false;
}

Net_Invitation::~Net_Invitation() {
}

void Net_Invitation::setIID(uint4 iid_) {

  iid = iid_;
  updateInfo();

}


void Net_Invitation::updateInfo()
{

  info = Invit_Disc::compose(iid,
                             playerCount,
                             owner,
                             name,
                             maxPlayers,
                             invisible,
                             anony_ids,
                             passprotected,
                             gameType,
                             options);

  cout << "Net_Invitation -> unpdateinfo: invit_disc: " << info << endl;
}


sint4 Net_Invitation::addPlayer(const string &name_, const string &alias_)
{
	// doing this when readying now, to allow any number of observers
  //if (playerCount == maxPlayers) return -1;
  
  cout << "Adding player, name=" << name_ << ", alias=" << alias_ << endl; 
  
  if (getPlayer(name_) >= 0) return -1;
  
  // was he/she previously in this list, if so, present = false but reuse same pid
  int pid = getPlayerIndex(name_);
  cout << "Adding player, pid=" << pid << endl;    
  if (pid >= 0 && present[pid] == false)
  {
      cout << "Detected player in list, pid = " << pid; 
      
      aliases[pid] = alias_;
      players[pid] = name_;
      ready[pid] = false;
      present[pid] = true;
      playerCount++;
      updateInfo();
      return pid;    
  }

  /** find an empty spot, but don't take over the dictator's spot unless you are him/her! */
  for(uint4 i = 1; i < players.size(); i++){
    if(!present[i]){
      aliases[i] = alias_;
      players[i] = name_;
      ready[i] = false;
      present[i] = true;
      playerCount++;
      updateInfo();
      return i;
    }
  }

  aliases.push_back(alias_);
  players.push_back(name_);
  ready.push_back(false);
  present.push_back(true);
  playerCount++;
  updateInfo();
  return (players.size()-1);
}

sint4 Net_Invitation::removePlayer(const string &name_)
{

  int index = getPlayer(name_);

  if(index >= 0){
    present[index] = false;
    ready[index]   = false;
    playerCount--;
    readyCount--;
    leaveTeam(name_);
    updateInfo();
  }


  return index;
}

sint4 Net_Invitation::removePlayer(sint4 pid)
{
  if (pid >= (sint4)maxPlayers || pid < 0) return playerCount;

  present[pid] = false;
  ready[pid]   = false;
  playerCount--;
  readyCount--;
  leaveTeam(players[pid]);
  updateInfo();
  return pid;
}

sint4 Net_Invitation::getPlayerIndex(const string& name_)
{
  for (uint4 i = 0; i < players.size(); i++) {
    if(players[i] == name_) return i;
  }
  return -1;
}

sint4 Net_Invitation::getPlayer(const string& name_)
{
  for (uint4 i = 0; i < players.size(); i++) {
    if(present[i] && players[i] == name_) return i;
  }
  return -1;
}

sint4 Net_Invitation::banPlayer(sint4 pid)
{
  if (pid >= (sint4)maxPlayers || pid < 0) return playerCount;
  REM("adding " + players[pid] + " to banned players list");
  bannedPlayers.push_back(players[pid]);
  return pid;
}

bool Net_Invitation::isPlayerBanned(const string& name)
{
  for (uint4 i = 0; i < bannedPlayers.size(); i++) {
    if(bannedPlayers[i] == name)
      return true;
  }
  return false;
}

int Net_Invitation::setReady(const string& name, bool ready_)
{
  int i = getPlayer(name);

  if (i < 0) return i;

  if(ready[i] == ready_) return i;

  if(ready_) { readyCount++; } else { readyCount--; }

  ready[i] = ready_;

  return i;
}


void Net_Invitation::print()
{
  cout << endl << "\t-----Game-----" << endl;
  cout << info << endl;
  for(uint4 i = 0; i < players.size(); i++){
    if(!present[i]){
      continue;
    }
    cout << "\t   ["<< i <<"] " << players[i];
    if(ready[i]){
      cout << " R";
    } else{
      cout << " U";
    }

    if(present[i]){
      cout << " P" << endl;
    } else{
      cout << " N" << endl;
    }
  }
}


bool Net_Invitation::joinTeam(string name, int team)
{
  if (team > 0)
  {
    FORALL (teamAssignments, iter)
    {
      const string& aname = iter->first; 
      int ateam = iter->second;
      
      if (ateam != 0 && ateam != team && name == aname) 
      {
        cout << "Error: Another player("<< name <<") is assigned to team " << team << endl;
        return false;
      }  
    }

    cout << "Adding " << name << " to team " << team << endl;
    teamAssignments[name] = team;
  }
  else
  {
    cout << "Removing " << name << " from team " << teamAssignments[name] << endl;
    teamAssignments.erase(name);
  }
  
  return true;
} 

bool Net_Invitation::leaveTeam(string name)
{
  joinTeam(name, 0);
  return true;
}

sint4 Net_Invitation::getTeamCount()
{
  map<int, string> teams;
  
  FORALL (teamAssignments, iter)
  {
    const string& aname = iter->first; 
    int ateam = iter->second;
    
    teams[ateam] = aname; 
  }
  
  return teams.size(); 
}

sint4 Net_Invitation::getPlayerCount(int teamNum)
{
  int count = 0;
  
  FORALL (teamAssignments, iter)
  {
    int ateam = iter->second;
    
    if (ateam == teamNum)
      count++;
  }
  
  return count; 
}

string Net_Invitation::getidstring() {
  stringstream idstr;
  idstr << "-ids ";

  int numTeamsLeft = getTeamCount();
  for (int t=0; numTeamsLeft > 0; t++) {

    if (getPlayerCount(t+1))
      --numTeamsLeft;

    FORALL(teamAssignments, iter) {

      const string& aname = iter->first;
      int ateam = iter->second;

      if (ateam == (t+1)) {
        cout << "adding "<< aname << " for team "<< to_string(t+1) << endl;

        idstr << aname << ",";
      }

    }

  }

  string s = idstr.str();   
  s = s.substr(0, s.length()-1); 
  
  return s;
}


bool Net_Invitation::readyToStart()
{
  // for game types > 1
  
  // at least 2 teams with at least 1 player ready in each 
  
  sint4 teamCount = getTeamCount();
  if (teamCount < 2) return false;
  
  cout << "Team count = " << teamCount << endl; 
  
  map<int, string> teamsReady;
   
  FORALL (teamAssignments, iter)
  {
    const string& aname = iter->first; 
    int ateam = iter->second;
    
    sint4 pid = getPlayer(aname);
    
    if (ready[pid])
    {
      cout << "Team " << ateam << " ready!" << endl;
      teamsReady[ateam] = aname;
    }  
  }
  
  cout << "TeamsReady.size() = " << (sint4)teamsReady.size() << endl;
  if ((sint4)teamsReady.size() >= teamCount) 
    return true;
   
  return false;
}

void Net_Invitation::setStarted(bool _started)
{
  started = _started;
} 



