// $Id: Net_Invitation.H 5693 2007-08-16 22:59:15Z krysta $

// (c) Michael Buro 2003, mburo@cs.ualberta.ca, licensed under the GPL

/**
 * An object that holds an invitation
 */

#ifndef Net_Invitation_H
#define Net_Invitation_H

#include "Global.H"
#include "Net_Protocol.H"

class Net_Invitation
{

public:

  /**
   * \brief does nothing
   * for the default call by derived classes
   */
  Net_Invitation(){
  }
  Net_Invitation(Invite invite);
  Net_Invitation(const Net_Invitation& n);
  virtual ~ Net_Invitation();

  /** returns the info if the current game */
  const std::string& getInfo() const {
    return info;
  }

  /** returns the iid of this invitation */
  sint4 getIID() const {
    return iid;
  }
  sint4 getGameType() const {
    return gameType;
  }
  std::string getOwner() const {
    return owner;
  }
  std::string getPass() const {
    return password;
  }
  bool getAnony() const {
    return anony_ids;
  }
  bool getInvisible() const {
    return invisible;
  }
  bool getStarted() const {
    return started;
  }  

  const Vector<std::string>& getAliases() const {
    return aliases;
  }
  const Vector<std::string>& getPlayers() const {
    return players;
  }
  const Vector<uint1>& getPresent() const {
    return present;
  }
  const Vector<uint1>& getReady() const {
    return ready;
  }
  const Vector<std::string>& getBanned() const {
    return bannedPlayers;
  }
  std::map<std::string, uint4>& getTeamAssignments  () {
    return teamAssignments;
  }

  void setIID(uint4 iid_);

  /**
     sets ready to ready_ (true = ready, false = unready)
     @return the iid of the player
   */
  int setReady(const std::string& name, bool ready_);
  
  void setStarted(bool _started); 

  sint4 addPlayer(const std::string& name, const std::string& alias);

  /**
     removes a player
     @return player alias
   */
  sint4 removePlayer(const std::string& name);

  /**
     removes a player
     @return player alias
   */
  sint4 removePlayer(sint4 pid);
  
  /**
   * removes a player and prevents them from
   * rejoining 
   * @return player alias
   */
  sint4 banPlayer(sint4 pid);

  /** 
   * @return true iff this player is banned from 
   * this invitation
   */ 
  bool isPlayerBanned(const std::string& name);
  
  uint4 getPlayerCount() const {
    return playerCount;
  }

  bool allReady() const {
    return readyCount == maxPlayers;
  }
  
  sint4 getReadyCount() const {
    return readyCount;
  }
  
  //const std::string& getStd::String(){ return inviteStd::String; }

  void print();
  sint4 getTeamCount();  
  sint4 getPlayerCount(int teamNum); 
  
  std::string getidstring();  
  
  bool joinTeam(std::string name, int team); 
  bool leaveTeam(std::string name); 
  bool readyToStart(); 

  std::string getOptions() { //KM07 took out const
    
    return "-nplayers " + to_string(getPlayerCount()) + options; //KM07 do update of options each time you retrieve them
  }

protected:
  /** updates the info stirng of the current game */
  void updateInfo();

  std::string owner;
  std::string name;
  bool invisible;
  bool anony_ids;
  bool passprotected;
  bool started;
  std::string alias;
  std::string password;
  std::string options;

  std::string info;

  uint4 readyCount;
  uint4 playerCount;
  uint4 maxPlayers;
  sint4 gameType;

  uint4 iid;
  Vector<std::string> aliases;
  Vector<std::string> players;
  Vector<std::string> bannedPlayers; //KM07 - whether a player is banned or not
  Vector<uint1> ready;
  Vector<uint1> present; /** to keep whether a player is present or not... to avoid using NULL pointers */
  std::map<std::string, uint4> teamAssignments;      

  /**
     searches linearly for the player
     returns players index
   */
  sint4 getPlayer(const std::string& name);
  sint4 getPlayerIndex(const std::string& name);
};

#endif

