// $Id: Net_Worker.C 5745 2007-08-23 17:59:09Z krysta $

// This is an ORTS file (c) Tu Hoang, licensed under the GPL

/*
 * ====================================================================================
 *
 *        Filename:  Net_Worker.C
 *
 *     Description:  main files
 *
 *         Version:  1.0
 *         Created:  05/25/04 15:39:07 MDT
 *        Revision:  none
 *        Compiler:  gcc
 *
 *          Author:  Tu Hoang
 *
 * ====================================================================================
 */

#include "Net_Worker.H"
#include "Options.H"

using namespace std;

extern void create_orts_world(stringstream &w);

const sint4 Net_Worker::FROM        = EventFactory::new_who();
const sint4 Net_Worker::READY_MSG   = EventFactory::new_what();    // sent when game loop starts
const sint4 Net_Worker::STEP_MSG    = EventFactory::new_what();     // sent after views are sent

Net_Worker::Net_Worker(const string &world_)
{
  signals_shut_down(false);
  world = world_;
}


Net_Worker::~Net_Worker()
{
  if (client) delete client;
}

/**
 * Makes accessible both the standard input and output of the shell 
 * process it creates. Two pipes are created: one provides standard
 * input for the shell commmand, and the other for passing back its 
 * standard output. The pipe file descriptors for the caller to use 
 * are pointed to by p_fd_in and p_fd_out.
 * 
 * @param  shell_cmd  the shell command to be executed  
 * @param  p_fd_in    the input pipe file descriptor for the caller to 
 *                    use (value of 0 indicates the shell command doesn't
 *                    use stdin)
 * @param  p_fd_out   the output pipe file descriptor for the caller to 
 *                    use (value of 0 indicates the shell command doesn't
 *                    use stdout) 
 * @return            the parent process id
 */      
pid_t Net_Worker::popen2(const string &shell_cmd, int *p_fd_in, int *p_fd_out)
{
  cout << "executing a shell command " << shell_cmd << " in popen2!\n";
  // CREATING TWO PIPES:
  int fds_processInput[2];  //pipe for process input
  int fds_processOutput[2]; //pipe for process output
  
  if (pipe(fds_processInput) != 0) {
    // create process input pipe
    ERR("pipe (process input) failed");
  }
  
  if (pipe(fds_processOutput) != 0) {
    //create process output pipe
    ERR("pipe (process output) failed");
  }
  // FORKING A CHILD PROCESS:
   pid_t pid;
   if ((pid = fork()) < 0) {
     ERR("fork failed");
   }
   
   // CONNECT THE CORRECT PIPE ENDS IN THE CHILD:

   if (pid == 0) {

     // child process

     // for process input pipe:
     close(fds_processInput[1]);    //close output
     dup2(fds_processInput[0], 0);  //close fd 0, fd 0 = fds_processInput[0]
     
     // for process output pipe:
     close(fds_processOutput[0]);   //close input
     dup2(fds_processOutput[1], 1); //close fd 1, fd 1 = fds_processOutput[1]
     
     //execl("/bin/sh", "sh", "-c", shell_cmd.c_str(), NULL ); 
     execl("/usr/sunkay5/cshome/krysta/svn_projects/orts/trunk/bin/orts", "bin/orts", 
         "-nplayers", "1", "-game1", "-bp", "tournament-2007/game1.bp", "-debug", "0", "-port", "7777", "-host", "129.128.23.26", NULL);
     
     ERR("failed to run shell_cmd");

   } else {

     // parent process

     // for process input pipe:
     close(fds_processInput[0]);    //close input
       
     // for process output pipe:
     close(fds_processOutput[1]);   //close output

     if (p_fd_in == 0)
       close(fds_processInput[1]);
     else
       *p_fd_in = fds_processInput[1];
       
     if (p_fd_out == 0)
       close(fds_processOutput[0]);
     else
       *p_fd_out = fds_processOutput[0];

   }
   return pid; 
}

/**
 * Reads the output of command in 's' and stores it in s
 *
 * @param s      shell command, will also hold the resulting output
 * @param client ggs client
 * @return       true <=> the results are valid
 */
bool Net_Worker::system_output(string &s, Net_GGSClient *client, string servicename)
{
  
  const string cmd = s;
  bool isRes = false;
  string temp = "";
   
  cout << "in system_output\n";
  
  FILE *out = popen(cmd.c_str(), "r");
  s.clear();
  s = Game_End::INVALID_RES;
  char buffer[1024];
  
  while (1) {
  
    //if ((r = fgets(buffer, sizeof(buffer), out)) <= 0) {
    if (fgets(buffer, sizeof(buffer), out) == NULL) {
      cout << "No more input from orts process...done" << endl;

      break; // no data
    }
    printf("%s\n", buffer);

    temp.assign(buffer);

    if(isRes) {
      s = temp;
      cout << "result string is: " + s << endl;
      isRes = false;
    }
    else if(temp.find("Waiting for", 0) != string::npos) {
      cout << "FOUND THE WAITING FOR STRING ..." << endl;
      REM("------ Notify Service: worker ready");
      client->send("tell " + servicename + " " + Worker_Ready::compose());

    }

    if(buffer[0] == '@') {
      isRes = true;
      cout << "@ raw result string is: " << buffer << endl;

    }
  }
  
  pclose(out);
  
  string::size_type pos = s.find('\n', 0);
  if(pos != string::npos) {
    s = s.substr(0, pos);
  }

  return (s != Game_End::INVALID_RES);
}


bool Net_Worker::handle_event(const Event& e)
{ 
  if (!client) {
    cerr << "ERROR: {Net_Worker::handle_event}: no GGS client make sure run Net_Worker::run first.\n";
    exit(-1);
  }
  
  // wait for the server to be ready to accept incoming connections

  if (e.get_who() == Net_Worker::FROM && e.get_what() == Net_Worker::READY_MSG ){
    REM("------ Notify Service: worker ready");
    client->send("tell " + servicename + " " + Worker_Ready::compose());
    return 1;
  }

  return 0;
}


sint4 Net_Worker::run()
{
  // message parsers
  Finger finger;

  string server_ip;
  sint4 sport;

  Options::get("-username", user_name);
  Options::get("-password", password);
  Options::get("-server", server_ip);
  Options::get("-sport", sport);
  Options::get("-servicename", servicename);

  client = new Net_GGSClient(user_name, password, server_ip, sport, true);
  string message;
  Net_Message net_msg;

  // get port
  sint4 port = 0;
  Options::get("-port", port);

  // get IP
  string hostIP;

  client->send(Finger::compose(user_name));
  bool ipOk = (waitForResponse(finger) && finger.get("host") != "NULL");

  if (!ipOk) { ERR("unable to retrieve local host ip"); }

  hostIP = finger.get("host");

  /***************************************************/
  // GGS inserts 0's to make all three sections of ip addresses
  // three chars long... i donno why
  // Ex: 123.124.22.65 -> 123.124.022.065
  //                              ^   ^
  // this cannot be used as an address to
  // connect to so it must be cleaned up

  stringstream cleanIP;

  FORU (i, hostIP.length()){
    if (hostIP[i] == '.') hostIP[i] = ' ';
  }

  stringstream ss(hostIP);
  int tint;
  ss >> tint;

  FOREVER {
    cleanIP << tint;
    if(!(ss >> tint)){ break; } // skip the last dot
    cleanIP << ".";
  }

  if (ss.bad()){
    ERR("could not clean up IP");
  } else {
    hostIP = cleanIP.str();
  }

  /****************************************************/

  // login into the service
  string send_msg = "tell " + servicename + " " + Login::compose(WORKER, password, hostIP, port) ;
  client->send (send_msg);

  Login_Notify notify;
  bool loginOK = waitForResponse(notify) && notify.get_player_name() == user_name;

  if (!loginOK) {
    ERR("Unable to login to the Service");
  } else {
    REM("login successful");
  }

  Worker_Start start;

  while (!shut_down) {
  
    REM("Waiting for START command");
    waitForResponse(start);
    if (start.get_sender() != servicename) continue;
    REM("START command received");

    if (Options::process(start.get_options(), cerr, "version ")) {
      REM2("\nInvalid options : ", start.get_options());
      continue;
    }

    REM2("Options : ", start.get_options());

    if (Options::process(start.get_options(), cerr, "started as ortsnet worker"))
      exit(20);


    string res = "bin/orts " + start.get_options();
    bool valid = Net_Worker::system_output(res, client, servicename);
    
    cout << "result string is: " + res << endl;
    
    //send out game results to service
    string msg = "tell " + servicename + " " + Game_End::compose(start.getIID(), valid, res, hostIP, port);
    client->send(msg);
        

//end popen
#if 0
    stringstream world;
    create_orts_world(world);
    
    Server server;

    server.add_handler(this); // server calls handle_event each frame

    // todo: call server.add_login_info(...) with client login information

    /*
     * Redefining options causes the worker to abort... and some options are set before hand
    MapTool::Options::add();
    GameStateModule::Options::add();
    Server::Options::add();
    GUI::add_options();
    Game::Options::add();
    */
            
    Server::Options so;         
    server.run(world.str(), so);

    // todo: send out game result to players and service

    
    
    //KM07
    Game *g = &server.get_game();
    bool flag = (g)->get_finished();
    string res;
    if( flag )   res = "THIS GAME IS FINISHED!!!!!!!!!!!!";
    else      res = "THIS GAME IS NOT NOT NOT FINISHED!!!!!!!!!";
    string msg;

    //send out game results to service
    msg = "tell " + servicename + " " + Game_End::compose(start.getIID(), flag, g->get_results());
    client->send(msg);
    
    // tell the service this worker is available to start playing a new game
    
    msg = "tell " + servicename + " " + Worker_Available::compose(hostIP, to_string(port));
    client->send(msg);
#endif //but still need to send messages...
    // ~Server() disconnects clients and deletes game
  }

  return 0;
}


bool Net_Worker::waitForResponse(Message_Base& expected)
{
  string message;
  Net_Message m;

  while (!shut_down) {
    client->recv(message);
    cout << message << endl;
    m.setMessage(message);
    if (expected.parse(m)) return true;
  }

  return false;
}
