#ifndef NUS_GROUP_H
#define NUS_GROUP_H

// $Id: NUS_Group.h,v 1.1 2006/08/10 17:16:01 nicholas Exp $

#include <algorithm>
#include "Global.H"
#include "Vector.H"

#include "NUS_Unit.h"

#include "NUS_MapObject.h"

#include "NUS_SimulationState.h"

namespace NUS_Simul{

  const real8 MIN_GROUP_SIZE=0.5;

  class Order;

  typedef Vector<GameObj*> ObjectVector;
  typedef Vector<Unit*> UnitVector;

  class Group : public MapObject{
  public:
    Group(uint4 playerID);
    Group(uint4 playerID, UnitVector &);
	Group(uint4 playerID, Unit* unit);
    ~Group();
    Group(const Group& b);
    Group* clone() const;

    bool withinRange(Group* otherGroup);

    //void recalculateRadius() { this->radius = sqrt((real8)getNumUnits()/M_PI) + MIN_GROUP_SIZE;}
	void recalculateRadius();
	
    /*
    void draw();
    void drawInSelectionMode();
  */

    real8 getExtents() const {return radius;}

    uint4 getNumUnits(){return units.size();}

    void setOrder(Order* newOrder);

    Order* getOrder() const {return currentOrder;}
    void executeOrders(real8 timeElapsed);
    void executeOrderAction(SimulationState& state);
    void cleanupOrder();

    real8 getMaxSpeed() const {return maxSpeed;}
    real8 getMaxAttackRange() const {return maxAttackRange;}
    real8 getLastDistanceMoved() const { return lastDistanceMoved;}

	void setUnits(UnitVector &u);
	ObjectVector getUnitsObject();	//return vector of GameObj* of individual units in this group
	UnitVector& getUnits() { return units; }
	
	bool unitInGroup(Unit *u) {
		Vector<Unit*>::iterator it = find(units.begin(), units.end(), u);
		
		return (it!=units.end());
	}
	
	bool gameObjInGroup(GameObj *obj) {
		Vector<Unit*>::const_iterator end = units.end();
		
		for (Vector<Unit*>::iterator it = units.begin(); it != units.end(); it++)
			if ((*it)->getObject() == obj) return true;
		
		return false;
	}
	
	real8 distance_to(GameObj *obj) const {
		return sqrt(square(real8(position.x-obj->get_int("x"))) + square(real8(position.y-obj->get_int("y"))));		
	}
	
	real8 distance_to(Group *g) const {
		return sqrt(square(real8(position.x-g->getPosition().x)) + square(real8(position.y-g->getPosition().y)));		
	}
	
	real8 distance_to(Unit *unit) const {
		return sqrt(square(real8(unit->getPosition().x)) + square(real8(unit->getPosition().y)));		
	}
	
	// NOTE: This should be called from Player::merge(Group *, Group *)
	// add the units of other group to this group
	void merge(Group *other);
		
	bool hurry;
	
  private:
    Group& operator=(const Group& b);// {return *b.clone();}; // no = operator for now

    Order* currentOrder;
    real8 maxSpeed;
    real8 maxAttackRange;
    real8 lastDistanceMoved;

    real8 radius;

    UnitVector units;
  };
}

#endif
