#ifndef NUS_SIMULATION_AI_H
#define NUS_SIMULATION_AI_H


#include "Global.H"

#include "GameObj.H"
#include "Map.H"
#include "GameTile.H"
#include "LowAI.H"
#include "EventHandler.H"
#include "ClientModule.H"
#include "NUS_TerrainBase.H"

#include "NUS_SimpleTerrain.H"

#include "NUS_ST_ForceField.H"

#include "NUS_Vector3D.h"
#include "NUS_Strategies.h"
#include "NUS_Order.h"

//#include "NUS_Group.h"
//#include "NUS_Player.h"

//class NUS_SimpleTerrain::NUS_ST_ForceField;


class GameStateModule;

namespace NUS_Simul{
//===================================================================

	typedef NUS_SimpleTerrain::NUS_ST_ForceField Terrain;   //pathfinding module

	typedef NUS_TerrainBase::Loc Loc;
	typedef NUS_TerrainBase::Path Path;

	class Task;
	class SimulationState;
	class Player;
	
	struct Tile {
    enum Tile_Status {FREE, FOG, OBSTACLE, UNIT} status; ///<Indicates the status of the tile.
    sint4 freeValue;                                     ///<Indicates the distance to the nearest obstacle.
    sint4 x,y;                                           ///<The x,y coordinates of this tile, in tile units!

    /** A simple constructor that will zero this tile.
     */
    Tile(){ freeValue = 0; status = FOG; x = 0; y = 0; }; 
    ~Tile(){};

  	};

	class ExtraObjectData {
	public:

		ExtraObjectData(){
			targetBase = NULL;
			targetUnit = NULL;
			location = NUS_TerrainBase::Loc(-1,-1);
			state = NONE;
			isDone = false;
			hasReplanned = false;
			executed = false;
			isExecuting = false;
		}

		Base *targetBase;
		Unit *targetUnit;
		NUS_TerrainBase::Loc location;
		enum State {NONE,MOVE,DEFEND,ATTACK};
		State state;
		bool isDone;
		bool hasReplanned;
		bool executed;
		bool isExecuting;
	};

//-------------------------------------------------------------------

	struct SavedData;
	typedef struct SavedData SimulationResult;
	
/**
	* This AI module causes all units to attack the nearest (geometric distance)
		* base.
*/
	class SimulationAI : public LowAI<ExtraObjectData>, public EventHandler, public ClientModule{
	protected:
		TaskCreateFunctionPtr bestStrategy;
		real8 lastSearchTime;

//std::set<GameObj*> bases; ///< Enemy Bases

		GameStateModule &gsm;

		Terrain &timp;    //pathfinding module

		SimulationState* simulationState;

		SimulationResult *simulationResult;
		
		sint4 height, width;           ///<The dimensions of the game map.
	    sint4 tile_points;             ///<The conversion factor between tiles and coordinates
		Tile **map;
	    const Map<GameTile> *game_map; ///<A pointer to the game map
		Vector<GameObj*> sheep;

//Vector<GameObj*> need_act; ///< GameObjects needing an action in the next tick
/**
		* Removes the given GameObj from the control of this class.
*/
//void stop(GameObj *obj);


/**
			* Finds the nearest base to travel to, based on geometric distance.
*/
//GameObj* find_target(GameObj *targeter, GameObj *baseToAvoid = 0);
		public:

			static const sint4 FROM;
			static void add_options(void);
			SimulationAI(GameStateModule &gsm, Terrain &timp, sint4 seed);
			~SimulationAI();

/**
			* Causes the units in the need_act vector to act according to
				* their current state.
*/
				void act(GameStateModule &gsm);

//static const std::string bp_name; ///< ?

			void add(GameObj*) { ERR("?"); }

/**
			* Addes an enemy base to the list of enemy bases, and updates targets
*/
//void add_base(GameObj *gob);

				void del(GameObj*) { ERR("?"); }

/**
			* Deletes a base from the list of enemy bases, and updates targets
*/
//void del_base(GameObj *gob);

/**
				* Handles events sent to SuicideAttackAI from other modules
*/
			bool handle_event(const Event &e);

			bool informObjectsOfDeath(GameObj* obj);
			bool isControllable(GameObj* obj);

			sint4 doPath(NUS_TerrainBase::Task& task);
			sint4 stopPath(NUS_TerrainBase::Task& task);
			sint4 doCombat(Vector<GameObj::WPtr> units, GameObj::WPtr target);
			sint4 stopCombat(Vector<GameObj::WPtr> units);

			real8 planPath(const NUS_TerrainBase::Loc startPt, const NUS_TerrainBase::Loc endPt, sint4 radius, WayPointVector& wayPoints); 
			real8 planPath(const NUS_TerrainBase::Loc startPt, const GameObj *obj, sint4 radius, WayPointVector& wayPoints); 
			
			//sint4 doPath(GameObj* currObj, Vector3D dest, sint4 radius);
			sint4 doPath(Player* currPlayer, Vector<Unit*> &unitVector, Vector3D dest, sint4 radius, bool IGNORE_ALL = true);
			sint4 stopPath(Player* currPlayer, Vector<Unit*> &unitVector);

			void replan(UnitVector &unitVector);
			void executePath(Group* currGroup);
			void executeAttack(Group* currGroup, int phase = 0);
			bool updatePath(Group* group);
			
			void initObstacles();
			void addBuilding(GameObj *obj); 
			void removeBuilding(GameObj *obj);
			void rotate(real8 &nx, real8 &ny, NUS_TerrainBase::Loc pos, real8 vecX, real8 vecY, real8 angle);
			void steerAway(Unit *unit, sint4 vecX, sint4 vecY);
			int vectorCollisions(NUS_TerrainBase::Loc pos, sint4 radius, real8 vecX, real8 vecY);
			int unitCollisions(NUS_TerrainBase::Loc pos, sint4 speed, sint4 radius, real8 vecX, real8 vecY);
			void pushFriendly(Unit *unit, real8 vecX, real8 vecY);
			void pushFriendly(Unit *unit, NUS_TerrainBase::Loc pos, sint4 radius, real8 vecX, real8 vecY);
			bool besideObstacle(NUS_TerrainBase::Loc pos, sint4 radius);
			
			bool firstInit;
			
			Player* getMe();
			GameStateModule& get_gsm(){ return gsm;}
			Random rand; // local RNG
	public:
		//travoattack functions
		float dist2dCompare(NUS_TerrainBase::Loc a, NUS_TerrainBase::Loc b);
		float dist2dCompare(float a,float b,float x,float y);
		
		Unit* GetNearestEnemy(NUS_TerrainBase::Loc pos);
		float dist2d(NUS_TerrainBase::Loc a, NUS_TerrainBase::Loc b);
		NUS_TerrainBase::Loc ToTerrainBase(Vector3D vec3);
		Vector3D ToVector3D(NUS_TerrainBase::Loc pos);
		void MoveTo(Unit* unit, Vector3D destination, float recommendedspeed);
		float angfrom (float x1, float y1, float x2, float y2);
		bool OffMapLimits(Vector3D destination);   
		bool TestPtClear(float x, float y, Unit* ourunit);
		Vector3D MTFA(float x, float y, float heading, float speed, Unit* ourunit);
		void Push(Group* unit, Vector3D newpos);
		int turnangle (float curDirection, float desDirection, float turnspeed);
		float angdist (float curDirection, float desDirection);
		
		bool TestTerrainClear(float x, float y);
		bool MTTA (float x, float y, float heading);
		};

	}

#endif

