// $Id: NUS_Strategies.cpp,v 1.1 2006/08/10 17:16:02 nicholas Exp $


#include "NUS_Strategies.h"

#include "NUS_SimulationAI.H"
#include "NUS_SimulationState.h"

#include "GameStateModule.H"
#include "Game.H"
#include "GameChanges.H"
#include "GameObj.H"
//#include "GfxModule.H"
#include "NUS_TerrainModule.H"
#include "TerrainClassifier.H"
#include "NUS_Order.h"
#include "Options.H"

static const bool debug = false;

namespace NUS_Simul {

#define DEFINE_TASK(name) \
	name::name(uint4 playerID, \
		SimulationState* state, \
		GroupVector& groupVector, \
		TaskCreateFunctionPtr f, \
		std::string n, \
		real8 timeout, \
		MoveStruct* extraData): \
	Task(playerID,state,groupVector,f,n,timeout,extraData){ \
		exec();	\
		} \
		name::name(Task* parent, \
		GroupVector& groupVector, \
		TaskCreateFunctionPtr function, \
		std::string n, \
		real8 timeout, \
		MoveStruct* extraData) : \
	Task(parent,groupVector,function,n,timeout,extraData){ \
		exec(); \
		} \
		name::name(const name& t) : Task(t){} \
	Task* name::clone(SimulationState* state, Task* newParent){ \
		name* newTask = new name(*this); \
		newTask->parent = newParent; \
		newTask->state = state; \
	FORALL(children,it){ \
		newTask->children.push_back((*it)->clone(state,newTask)); \
		} \
		return newTask;	\
		} \
		bool name::exec()

// ----------------
// Basic Tasks

	DEFINE_TASK(SpreadAttack) {
		UnitVector units;	//all units in all newGroups
		GroupVector temp;
		
		if (debug) std::cout << "SpreadAttack: Start by player " << playerID << std::endl;
		
		switch(lineNum) {
			case 0:

			temp = groups;
			FORALL(temp, it) {
				(*it)->hurry = true;
				
				Base* nearestBase = state->getNearestEnemyBaseOfGroup(playerID, (*it)->getUnits());
				if(nearestBase != NULL) {
					GroupVector gv;
					gv.push_back(*it);
					new Attack(this, gv, function, "Attack", 0.0, new MoveStruct(nearestBase));
				} else return true; //ERR("No base to attack?");
			}
			default:
			return false;
		}
		return false;
	}

	DEFINE_TASK(Join) {
		assert(extraData == NULL); // extradata should be NULL for this task
		UnitVector units;

		if (debug) std::cout << "Join: Start" << std::endl;

		switch(lineNum) {
			case 0:{
				if (debug) std::cout << "Join: creating newGroups consisting of singleton units" << std::endl;
				
				// algo: get average position, then get nearest group to average point
				
				Vector3D average(0.0, 0.0, 0.0);
				int totalNumUnits = 0;
				
				FORALL(groups, it){
					average += (*it)->getPosition() * (double)(*it)->getNumUnits();
					totalNumUnits += (*it)->getNumUnits();
				}	
				
				average /= (double) totalNumUnits;
				
				real8 nearest = 2e8;
				Group *nearestGroup = NULL;
				
				FORALL (groups, it) {
					real8 dist = ((*it)->getPosition() - average).length();
					if (dist < nearest) {
						nearest = dist;
						nearestGroup = (*it);
					}
				}
				
				NUS_TerrainBase::Loc center_loc(nearestGroup->getPosition().x, nearestGroup->getPosition().y);
								
				GroupVector &farAwayGroups(groups);
			
				if(state->findspot){
					GameStateModule & gsm = state->simAI->get_gsm();
					center_loc = find_a_spot(center_loc, gsm, state);
				}

				if (!farAwayGroups.empty()) {
					real8 timeoutJoin = state->timeoutJoin;
					
					if (debug) std::cout << "Join: Pathfinding for individual units for Join" << std::endl;
					new MoveToLocation(this, farAwayGroups, function, "MoveToLocation", timeoutJoin, new MoveStruct(center_loc));
					++lineNum;
					break;
				} else 
					++lineNum; // close enough, proceed to next step
			}
			case 1: {
				//re-group all the newGroups into one group after joining	
				//add all units from all newGroups into a unit vector and remove the old newGroups
				UnitVector units;
			
				FORALL(groups, it){
					FORALL((*it)->getUnits(), it2){
						units.push_back((*it2));
					}
				}
			
				removeAllGroups();
			
				Group *group = new Group(playerID, units);
				addGroup(group);
				
				++lineNum;
				return true;
			}
			default:
			return true;
		}
		return false;
	}
	
	DEFINE_TASK(AJoin) {
		assert(extraData == NULL); // extradata should be NULL for this task
		UnitVector units;

		if (debug) std::cout << "Join: Start" << std::endl;

		switch(lineNum) {
			case 0:{
				FORALL(groups, it){
					FORALL((*it)->getUnits(), it2){
						units.push_back(*it2);
					}
				}
				
				Unit* friendliestUnit = groups[0]->getUnits()[0];
				real4 friendliness = -2e8;
								
				FORALL(units, it){
					UnitVector myUnit;
					
					myUnit.push_back(*it);
					
					Unit *nearestEnemy = state->getAverageNearestEnemyUnitOfGroup(playerID, myUnit);

					real4 influence = 2*state->getFriendlyInfluence(playerID, (*it)->getPosition()) - state->getEnemyInfluence(playerID, (*it)->getPosition());
					
					real8 power;
					
					power = 1.0/(real8((*it)->distance_to(*nearestEnemy))) * influence;

					if(power > friendliness){
						friendliness = power;
						friendliestUnit = *it;
					}
				}
				
				NUS_TerrainBase::Loc center_loc = friendliestUnit->getPosition();
								
				GroupVector &farAwayGroups(groups);
			
				if(state->findspot){
					GameStateModule & gsm = state->simAI->get_gsm();
					center_loc = find_a_spot(center_loc, gsm, state);
				}

				if (!farAwayGroups.empty()) {
					real8 timeoutJoin = state->timeoutJoin;
					
					if (debug) std::cout << "Join: Pathfinding for individual units for Join" << std::endl;
					new MoveToLocation(this, farAwayGroups, function, "MoveToLocation", timeoutJoin, new MoveStruct(center_loc));
					++lineNum;
					break;
				} else 
					++lineNum; // close enough, proceed to next step
			}
			case 1: {
				//re-group all the newGroups into one group after joining	
				//add all units from all newGroups into a unit vector and remove the old newGroups
				UnitVector units;
			
				FORALL(groups, it){
					FORALL((*it)->getUnits(), it2){
						units.push_back((*it2));
					}
				}
			
				removeAllGroups();
			
				Group *group = new Group(playerID, units);
				addGroup(group);
				
				++lineNum;
				return true;
			}
			default:
			return true;
		}
		return false;
	}

	DEFINE_TASK(Attack) {
		assert(extraData != NULL);
		NUS_TerrainBase::Loc targetLoc;
		GameObj *target;
		
		if (debug) std::cout << "Attack: Start" << std::endl;

		switch(lineNum) {
			case 0:
			// everyone should arrive at the same time

			//get the location of the target
			if (extraData->unit) {
				targetLoc.x = sint4(extraData->unit->getPosition().x);
				targetLoc.y = sint4(extraData->unit->getPosition().y);
				target = NULL;
			}
			else {
				if (extraData->base) {
					targetLoc.x = sint4(extraData->base->getPosition().x);
					targetLoc.y = sint4(extraData->base->getPosition().y);
					target = extraData->base->getObject();
				}
				else ERR("Attack: No target set");
			}
			
			//if (debug) std::cout << "Target Loc: (" << targetLoc.x << "," << targetLoc.y << ")" << std::endl;

			FORALL(groups, it){
				NUS_TerrainBase::Loc startPt((*it)->getPosition().x, (*it)->getPosition().y);
				WayPointVector wayPoints;

				real8 length = 0.0;
				
				if (target) 
					length = state->simAI->planPath(startPt, target, (*it)->getExtents(), wayPoints);
				else
					length = state->simAI->planPath(startPt, targetLoc, (*it)->getExtents(), wayPoints);
				
				Order* order = new Order(wayPoints, (*it)->getMaxSpeed());
				
				if (extraData->unit) {
					order->setTarget(extraData->unit);
				}
				else {
					if (extraData->base) {
						order->setTarget(extraData->base);
					}
					else ERR("Attack: No target set");
				}
				
				order->setActionType("ATTACK");
				//order->setActionType("IDLE");
				
				(*it)->setOrder(order);
			}
		
			++lineNum;
			break;
			default:
				return true;
		}
		return false;
	}

	DEFINE_TASK(MoveToLocation) {
		assert(extraData != NULL);

		if (debug) std::cout << "MoveToLocation: Start" << std::endl;
		
		switch(lineNum) {
			case 0:
	// everyone should arrive at the same time
			FORALL(groups, it){
				NUS_TerrainBase::Loc startPt((*it)->getPosition().x, (*it)->getPosition().y);
				WayPointVector wayPoints;
				
				real8 length = state->simAI->planPath(startPt, extraData->pos, (*it)->getExtents(), wayPoints);
				//std::cout << "planned path" << std::endl;
				if(wayPoints.empty()) ERR("planned path is empty");
				Order* order = new Order(wayPoints, (*it)->getMaxSpeed());
				/*		FORALL(wayPoints, it2){
					std::cout << it2->toString() <<"\t" ;
				}
				std::cout << std::endl;*/
				//order->setActionType("IDLE");
				order->setActionType("MOVE");
				(*it)->setOrder(order);	
			}

			++lineNum;
			break;
			default:
			return true;
		}
		return false;
	}

// ----------------
// Hunter 
/*
DEFINE_TASK(Hunter) {
	UnitVector units;
	
	if (debug) std::cout << "Hunter: Start" << std::endl;
	
	switch (lineNum){
					
		case 0:
			// make newGroups of X units(in this case, 5).  kind of a dumb way to do it... basically, just take one unit, find X-1
			// closest units, finding their average position, and then joining at the unit closest to that position
			
			if (debug) std::cout << "Hunter: Start Join" << std::endl;
			
			while(groups.size() > 0) {
				
				GroupVector newGroupsToJoin;
				GroupVector remainingGroups(groups);
				
				int numUnitsInGroups = groups[0]->getNumUnits();
				Vector3D leaderPosition = groups[0]->getPosition();
				
				newGroupsToJoin.push_back(groups[0]);
				remainingGroups.erase(remainingGroups.begin());
				
				while (newGroupsToJoin.size() < groups.size() && numUnitsInGroups < state->groupSize) {
					real8 nearest = 2e8;
					Group *nearestGroup = NULL;
					GroupVector::iterator nit;
					
					FORALL (remainingGroups, it) {
						real8 dist = ((*it)->getPosition() - leaderPosition).length();
						if (dist < nearest) {
							nearest = dist;
							nearestGroup = (*it);
							nit = it;
						}
					}
					
					numUnitsInGroups += nearestGroup->getNumUnits();
					newGroupsToJoin.push_back(nearestGroup);
					remainingGroups.erase(nit);
				}
				
				new JoinAndHunt(this, newGroupsToJoin, function, "JoinAndHunt", 0.0);				
			}
			
			if (debug) std::cout << "Hunter: Join End" << std::endl;

			lineNum++;
			break;
		case 1:
			if (debug) std::cout << "Hunter: Start Spread Attack" << std::endl;
			//new MassAttack(this, objects, function, name, 0.0);
			new SpreadAttack(this, groups, function, name, 0.0);

			lineNum++;
			break;
			default:
				return true;
	}
	return false;
}*/

DEFINE_TASK(JoinAndHunt){
	assert(extraData == NULL);

	if (debug) std::cout << "JoinAndHunt: Start" << std::endl;
	
	switch (lineNum){
		case 0:{
			if (debug) std::cout << "JoinAndHunt: Start Join" << std::endl;

			if(state->smartJoin){
				new SmartJoin(this, groups, function, name, 0.0); 
			}else{
				new Join(this, groups, function, "Join", 0.0);
			}
			lineNum++;
			break;
		}
		case 1:{		
			//note: the is only 1 group	
			if (debug) std::cout << "JoinAndHunt: Start attack" << std::endl;

			if(groups.size()!=1) ERR(("JoinAndHunt cannot have more than 1 group ??"));

			if (debug) std::cout << "JoinAndHunt: Find get nearest enemy" << std::endl;
			
			Unit* target = state->getNearestEnemyUnitOfGroup(playerID, groups[0]->getUnits());

			if (debug) std::cout << "JoinAndHunt: Done finding get nearest enemy" << std::endl;

			if (target != NULL){
				//if(state->useMoveForAttack){
					//new MoveToObject(this, newGroups, function,name,0.0, new MoveStruct(target));
				//}else{
					new Attack(this, groups, function, "Attack",0.0, new MoveStruct(target));
				//}
					} else return true;
			break;
		}
		default:
		return true;
	}
	return false;
}

// ----------------
// Hunter 2

DEFINE_TASK(Hunter2) {
	UnitVector units;
	
	if (debug) std::cout << "Hunter2: Start" << std::endl;
	
	switch (lineNum){
					
		case 0:
			// make newGroups of X units(in this case, 5).  kind of a dumb way to do it... basically, just take one unit, find X-1
			// closest units, finding their average position, and then joining at the unit closest to that position
			
			if (debug) std::cout << "Hunter2: Start Join" << std::endl;
			
			while(groups.size() > 0) {
				
				GroupVector newGroupsToJoin;
				GroupVector remainingGroups(groups);
				
				int numUnitsInGroups = groups[0]->getNumUnits();
				Vector3D leaderPosition = groups[0]->getPosition();
				
				newGroupsToJoin.push_back(groups[0]);
				remainingGroups.erase(remainingGroups.begin());
				
				while (newGroupsToJoin.size() < groups.size() && numUnitsInGroups < state->groupSize) {
					real8 nearest = 2e8;
					Group *nearestGroup = NULL;
					GroupVector::iterator nit;
					
					FORALL (remainingGroups, it) {
						real8 dist = ((*it)->getPosition() - leaderPosition).length();
						if (dist < nearest) {
							nearest = dist;
							nearestGroup = (*it);
							nit = it;
						}
					}
					
					numUnitsInGroups += nearestGroup->getNumUnits();
					newGroupsToJoin.push_back(nearestGroup);
					remainingGroups.erase(nit);
				}
				
				new JoinAndHunt2(this, newGroupsToJoin, function, "JoinAndHunt2", 0.0);
			}
			
			if (debug) std::cout << "Hunter2: Join End" << std::endl;

			lineNum++;
			break;
		case 1:
			if (debug) std::cout << "Hunter2: Start Spread Attack" << std::endl;
			//new MassAttack(this, objects, function, name, 0.0);
			new SpreadAttack(this, groups, function, name, 0.0);

			lineNum++;
			break;
			default:
				return true;
	}
	return false;
}

DEFINE_TASK(JoinAndHunt2){
	assert(extraData == NULL);

	if (debug) std::cout << "JoinAndHunt2: Start" << std::endl;
	
	switch (lineNum){
		case 0:{
			if (debug) std::cout << "JoinAndHunt2: Start Join" << std::endl;

			if(state->smartJoin){
				new SmartJoin(this, groups, function, name, 0.0);  
			}else{
				new Join(this, groups, function, "Join", 0.0);
			}
			lineNum++;
			break;
		}
		case 1:{			
			if (debug) std::cout << "JoinAndHunt2: Start attack" << std::endl;

			if(groups.size()!=1) ERR(("JoinAndHunt cannot have more than 1 group ??"));

			if (debug) std::cout << "JoinAndHunt2: Find get nearest enemy" << std::endl;

			Group *group = state->getNearestGroupOfGroup(playerID, groups[0]);
			
			if (group) {
				if ( (group->getPosition()-groups[0]->getPosition()).length() < 100.0 ) {
					if (debug) std::cout << "Nearby friendly group, let's merge!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << std::endl;
					merge(groups[0], group);
				}
			}			
			
			Unit* target = state->getAverageNearestEnemyUnitOfGroup(playerID, groups[0]->getUnits());
			//Unit* target = state->getNearestEnemyUnitOfCGOfGroup(playerID, groups[0]->getUnits());

			if (debug) std::cout << "JoinAndHunt2: Done finding get nearest enemy" << std::endl;

			if (target != NULL){
				//if(state->useMoveForAttack){
					//new MoveToObject(this, newGroups, function,name,0.0, new MoveStruct(target));
				//}else{
					new Attack(this, groups, function, "Attack",0.0, new MoveStruct(target));
				//}
					} else return true;
			break;
		}
		default:
		return true;
	}
	return false;
}
/*
DEFINE_TASK(JoinAndHunt3){
	assert(extraData == NULL);
	Group *group = NULL;
	if (debug) std::cout << "JoinAndHunt3: Start" << std::endl;
	
	switch (lineNum){
		case 0:{
			if (debug) std::cout << "JoinAndHunt3: Start Join" << std::endl;

			if(state->smartJoin){
				new SmartJoin(this, groups, function, name, 0.0); 
			}else{
				new Join(this, groups, function, "Join", 0.0);
			}
			lineNum++;
			break;
		}
		case 1:{		
			groups[0]->hurry = false;
			
			//note: the is only 1 group	
			if (debug) std::cout << "JoinAndHunt3: Start attack" << std::endl;

			if(groups.size()!=1) ERR(("JoinAndHunt3 cannot have more than 1 group ??"));

			if (debug) std::cout << "JoinAndHunt3: Find get nearest enemy" << std::endl;
			
			Group *nearestGroup = state->getNearestGroupOfGroup(playerID, groups[0]);
			
			if (nearestGroup) {
				if ( (nearestGroup->getPosition()-groups[0]->getPosition()).length() < 70.0 ) {
					if (debug){
						 std::cout << "Nearby friendly group, let's merge!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << std::endl;
							std::cout << "distance between merging units is " << (nearestGroup->getPosition()-groups[0]->getPosition()).length() << std::endl;
					}
					merge(groups[0], nearestGroup);
				}
			}
			
			Unit* target = state->getNearestWeakEnemyUnitOfGroup(playerID, groups[0]->getUnits(), groups[0]->getNumUnits());
			
			//nearestGroup = state->getStrongestFriendlyGroup(playerID, groups[0]);
			nearestGroup = state->getStrongerFriendlyGroup(playerID, groups[0]);
			
			if (debug) std::cout << "JoinAndHunt3: Done finding get nearest enemy" << std::endl;

			NUS_TerrainBase::Loc groupPosition(groups[0]->getPosition().x,groups[0]->getPosition().y);
			
			if (debug) if (target) std::cout << "Distance to enemy target:" << target->distance_to(groupPosition) << std::endl;
			if (debug) if (nearestGroup) std::cout << "Distance to nearest strong group:" << (groups[0]->getPosition()-nearestGroup->getPosition()).length() << std::endl;
			
			if ((nearestGroup == NULL && target != NULL) || \
			(target != NULL && target->distance_to(groupPosition) < (groups[0]->getPosition()-nearestGroup->getPosition()).length())){
				//if(state->useMoveForAttack){
					//new MoveToObject(this, newGroups, function,name,0.0, new MoveStruct(target));
				//}else{
					new Attack(this, groups, function, "Attack",0.0, new MoveStruct(target));
				//}
			} else{
				std::cout <<"JoinAndHunt3 no weaker enemy influence, move to the  nearest group" <<std::endl;
				//ERR("better merge");
				
					if(nearestGroup){
						NUS_TerrainBase::Loc move_loc(nearestGroup->getPosition().x, nearestGroup->getPosition().y);
						//if(groups[0]->getPosition().x != move_loc.x || groups[0]->getPosition().y != move_loc.y  ){
						if ((groups[0]->getPosition()-nearestGroup->getPosition()).length() > 70.0) 
						{
							if (debug) std::cout << "trying to move to another group location" << std::endl;
							//groups[0]->hurry = true;
							
							if (groups.size() != 1) ERR("more than one group in hunter3");
						 
							//better if it is a MoveToObj
							new MoveToLocation(this,groups,function,name,0.0,new MoveStruct(move_loc)); //ERR("moving");
							
							// actually attack might be nice since if the nearest group dies we will be informed. 
							// but alas, currently this seg faults!
							//new Attack(this, groups, function, "Attack",0.0, new MoveStruct(nearestGroup->getUnits()[0]));
							break;
						} 
						else 
						{
							std::cout <<"merged!!" << nearestGroup->getNumUnits() <<" and " << groups[0]->getNumUnits() <<std::endl;
							merge(nearestGroup, groups[0]);
							
							if (nearestGroup->getOrder()) nearestGroup->getOrder()->setCompleted(true);
							return true;
						}
					}
					else { 
						Unit* target = state->getAverageNearestEnemyUnitOfGroup(playerID, groups[0]->getUnits());
						//Unit* target = state->getNearestEnemyUnitOfCGOfGroup(playerID, groups[0]->getUnits());

						if (debug) std::cout << "JoinAndHunt3: Resorting to finding nearest enemy" << std::endl;

						if (target != NULL){
							//if(state->useMoveForAttack){
								//new MoveToObject(this, newGroups, function,name,0.0, new MoveStruct(target));
							//}else{
								new Attack(this, groups, function, "Attack",0.0, new MoveStruct(target));
							//}
						} else return true;
						break;
					}
				}
		}
		break;
		default:
		return true;
	}
	return false;
}*/


DEFINE_TASK(JoinAndHunt3){
	assert(extraData == NULL);
	Group *group;
	if (debug) std::cout << "JoinAndHunt3: Start" << std::endl;
	
	switch (lineNum){
		case 0:{
			if (debug) std::cout << "JoinAndHunt3: Start Join" << std::endl;

			if(state->smartJoin){
				new SmartJoin(this, groups, function, name, 0.0); 
			}else{
				new AJoin(this, groups, function, "Join", 0.0);
			}
			lineNum++;
			break;
		}
		case 1:{		
			groups[0]->hurry = false;
			
			//note: the is only 1 group	
			if (debug) std::cout << "JoinAndHunt3: Start attack" << std::endl;

			if(groups.size()!=1) ERR(("JoinAndHunt3 cannot have more than 1 group ??"));

			if (debug) std::cout << "JoinAndHunt3: Find get nearest enemy" << std::endl;
			
			Group *nearestGroup = state->getNearestGroupOfGroup(playerID, groups[0]);
			
			if (nearestGroup) {
				if ( (nearestGroup->getPosition()-groups[0]->getPosition()).length() < 70.0 ) {
					if (debug){
						 std::cout << "Nearby friendly group, let's merge!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << std::endl;
						 std::cout << "distance between merging units is " << (nearestGroup->getPosition()-groups[0]->getPosition()).length() << std::endl;
					}
					merge(groups[0], nearestGroup);
				}
			}
			
			
			Unit* target = state->getNearestWeakEnemyUnitOfGroup(playerID, groups[0]->getUnits(), groups[0]->getNumUnits());
			
			//nearestGroup = state->getStrongestFriendlyGroup(playerID, groups[0]);
			nearestGroup = state->getStrongerFriendlyGroup(playerID, groups[0], groups);
			
			if (debug) std::cout << "JoinAndHunt3: Done finding get nearest enemy" << std::endl;

			NUS_TerrainBase::Loc groupPosition(groups[0]->getPosition().x,groups[0]->getPosition().y);
			
			if (debug) if (target) std::cout << "Distance to enemy target:" << target->distance_to(groupPosition) << std::endl;
			if (debug) if (nearestGroup) std::cout << "Distance to nearest strong group:" << (groups[0]->getPosition()-nearestGroup->getPosition()).length() << std::endl;
			
			if ((nearestGroup == NULL && target != NULL) || \
			(target != NULL && target->distance_to(groupPosition) < (groups[0]->getPosition()-nearestGroup->getPosition()).length())){
				//if(state->useMoveForAttack){
					//new MoveToObject(this, newGroups, function,name,0.0, new MoveStruct(target));
				//}else{
					new Attack(this, groups, function, "Attack",0.0, new MoveStruct(target));
				//}
			} else{
				if (debug) std::cout <<"JoinAndHunt3 no weaker enemy influence, move to the  nearest group" <<std::endl;
				//ERR("better merge");
				//nearestGroup = state->getNearestGroupOfGroup(playerID, groups[0]);
				
					if(nearestGroup){
						NUS_TerrainBase::Loc move_loc(nearestGroup->getPosition().x, nearestGroup->getPosition().y);
						//if(groups[0]->getPosition().x != move_loc.x || groups[0]->getPosition().y != move_loc.y  ){
						if ((groups[0]->getPosition()-nearestGroup->getPosition()).length() > 70.0) 
						{
							if (debug) std::cout << "trying to move to another group location" << std::endl;
							//groups[0]->hurry = true;
							
							if (groups.size() != 1) ERR("more than one group in hunter3");
						
							GroupVector groupsToJoin;
							groupsToJoin.push_back(groups[0]);
							groupsToJoin.push_back(nearestGroup);
							
							Vector3D average(0.0, 0.0, 0.0);
							int totalNumUnits = 0;

							FORALL(groupsToJoin, it){
								average += (*it)->getPosition() * (double)(*it)->getNumUnits();
								totalNumUnits += (*it)->getNumUnits();
							}	

							average /= (double) totalNumUnits;

							NUS_TerrainBase::Loc center_loc(average.x, average.y);
						
							merge(groups[0], nearestGroup);
							
							if (debug) std::cout << "meeting location is " << average.toString() << std::endl;
							new MoveToLocation(this,groups,function,name,0.0,new MoveStruct(center_loc)); //ERR("moving");
							
							// actually attack might be nice since if the nearest group dies we will be informed. 
							// but alas, currently this seg faults!
							//new Attack(this, groups, function, "Attack",0.0, new MoveStruct(nearestGroup->getUnits()[0]));
							break;
						} 
						else 
						{
							if (debug) std::cout <<"merged!!" << nearestGroup->getNumUnits() <<" and " << groups[0]->getNumUnits() <<std::endl;
							merge(nearestGroup, groups[0]);
							
							if (nearestGroup->getOrder()) nearestGroup->getOrder()->setCompleted(true);
							return true;
						}
					}
					else { 
						Unit* target = state->getAverageNearestEnemyUnitOfGroup(playerID, groups[0]->getUnits());
						//Unit* target = state->getNearestEnemyUnitOfCGOfGroup(playerID, groups[0]->getUnits());

						if (debug) std::cout << "JoinAndHunt3: Resorting to finding nearest enemy" << std::endl;

						if (target != NULL){
							//if(state->useMoveForAttack){
								//new MoveToObject(this, newGroups, function,name,0.0, new MoveStruct(target));
							//}else{
								new Attack(this, groups, function, "Attack",0.0, new MoveStruct(target));
							//}
						} else return true;
						break;
					}
				}
		}
		break;
		default:
		return true;
	}
	return false;
}

// ----------------
// Hunter 3

DEFINE_TASK(Hunter3) {
	UnitVector units;
	
	if (debug) std::cout << "Hunter3: Start" << std::endl;
	
	switch (lineNum){
					
		case 0:
			// make newGroups of X units(in this case, 5).  kind of a dumb way to do it... basically, just take one unit, find X-1
			// closest units, finding their average position, and then joining at the unit closest to that position
			
			if (debug) std::cout << "Hunter: Start Join" << std::endl;
			
			while(groups.size() > 0) {
				
				GroupVector newGroupsToJoin;
				GroupVector remainingGroups(groups);
				
				int numUnitsInGroups = groups[0]->getNumUnits();
				Vector3D leaderPosition = groups[0]->getPosition();
				
				newGroupsToJoin.push_back(groups[0]);
				remainingGroups.erase(remainingGroups.begin());
				
				while (newGroupsToJoin.size() < groups.size() && numUnitsInGroups < state->groupSize) {
					real8 nearest = 2e8;
					Group *nearestGroup = NULL;
					GroupVector::iterator nit;
					
					FORALL (remainingGroups, it) {
						real8 dist = ((*it)->getPosition() - leaderPosition).length();
						if (dist < nearest) {
							nearest = dist;
							nearestGroup = (*it);
							nit = it;
						}
					}
					
					numUnitsInGroups += nearestGroup->getNumUnits();
					newGroupsToJoin.push_back(nearestGroup);
					remainingGroups.erase(nit);
				}
								
				new JoinAndHunt3(this, newGroupsToJoin, function, "JoinAndHunt3", 0.0);
			}
			
			if (debug) std::cout << "Hunter3: Join End" << std::endl;

			lineNum++;
			break;
		case 1:
			if (debug) std::cout << "Hunter3: Start Spread Attack" << std::endl;
			//new MassAttack(this, objects, function, name, 0.0);
			new SpreadAttack(this, groups, function, name, 0.0);

			lineNum++;
			break;
			default:
				return true;
	}
	return false;
}

DEFINE_TASK(Hunter4) {
	UnitVector units;
	
	if (debug) std::cout << "Hunter4: Start" << std::endl;
	bool safe = true;
	
	switch (lineNum){
					
		case 0:
			// make newGroups of X units(in this case, 5).  kind of a dumb way to do it... basically, just take one unit, find X-1
			// closest units, finding their average position, and then joining at the unit closest to that position
			
			if (debug) std::cout << "Hunter4: Start Join" << std::endl;
			
			//check safety for mass attack
			if (debug) std::cout << "enemy influence at each position" << std::endl;
			FORALL(groups, it){
				if(state->getEnemyInfluence(playerID, (*it)->getUnits()[0]->getPosition())>3.5){
					if (debug) std::cout << state->getEnemyInfluence(playerID, (*it)->getUnits()[0]->getPosition()) << "\t";
					safe = false;
				}
			}
			if (debug) std::cout << std::endl;
	
			if (safe) {
				if (debug) std::cout << "ray check" << std::endl;
				
				Player *me = state->getPlayer(playerID);
				
				Vector<Base*> myBases;
				
				std::map<Base*, ExtraObjectData> &baseMap(me->getOrtsBases());
				
				FORALL(baseMap, it) 
					myBases.push_back(it->first);
				
				PlayerMap &allPlayers(state->getPlayers());
				
				Vector<Base*> enemyBases;
				
				FORALL(allPlayers, pit) {
					if (pit->second != me) {
						std::map<Base*, ExtraObjectData> &enemyBaseMap((pit->second)->getOrtsBases());
						FORALL(enemyBaseMap, it) 
							enemyBases.push_back(it->first);
					}
				}
				
				for (int i = 0; i < myBases.size(); ++i) {
					if (!safe) break;

					for (int j = i+1; j < myBases.size(); ++j) {
						if (!safe) break;

						// shoot a ray from base i to base j
						Vector3D vec(myBases[j]->getPosition()-myBases[i]->getPosition());
						
						const int steps = 16;
						
						for (int k = 0; k < steps; ++k) {
							if (!safe) break;
							
							Vector3D point = myBases[i]->getPosition() + vec * ((k+1)/(double)steps);
							
							for (int l = 0; l < enemyBases.size(); ++l) {
								if ((enemyBases[l]->getPosition()-point).length() < enemyBases[l]->getExtents() * 0.51) {
									if (debug) std::cout << "base in line. detected collision of length " << (enemyBases[l]->getPosition()-point).length() << std::endl;
									if (debug) std::cout << "base extents:" << enemyBases[l]->getExtents() << std::endl;
									safe = false;
									break;
								}
							}
						}
					}
				}
			}
	
			if(safe){
				if(debug) std::cout << "Mass Join Strategy" << std::endl;
				::Options::set("-groupsize", 25, false);
				::Options::set("-findspot", true, false);
				::Options::set("-timeoutJoin", 12.5, false);
				state->groupSize = 25;
				state->findspot = true;
				state->timeoutJoin = 12.5;
			}
			
			while(groups.size() > 0) {
				
				GroupVector newGroupsToJoin;
				GroupVector remainingGroups(groups);
				
				int numUnitsInGroups = groups[0]->getNumUnits();
				Vector3D leaderPosition = groups[0]->getPosition();
				
				newGroupsToJoin.push_back(groups[0]);
				remainingGroups.erase(remainingGroups.begin());
				
				while (newGroupsToJoin.size() < groups.size() && numUnitsInGroups < state->groupSize) {
					real8 nearest = 2e8;
					Group *nearestGroup = NULL;
					GroupVector::iterator nit;
					
					FORALL (remainingGroups, it) {
						real8 dist = ((*it)->getPosition() - leaderPosition).length();
						if (dist < nearest) {
							nearest = dist;
							nearestGroup = (*it);
							nit = it;
						}
					}
					
					numUnitsInGroups += nearestGroup->getNumUnits();
					newGroupsToJoin.push_back(nearestGroup);
					remainingGroups.erase(nit);
				}
								
				new AJoin(this, newGroupsToJoin, function, "Ajoin", 0.0);
			}
			
			if (debug) std::cout << "Hunter4: Join End" << std::endl;

			lineNum++;
			break;
			
		case 1:{
			if (debug) std::cout << "Hunter4: Hunt Start" << std::endl;
			if(state->getNearestEnemyUnitOfGroup(playerID, groups[0]->getUnits())==NULL){
				lineNum++;
				if (debug) std::cout << "no enemy left" << std::endl;
			}
			else {
				real8 distance[groups.size()];
				Unit* nearestEnemy[groups.size()];
				uint4 i = 0;
				FORALL(groups, it){
					nearestEnemy[i] = state->getNearestEnemyUnitOfGroup(playerID, (*it)->getUnits());
					distance[i] =  (*it)->distance_to(nearestEnemy[i]);
					i++;
				}
			
				uint4 j;
				bool swapped = true;
				while(swapped){
					swapped = false;
					for(j=0;j<groups.size()-1;j++){
						if(distance[j] > distance[j+1]){
							swapped = true;
							Group* gtemp = groups[j];
							groups[j] = groups[j+1];
							groups[j+1] = gtemp;
						
							Unit* utemp = nearestEnemy[j];
							nearestEnemy[j] = nearestEnemy[j+1];
							nearestEnemy[i+1] = utemp;
						
							real8 dtemp = distance[j];
							distance[j] = distance[j+1];
							distance[j+1] = dtemp;
						}
					}
				}
			
				//checker
				if (debug) {
					FORALL(groups, it){
						std::cout << (*it)->distance_to(state->getNearestEnemyUnitOfGroup(playerID, (*it)->getUnits())) << "\t";
						std::cout << (*it)->getNumUnits() << std::endl;
					}
					std::cout << std::endl;
				}
				
				//do hunter3 algo
				GroupVector temp = groups;
				FORALL(temp, it){
					Vector<Group*>::iterator git = std::find(groups.begin(), groups.end(), *it);
					
					if (git == groups.end()) continue;
					
					Unit* target = state->getNearestWeakEnemyUnitOfGroup(playerID, (*it)->getUnits(), (*it)->getNumUnits());

					Group* nearestGroup = state->getNearestGroupOfGroup(playerID, *it, groups);

					if (nearestGroup) {
						if ( (nearestGroup->getPosition()-groups[0]->getPosition()).length() < 100.0 ) {
							if (debug) std::cout << "Nearby friendly group, let's merge!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << std::endl;

							merge(*it, nearestGroup);
						}
					}			
				
					if (target != NULL)
					{
						GroupVector toAttack;
						toAttack.push_back(*it);
						new Attack(this, toAttack, function, "Attack",0.0, new MoveStruct(target));
					}else{
						if(debug) std::cout <<"Hunter4: no weaker enemy" << std::endl;
					
						if(target == NULL){
							Unit* target = state->getNearestEnemyUnitOfGroup(playerID, (*it)->getUnits());
							if(target){
								if(debug) std::cout << "No weaker enemy, attack stronger enemy anyway" << std::endl;
								GroupVector toAttack;
								toAttack.push_back(*it);
								new Attack(this, toAttack, function, "Attack",0.0, new MoveStruct(target));
							}else{
								if(debug) std::cout << "No enemy unit left" << std::endl;
								lineNum++;
							}
						}
					}
				}
				if (lineNum == 1) break;
			}
		}
		case 2:
			if (debug) std::cout << "Hunter4: Start Spread Attack" << std::endl;
			//new MassAttack(this, objects, function, name, 0.0);
			new SpreadAttack(this, groups, function, name, 0.0);

			lineNum++;
			break;
			default:
				return true;
	}
	return false;
}


DEFINE_TASK(JoinSpreadAttack) {
	
	UnitVector units;
	
	if (debug) std::cout << "JoinSpreadAttack: Start" << std::endl;
	
	switch (lineNum){
		case 0:
			// make newGroups of X units(in this case, 5).  kind of a dumb way to do it... basically, just take one unit, find X-1
			// closest units, finding their average position, and then joining at the unit closest to that position
			
			if (debug) std::cout << "JoinSpreadAttack: Start Join" << std::endl;
			
			// make newGroups of X units(in this case, 5).  kind of a dumb way to do it... basically, just take one unit, find X-1
			// closest units, finding their average position, and then joining at the unit closest to that position
			while(groups.size() > 0) {
				
				GroupVector newGroupsToJoin;
				GroupVector remainingGroups(groups);
				
				int numUnitsInGroups = groups[0]->getNumUnits();
				Vector3D leaderPosition = groups[0]->getPosition();
				
				newGroupsToJoin.push_back(groups[0]);
				remainingGroups.erase(remainingGroups.begin());
				
				while (newGroupsToJoin.size() < groups.size() && numUnitsInGroups < state->groupSize) {
					real8 nearest = 2e8;
					Group *nearestGroup = NULL;
					GroupVector::iterator nit;
					
					FORALL (remainingGroups, it) {
						real8 dist = ((*it)->getPosition() - leaderPosition).length();
						if (dist < nearest) {
							nearest = dist;
							nearestGroup = (*it);
							nit = it;
						}
					}
					
					numUnitsInGroups += nearestGroup->getNumUnits();
					newGroupsToJoin.push_back(nearestGroup);
					remainingGroups.erase(nit);
				}
				
				new JoinAndAttack(this, newGroupsToJoin, function, "JoinAndAttack", 0.0);
			}
			
			if (debug) std::cout << "JoinSpreadAttack: Join End" << std::endl;

			lineNum++;
			break;
		case 1:
			if (debug) std::cout << "JoinSpreadAttack: Start Spread Attack" << std::endl;
			//new MassAttack(this, objects, function, name, 0.0);
			new SpreadAttack(this, groups, function, name, 0.0);

			lineNum++;
			break;
			default:
				return true;
	}
	return false;
}


/*
	DEFINE_TASK(MoveToObject) {
		assert(extraData != NULL);
		assert(extraData->obj != NULL);

		switch(lineNum) {
			case 0:
	// everyone should arrive at the same time
			FORALL(objects,it) {
				if(*it == extraData->obj){ // the center object does not need to move
					ExtraObjectData newData;
				newData.isDone = true;
				Player* currPlayer = state->getPlayer(playerID);
				currPlayer->getOrtsUnits()[*it] = newData;
				continue;
			}
			NUS_TerrainBase::Task task(NUS_TerrainBase::Goal::OBJ, NUS_TerrainBase::Goal::TOUCH,
				NUS_TerrainBase::Task::INDIVIDUAL);

			ExtraObjectData newData;
			newData.target = extraData->obj;
			newData.state = ExtraObjectData::MOVE;

			Player* currPlayer = state->getPlayer(playerID);

			currPlayer->getOrtsUnits()[*it] = newData;

			task.goal.obj = extraData->obj;
			task.objs.insert(*it);
			state->simAI->doPath(task);
		}
		++lineNum;
		break;
		default:
		return false;
	}
	return false;
}
*/
/*
DEFINE_TASK(DefendObject) {
	assert(extraData != NULL);

	switch(lineNum) {
		case 0:
	// everyone should arrive at the same time
		FORALL(objects,it){
		// this should NEVER happen, since you should not be defending an object that is getting orders
		// to defend itself!
			if(*it == extraData->obj){
				ExtraObjectData newData;
				newData.isDone = true;
				Player* currPlayer = state->getPlayer(playerID);
				currPlayer->getOrtsUnits()[*it] = newData;
				continue;
			}
			NUS_TerrainBase::Task task(NUS_TerrainBase::Goal::OBJ, NUS_TerrainBase::Goal::TOUCH,
				NUS_TerrainBase::Task::INDIVIDUAL);

			ExtraObjectData newData;
			newData.target = extraData->obj;
			newData.state = ExtraObjectData::DEFEND;

			Player* currPlayer = state->getPlayer(playerID);

			currPlayer->getOrtsUnits()[*it] = newData;

			task.goal.obj = extraData->obj;
			task.objs.insert(*it);
			state->simAI->doPath(task);
		}
		++lineNum;
		break;
		default:
		return false;
	}
	return false;
}
*/

/*
DEFINE_TASK(HalfUnitsBaseDefence_MassAttack){

	switch(lineNum) {
		case 0:{
			Player* currPlayer = state->getPlayer(playerID);
	// make sure that we have enough units to do anything.. if not, then just don't defend bases
			uint4 numUnits = currPlayer->getOrtsUnits().size();
			uint4 numBases = currPlayer->getOrtsBases().size();
			if(numUnits/2 >= numBases){
				uint4 numUnitsPerBase = (numUnits/2) / numBases;
				FORALL(currPlayer->getOrtsBases(),it){
					ObjectVector unitsToDefendBase;
					state->getNearestObjectsToObject(unitsToDefendBase,objects,it->first,numUnitsPerBase);
					new DefendObject(this,unitsToDefendBase,function,name,0.0,new MoveStruct(it->first));

				}
			}
	// the rest of the units that are not defending go join up and attack!
			new MassAttack(this,objects,function,name,0.0);
			++lineNum;
			break;
		}
		default:
		return false;
	}
	return false;
}
*/
/*
DEFINE_TASK(TouchObj) {
	assert(extraData != NULL);
	//assert(extraData->obj != NULL);

	switch(lineNum){
		case 0:
		if (extraData->obj != NULL){
			if (extraData->obj->getMaxSpeed() > 0 && *extraData->obj->sod.owner == (sint4)playerID){
				NUS_TerrainBase::Loc center_loc;
				extraData->obj->get_center(center_loc.x, center_loc.y);
				new MoveToLocation(this,objects,function,name,0.0,new MoveStruct(center_loc));
			}
			else{
				new MoveToObject(this, objects, function,name,0.0, new MoveStruct(extraData->obj));
			}
		}
		++lineNum;
		break;
		default:
		return false;
	}
	return false;
}
*/


DEFINE_TASK(JoinAndMoveToObj) {
	
	assert(extraData != NULL); // extradata should not be NULL for this task

	assert(extraData->pos != NUS_TerrainBase::Loc(-1, -1));
	UnitVector units;

	switch(lineNum) {
		case 0:{
			FORALL(groups, it){
				FORALL((*it)->getUnits(), it2){
					units.push_back((*it2));
				}
			}
			
			Unit* centerObject = state->getNearestObject(units, state->getCenterOfObjects(units));
			assert(centerObject != NULL);// takje this assert OUT later
			NUS_TerrainBase::Loc center_loc = centerObject->getPosition();
			
			GroupVector &farAwayGroups(groups);
			
		/*	FORALL(groups, it){
				NUS_TerrainBase::Loc groupLoc(sint4((*it)->getPosition().x),sint4((*it)->getPosition().y));
				
				if (sqrt((groupLoc.x-center_loc.x)*(groupLoc.x-center_loc.x) \
					+ (groupLoc.y-center_loc.y)*(groupLoc.y-center_loc.y)) > 100.0)
					farAwayGroups.push_back(*it);
			}*/
			
			
			
			if (!farAwayGroups.empty())
			{
				new MoveToLocation(this,farAwayGroups,function,name,0.0,new MoveStruct(center_loc));

				++lineNum;
				break;
				} else ++lineNum;
		}
		case 1:
		new MoveToLocation(this,groups,function,name,0.0,new MoveStruct(extraData->pos));

		++lineNum;
		break;
		default:
		return false;
	}
	return false;
}



DEFINE_TASK(SmartJoin) {
	
	assert(extraData == NULL); // extradata should be NULL for this task

	if (debug) std::cout << "SmartJoin: Start" << std::endl;

	switch(lineNum) {
		case 0:{
			UnitVector units;
			//add all units from all newGroups into a unit vector

			FORALL(groups, it){
				FORALL((*it)->getUnits(), it2){
					units.push_back((*it2));
				}
			}

			Unit* centerObject = state->getNearestObject(units, state->getCenterOfObjects(units));
			//assert(centerObject != NULL && centerObject->get_shape() == Object::CIRCLE);// takje this assert OUT later
			NUS_TerrainBase::Loc center_loc;
			//centerObject->get_center(center_loc.x, center_loc.y);
			center_loc = centerObject->getPosition();

			/*
			ignore this for now
			if(state->findspot){
				GameStateModule & gsm = state->simAI->get_gsm();
				center_loc = find_a_spot(center_loc, gsm);
			}*/
			removeAllGroups();
			
			while(!units.empty()) {
				if (debug) std::cout << "SmartJoin: start JoinAndMoveToObj" << std::endl;
				
				UnitVector unitsToJoin;

				state->getNearestObjectsToObject(unitsToJoin,units,units[0],state->smartJoinSize);
				
				GroupVector newGroupsToJoin;
				FORALL(unitsToJoin, it){
					Group* group = new Group(playerID, *it);
					addGroup(group);
					
					newGroupsToJoin.push_back(group);
				}
				
				new JoinAndMoveToObj(this,newGroupsToJoin,function,name,0.0, new MoveStruct(center_loc));
				
				// remove unitToJoin from units
				FORALL(unitsToJoin, it) {
					Vector<Unit*>::iterator uit = std::find(units.begin(), units.end(), *it);
					
					if (uit != units.end())
						units.erase(uit);
				}
			}
		}
		case 1: {
			//re-group all the newGroups into one group after joining	
			//add all units from all newGroups into a unit vector and remove the old newGroups
			UnitVector units;

			FORALL(groups, it){
				FORALL((*it)->getUnits(), it2){
					units.push_back((*it2));
				}
			}

			removeAllGroups();

			Group *group = new Group(playerID, units);
			addGroup(group);

			++lineNum;
			return true;
		}
		default:
		return true;
	}
	return false;
}


DEFINE_TASK(JoinAndAttack){
	
	assert(extraData == NULL);

	if (debug) std::cout << "JoinAndAttack: Start" << std::endl;

	switch (lineNum){
		case 0:{
			if (debug) std::cout << "JoinAndAttack: Start Join" << std::endl;

			if(state->smartJoin){
				new SmartJoin(this, groups, function, "Smart Join", 0.0);  //no smart joins for now
			}else{
				new Join(this, groups, function, "Join", 0.0);
			}
			lineNum++;
			break;
		}
		case 1:{			
			if (debug) std::cout << "JoinAndAttack: Start attack" << std::endl;
			if (debug) std::cout << "JoinAndAttack: Find get nearest enemy base" << std::endl;

			Group *group = state->getNearestGroupOfGroup(playerID, groups[0]);
			
			if (group) {
				if ( (group->getPosition()-groups[0]->getPosition()).length() < 75.0 ) {
					if (debug) std::cout << "Nearby friendly group, let's merge!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << std::endl;
					merge(groups[0], group);
				}
			}			
			
			Base* target = state->getNearestEnemyBaseOfGroup(playerID, groups[0]);

			if (debug) std::cout << "JoinAndAttack: Done finding get nearest enemy base" << std::endl;

			if (target != NULL){
				//if(state->useMoveForAttack){
					//new MoveToObject(this, newGroups, function,name,0.0, new MoveStruct(target));
				//}else{
					new Attack(this, groups, function, "Attack",0.0, new MoveStruct(target));
				//}
					} else return true;
			break;
		}
		default:
		return true;
	}
	return false;
}


/*
	DEFINE_TASK(MassAttack) {
	// ObjectVector temp;
		switch(lineNum){

			case 0:
			if(state->smartJoin){
				new SmartJoin(this, objects, function, name, 0.0);
			}else{
				new Join(this, objects, function, name, 0.0);
			}

			++lineNum;
			break;
			case 1:{
	// need a copy of this vector, since the constructor for tasks modifies the original by removing objects from it
				GameObj* nearestBase = state->getNearestEnemyBaseOfGroup(playerID,objects);
		//temp = objects;

				if(nearestBase != NULL){
					new Attack(this,objects,function,name,0.0,new MoveStruct(nearestBase));
				}
				break;
			}
			default:
			return false;
		}
		return false;
	}
*/

Task* createSmartJoin(uint4 id,SimulationState* s,GroupVector& o,real8 timeout) {
	return new SmartJoin(id,s,o,createSmartJoin,"SmartJoin",timeout);
}
Task* createSpreadAttack(uint4 id,SimulationState* s,GroupVector& o,real8 timeout) {
	return new SpreadAttack(id,s,o,createSpreadAttack,"SpreadAttack",timeout);
}
Task* createMassAttack(uint4 id,SimulationState* s,GroupVector& o,real8 timeout) {
	return NULL; //return new MassAttack(id,s,o,createMassAttack,"MassAttack",timeout);
}
Task* createHalfUnitsBaseDefence_MassAttack(uint4 id,SimulationState* s, GroupVector& o,real8 timeout) {
	return NULL; //return new HalfUnitsBaseDefence_MassAttack(id,s,o,createHalfUnitsBaseDefence_MassAttack,"HalfUnitsBaseDefence_MassAttack",timeout);
}
Task* createHunter4(uint4 id,SimulationState* s, GroupVector& o,real8 timeout) {
	return new Hunter4(id,s,o,createHunter4,"Hunter",timeout);
}
Task* createHunter2(uint4 id,SimulationState* s, GroupVector& o,real8 timeout) {
	return new Hunter2(id,s,o,createHunter2,"Hunter2",timeout);
}

Task* createHunter3(uint4 id,SimulationState* s, GroupVector& o,real8 timeout) {
	return new Hunter3(id,s,o,createHunter2,"Hunter3",timeout);
}

Task* createJoinSpreadAttack(uint4 id,SimulationState* s, GroupVector& o,real8 timeout) {
	return new JoinSpreadAttack(id,s,o,createJoinSpreadAttack,"JoinSpreadAttack",timeout);
}

/*********************JAMES' STUFF **************************/  
NUS_TerrainBase::Loc find_a_spot(NUS_TerrainBase::Loc center_loc, GameStateModule& gsm, SimulationState *state){

	mapTerrain::TerrainMap terrMap(&gsm.get_game(), state);
	NUS_TerrainBase::Loc loc(0,0);
	sint4 standard = 9; sint4 range = 128;
	bool found = terrMap.find_free_loc(center_loc, loc, range, standard);
	range+= 32;
	//std::cout << "found = " << found << ", and meeting spot is (" <<  center_loc.x / 16 << "," << center_loc.y / 16 << ")" << std::endl;
	while(!found  && standard > 4){
		found = terrMap.find_free_loc(center_loc, loc, range, standard--);
	//std::cout << "found = " << found << ", and meeting spot is (" <<  center_loc.x / 16 << "," << center_loc.y / 16 << ")" << std::endl;
	}
	terrMap.print_map();
	return NUS_TerrainBase::Loc(loc.x, loc.y);
}
/*********************JAMES' STUFF **************************/ 


}



