#ifndef NUS_TASK_H
#define NUS_TASK_H


#include "Global.H"
#include "LowAI.H"
#include "EventHandler.H"
#include "ClientModule.H"

#include "NUS_TerrainBase.H"
#include "NUS_Vector3D.h"

namespace NUS_Simul {

	class Task;
	class SimulationState;
	class Base;

	class Group;
	typedef Vector<Group*> GroupVector;

	class Unit;
	typedef Vector<Unit*> UnitVector;
	
//typedef Vector<uint4> GroupVector;
	typedef Vector<Task*> TaskVector;

//typedef Task* (*TaskCreateFunctionPtr)(uint4 id,GameState* s,GroupVector& g);
	typedef Task* (*TaskCreateFunctionPtr)(uint4 id,SimulationState* s, GroupVector& o, real8 timeout);

// timeout refers to time it would take to complete moving a certain distance
	class MoveStruct
	{
	public:

		MoveStruct(const NUS_TerrainBase::Loc& p); 

		MoveStruct(Base *base); 
		
		MoveStruct(Unit *unit); 
		
		MoveStruct* clone() { return new MoveStruct(*this); }
		NUS_TerrainBase::Loc pos;
		Base* base;
		Unit* unit;
		
	private:
		MoveStruct();
		MoveStruct &operator=(const MoveStruct &);
	};

	class Task
	{
	public:

		Task();
	/*
		Task(Task* parent,GroupVector& origGroups,TaskCreateFunctionPtr function,std::string name,
			bool baseAttackPreference = false,MoveStruct* extraData = NULL);
		Task(uint4 playerID,GameState* state,GroupVector& origGroups,TaskCreateFunctionPtr function,std::string name,
			bool baseAttackPreference = false,MoveStruct* extraData = NULL);
*/
		Task(Task* parent, GroupVector& groupVector,TaskCreateFunctionPtr function,std::string name,real8 timeout,
			MoveStruct* extraData = NULL);
		Task(uint4 playerID,SimulationState* state,GroupVector& groupVector,TaskCreateFunctionPtr function,
			std::string name,real8 timeout,MoveStruct* extraData = NULL);

		virtual ~Task();

		virtual Task* clone(SimulationState* state,Task* parent)=0;

		SimulationState* getSimulationState(){ return state;}
		Task* getParent(){ return parent;}

		GroupVector& getGroups(){ return groups; }
		//ObjectVector& getObjects() { return objects;}

		uint4 getPlayerID() const {return playerID;}

		void setLineNum(uint4 lineN) { lineNum = lineN;}
		uint4 getLineNum() const { return lineNum;}

		TaskCreateFunctionPtr getFunction(){return function;}
		std::string getName() const {return name;}

		// Design: To create new groupings of units, delete the old ones
		// and create new ones with what you want. Calling removeGroup will
		// deallocate memory for that group.
		void addGroup(Group *group);
	 	void removeGroup(Group* group);
		void removeAllGroups();
		
		// Merge will copy the units from other to original, and delete other
		void merge(Group *original, Group *other);
	
		// Do NOT use this function, it is used by Player to remove units from groups
		// when they have actually died in the game
		bool removeUnit(Unit* unit);
		//void removeObject(GameObj* obj);
		
		void childCompleted(Task* child);
		bool update();

		void setTimeout(real8 time){timeout = time;}
		real8 getTimeout() const{return timeout;}

		void setStartTime(real8 time){startTime = time;}
		real8 getStartTime() const {return startTime;}

		virtual bool exec()=0;

	protected:

		Task* parent;
		TaskVector children;

		SimulationState* state;

		GroupVector groups;

		uint4 playerID;
		uint4 lineNum;
		MoveStruct* extraData;

		TaskCreateFunctionPtr function; // right now, ONLY used for identification!

		Task(const Task&);
		std::string name;

		real8 startTime;
		real8 timeout;

	private:
		Task &operator=(const Task &);
	public:
		//travoattack functions
		float dist2dCompare(NUS_TerrainBase::Loc a, NUS_TerrainBase::Loc b);
		float dist2dCompare(float a,float b,float x,float y);
		
		Unit* GetNearestEnemy(NUS_TerrainBase::Loc pos);
		float dist2d(NUS_TerrainBase::Loc a, NUS_TerrainBase::Loc b);
		NUS_TerrainBase::Loc ToTerrainBase(Vector3D vec3);
		Vector3D ToVector3D(NUS_TerrainBase::Loc pos);
		void MoveTo(Group* unit, Vector3D destination, float recommendedspeed);
		float angfrom (float x1, float y1, float x2, float y2);
		bool OffMapLimits(Vector3D destination);   
		bool TestPtClear(float x, float y, Unit* ourunit);
		Vector3D MTFA(float x, float y, float heading, float speed, Unit* ourunit);
		void Push(Group* unit, Vector3D newpos);
		int turnangle (float curDirection, float desDirection, float turnspeed);
		float angdist (float curDirection, float desDirection);
		int HowManyInFrontOfMe(Group* me, Vector3D dest, float angwidth);
		float HowMuchFriendlyInfluence(Vector3D pos, Vector3D centerpos);
		
		
	
	};

}

REGISTER_TYPEOF(2821, NUS_Simul::TaskVector::iterator);
REGISTER_TYPEOF(2822, NUS_Simul::TaskVector::const_iterator);

#endif

