// $Id: NUS_Unit.cpp,v 1.1 2006/08/10 17:16:02 nicholas Exp $

#include "NUS_Unit.h"


namespace NUS_Simul{

	Unit::Unit(){
		position = NUS_TerrainBase::Loc(0,0);
		hitPoints = 0;
	}

	Unit::Unit(GameObj* obj,real8 hp, NUS_TerrainBase::Loc loc){
		this->obj = obj;
		hitPoints = hp;
		position = loc;
	}
	
	Unit::Unit(GameObj *obj) {
		this->obj = obj;
		hitPoints = real8(obj->get_int("hp"));
		position = NUS_TerrainBase::Loc(*obj->sod.x, *obj->sod.y);
		lastFrame = 0;
		
		for  (int i = 0; i < FPS; ++i) 
			previousLocs[i] = position;		
	}

	Unit::Unit(const Unit &u):hitPoints(u.hitPoints), position(u.position), obj(u.obj) {
		lastFrame = u.lastFrame;
		
		for (int i = 0; i < FPS; ++i) {
			previousLocs[i] = u.previousLocs[i];
		}
	}

	Unit* Unit::clone() const {
		return new Unit(*this);
	}	
	
	void Unit::initPreviousLocs() {
		lastFrame = 0;
		
		for (int i = 0; i < FPS; ++i) 
			previousLocs[i] = position;		
	}
	
	Vector3D Unit::velocity() {
		int first = lastFrame;
		int last = lastFrame - 1;
		
		if (last < 0) last = FPS-1;
		
		return Vector3D( previousLocs[first].x - previousLocs[last].x, previousLocs[first].y - previousLocs[last].y, 0.0f );
	}
	
	void Unit::addNewLoc(NUS_TerrainBase::Loc pos) {
		previousLocs[lastFrame++] = pos;
		
		if (lastFrame == FPS) lastFrame = 0;
	}

}


