// $Id: MyPathfindEnvironment.C 5279 2007-06-23 02:49:08Z mburo $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include"MyPathfindEnvironment.H"

MyPathfindEnvironment::MyPathfindEnvironment()
    :PathFind::Environment(),
     graph(NULL),
     m_start(-1),
     m_goal(-1)
{}

void MyPathfindEnvironment::attach_graph(const PolyGraph* new_graph)
{
  if (new_graph==NULL) return;
  graph = new_graph;
}

void MyPathfindEnvironment::detach_graph()
{
  graph = NULL;
}

node_id_type MyPathfindEnvironment::point_to_id(const ScalarPoint &/*pt*/) const
{
  return 0;
}

Point2i MyPathfindEnvironment::id_to_point(node_id_type id) const
{
  Graph_node *node = graph->all_nodes.at(id);
  return node->p;
}

void MyPathfindEnvironment::set_start(node_id_type id)
{
  m_start = id;
}

void MyPathfindEnvironment::set_goal(node_id_type id)
{
  m_goal = id;
}

int MyPathfindEnvironment::getHeuristic(node_id_type start, node_id_type target) const
{
  Point2i p1 = id_to_point(start);
  Point2i p2 = id_to_point(target);

  real4 x = abs(p1.x - p2.x);
  real4 y = abs(p1.y - p2.y);

  return (int)(sqrt(x*x+y*y));
}

int MyPathfindEnvironment::getMaxCost() const
{
  return 1;
}

int MyPathfindEnvironment::getMinCost() const
{
  return 1000;
}

int MyPathfindEnvironment::getCost(node_id_type source, node_id_type target) const
{
  Point2i p1 = id_to_point(source);
  Point2i p2 = id_to_point(target);

  real4 x = abs(p1.x - p2.x);
  real4 y = abs(p1.y - p2.y);

  return (int)(sqrt(x*x+y*y));
}

int MyPathfindEnvironment::getNumberNodes() const
{
  return graph->all_nodes.size();
}
    
void MyPathfindEnvironment::getSuccessors(node_id_type nodeId, node_id_type lastNodeId,
					  std::vector<Successor>& result,
					  bool usePruning) const
{
  Graph_node* curNode = graph->all_nodes.at(nodeId);

  result.reserve(curNode->neighbours.size());
  result.clear();

  FORALL (curNode->neighbours, i) {

    Graph_node* neighbour = *i;

    node_id_type id_next = neighbour->index;
	  
    if (usePruning && lastNodeId == id_next) continue;

    int cost = getCost(nodeId, id_next);

    result.push_back(Successor(id_next,cost));
  }
}

bool MyPathfindEnvironment::isValidNodeId(node_id_type nodeId) const
{
  return nodeId > -1 && nodeId < (int)graph->all_nodes.size();
}
