// $Id: PolyGraph.C 5279 2007-06-23 02:49:08Z mburo $

#include"PolyGraph.H"

PolyGraph::~PolyGraph() {
  while (!all_nodes.empty()) {
    Graph_node* current = all_nodes.back();
    while (!current->neighbours.empty()) {
      current->neighbours.pop_back();
    }
    all_nodes.pop_back();
  }
}

void PolyGraph::clear_graph() {
    if (all_nodes.empty()) {
	  return;
    }

    FORALL(all_nodes,i) {
	  Graph_node* current = *i;
	  while(!current->neighbours.empty()) {
		current->neighbours.pop_back();
	  }
    }

    while(!all_nodes.empty()) {
	  Graph_node* current = all_nodes.back();
	  all_nodes.pop_back();
	  
	  delete current;
    }
}
void PolyGraph::build_graph(Decomp* decomposition) {
  count = 0;
    
  //mark them as not visited
  FORALL(decomposition->nodes,i) {
    Decomp::Node *n = *i;
    if (n->type!=Decomp::Node::R_NODE) continue;

    n->visited = false;
  }
    
  Vector<Graph_node*> pts;
    
  //start building the graph
  FORALL(decomposition->nodes,j) {
    Decomp::Node *n = *j;
	  
    if (n->type!=Decomp::Node::R_NODE) {
      continue;
    }

    // n->write();
    
    //current rectangle
    Decomp::Rect r0 = n->r;

    Graph_node* new_node = new Graph_node();

    new_node->index = all_nodes.size();

    new_node->p.x = (r0.x0 + r0.x1)/2;
    new_node->p.y = (r0.y0 + r0.y1)/2;
    new_node->n1 = n;
    new_node->n2 = n;
    
    pts.push_back(new_node);
    all_nodes.push_back(new_node);
    
    //north neighbour
#if 0
    if (n->nl[0].empty()) {
      Point2i p;
      p.x = (r0.x0 + r0.x1)/2;
      p.y = r0.y0;

      Graph_node* new_node = new Graph_node();

      new_node->index = all_nodes.size();

	  
      new_node->p.x = p.x;
      new_node->p.y = p.y;

      new_node->n1 = n;
      new_node->n2 = NULL;

      pts.push_back(new_node);

      all_nodes.push_back(new_node);
    }
#endif
    FORALL(n->nl[0],it0) {
      Decomp::Node *neighbour = *it0;

	
      if (neighbour->type!=Decomp::Node::R_NODE) {
	cout<<"??"<<endl;
	continue;
      }

      if (neighbour->data != n->data || n->data == 1 || neighbour->data == 1) {
	continue;
      }
	
      //handle it differently if the neighbour rectangle is visited
      if (neighbour->visited) {
	Graph_node *already_exist = get_node(n,neighbour);
	if (already_exist==NULL) {
	  ERR("should not be happening");
	}

	pts.push_back(already_exist);

	continue;
      }

      Decomp::Rect r1 = neighbour->r;

      Point2i p1,p2,p3,p4;

      p1.x = r0.x0;
      p1.y = r0.y1;
      p2.x = r0.x1;
      p2.y = r0.y1;

      p3.x = r1.x0;
      p3.y = r1.y0;
      p4.x = r1.x1;
      p4.y = r1.y0;
		    
      Point2i p = get_point(p1,p2,p3,p4);

      Graph_node* new_node = new Graph_node;

      new_node->p = p;
      new_node->n1 = n;
      new_node->n2 = neighbour;

      pts.push_back(new_node);

      //add it to the list if it is not added yet

      new_node->index = all_nodes.size();
      all_nodes.push_back(new_node);
    }
    //east neighbour
#if 0
    if (n->nl[1].empty()) {
      Point2i p;
      p.x = r0.x1;
      p.y = (r0.y0 + r0.y1)/2;

      Graph_node* new_node = new Graph_node();
      new_node->p.x = p.x;
      new_node->p.y = p.y;

      new_node->n1 = n;
      new_node->n2 = NULL;

      pts.push_back(new_node);

      new_node->index = all_nodes.size();
      all_nodes.push_back(new_node);
    }
#endif
    FORALL(n->nl[1],it1) {
      Decomp::Node *neighbour = *it1;

      if (neighbour->type!=Decomp::Node::R_NODE) {
	cout<<"??"<<endl;
	continue;
      }
      if (neighbour->data != n->data || n->data == 1 || neighbour->data == 1) {
	continue;
      }

      //handle it differently if the neighbour rectangle is visited
      if (neighbour->visited) {
	Graph_node *already_exist = get_node(n,neighbour);
	if (already_exist==NULL) {
	  ERR("should not be happening");
	}

	pts.push_back(already_exist);

	continue;
      }
		
      Decomp::Rect r1 = neighbour->r;

      Point2i p1,p2,p3,p4;

      p1.x = r0.x1;
      p1.y = r0.y1;
      p2.x = r0.x1;
      p2.y = r0.y0;

      p3.x = r1.x0;
      p3.y = r1.y1;
      p4.x = r1.x0;
      p4.y = r1.y0;
		    
      Point2i p = get_point(p1,p2,p3,p4);

      Graph_node* new_node = new Graph_node;

      new_node->p = p;
      new_node->n1 = n;
      new_node->n2 = neighbour;

      pts.push_back(new_node);

      //add it to the list if it is not added yet
      new_node->index = all_nodes.size();
      all_nodes.push_back(new_node);
    }

    //south neighbour
#if 0
    if (n->nl[2].empty()) {
      Point2i p;
      p.x = (r0.x0 + r0.x1)/2;
      p.y = r0.y1;

      Graph_node* new_node = new Graph_node();
      new_node->p.x = p.x;
      new_node->p.y = p.y;

      new_node->n1 = n;
      new_node->n2 = NULL;

      pts.push_back(new_node);
	  
      new_node->index = all_nodes.size();
      all_nodes.push_back(new_node);
    }
#endif
    FORALL(n->nl[2],it2) {
      Decomp::Node *neighbour = *it2;

      if (neighbour->type!=Decomp::Node::R_NODE) {
	cout<<"??"<<endl;
	continue;
      }

      if (neighbour->data != n->data || n->data ==1 || neighbour->data ==1) {
	continue;
      }

      //handle it differently if the neighbour rectangle is visited
      if (neighbour->visited) {
	Graph_node *already_exist = get_node(n,neighbour);
	if (already_exist==NULL) {
	  ERR("should not be happening");
	}

	pts.push_back(already_exist);
	continue;
      }

      Decomp::Rect r1 = neighbour->r;

      Point2i p1,p2,p3,p4;

      p1.x = r0.x0;
      p1.y = r0.y0;
      p2.x = r0.x1;
      p2.y = r0.y0;

      p3.x = r1.x0;
      p3.y = r1.y1;
      p4.x = r1.x1;
      p4.y = r1.y1;
		    
      Point2i p = get_point(p1,p2,p3,p4);

      Graph_node* new_node = new Graph_node;

      new_node->p = p;
      new_node->n1 = n;
      new_node->n2 = neighbour;

      pts.push_back(new_node);

      //add it to the list if it is not added yet
      new_node->index = all_nodes.size();
      all_nodes.push_back(new_node);
    }

    //west neighbour
#if 0
    if (n->nl[3].empty()) {
      Point2i p;
      p.x = r0.x0;
      p.y = (r0.y0 + r0.y1)/2;

      Graph_node* new_node = new Graph_node();
      new_node->p.x = p.x;
      new_node->p.y = p.y;

      new_node->n1 = n;
      new_node->n2 = NULL;

      pts.push_back(new_node);
	  
      new_node->index = all_nodes.size();
      all_nodes.push_back(new_node);
    }
#endif
    FORALL(n->nl[3],it3) {
      Decomp::Node *neighbour = *it3;

      if (neighbour->type!=Decomp::Node::R_NODE) {
	cout<<"??"<<endl;
	continue;
      }

      if (neighbour->data != n->data || n->data ==1 || neighbour->data ==1) {
	continue;
      }

      //handle it differently if the neighbour rectangle is visited
      if (neighbour->visited) {
	Graph_node *already_exist = get_node(n,neighbour);
	if (already_exist==NULL) {
	  ERR("should not be happening");
	}

	pts.push_back(already_exist);
	continue;
      }
		
      Decomp::Rect r1 = neighbour->r;

      Point2i p1,p2,p3,p4;

      p1.x = r0.x0;
      p1.y = r0.y0;
      p2.x = r0.x0;
      p2.y = r0.y1;

      p3.x = r1.x1;
      p3.y = r1.y1;
      p4.x = r1.x1;
      p4.y = r1.y0;
		    
      Point2i p = get_point(p1,p2,p3,p4);

      Graph_node* new_node = new Graph_node;

      new_node->p = p;
      new_node->n1 = n;
      new_node->n2 = neighbour;

      pts.push_back(new_node);

      //add it to the list if it is not added yet
      new_node->index = all_nodes.size();
      all_nodes.push_back(new_node);
    }

#if 0
    //interconnect all nodes
    FORALL(pts,i) {
      Graph_node *node = *i;
		
      FORALL(pts,j) {
	Graph_node *another = *j;
	if (node!=another) {

	  if (!node->has_neighbour(another)) {
	    node->neighbours.push_back(another);
	    another->neighbours.push_back(node);
			
	    //cout<<"connecting:("<<node->p.x<<","<<node->p.y<<")-->("<<another->p.x<<","<<another->p.y<<")\n";
	  }
	}
      }
    }
#endif

    FORALL(pts,i) {
      if (i!=pts.begin()) {

	Graph_node *node = *i;
		
	node->neighbours.push_back(pts.front());

	pts.front()->neighbours.push_back(node);
      }
    }

    pts.clear();
    
    while(!pts.empty()) {
      pts.pop_back();
    }

    //mark the current as visited
    n->visited = true;
	  
  }
	  
}

void PolyGraph::build_graph_redundant(Decomp* decomposition) {

  //mark them as not visited
  FORALL(decomposition->nodes,i) {
    Decomp::Node *n = *i;
    if (n->type!=Decomp::Node::R_NODE) continue;

    n->visited = false;

  }
    
  Vector<Graph_node*> pts;
    
  //start building the graph
  FORALL(decomposition->nodes,j) {
    Decomp::Node *n = *j;
	  
    if (n->type!=Decomp::Node::R_NODE) {
      continue;
    }

    // n->write();
    
    //current rectangle
    Decomp::Rect r0 = n->r;

    Graph_node* new_node = new Graph_node();

    new_node->p.x = (r0.x0 + r0.x1)/2;
    new_node->p.y = (r0.y0 + r0.y1)/2;
    new_node->n1 = n;
    new_node->n2 = n;
    pts.push_back(new_node);

    new_node->index = all_nodes.size();
    all_nodes.push_back(new_node);
    
    //north neighbour
    FORALL(n->nl[0],it0) {
      Decomp::Node *neighbour = *it0;

	
      if (neighbour->type!=Decomp::Node::R_NODE) {
	cout<<"??"<<endl;
	continue;
      }

      if (neighbour->data != n->data || n->data == 1 || neighbour->data == 1) {
	continue;
      }
	
      //handle it differently if the neighbour rectangle is visited
      if (neighbour->visited) {
	Vector<Graph_node*> temp;
	    
	if (!get_nodes(n, neighbour, temp)) {
	  ERR("should not be happening");
	}

	while(!temp.empty()) {
	  pts.push_back(temp.back());
	  temp.pop_back();
	}
	continue;
      }

      Decomp::Rect r1 = neighbour->r;

      Point2i p1,p2,p3,p4;

      p1.x = r0.x0;
      p1.y = r0.y1;
      p2.x = r0.x1;
      p2.y = r0.y1;

      p3.x = r1.x0;
      p3.y = r1.y0;
      p4.x = r1.x1;
      p4.y = r1.y0;

      Vector<Point2i> temp_pts;
	
      get_points(p1, p2, p3, p4, temp_pts);

      while(!temp_pts.empty()) {
	    
	Graph_node* new_node = new Graph_node;
	    
	new_node->p = temp_pts.back();
	temp_pts.pop_back();
	new_node->n1 = n;
	new_node->n2 = neighbour;
	    
	pts.push_back(new_node);

	//add it to the list if it is not added yet
	new_node->index = all_nodes.size();
	    
	all_nodes.push_back(new_node);
      }
    }
    //east neighbour
    FORALL(n->nl[1],it1) {
      Decomp::Node *neighbour = *it1;

      if (neighbour->type!=Decomp::Node::R_NODE) {
	cout<<"??"<<endl;
	continue;
      }
      if (neighbour->data != n->data || n->data == 1 || neighbour->data == 1) {
	continue;
      }

      //handle it differently if the neighbour rectangle is visited
      if (neighbour->visited) {
	Vector<Graph_node*> temp;
	    
	if (!get_nodes(n,neighbour,temp)) {
	  ERR("should not be happening");
	}

	while(!temp.empty()) {
	  pts.push_back(temp.back());
	  temp.pop_back();
	}
	continue;
      }
		
      Decomp::Rect r1 = neighbour->r;

      Point2i p1,p2,p3,p4;

      p1.x = r0.x1;
      p1.y = r0.y1;
      p2.x = r0.x1;
      p2.y = r0.y0;

      p3.x = r1.x0;
      p3.y = r1.y1;
      p4.x = r1.x0;
      p4.y = r1.y0;

      Vector<Point2i> temp_pts;
	
      get_points(p1,p2,p3,p4,temp_pts);

      while(!temp_pts.empty()) {
	    
	Graph_node* new_node = new Graph_node;
	    
	new_node->p = temp_pts.back();
	temp_pts.pop_back();
	new_node->n1 = n;
	new_node->n2 = neighbour;
	    
	pts.push_back(new_node);

	//add it to the list if it is not added yet
	new_node->index = all_nodes.size();
	    
	all_nodes.push_back(new_node);
      }
	
    }

    //south neighbour
    FORALL(n->nl[2],it2) {
      Decomp::Node *neighbour = *it2;

      if (neighbour->type!=Decomp::Node::R_NODE) {
	cout<<"??"<<endl;
	continue;
      }

      if (neighbour->data != n->data || n->data ==1 || neighbour->data ==1) {
	continue;
      }

      //handle it differently if the neighbour rectangle is visited
      if (neighbour->visited) {
	Vector<Graph_node*> temp;
	    
	    
	if (!get_nodes(n,neighbour,temp)) {
	  ERR("should not be happening");
	}

	while(!temp.empty()) {
	  pts.push_back(temp.back());
	  temp.pop_back();
	}
	continue;
      }

      Decomp::Rect r1 = neighbour->r;

      Point2i p1,p2,p3,p4;

      p1.x = r0.x0;
      p1.y = r0.y0;
      p2.x = r0.x1;
      p2.y = r0.y0;

      p3.x = r1.x0;
      p3.y = r1.y1;
      p4.x = r1.x1;
      p4.y = r1.y1;

      Vector<Point2i> temp_pts;
	
      get_points(p1,p2,p3,p4,temp_pts);

      while(!temp_pts.empty()) {
	    
	Graph_node* new_node = new Graph_node;
	    
	new_node->p = temp_pts.back();
	temp_pts.pop_back();
	new_node->n1 = n;
	new_node->n2 = neighbour;
	    
	pts.push_back(new_node);

	//add it to the list if it is not added yet
	new_node->index = all_nodes.size();
	    
	all_nodes.push_back(new_node);
      }
      
    }

    //west neighbour
    FORALL(n->nl[3],it3) {
      Decomp::Node *neighbour = *it3;

      if (neighbour->type!=Decomp::Node::R_NODE) {
	cout<<"??"<<endl;
	continue;
      }

      if (neighbour->data != n->data || n->data ==1 || neighbour->data ==1) {
	continue;
      }

      //handle it differently if the neighbour rectangle is visited
      if (neighbour->visited) {
	Vector<Graph_node*> temp;
	    
	    
	if (!get_nodes(n,neighbour,temp)) {
	  ERR("should not be happening");
	}

	while(!temp.empty()) {
	  pts.push_back(temp.back());
	  temp.pop_back();
	}
	continue;
      }
		
      Decomp::Rect r1 = neighbour->r;

      Point2i p1,p2,p3,p4;

      p1.x = r0.x0;
      p1.y = r0.y0;
      p2.x = r0.x0;
      p2.y = r0.y1;

      p3.x = r1.x1;
      p3.y = r1.y1;
      p4.x = r1.x1;
      p4.y = r1.y0;
		    
      Vector<Point2i> temp_pts;
	
      get_points(p1,p2,p3,p4,temp_pts);

      while(!temp_pts.empty()) {
	    
	Graph_node* new_node = new Graph_node;
	    
	new_node->p = temp_pts.back();
	temp_pts.pop_back();
	new_node->n1 = n;
	new_node->n2 = neighbour;
	    
	pts.push_back(new_node);

	//add it to the list if it is not added yet
	new_node->index = all_nodes.size();
	    
	all_nodes.push_back(new_node);
      }
    }

    //interconnect all nodes
    FORALL(pts,i) {
      Graph_node *node = *i;
		
      FORALL(pts,j) {
	Graph_node *another = *j;
	if (node!=another) {

	  if (!node->has_neighbour(another)) {
	    node->neighbours.push_back(another);
	    another->neighbours.push_back(node);
			
	    //cout<<"connecting:("<<node->p.x<<","<<node->p.y<<")-->("<<another->p.x<<","<<another->p.y<<")\n";
	  }
	}
      }
    }

    while(!pts.empty()) {
      pts.pop_back();
    }

    //mark the current as visited
    n->visited = true;
	  
  }
	  
}
//return midpoint of the interval

Point2i PolyGraph::get_point(Point2i p1, Point2i p2, Point2i p3, Point2i p4) {

  Point2i result;

  
    
  if (p1.x == p2.x && p1.x == p3.x && p1.x == p4.x) {
    std::priority_queue<sint4, Vector<sint4>, std::less<sint4> > pq; 
    pq.push(p1.y);
    pq.push(p2.y);
    pq.push(p3.y);
    pq.push(p4.y);

    sint4 s1,s2;
    pq.pop();
    s2 = pq.top();
    pq.pop();
    s1 = pq.top();
    pq.pop();
    pq.pop();

        
    result.y = (s1+s2)/2;

    result.x = p1.x; 
  }

  if (p1.y == p2.y && p1.y == p3.y && p1.y == p4.y) {
	  
    std::priority_queue<sint4, Vector<sint4>, std::less<sint4> > pq;
    pq.push(p1.x);
    pq.push(p2.x);
    pq.push(p3.x);
    pq.push(p4.x);

    sint4 s1,s2;
    pq.pop();
    s2 = pq.top();
    pq.pop();
    s1 = pq.top();
    pq.pop();
    pq.pop();

    result.x = (s1+s2)/2;

    result.y = p1.y;
	  
  }

  /*
  cout<<p1.x<<"(1)"<<p1.y<<endl;
  cout<<p2.x<<"(2)"<<p2.y<<endl;
  cout<<p3.x<<"(3)"<<p3.y<<endl;
  cout<<p4.x<<"(4)"<<p4.y<<endl;
  
  cout<<result.x<<" "<<result.y<<endl;
  */
  return result;
}

//return a set of points at the boundary
void PolyGraph::get_points(Point2i p1, Point2i p2, Point2i p3, Point2i p4,
			   Vector<Point2i> &results)
{
  Point2i result1;
  Point2i result2;
  Point2i result3;
  
  if (p1.x == p2.x && p1.x == p3.x && p1.x == p4.x) {
    std::priority_queue<sint4, Vector<sint4>, std::less<sint4> > pq;
    pq.push(p1.y);
    pq.push(p2.y);
    pq.push(p3.y);
    pq.push(p4.y);

    sint4 s1,s2;
    pq.pop();
    s2 = pq.top();
    pq.pop();
    s1 = pq.top();
    pq.pop();
    pq.pop();

    result1.x = p1.x;
    result1.y = s1;

    result2.x = p1.x;
    result2.y = s2;
    
    result3.y = (s1+s2)/2;

    result3.x = p1.x; 
  }

  if (p1.y == p2.y && p1.y == p3.y && p1.y == p4.y) {
	  
    std::priority_queue<sint4, Vector<sint4>, std::less<sint4> > pq;
    pq.push(p1.x);
    pq.push(p2.x);
    pq.push(p3.x);
    pq.push(p4.x);

    sint4 s1,s2;
    pq.pop();
    s2 = pq.top();
    pq.pop();
    s1 = pq.top();
    pq.pop();
    pq.pop();

    result1.x = s1;
    result1.y = p1.y;

    result2.x = s2;
    result2.y = p1.y;
    
    result3.x = (s1+s2)/2;

    result3.y = p1.y;
	  
  }

  results.push_back(result1);
  results.push_back(result2);
  results.push_back(result3);

  /*
  cout<<p1.x<<"(1)"<<p1.y<<endl;
  cout<<p2.x<<"(2)"<<p2.y<<endl;
  cout<<p3.x<<"(3)"<<p3.y<<endl;
  cout<<p4.x<<"(4)"<<p4.y<<endl;
  
  cout<<result.x<<" "<<result.y<<endl;
  */
  return;
}


Graph_node* PolyGraph::get_node(Decomp::Node *n, Decomp::Node *neighbour)
{
  FORALL(all_nodes,it) {
    Graph_node *cur = *it;
    if (cur->n1 == n && cur->n2 == neighbour) return cur;
    if (cur->n2 == n && cur->n1 == neighbour) return cur;
  }

  return NULL;
}

bool PolyGraph::get_nodes(Decomp::Node *n, Decomp::Node *neighbour, Vector<Graph_node*> &nodes)
{
  FORALL(all_nodes,it) {
    Graph_node *cur = *it;
    if (cur->n1 == n && cur->n2 == neighbour) {
      nodes.push_back(cur);
      continue;
    }
    if (cur->n2 == n && cur->n1 == neighbour) {
      nodes.push_back(cur);
      continue;
    }
  }
  if (nodes.empty()) {
    return false;
  }
  return true;
}

int PolyGraph::insert_node(Point2i p, Decomp::Node *n)
{
    vector<Graph_node*> pts;

#if 0
    //handle overshooting
    if (n->data == 1) {
	  Decomp::Node* nearest=NULL;

	  double distance = 0;
	  Point2i exit;

	  bool found = false;

	  Decomp::Rect r0 = n->r;

	  //point on the west boundary
	  if (p.x == r0.x0) {
		FORALL(n->nl[3],i) {
		    (*i)->write();
		    if ((*i)->data==0) {
			  Decomp::Rect r1 = (*i)->r;

			  if (p.y>= r1.y0 && p.y <= r1.y1) {
				found = true;
				nearest = *i;
				exit.x = p.x;
				exit.y = (r1.y0+r1.y1)/2;
				break;
			  }
		    }
		}
	  }
	  //point on the east boundary
	  if (p.x == r0.x1) {
		FORALL(n->nl[1],i) {
		    (*i)->write();
		    if ((*i)->data==0) {
			  Decomp::Rect r1 = (*i)->r;

			  if (p.y>= r1.y0 && p.y <= r1.y1) {
				found = true;
				nearest = *i;
				exit.x = p.x;
				exit.y = (r1.y0+r1.y1)/2;
				break;
			  }
		    }
		}
	  }
	  //point on the north boundary
	  if (p.y == r0.y0) {
		FORALL(n->nl[2],i) {
		    (*i)->write();
		    if ((*i)->data==0) {
			  Decomp::Rect r1 = (*i)->r;

			  if (p.x>= r1.x0 && p.x <= r1.x1) {
				found = true;
				nearest = *i;
				exit.x = (r1.x0+r1.x1)/2;
				exit.y = p.y;
				break;
			  }
		    }
		}
	  }
	  //point on the south boundary
	  if (p.y == r0.y1) {
		FORALL(n->nl[0],i) {
		    (*i)->write();
		    if ((*i)->data==0) {
			  Decomp::Rect r1 = (*i)->r;

			  if (p.y>= r1.y0 && p.y <= r1.y1) {
				found = true;
				nearest = *i;
				exit.x = (r1.x0+r1.x1)/2;
				exit.y = p.y;
				break;
			  }
		    }
		}
	  }
	  if (!found) {
		cout<<p.x<<" "<<p.y<<endl;
		n->write();
		cout<<exit.x<<" "<<exit.y<<endl;
		cout<<"not on boundary"<<endl;
		return -1;
	  }
	  Graph_node* new_node = new Graph_node(p);
	  new_node->n1 = n;
	  new_node->n2 = n;

	  Graph_node* new_node2 = new Graph_node(exit);
	  new_node2->n1 = n;
	  new_node2->n2 = nearest;

	  new_node->neighbours.push_back(new_node2);
	  new_node2->neighbours.push_back(new_node);


	  cout<<"result"<<endl;
	  
	  n->write();

	  if (nearest==NULL) {
		cout<<"NULL"<<endl;
	  } else {
		nearest->write();
	  }
	  cout<<p.x<<" "<<p.y<<endl;
	  
	  cout<<exit.x<<" "<<exit.y<<endl;
	  
	  FORALL(all_nodes,i) {
		Graph_node* cur = *i;
		
		if (cur->n1 == n || cur->n2 == n) {
		    pts.push_back(cur);
		}
	  }

	  FORALL(pts,j) {
		new_node2->neighbours.push_back(*j);
		(*j)->neighbours.push_back(new_node2);
	  }

	  new_node2->index = all_nodes.size();
	  all_nodes.push_back(new_node2);

	  new_node->index = all_nodes.size();
	  all_nodes.push_back(new_node);

	  pts.clear();
	  
	  return new_node->index;
	  
    }
#endif

    FORALL(all_nodes,i) {
	  Graph_node* cur = *i;

	  if (cur->n1 == n || cur->n2 == n) {
		pts.push_back(cur);
	  }
    }

    Graph_node* new_node = new Graph_node(p);

    new_node->n1 = n;
    new_node->n2 = n;
    
    FORALL(pts,j) {
	  new_node->neighbours.push_back(*j);
	  (*j)->neighbours.push_back(new_node);
    }

    new_node->index = all_nodes.size();
    all_nodes.push_back(new_node);

    pts.clear();
    
    return new_node->index;
}
void PolyGraph::delete_last_two_nodes()
{
    Graph_node* cur_1 = all_nodes.back();

    for(vector<Graph_node*>::iterator i = cur_1->neighbours.begin();
	  i!=cur_1->neighbours.end();i++) {

	  Graph_node *node = *i;

	  if (node->neighbours.back()==cur_1) {
		node->neighbours.pop_back();
	  } else {
		cout<<"neighbour list corrupted, should not be happenning"<<endl;
	  }
    }

    cur_1->neighbours.clear();

    all_nodes.pop_back();
    
    Graph_node* cur_2 = all_nodes.back();

    for(vector<Graph_node*>::iterator i = cur_2->neighbours.begin();
	  i!=cur_2->neighbours.end();i++) {

	  Graph_node *node = *i;

	  if (node->neighbours.back()==cur_2) {
		node->neighbours.pop_back();
	  } else {
		cout<<"neighbour list corrupted, should not be happening"<<endl;
	  }
    }

    cur_2->neighbours.clear();

    all_nodes.pop_back();

    delete cur_1;
    delete cur_2;

    return;
    
}
void PolyGraph::delete_last_three_nodes()
{
  Graph_node* cur_1 = all_nodes.back();

  for(vector<Graph_node*>::iterator i = cur_1->neighbours.begin();
      i!=cur_1->neighbours.end();i++) {

    Graph_node *node = *i;

    if (node->neighbours.back()==cur_1) {
      node->neighbours.pop_back();
    } else {
      cout<<"neighbour list corrupted, should not be happenning"<<endl;
    }
  }

  cur_1->neighbours.clear();

  all_nodes.pop_back();
    
  Graph_node* cur_2 = all_nodes.back();

  for(vector<Graph_node*>::iterator i = cur_2->neighbours.begin();
      i!=cur_2->neighbours.end();i++) {

    Graph_node *node = *i;

    if (node->neighbours.back()==cur_2) {
      node->neighbours.pop_back();
    } else {
      cout<<"neighbour list corrupted, should not be happening"<<endl;
    }
  }

  cur_2->neighbours.clear();

  all_nodes.pop_back();

  Graph_node* cur_3 = all_nodes.back();

  for(vector<Graph_node*>::iterator i = cur_3->neighbours.begin();
      i != cur_3->neighbours.end(); i++) {

    Graph_node *node = *i;

    if (node->neighbours.back()==cur_3) {
      node->neighbours.pop_back();
    } else {
      cout<<"neighbour list corrupted, should not be happening"<<endl;
    }
  }
  cur_3->neighbours.clear();

  all_nodes.pop_back();
    

  delete cur_1;
  delete cur_2;
  delete cur_3;

  return;
}
void PolyGraph::print_graph()
{
  cout<<"number of nodes: "<<all_nodes.size()<<endl;
    
  FORALL (all_nodes, it) {
    Graph_node *n = *it;

    cout<<"node: ("<<n->p.x<<","<<n->p.y<<")\n";
    cout<<"degrees:"<<n->neighbours.size()<<endl;

    FORALL (n->neighbours, j) {
      cout <<"edge:(" << n->p.x << ","<< n->p.y << ")-->("
	   << (*j)->p.x << "," << (*j)->p.y << ")"
	   << endl;
    }
  }
}
