// $Id: preplay_main.C 5279 2007-06-23 02:49:08Z mburo $

// This is an ORTS file (c) Tim Furtak, licensed under the GPL

#include "Options.H"
#include "ShutDown.H"
#include "SDLinit.H"
#include "Game.H"

#include "Replayer.H"

using namespace std;


const char *ORTSG_VERSION = "GfxClient version 0.91 (2005-Apr-10)";

//===================================================================

int main(int argc, char **argv)
{
 char mydir[81];
 getcwd(mydir, 80);
 chdir(mydir);

 try {

  signals_shut_down(true);
  atexit(SDL_Quit);

  // populate opt with options from various modules
  
  Replayer::Options::add();
  Game::Options::add();
  
  // process command line arguments
  if (Options::process(argc, argv, cerr, ORTSG_VERSION)) exit(20);

  // change seed value to time(0) if 0
  sint4 seed;
  Options::get("-seed", seed);
  if (!seed) seed = time(0);
  Options::set("-seed", seed);
  cout << "seed=" << seed << endl;

  srand48(seed); // used by STL, fixme: avoid (global)

  SDLinit::video_init();

  // populate option objects with command line values
  // and create modules
  
  Vector<string> views;

  Replayer rep;
  Replayer::Options op;
  rep.init("", op, views);

  sint4 p = rep.get_nplayers();

  FILE **files = new FILE*[p];
  FORU (i, p) {
    string num(1, '0'+i);
    string fname = op.load_replay + "." + num;
    fname = sanitize_path(fname);
    files[i] = fopen(fname.c_str(), "w");
    if (!files[i]) ERR2("could not open output file", fname);

    uint4 val = views[i].size();
    fwrite(&val, sizeof(val), 1, files[i]);
    fwrite(views[i].data(), sizeof(char), val, files[i]);
  }

  cout << "start loop" << endl;
  
  while (rep.is_good()) {
    rep.tick(views);

    FORU (i, p) {
      uint4 val = views[i].size();
      fwrite(&val, sizeof(val), 1, files[i]);
      fwrite(views[i].data(), sizeof(char), val, files[i]);
    }
  }

  FORU (i, p) {
    fclose(files[i]);
  }
  
 }
 catch (ExitException& ee)
 {
 	errstr << std::endl << *(ee.get_reason()) << std::endl;
#ifdef GCC
    abort();
#else
 	ABORT;
#endif
 	exit(20);
 }
 return 0;
}
