// $Id: replayer_main.C 5279 2007-06-23 02:49:08Z mburo $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "Options.H"
#include "GfxModule.H"
#include "ShutDown.H"
#include "SDLinit.H"
#include "SoundMixer.H"
#include "LoopFunctor.H"
#include "Game.H"
#include "GameChanges.H"

#include "ClientState.H"
#include "TerrainModule.H"

#include "Replayer.H"
#include "VCRWidget.H"

using namespace std;


const char *ORTSG_VERSION = "GfxClient version 0.91 (2005-Apr-10)";

#define USE_EFENCE 0

extern int EF_PROTECT_FREE, EF_FILL, EF_PROTECT_BELOW, EF_ALLOW_MALLOC_0;


static bool just_drew = false;
sint4 speed;
sint4 last_frame;

struct GfxEventHandler : public EventHandler
{
  GfxEventHandler() {}

  bool handle_event(const Event& e)
  {
    if (e.get_what() == VCRWidget::SET_SPEED) {
      const VCREvent *ve = dynamic_cast<const VCREvent*>(&e);
      assert(ve && "VCR Event");
      speed = ve->get_speed();
      return true;
    }
    return false;
  }
};


class ortsgLooper : public LoopFunctor
{
private:
  GfxModule *gfxm;
  ViewReplayer *rep;
  bool refresh;
  
public:

  ortsgLooper(GfxModule *g, ViewReplayer *r) : gfxm(g), rep(r)
  {
    ::Options::get("-refresh", refresh);
    last_frame = 0;
    speed = 1000;
  }
  
  void loop() const {

    sint4 now = SDL_GetTicks();
    if (now - last_frame > speed) {
      last_frame = now;
      rep->tick();
      gfxm->process_changes();

      const Game &game = gfxm->get_game();
      sint4 vf = game.get_view_frame();
      sint4 ai = game.get_action_frame();
      sint4 sk = game.get_skipped_actions();
      sint4 md = game.get_merged_actions();
      
      if (vf != ai || sk || md > 1) {
        
        cout << "frame " << vf;
        if (ai < 0) cout << " [no action]";
        else if (vf != ai) cout << " [behind by " << (vf - ai) << " frame(s)]";
        if (sk) cout << " [skipped " << sk << "]";
        if (md > 1) cout << " [merged " << md << "]";
        cout << endl;
      }
      
      // don't draw if we are far behind
      if (abs(vf-ai) < 10) {
        gfxm->draw();
      }
      just_drew = true;
    }

    if (gfxm->get_quit()) exit(0);
    
    if (refresh) {

      if (!just_drew) // allow some waiting time after a message
        gfxm->draw();
      else
        just_drew = false;
    }
  }
};

int main(int argc, char **argv)
{
 char mydir[81];
 getcwd(mydir, 80);
 glutInit(&argc, argv);
 chdir(mydir);

 try {
#if USE_EFENCE
  // aggressive efence checks
  EF_PROTECT_FREE = 1;
  EF_FILL = 0xee;
  EF_PROTECT_BELOW = 1;
  EF_ALLOW_MALLOC_0 = 1;
#endif

  signals_shut_down(true);
  atexit(SDL_Quit);

  // populate opt with options from various modules
  
  GfxModule::Options::add();
  
  // process command line arguments
  if (Options::process(argc, argv, cerr, ORTSG_VERSION)) exit(20);


  // change seed value to time(0) if 0
  sint4 seed;
  Options::get("-seed", seed);
  if (!seed) seed = time(0);
  Options::set("-seed", seed);
  cout << "seed=" << seed << endl;

  srand48(seed); // used by STL, fixme: avoid (global)

  SDLinit::video_init();
  SDLinit::network_init();

  // populate option objects with command line values
  // and create modules
  
  GfxModule::Options gfxmo;
  GfxModule gfxm;

  // ortsg event handler
  GfxEventHandler gev;

  //=====================================

  GameChanges gc;
  ActionChanges ac;
  
  ViewReplayer rep;
  ViewReplayer::Options repo;
  rep.init(repo);

  VCRWidget vcr;
  vcr.add_handler(&gev);

  gfxm.init(rep.get_game(), gc, ac, gfxmo);

  // gfx loop (should become a thread)
  ortsgLooper l(&gfxm, &rep);

  cout << "start loop" << endl;
  
  gfxm.start_loop(&l);
    
 } 
 catch (ExitException& ee)
 {
 	errstr << std::endl << *(ee.get_reason()) << std::endl;
#ifdef GCC
    abort();
#else
 	ABORT;
#endif
 	exit(20);
 }
 return 0;
}
