
// $Id: simtest_nogfx_main.C 5279 2007-06-23 02:49:08Z mburo $

#include "Options.H"
#include "ShutDown.H"
#include "SDLinit.H"
//#include "SoundMixer.H"
//#include "LoopFunctor.H"
#include "Game.H"
#include "GameChanges.H"

//#include "GatherAI.H"

#include "SquadCombatAI.H"
//#include "SquadCombatAIWidget.H"

#include "SI_SimulationAI.H"

//#include "ClientState.H"
#include "TerrainModule.H"

#include "SimpleTerrain.H"
//#include "SimpleTerrainWidget.H"

#include "ST_ForceField.H"

//#include "PRATerrain.H"

using namespace std;

//typedef SimpleTerrain::ST_Terrain Terrain;
typedef SimpleTerrain::ST_ForceField Terrain;

//===================================================================

const char *ORTSG_VERSION = "GfxClient version 0.91 (2005-Apr-10)";

#define USE_EFENCE 0

extern int EF_PROTECT_FREE, EF_FILL, EF_PROTECT_BELOW, EF_ALLOW_MALLOC_0;

//===================================================================

 void add_options()
 {
   Options o("simtest nofgx main");
   o.put("-seed", sint4(0), "random number generator seed");
 }


struct NoGfxEventHandler : public EventHandler
{
  GameStateModule &gsm;

  NoGfxEventHandler(GameStateModule &state_) : gsm(state_) { }

  bool handle_event(const Event& e)
  {
    if (e.get_who() == GameStateModule::FROM) {

      //cout << "Event: " << e.who() << " " << e.what() << endl;

      if (e.get_what() == GameStateModule::STOP_MSG) exit(0);

      if (e.get_what() == GameStateModule::READ_ERROR_MSG) exit(10);

      if (e.get_what() == GameStateModule::VIEW_MSG) {


        ////////////////
        Vector<string> temp;
        Game &game =gsm.get_game();
        FORS (i, game.get_player_num()) temp.push_back(""); // dummy list of commands
        game.exec_actions(temp); // evaluate client actions and send actions to the server
        ////////////////

        gsm.send_actions();

        sint4 vf = game.get_view_frame();
        sint4 ai = game.get_action_frame();
        sint4 sk = game.get_skipped_actions();
        sint4 md = game.get_merged_actions();

        if (vf != ai || sk || md > 1) {

          cout << "frame " << vf;
          if (ai < 0) cout << " [no action]";
          else if (vf != ai) cout << " [behind by " << (vf - ai) << " frame(s)]";
          if (sk) cout << " [skipped " << sk << "]";
          if (md > 1) cout << " [merged " << md << "]";
          cout << endl;
        }
        return true;
      }

    }

    return false;
  }
};

// //===================================================================

// class ortsgLooper : public LoopFunctor
// {
// private:
//   ClientState &state;
//   bool refresh;

// public:

//   ortsgLooper(ClientState &s) : state(s)
//   {
//     ::Options::get("-refresh", refresh);
//   }

//   void loop() const {

//     if (state.gfxm->get_quit()) exit(0);

//     if (refresh) {

//       if (!just_drew) // allow some waiting time after a message
//         state.gfxm->draw();
//       else
//         just_drew = false;
//     }

//     // server events
//     if (!state.gsm->recv_view()) SDL_Delay(1);
//   }
// };

//===================================================================

int main(int argc, char **argv)
{
  char mydir[81];
  getcwd(mydir, 80);
  //glutInit(&argc, argv);
  chdir(mydir);

  try {

#if USE_EFENCE
    // aggressive efence checks
    EF_PROTECT_FREE = 1;
    EF_FILL = 0xee;
    EF_PROTECT_BELOW = 1;
    EF_ALLOW_MALLOC_0 = 1;
#endif

    signals_shut_down(true);
    atexit(SDL_Quit);

    // populate opt with options from various modules

    add_options();
    Game::Options::add();
    GameStateModule::Options::add();
    //GfxModule::Options::add();
    Terrain::add_options();
    Simul::SimulationAI::add_options();
    SquadCombatAI::add_options();

    ::Options::set("-localstrategy", 3, false);

    // process command line arguments
    if (Options::process(argc, argv, cerr, ORTSG_VERSION)) exit(20);

    // change seed value to time(0) if 0
    sint4 seed;
    Options::get("-seed", seed);
    if (!seed) seed = time(0);
    Options::set("-seed", seed);
    cout << "seed=" << seed << endl;

    srand48(seed); // used by STL, fixme: avoid (global)

    SDLinit::video_init();
    SDLinit::network_init();

    // populate option objects with command line values
    // and create modules

    GameStateModule::Options gsmo;
    GameStateModule gsm(gsmo);

    // GfxModule::Options gfxmo;
    // GfxModule gfxm;

    // gives event handler access to the state
    //ClientState state(gsm, gfxm);

    // ortsg event handler
    NoGfxEventHandler ngev(gsm);

    //=====================================
    // create pathfinding module and implementation
    Terrain timp;
    TerrainModule tm(gsm, timp, 30);

    // hook up terrain widget to gfx module
    //SimpleTerrain::SimpleTerrainWidget st_widget(timp, &gfxm);
    //gfxm.add_cwidget(st_widget);
    //st_widget.enable();
    //=====================================

    Simul::SimulationAI suicide_ai(gsm);
    suicide_ai.add_handler(&tm);
    gsm.add_handler(&suicide_ai);
    tm.add_handler(&suicide_ai);
    //=====================================
    //   GatherAI gather_ai(gsm);
    //   gather_ai.add_handler(&tm);
    //   gfxm.add_handler(&gather_ai);
    //   gsm.add_handler(&gather_ai);
    //   tm.add_handler(&gather_ai);
    //=====================================
    SquadCombatAI scai(gsm);
    //gfxm.add_handler(&scai);
    scai.enable_motion();
    scai.enable_attack();

    //    SquadCombatAIWidget scai_widget(scai, &gfxm);
    //    gfxm.add_cwidget(scai_widget);

    //    shooter_ai.add_handler(&tm);
    //    tm.add_handler(&shooter_ai);
    //=====================================

    // register pathfinder as gfx and gsm event handler
    //gfxm.add_handler(&tm);
    //gsm.add_handler(&tm); done in TerrainModule

    // gsm events also handled in ngev
    gsm.add_handler(&ngev);

    // connect game state module to server
    if (!gsm.connect()) ERR("connection problems");
    //gfxm.init(gsm, gfxmo);

    // gfx loop (should become a thread)

    cout << "start loop" << endl;
    for (;;){
       if (!gsm.recv_view()) SDL_Delay(1);
    }

    //ortsgLooper l(state);
  }
  catch (ExitException& ee) {
    errstr << std::endl << *(ee.get_reason()) << std::endl;
#ifdef GCC
    abort();
#else
    ABORT;
#endif
    exit(20);
  }
  return 0;
}

//===================================================================

