#include "General.H"

bool General::onEvent(const Event &event) {
  if (event.get_what() == GameStateModule::VIEW_MSG) {
    if (!receivedFirstView) {
      this->me = moduleSet.get_game_state().get_game().get_client_player();
      receivedFirstView = !receivedFirstView;
      std::set<GameObj*> friendlies;
      myWatcher.queryUnits(And(OwnerIs(me),Or(TypeIs(TANK),TypeIs(MARINE),TypeIs(WORKER))), friendlies);
        
      insertUnits(friendlies);
      exec();
    } 
    this->update();
    return true;
    
  }
  return false;
}

void General::insertUnits(std::set<GameObj*> friendlies)
{
  FORALL(friendlies, it) {
    GameObj* gob = (*it)->get_GameObj();
    if( !gob )
      continue;
    
    unitVector.push_back(gob);
  }
}

/*********************JAMES' STUFF **************************/  
TerrainBase::Loc General::find_a_spot(TerrainBase::Loc center_loc){
  
  mapTerrain::TerrainMap terrMap(&moduleSet.get_game_state().get_game());
  TerrainBase::Loc loc(0,0);
  sint4 standard = 9; sint4 range = 64;
  bool found = terrMap.find_free_loc(center_loc, loc, range, standard);
  range+= 32;
  
  while(!found  && standard > 4){
    found = terrMap.find_free_loc(center_loc, loc, range, standard--);
  }
  terrMap.print_map();
  return TerrainBase::Loc(loc.x, loc.y);
}
/*********************JAMES' STUFF **************************/ 




void General::exec(){
  
  while(unitVector.size() > 0){
    
    Vector<GameObj*> unitsToJoin;
    
    getGroup(unitsToJoin,unitVector,unitVector[0]);
    
    // TODO: Store this commander somewhere explicitly?
    // The base class will have a record of it, but do we also need one?
    new AttackCommander(this, unitsToJoin, me);
  }	
}


void General::getGroup(Vector<GameObj*>& closestObjects,Vector<GameObj*>& originalObjects,
		       GameObj* obj){
  
  
  closestObjects.clear();
  
  real8 edist = 100.0;
  real8 dist = 0.0;
  real8 extended;
  
  sint4 x,y;
  sint4 x2,y2;
  obj->get_center(x,y);
  
  while(originalObjects.size() > 0){
    GameObj* mostNearestGroup = getNearestObject(originalObjects,obj);
    edist =  getNearestEnemyUnitOfGroupDist(obj);
    mostNearestGroup->get_center(x2,y2);
    dist = sqrt(pow((real8)(x - x2),(real8)2) + pow((real8)(y - y2),(real8)2));
    
    extended = 50;
    
    if (edist+extended > dist){
      closestObjects.push_back(mostNearestGroup);
      removeObjectFromVector(mostNearestGroup,originalObjects);
      removeObjectFromVector(mostNearestGroup,originalObjects);
    }
    else{
      break;
    } 
  } 
}

GameObj* General::getNearestObject(const Vector<GameObj*>& objects,const TerrainBase::Loc& loc){
  if(objects.empty()) return NULL;
  real8 closestDistance = 2e8;
  GameObj* closestObject = NULL;
  FORALL(objects,it){
    sint4 x,y;
    (*it)->get_center(x,y);
    real8 distance = sqrt(pow((real8)(x - loc.x),2.0) + pow((real8)(y - loc.y),2.0));
    if(distance < closestDistance){
      closestDistance = distance;
      closestObject = *it;
    }
  }
  return closestObject;
}

GameObj* General::getNearestObject(const Vector<GameObj*>& objects,GameObj* obj){
  if(obj == NULL) ERR("can't find nearest group to a NULL group");
  sint4 x,y;
  obj->get_center(x,y);
  return getNearestObject(objects,TerrainBase::Loc(x,y));
}

real8 General::getNearestEnemyUnitOfGroupDist(GameObj* obj){
  
  GameObj* nearestUnit = NULL;
  
  myWatcher.queryUnits(And(Not(OwnerIs(me)),Not(OwnerIs(2))), enemies);
  
  
  real8 nearDist = 1000000;
  
  FORALL(enemies,it){
    GameObj* curr = *it;
    sint4 x,y;
    curr->get_center(x,y);
    sint4 targetX, targetY;
    obj->get_center(targetX,targetY);
    real8 dist = sqrt( (real8)((targetX - x)*(targetX - x)) + ((targetY - y)*(targetY - y)) );
    if (dist < nearDist){
      nearDist = dist;
      nearestUnit = curr;
    }
  }
  return nearDist;
  
}

bool General::removeObjectFromVector(GameObj* obj, Vector<GameObj*>& vector){
  FORALL(vector,it){
    if(*it == obj){
      vector.erase(it);
      return true;
    }
  }
  return false;
}



