#ifndef GENERAL_H
#define GENERAL_H

#include "Commander.H"
#include "SquadCombatAI.H"
#include "Watcher.H"
#include "WatcherAux.H"
#include "ClientModule.H"
#include "TerrainModule.H"
#include "TerrainBase.H"
#include "TerrainClassifier.H"
#include "GameObj.H"
#include "Game.H"
#include "GameChanges.H"
#include "AttackCommander.H"



//currently the defence commander from game3

using namespace Watcher;


class General: public Commander, ClientModule {
private:
  //sint4 combatEventID;
  //a vector of enemy units
    std::set<GameObj*> enemies;
  //stores if an notification has been recived
  bool receivedNotification;
  //sotres the mode the general is in the ID value of this player
  sint4 mode, me;	
  //stores information about the target
  GameObj* target;
	Watcher::WatcherModule & myWatcher;
  bool receivedFirstView;
  //stores the location that a group of units will be moved to
  TerrainBase::Loc loc; 
  //pointer to the SquadCombat AI that get passed to other commanders so the can subcribe to it
  SquadCombatAI* combat;
  
    Vector<GameObj*> unitVector;

public:

	/**
     * Constructor for General Commander.
     * @param moduleSet reference to the global module set.
     * @param units Vector<GameObj*> ofall of the units being placed under the Commander's control
     * @see unitVector
     * @param combat_ The squad combat AI 
     * @param maxUpdate an integer that will be used as the highest possible random number of update() calls before exec() is run.
     * @see maxUpdateCount
     * @see updateCounter
     * @see ~General()
     */  
  General(ModuleSet& ms, Vector<GameObj*> units, SquadCombatAI* combat, sint4 updateTime=25)
    : Commander(ms, updateTime),
	  myWatcher(ms.get_watcher()),
      receivedFirstView(false),
      combat(combat)
  {
    std::cerr << "adding gsm handler" << std::endl;
    moduleSet.get_game_state().add_handler(this);

    moduleSet.get_combat().add_handler(this);
    this->add_handler(&moduleSet.get_combat());
    std::cerr << "gsm handler added" << std::endl;

    this->addUnits(units);
  }
  
  /**
     * Destroys the General Commander and all subcommanders recursively.
     * Sends units to superior Commander.
     * @see commVector
     * @see myCommander
     * @see unitVector
     * @see Commander()
     */
  ~General()
  {
      moduleSet.get_game_state().remove_handler(this);	
      moduleSet.get_combat().remove_handler(this);
  }    
  
  	/**
     * Finds a spot to group up
     * @param center_loc the center of the group 
     * @param gsm the game state module 
     * @see unitVector
     */  
    TerrainBase::Loc find_a_spot(TerrainBase::Loc center_loc);
  
  /**
     * Creates groups of object that are close
     * @param closestObjects fills in this vector with grouping 
     * @param originalObjects vector of all units that need to be grouped
     * @param obj the first gameobj in the vector
     */ 
  void getGroup(Vector<GameObj*>& closestObjects,
		Vector<GameObj*>& originalObjects,
		GameObj* obj);
  
   /**
     * Gets nearest object
     * @param objects to find the nearest one of 
     * @param loc the location we want the closest enemy to
     */ 
  GameObj* getNearestObject(const Vector<GameObj*>& objects,const TerrainBase::Loc& loc);
  /**
     * Gets nearest object
     * @param objects to find the nearest one of 
     * @param obj the object we want the closest object to
     */ 
  GameObj* getNearestObject(const Vector<GameObj*>& objects,GameObj* obj);
  
  /**
     * removes a game object from a game object vector
     * @param obj the game object to remove from the vector
     * @param vector the vector from which the object is to be removed
     * @return whether it was successful
     */ 
  bool removeObjectFromVector(GameObj* obj, Vector<GameObj*>& vector);
  
  /**
     * Gets nearest enemy distance
     * @param obj the object we want the closest object to
     * @return real8 that is the distance
     */
  real8 getNearestEnemyUnitOfGroupDist(GameObj* obj);
  
  /**
     * Inserts units to this generals unitVector
     * @param friendlies friendly units
     * @see unitVector 
     */
    void insertUnits(std::set<GameObj*> friendlies);
  
 /**
     * The main function (or brain) of the general.
     * Where it executes commands to execute
     */
  void exec();
  
  /**
     * Handles the event, sets everything up on the first event
     * @param event the event that happened
     */
  bool onEvent (const Event& event);

    void onReceiveUnit (GameObj * unit, Watcher::WatchID)
    {
        unitVector.push_back(unit);
    }

    void onLoseUnit (GameObj * unit)
    {
        removeObjectFromVector(unit, unitVector);
    }
  
   /**
     * Converts gameobject pointers to weak pointers
     * @param vec the vector of objects
     */
  Vector<GameObj::WPtr> convertWeakPtrs(std::vector<GameObj*> vec) 
  {
      Vector<GameObj::WPtr> Wvec;
      FORALL(vec, unit) 
      {
          Wvec.push_back(*unit);
      }
      return Wvec;
  }   
};

#endif // GENERAL_H
