#ifndef __GAME_3_GENERAL_H__
#define __GAME_3_GENERAL_H__

#include "Commander.H"
#include "ClientModule.H"
#include "TerrainBase.H" // For TerrainBase::Loc
#include "BaseClassifier.H"
#include "Movement.H"

class GameObj;
class GameStateModule;
namespace Finance { class Module; class Hold; class Ticket; }
class BuildCommander;
class BaseCommander;
class DefenseCommander;
class ScoutCommander;

class General : public Commander, public ClientModule
{
    typedef TerrainAnalysis::BaseInfo BaseInfo;

    bool receivedFirstView;
    GameStateModule & gsm;
    Movement::Context mc;
    Finance::Module & fm;
    TerrainAnalysis::BaseClassifier bc;
    
    struct ExpandTask
    {
        enum STATE { UNPICKED, PICKED, MOVING, READY, EXPANDING };

        GameObj *          worker;
        DefenseCommander * cover;
        TerrainBase::Loc   location;
        Finance::Hold *    hold;
        Finance::Ticket *  funds;
        STATE              state;

        ExpandTask(GameObj * worker, 
                   const TerrainBase::Loc & location,
                   Finance::Hold * hold,
                   Finance::Ticket * funds) :
            worker(worker), cover(NULL),
            location(location), hold(hold), 
            funds(funds), state(UNPICKED)
            {}
    };
    ExpandTask expansion;

    std::set<BuildCommander*> activeBuilders;
    std::set<BuildCommander*> finishedBuilders;

    std::set<BaseCommander*> bases;
    ScoutCommander * scoutCommander;

    std::set<GameObj*> controlCenters;
    std::set<GameObj*> otherBuildings;
    std::set<GameObj*> workers;
    std::set<GameObj*> army;
    DefenseCommander * assaultCommander;

    bool hasFactory;

    const BaseInfo * getNearestAlliedBase(const TerrainBase::Loc & loc) const;
    const BaseInfo * getNearestEnemyBase(const TerrainBase::Loc & loc) const;
    static real8 getDistanceFromBase(const TerrainBase::Loc & loc,
                                     const BaseInfo * base);

    sint4 getExpansionScore(const BaseInfo * base) const;
    sint4 getTargetScore(const BaseInfo * base) const;

    const BaseInfo * getBestExpansion() const;
    const BaseInfo * getBestTarget() const;

    BaseCommander * getBaseWithMostRequestable(const std::string & bp_name);
    BaseCommander * getBaseWithMostOwned(const std::string & bp_name);
    BaseCommander * getBaseWithLeastOwned(const std::string & bp_name);

    size_t acquireUnits(const std::string & bp_name, 
                        size_t amount, bool priority);
    size_t acquireAllUnits(const std::string & bp_name, bool priority);
    size_t countAllUnits(const std::string & bp_name, bool priority);

    void pickExpansionTarget(TerrainBase::Loc & target);
    bool pickExpansionLocation();
    void startExpansion();
    bool pickAttackTarget();
    void insertUnits(const Vector<ScriptObj::WPtr>& objects);

    void cleanupFinishedBuilders();
    void handleNewBases();
    void decideIfAttacking();
    void decideIfExpanding();

public:
    General(ModuleSet & ms);

    ~General();

    void exec();
    void onReceiveUnit(GameObj * unit, Watcher::WatchID);
    void onLoseUnit(GameObj * unit);
    bool onEvent(const Event & event);
    void onMovement(const Movement::Task & task);
};

#endif
