#include "AssetManager.H"
#include "GameObj.H"

AssetManager::AssetManager()
{

}

AssetManager::~AssetManager()
{

}

size_t AssetManager::getNumOwned(const std::string & bpName) const
{
    StatMap::const_iterator it = stats.find(bpName);
    if( it != stats.end() )
    {
        return it->second.numOwned;
    }
    return 0;
}

size_t AssetManager::getNumQueued(const std::string & bpName) const
{
    StatMap::const_iterator it = stats.find(bpName);
    if( it != stats.end() )
    {
        return it->second.numQueued;
    }
    return 0;
}

size_t AssetManager::getNumTotal(const std::string & bpName) const
{
    StatMap::const_iterator it = stats.find(bpName);
    if( it != stats.end() )
    {
        return it->second.numOwned + it->second.numQueued;
    }
    return 0;
}

void AssetManager::onReceiveUnit(GameObj * unit)
{
    const std::string & bpName = unit->bp_name();
    units.insert( std::make_pair(bpName, unit) );
    stats[bpName].numOwned++;
}

void AssetManager::onLoseUnit(GameObj * unit)
{
    const std::string & bpName = unit->bp_name();
    std::pair<UnitIt, UnitIt> range = units.equal_range(bpName);
    for(UnitIt it = range.first; it != range.second; it++)
    {
        if(it->second == unit)
        {
            units.erase( it );
            stats[bpName].numOwned--;
            return;
        }
    }
}

void AssetManager::enqueueUnit(const std::string & bpName)
{
    stats[bpName].numQueued++;
}

void AssetManager::dequeueUnit(const std::string & bpName)
{
    stats[bpName].numQueued--;
}

size_t AssetManager::retrieveUnits(const std::string & bpName, 
                                   size_t count, 
                                   std::set<GameObj*> & result)
{
    size_t addCount = 0;
    std::pair<UnitIt, UnitIt> range = units.equal_range(bpName);
    for(UnitIt it = range.first; it != range.second; it++)
    {
        if(addCount < count)
        {
            result.insert( it->second );
            addCount++;
        }
        else
        {
            break;
        }
    }
    return count;
}
