#ifndef __ASSET_MANAGER_H__
#define __ASSET_MANAGER_H__

#include <string>
#include <set>
#include <map>

class GameObj;

class AssetManager
{
    struct UnitStats
    {
        size_t numOwned;
        size_t numQueued;

        UnitStats() : numOwned(0), numQueued(0) {}
    };

    typedef std::multimap<std::string, GameObj*>  UnitMap;
    typedef std::map<std::string, UnitStats>      StatMap;
    typedef UnitMap::iterator                     UnitIt;

    UnitMap units;
    StatMap stats;
public:
    AssetManager();
    ~AssetManager();

    size_t getNumOwned(const std::string & bpName) const;
    size_t getNumQueued(const std::string & bpName) const;
    size_t getNumTotal(const std::string & bpName) const;

    void onReceiveUnit(GameObj * unit);
    void onLoseUnit(GameObj * unit);

    void enqueueUnit(const std::string & bpName);
    void dequeueUnit(const std::string & bpName);

    size_t retrieveUnits(const std::string & bpName, size_t count, 
                         std::set<GameObj*> & result);
};

#endif
