#include "Commander.H"
#include "Game.H"
#include "SquadCombatAI.H"
#include "Watcher.H"
#include "WatcherAux.H"
#include "ClientModule.H"
#include "TerrainModule.H"
#include "TerrainBase.H"
#include "TerrainClassifier.H"
#include "Movement.H"
//currently the defence commander from game3

using namespace Watcher;

class AttackCommander : public Commander, ClientModule 
{
public:
    struct UnitState
    {
        enum MODE { IDLE, MOVING, FIGHTING };
        MODE mode;

        UnitState() : mode(IDLE) {}
    };
private:
    Movement::Context mc;
	Watcher::WatcherModule & myWatcher;

    std::map<GameObj*, UnitState> unit2state;

    Vector<GameObj::WPtr> attackers;

    std::set<GameObj*> enemies; ///< set of game objects of enemies
    TerrainBase::Loc center;    ///< location of a specficed unit
    GameObj * target;           ///< object of the target
    sint4 me;                   ///< our ID

    int mode; // 0 = forming, 1 = attacking
    TerrainBase::Loc formLoc;

    void findCentroid();
    void updateEnemies();
    GameObj * getNearestUnit(const std::string & bp_name,
                             real8 * distance = NULL) const;
public:
    AttackCommander(Commander* const mySuperior, 
                    Vector<GameObj*> units,
                    sint4 owner);

    ~AttackCommander()
    {
        remove_handler(&moduleSet.get_combat());
        moduleSet.get_combat().remove_handler(this);
    }

    void onMovement(const Movement::Task & task);
    
    /// Find an open spot, based on a circle and an area to search.
    /// @param TerrainBase::Loc the coordinates of an area to get closest too.
    /// @return the location where the circle fits near given inputs
    /// @see TerrainBase
    TerrainBase::Loc find_a_spot(TerrainBase::Loc center_loc);
  
    void exec();
  
    /// Handles pathfinding and combat messages
    /// @param event the event to handle
    bool onEvent (const Event& event);  

    void onReceiveUnit(GameObj * unit, Watcher::WatchID)
    {
        unit2state.insert(std::make_pair(unit, UnitState()));
    }

    void onLoseUnit(GameObj * unit)
    {
        unit2state.erase(unit);
        GameObj::WPtr wp(unit);
        Vector<GameObj::WPtr>::iterator it;
        it = std::find(attackers.begin(), attackers.end(), wp);
        if(it != attackers.end())
        {
            attackers.erase(it);
        }
    }
};

REGISTER_TYPEOF2(100201, std::map<GameObj*, AttackCommander::UnitState>::iterator);
REGISTER_TYPEOF2(100202, std::map<GameObj*, AttackCommander::UnitState>::const_iterator);
