#ifndef BuildCommander_H
#define BuildCommander_H

#include "Commander.H"
#include "ClientModule.H"
#include "TerrainBase.H"
#include "Watcher.H"
#include "Finance.H"
#include "Movement.H"

class BuildCommander : public Commander, ClientModule 
{
    enum STATE { PLANNING, ENROUTE, READY, BUILDING, 
                 STATE_FINISHED, STATE_FAILED };

    TerrainAnalysis::Module& tam;
    Watcher::WatcherModule&  wm;
    Movement::Context        mc;

    std::string            bp_name;
	TerrainBase::Loc       nearLoc;

    Finance::Ticket *      funds;

    GameObj *              worker;
    GameObj *              building;

    STATE                  state;

    Vector<TerrainBase::Loc> buildSites;
    size_t		             nextSite;

    bool                   tryingToBuild;
    sint4                  countDown;
    Watcher::WatchID       buildWatch;
    Watcher::WatchID       completeWatch;

    void doPlanning();
    void tryToBuild();
    bool findBuildSite();
    void moveToNextSite();

    void onFail();
public:
    BuildCommander(Commander*          mySuperior, 
                   GameObj*            worker,
                   const std::string & bp_name,
				   TerrainBase::Loc    nearLoc,
                   Finance::Ticket *   funds = 0,
                   sint4               updateTime_ = 25);
    ~BuildCommander();

    void onReceiveUnit(GameObj * unit, Watcher::WatchID watch);
    void onLoseUnit(GameObj * unit);
    bool onEvent(const Event& event);
    void handleMovementStatus(const Movement::Task & task);

    void exec();

    const std::string & getBuildingName() const { return bp_name; }

    static const sint4 FROM; 
    static const sint4 FINISHED;
    static const sint4 FAILED;
};

class BuildFinishedEvent : public Event 
{
    BuildCommander * source;
    GameObj *        worker;
    GameObj *        building;
public:
    BuildFinishedEvent(BuildCommander * source,
                       GameObj * worker, 
                       GameObj * building) :
        Event(BuildCommander::FROM, BuildCommander::FINISHED),
        source(source), worker(worker), building(building) {}
    ~BuildFinishedEvent() {}

    BuildCommander * getSource() const { return source; }
    GameObj *        getWorker() const { return worker; }
    GameObj *        getBuilding() const { return building; }
};

class BuildFailedEvent : public Event 
{
public:
    enum REASON { NO_BUILD_SITE, WORKER_KILLED, BUILDING_DESTROYED };
private:
    BuildCommander * source;
    GameObj *        worker;
    REASON           reason;
public:
    BuildFailedEvent(BuildCommander * source,
                     GameObj * worker,
                     REASON reason) :
        Event(BuildCommander::FROM, BuildCommander::FAILED),
        source(source), worker(worker), reason(reason) {}
    ~BuildFailedEvent() {}

    BuildCommander * getSource() const { return source; }
    GameObj *        getWorker() const { return worker; }
    REASON           getReason() const { return reason; }
};

REGISTER_TYPEOF(424001, std::set<BuildCommander*>::iterator);
REGISTER_TYPEOF(424002, std::set<BuildCommander*>::const_iterator);

#endif
