#ifndef __MODULE_SET_H__
#define __MODULE_SET_H__

#include <boost/shared_ptr.hpp>

class GameStateModule;
class SquadCombatAI;
namespace Watcher { class WatcherModule; }
namespace History { class HistoryModule; }
namespace TerrainAnalysis { class Module; }
namespace Finance { class Module; }
namespace Movement { class Module; }

class ModuleSet
{
  Finance::Module          *fm;
  GameStateModule          *gsm;
  History::HistoryModule   *hm;
  SquadCombatAI            *scai;
  TerrainAnalysis::Module  *tam;
  Watcher::WatcherModule   *wm;
  boost::shared_ptr<Movement::Module> mm;
public:
  ModuleSet();
  ~ModuleSet();
  
  void set_game_state(GameStateModule *gsm);
  void set_combat(SquadCombatAI *scai);
  void set_watcher(Watcher::WatcherModule *wm);
  void set_history(History::HistoryModule *hm);
  void set_terrain_analysis(TerrainAnalysis::Module *tam);
  void set_finance(Finance::Module *fm);
  void set_movement(boost::shared_ptr<Movement::Module> mm);
  
  bool has_gsm()              const { return gsm != 0; }
  bool has_combat()           const { return scai != 0; }
  bool has_watcher()          const { return wm != 0; }
  bool has_history()          const { return hm != 0; }
  bool has_terrain_analysis() const { return tam != 0; }
  bool has_finance()          const { return fm != 0; }
  
  GameStateModule& get_game_state() const;
  SquadCombatAI& get_combat() const;
  Watcher::WatcherModule& get_watcher() const;
  History::HistoryModule& get_history() const;
  TerrainAnalysis::Module& get_terrain_analysis() const;
  Finance::Module& get_finance() const;
  Movement::Module& get_movement() const;
};

#endif
