#include "ScoutCommander.H"
#include "History.H"

const sint4 ScoutCommander::FROM = EventFactory::new_who();

ScoutCommander::ScoutCommander(Commander * const mySuperior)
    : Commander(mySuperior, 8), finished(false),
      mc(moduleSet.get_movement(), "Default", "Default")
{
    
}

ScoutCommander::~ScoutCommander() 
{
    
}

bool ScoutCommander::isFinished() const
{
    return finished;
}

void ScoutCommander::onReceiveUnit( GameObj * unit, Watcher::WatchID )
{
    if(!finished)
    {
        explorers.insert( unit );
    }
}

void ScoutCommander::onLoseUnit( GameObj * unit )
{
    explorers.erase( unit );
    recon.erase( unit );
}

bool ScoutCommander::onEvent( const Event & ) 
{
    return false;
}

void ScoutCommander::exec() 
{
    const Game & game = moduleSet.get_game_state().get_game();

    Vector<TerrainBase::Loc> locations;

    FORALL(explorers, s) 
    {
        if(finished)
        {
            continue;
        }

        GameObj * scout = (*s);
        if (scout->get_int("speed") == 0)
        {
            if(locations.empty())
            {
                TerrainAnalysis::FindBorderLocations(game.get_map(),
                                                     game.get_tile_points(),
                                                     locations);
                if(locations.empty())
                {
                    finished = true;
                    continue;
                }
            }

            sint4 best = -1;
            sint4 bestDistance = 0;
            sint4 x, y; scout->get_center(x, y);
            for(sint4 i = 0; i < (sint4)locations.size(); i++)
            {
                const TerrainBase::Loc & l = locations[i];
                sint4 dx = l.x - x;
                sint4 dy = l.y - y;
                sint4 d2 = dx * dx + dy * dy;
                if(best == -1 || d2 < bestDistance)
                {
                    best = i;
                    bestDistance = d2;
                }
            }

	    mc.moveUnit(scout, Movement::TouchPoint(locations[best]));
        }
    }

    if(finished)
    {
        FORALL( explorers, e )
        {
            recon.insert( (*e) );
        }
        explorers.clear();
    }

    if(recon.empty() == false)
    {
        std::cerr << "Units doing recon: " << recon.size() << std::endl;

        std::vector<const History::Record*> records;
        moduleSet.get_history().queryActiveRecords(Watcher::TypeIs("mineral"),
                                                   records);
        if(records.empty())
        {
            return;
        }

        FORALL(recon, r)
        {
            GameObj * unit = (*r);
            if (unit->get_int("speed") == 0)
            {
                size_t target = rand.rand_uint4() % records.size();
		const TerrainBase::Loc & pos(records[target]->getPosition());
		mc.moveUnit(unit, Movement::TouchPoint(pos));
            }
        }
    }
}
