#include <string>
#include <map>

namespace TechTree
{
    struct Requirement
    {
        std::string creator;
        std::string action;

        Requirement() : creator(), action() {}
        Requirement(const std::string & creator,
                    const std::string & action) :
            creator(creator), action(action) {}
    };

    static std::map<std::string, Requirement> buildReqs;
    static std::map<std::string, Requirement> unitReqs;
    static bool initialized = false;

    static void BuildingReq(const std::string & building,
                            const std::string & builder,
                            const std::string & action)
    {
        buildReqs.insert(std::make_pair(building, 
                                        Requirement(builder, action)));
    }

    static void UnitReq(const std::string & unit,
                        const std::string & trainer,
                        const std::string & action)
    {
        unitReqs.insert(std::make_pair(unit, Requirement(trainer, action)));
    }

    static void Init()
    {
        if(initialized == false)
        {
            BuildingReq("controlCenter", "worker", "build_controlCenter");
            BuildingReq("barracks", "worker", "build_barracks");
            BuildingReq("factory", "worker", "build_factory");

            UnitReq("worker", "controlCenter", "train_worker");
            UnitReq("marine", "barracks", "train_marine");
            UnitReq("tank", "factory", "build_tank");

            initialized = true;
        }
    }

    const std::string & GetBuilder(const std::string & building)
    {
        Init(); return buildReqs[building].creator;
    }

    const std::string & GetBuildAction(const std::string & building)
    {
        Init(); return buildReqs[building].action;
    }

    const std::string & GetTrainer(const std::string & unit)
    {
        Init(); return unitReqs[unit].creator;
    }

    const std::string & GetTrainAction(const std::string & unit)
    {
        Init(); return unitReqs[unit].action;
    }
}
