#include "GatherCommander.H"

#include "RouteAllocator.H"
#include "Game.H"
#include "Route.H"
#include "GameStateModule.H"
#include "TerrainModule.H"
#include "WatcherAux.H"
#include "Delegate.H"
#include "GameObj.H"

#define TEST(x)

const sint4 GatherCommander::FROM = EventFactory::new_who();

const std::string GatherCommander::workerBPname("worker");
const std::string GatherCommander::mineralBPname("mineral");
const std::string GatherCommander::controlCenterBPname("controlCenter");

using namespace std;

////////////////////////////Constructor/Destructor//////////////////////////////

GatherCommander::GatherCommander(Commander* const mySuperior, 
                                 const Vector<GameObj*> & units,
                                 TerrainBase::Loc centre_,
                                 sint4 radius_,
                                 sint4 updateTime_=1)
    : Commander(mySuperior, updateTime_),
      centreLoc(centre_),
      radius(radius_),
      history(moduleSet.get_history()),
      gsm(moduleSet.get_game_state()),
      mc(moduleSet.get_movement(), "Default", "Default"),
      receivedFirstView(false),
      mineralDied(false)
{
    TEST(std::cerr <<"Adding Commander as gsm handler" <<std::endl);
    gsm.add_handler(this);       // gsm -> ai

    mc.setStatusProcedure(
        make_delegate(this, &GatherCommander::handleMovementStatus));

    TEST(std::cerr << "adding notify on creation to watcher" << std::endl);
#if 0
    newMineralsWatchID = 
        moduleSet.get_watcher().notifyOnCreation(this, 
            Watcher::And(Watcher::TypeIs(mineralBPname), 
                         Watcher::InCircle(centreLoc.x,
                                           centreLoc.y, 
                                           160)), 10, false);
#else
    newMineralsWatchID = 
        moduleSet.get_watcher().notifyOnCreation(this, 
                                                 Watcher::TypeIs(mineralBPname), 
                                                 10, false);
#endif
    routeAllocator = RouteAllocatorPtr(new RouteAllocator());

    addUnits( units );

    init();
}

GatherCommander::~GatherCommander() 
{
    gsm.remove_handler(this);

    moduleSet.get_watcher().cancelNotify(newMineralsWatchID);
    std::set<uint4>::iterator watchID = watcherIDs.begin();
    while (watchID != watcherIDs.end()) 
    {
        moduleSet.get_watcher().cancelNotify(*watchID);
        watchID++;
    }

    FORALL(workers, w) { delete (*w); }
}

/////////////////////////////////////PRIVATE////////////////////////////////////////

void GatherCommander::assessChangesToMinerals() 
{
    TEST(std::cout << (void*)this << " assessing changes to minerals " << newMinerals.size() << std::endl);
    std::set<const History::Record*> mineralRecsTemp = mineralRecs;

    bool isNew = false;

    if (!mineralDied)
    {
        FORALL(newMinerals, nmit) 
        {
            bool imNew = true;
            //sint4 x,y;
            //(*nmit)->get_center(x,y);
            FORALL(mineralRecsTemp, mrit) 
            {
                if (*nmit == (*mrit)->getObject()) 
                {
                    imNew = false;
                    break;
                }
            }
            if (imNew)
            {
                isNew = true;
                break;
            }
        }
    }
    if (isNew) 
    {
        std::vector<const History::Record*> rec;
        history.queryActiveRecords(Watcher::TypeIs("mineral"), rec);
        mineralRecs.clear();
        FORALL(rec, r) 
        {
            mineralRecs.insert(*r);
        }

        newMinerals.clear();
        
        //if there was a new mineral
        //if (mineralRecs.size() != mineralRecsTemp.size()) 
        //{
            routeAllocator = RouteAllocatorPtr(new RouteAllocator(workers.size()));
            
            computeMineralClusters();
            
            computeRoutes();
            
            if(controlCenters.empty() == false)
            {
                sint4 x,y;
                controlCenters[0]->get_center(x,y);
                routeAllocator->sortRoutesAccordingToLocation(TerrainBase::Loc(x,y));
            }
            
            FORALL(workers, wit) 
            {
                (*wit)->clearRoute();
            }        
            //}
    }
    mineralDied = false;
    newMinerals.clear();
}

void GatherCommander::handleMoveFailure(Worker *worker) 
{
    moveRandomly(worker->get_gob());
    worker->setState(Worker::IDLE);
}

void GatherCommander::handlePlanFailure(Worker *worker) 
{
    TEST(std::cerr << "(tPF\t" << worker->getState() << "\t" 
         << worker->getLocation() << ")");
    if (worker->getState() == Worker::MOVING)
    {
        if (worker->hasRoute())
        {
            RoutePtr route = worker->getRoute();
            if(getDistance(worker->get_gob(), route->getAccessPoint()) == -1)
            {
                TEST(std::cout << "path does not exist" << std::endl);
                worker->clearRoute();
                routeAllocator->removeRoute(route);
            }
        }
    }
    else if (worker->getState() == Worker::RETURNING)
    {
        moveRandomly(worker->get_gob());
    }
    worker->setState(Worker::IDLE);
}

void GatherCommander::handleSuccess(Worker *worker) 
{
    if(worker->hasRoute())
    {
        worker->getRoute()->planSuccessOccurred();
    }
    worker->setState(Worker::IDLE);
}

void GatherCommander::init() 
{
    TEST(std::cout << "initialising" << std::endl);
    
    locateNearbyMinerals();
}

void GatherCommander::locateNearbyMinerals() 
{
    TEST(std::cout << "locating minerals" << std::endl);

    moduleSet.get_watcher().queryUnits(
        Watcher::TypeIs(mineralBPname), newMinerals);
    
    FORALL(newMinerals, nmit) 
    {
        sint4 x,y;
        (*nmit)->get_center(x,y);
        TEST(std::cout << x << ", " << y << "\t");
    }
    TEST(std::cout << std::endl);
}

void GatherCommander::performWorkerActions() 
{
    FORALL(workers, wit) 
    {
        Worker * worker = (*wit);
        if (worker->hasRoute()) 
        {
            if (worker->getState() == Worker::RETURNING) 
            {
                if (worker->closeEnoughToDropOff()) 
                {
                    worker->dropOffMinerals();
                }
            }        

            if (worker->getState() == Worker::IDLE || 
                worker->getState() == Worker::MINING) 
            {                
                if (worker->getCurrentMinerals() < 10) 
                {
                    if (worker->closeEnoughToMine()) 
                    {
                        worker->mine();
                    } 
                    else if (worker->getMineralRec()->isActive()) 
                    {
                        worker->moveToMineral();
                    }
                    else
                    {
                        mineralDied = true;
                    }
                }
                else
                {
                    if (worker->closeEnoughToDropOff()) 
                    {
                        worker->dropOffMinerals();
                    }
                    else
                    {
                        worker->moveToCC();
                    }
                }
            }
        }
        else if(routeAllocator->getNumAvailableRoutes() > 0) 
        {
            if (routeAllocator->getNumAvailableRoutes() > workers.size())
            {
                boost::shared_ptr<Route> bestRoute(
                    routeAllocator->getBestRoute(worker->getLocation()));
                if(bestRoute.get() != 0)
                {
                    worker->setRoute(bestRoute);
                }
            }
            else
            {
                mineralDied = true;
            }
        }
        else
        {
            // Cannot use worker right now
        }

        if (mineralDied)
            break;
    }
}

real8 GatherCommander::getDistance(GameObj *obj, TerrainBase::Loc loc) 
{
    Movement::Path::ptr path(mc.findPath(obj, Movement::TouchPoint(loc)));
    if(!path) return -1;

    real8 dist = path->getLength();

    TEST(std::cout << "dist: " << dist << std::endl);

    real8 finalDist = dist; //0.0;

    /*    if (!path.locs.empty()) 
    {
        TEST(std::cout << "Path size: " << path.locs.size() << std::endl);
        TerrainBase::Loc behind;
        sint4 count = 0;
        FORALL(path.locs, loc) 
        {
            if (path.locs.size() == 1)
            {
                TerrainBase::Loc objLoc;
                obj->get_center(objLoc.x, objLoc.y);
                return loc->distance(objLoc);
            }
            if (count==0) 
            {
                behind = *loc;
            } 
            else 
            {
                finalDist += loc->distance(behind);
                behind = *loc;
            }
            count++;
        }
    }*/
    return finalDist;
}
/*
real8 GatherCommander::getDistance(TerrainBase::Loc loc1, 
                                   TerrainBase::Loc loc2, 
                                   sint4 radius) 
{
    TerrainBase::Path path;

    real8 dist = st.find_path(loc1, loc2, radius, &path, 
                              TerrainBase::IGNORE_ALL_MOBILE_OBJS);
    TEST(std::cout << "dist: " << dist << std::endl);

    if (dist == -1) 
        return dist;

    real8 finalDist = 0.0;

    if (path.locs.size() > 0) 
    {
        TEST(std::cout << "Path size: " << path.locs.size() << std::endl);
        TerrainBase::Loc behind;
        sint4 count = 0;
        FORALL(path.locs, loc) 
        {
            if (count==0) 
            {
                behind = *loc;
            } 
            else 
            {
                finalDist += loc->distance(behind);
                behind = *loc;
            }
            count++;
        }
    }
    return finalDist;
}
*/

/////////////////////////////////////PUBLIC/////////////////////////////////////////

size_t GatherCommander::countRequestable(const std::string & bp_name)
{
    if(bp_name == "worker") // If the request wants workers
    {
        // We can give them all (we can make more)
        return getNumUnits(bp_name);
    }
    else // If they want anything else (like CCs)
    {
        // Don't let them go without a fight
        return 0;
    }
}

size_t GatherCommander::countWorkersNeeded() const
{
    assert(routeAllocator != NULL);
    return routeAllocator->countWorkersNeeded();
}

void GatherCommander::exec() 
{
    if (!receivedFirstView) 
    {
        init();
        receivedFirstView = true;
    } 
    else 
    {
        if (!newMinerals.empty() || mineralDied) 
        {
            assessChangesToMinerals();
        }
        
        performWorkerActions();
    }
}

sint4 GatherCommander::getNumUnits(const std::string & unitBPname) 
{
    if (unitBPname == workerBPname) 
    {
        return workers.size();
    }
    if (unitBPname == controlCenterBPname)
    {
        return controlCenters.size();
    }
    if (unitBPname == mineralBPname)
    {
        return mineralRecs.size();
    }
    else 
    {
        return 0;
    }
}

void GatherCommander::moveRandomly(GameObj *unit)
{
    ScalarPoint t;
    unit->get_center(t.x, t.y);
    t.x += rand.ranged_sint4(-5, 5);
    t.y += rand.ranged_sint4(-5, 5);
        
    Vector<sint4> params;
    params.push_back(t.x);
    params.push_back(t.y);
    params.push_back(*(unit->sod.max_speed));
    unit->set_action("move", params);

}

void GatherCommander::moveTo(GameObj *unit, GameObj *gob)
{
    mc.moveUnit(unit, Movement::TouchObject(gob));
}

void GatherCommander::moveTo(GameObj *unit, const TerrainBase::Loc &location)
{
    mc.moveUnit(unit, Movement::TouchPoint(location));
}

bool GatherCommander::needsUnit(const std::string & bp_name) 
{
    if(bp_name == "worker")
    {
        return (routeAllocator->getNumAvailableRoutes() > 0);
    }
    else
    {
        return false;
    }
}

void GatherCommander::handleMovementStatus(const Movement::Task & task)
{
    FORALL( task.getUnits(), u )
    {
        const GameObj * unit(*u);

        FORALL(workers, wit) 
        {
            if ((*wit)->get_gob() == unit) 
            {
                Worker * worker(*wit);
                switch( task.getStatus() )
                {
                case Movement::Task::PLAN_FAILURE:
                    handlePlanFailure(worker);
                    break;
                case Movement::Task::MOVE_FAILURE:
                    handleMoveFailure(worker);
                    break;
                case Movement::Task::SUCCESS:
                    handleSuccess(worker);
                    break;
                default: break;
                }
            }
        }
    }
}

bool GatherCommander::onEvent(const Event &event) 
{   
    if (event.get_who() == Watcher::WatcherModule::FROM) 
    {
        if (event.get_what() == Watcher::WatcherModule::CREATION) 
        {
            const Watcher::CreationEvent* ce = 
                dynamic_cast<const Watcher::CreationEvent*>(&event);
            if (ce->getID() == newMineralsWatchID) 
            {
                TEST(std::cout << "Watcher found a new mineral" << std::endl);
                newMinerals = ce->getUnits();
            }
        }
    }    
    
    return false;
}

void GatherCommander::onLoseUnit( GameObj * object )
{
    if( object->bp_name() == mineralBPname )
    {
        // We are giving up a mineral that we were given because
        // we were watching for new minerals. Ignore it, because
        // this is the behaviour we want.
        return;
    }
    else if( object->bp_name() == workerBPname )
    {
        FORALL( workers, w )
        {
            if( (*w)->get_gob() == object )
            {
                TEST(std::cerr << "GATHERER lost worker" << std::endl);
                (*w)->clearRoute();
                routeAllocator->removeWorker();
                delete (*w);
                workers.erase( w );
                return;
            }
        }
    }
    else if( object->bp_name() == controlCenterBPname )
    {
        FORALL( workers, w )
        {
            delete (*w);
        }
        workers.clear();

        FORALL( controlCenters, cc )
        {
            if( (*cc) == object )
            {
                controlCenters.erase( cc );
                break;
            }
        }

        assessChangesToMinerals();
    }
    else // Should never happen.  Gather commanders should only have workers, 
        // minerals and control centres.
    {
        assert( false );
    }
}

void GatherCommander::onReceiveUnit( GameObj * object, Watcher::WatchID watch )
{
    if( watch == newMineralsWatchID )
    {
        this->removeUnit( object );
    }

    if( object->bp_name() == workerBPname ) 
    {
        TEST(std::cerr << "GATHERER received worker" << std::endl);
        Worker * worker = new Worker(object, this, &gsm);
        workers.push_back(worker);
        routeAllocator->addWorker();
    }

    if( object->bp_name() == controlCenterBPname ) 
    {
        controlCenters.push_back(object);
    }
}

size_t GatherCommander::requestUnits(Commander * receiver,
                                     const std::string & bp_name,
                                     size_t amount)
{
    size_t count = 0;
    if(bp_name == "worker") // If the requester wants workers
    {
        // Return as many as we can (we can make more)
        while(count < amount)
        {
            if( requestWorker(receiver) )
            {
                count++;
            }
            else
            {
                break;
            }
        }
    }
    return count;
}

bool GatherCommander::requestWorker(Commander * requestingCommander) 
{
    if (!workers.empty()) 
    {
        GameObj * worker = workers[0]->get_gob();
        stopWorker( worker );
        removeUnit( worker );
        requestingCommander->addUnit( worker );

        return true;
    } 
    else 
    {
        return false;
    }
}

void GatherCommander::stopWorker(GameObj *worker)
{
    Vector<GameObj*> vec;
    vec.push_back(worker);
    call_handlers( TerrainModule::PathStopEvent(FROM, 
                                                TerrainModule::FIND_PATH_STOP, 
                                                vec));
}

