#include "GatherCommander.H"

#include "Gather_accessories.H"
#include <numeric>
#include "Route.H"
#include "RouteAllocator.H"
#include <math.h>

using namespace std;

#define MINERAL_ACCESS_POINT_THRESHOLD 6

#define TEST(x)

const static real8 DIST = 10.0;
const static real8 LS = cos( M_PI * 0.125 ) * DIST;
const static real8 SS = sin( M_PI * 0.125 ) * DIST;

//****************************Function Objects**************************************

void GatherCommander::computeMineralClusters()
{
    cout << "compute MineralClusters " << endl;

    mineralClusters.clear();
    if (mineralRecs.empty())
    {
        // Do nothing
    }
    else if (mineralRecs.size() <= 10) 
    {
        boost::shared_ptr<MineralCluster> cluster(new MineralCluster(mineralRecs));
        mineralClusters.push_back(cluster);
    } 
    else 
    {
        // List all mineral locations
        Vector<sint4> clusterOf;
        Vector<ScalarPoint> mineralLocations;
        mineralLocations.reserve(mineralRecs.size());
        FORALL(mineralRecs, mit) 
        {
            const TerrainBase::Loc pos = (*mit)->getPosition();
            ScalarPoint point(pos.x, pos.y);
            mineralLocations.push_back(point);
        }
        
        // Cluster
        simple_clustering(mineralLocations, 2, 8, clusterOf);
        // Store the results
        sint4 clusterCount = *max_element(clusterOf.begin(), clusterOf.end()) + 1;
        mineralClusters.reserve(clusterCount);
        
        FORU(i, clusterCount) 
        {
            boost::shared_ptr<MineralCluster> cluster(new MineralCluster());
            mineralClusters.push_back(cluster);
        }

        Vector<sint4>::iterator cit = clusterOf.begin();
        FORALL(mineralRecs, mrit) 
        {
            mineralClusters[*cit]->addMineral(*mrit);
            ++cit;
        }
    }
    
    // Compute cluster centroids
    FORALL(mineralClusters, mcit) 
    {
        (*mcit)->calculateCentroid();
    }
    cout << "done" << endl;
}

void GatherCommander::computeRoutes() 
{
    TEST(cout << "computing Routes............" << endl);

    if(controlCenters.empty())
    {
        TEST(cout << "no control centers" << endl);
        return;
    }

    TerrainBase::Loc ccLoc;
    controlCenters[0]->get_center(ccLoc.x, ccLoc.y);

    TEST(cout << "mineral Clusters: " << mineralClusters.size() << endl);
    Vector<MineralClusterPtr>::iterator cluster = mineralClusters.begin();
    for (cluster = mineralClusters.begin(); 
         cluster != mineralClusters.end(); 
         cluster++)
    {
        set<const History::Record*> mineralsInCluster;
        (*cluster)->getMinerals(mineralsInCluster);
        TEST(cout << "\tminerals In Cluster: " << mineralsInCluster.size() 
             << endl);

        Vector<MineralPtr> minerals;

        FORALL(mineralsInCluster, record)
        {
            TEST(cout << "creating initial points " << minerals.size() 
                 << endl);
            TerrainBase::Loc c = (*record)->getPosition();
            Vector<TerrainBase::Loc> tempPoints;

            sint4 ls(static_cast<sint4>(LS));
            sint4 ss(static_cast<sint4>(SS));
            tempPoints.push_back(TerrainBase::Loc(c.x + ls, c.y + ss));
            tempPoints.push_back(TerrainBase::Loc(c.x + ss, c.y + ls));
            tempPoints.push_back(TerrainBase::Loc(c.x - ss, c.y + ls));
            tempPoints.push_back(TerrainBase::Loc(c.x - ls, c.y + ss));
            tempPoints.push_back(TerrainBase::Loc(c.x - ls, c.y - ss));
            tempPoints.push_back(TerrainBase::Loc(c.x - ss, c.y - ls));
            tempPoints.push_back(TerrainBase::Loc(c.x + ss, c.y - ls));
            tempPoints.push_back(TerrainBase::Loc(c.x + ls, c.y - ss));
            
            boost::shared_ptr<Mineral> mineral(new Mineral(tempPoints, c));
            minerals.push_back(mineral);
        }

        FORALL(minerals, mineralToTest)
        {
            TEST(cout << "mineral: " << (*mineralToTest)->centre << endl);
            
            //            FORALL_DYN((*mineralToTest)->points, pointToTest)
            FORALL_NI((*mineralToTest)->points, pointToTest)
            {
                TEST(cout << "\tpoint: " << *pointToTest << endl);
                bool clashes = false;
                if (pointToTest->distance((*cluster)->getCentroid()) <
                    (*mineralToTest)->centre.distance((*cluster)->getCentroid()))
                {
                    TEST(cout << "\t\tp: " << *pointToTest << " wrong side." << endl);
                    pointToTest = (*mineralToTest)->points.erase(pointToTest);
                    continue;
                }
                FORALL(minerals, otherMineral)
                {
                    TEST(cout << "\t\tagainst mineral: " 
                         << (*otherMineral)->centre << endl);
                    if (mineralToTest == otherMineral)
                    {
                        continue;
                    }
                    if (pointToTest->distance((*otherMineral)->centre) < 9) 
                    {
                        TEST(cout << "\t\t\tp: " << *pointToTest 
                             << " too close to m: " 
                             << (*mineralToTest)->centre << endl);
                        clashes = true;
                        pointToTest = (*mineralToTest)->points.erase(pointToTest);
                        break;
                    }
                }
                if (clashes) continue;

                FORALL(minerals, otherMineral)
                {
                    TEST(cout << "\t\tagainst mineral: " 
                         << (*otherMineral)->centre << endl);
                    if (otherMineral == mineralToTest ||
                        pointToTest->distance((*otherMineral)->centre) > 15)
                    {
                        continue;
                    }
                    FORALL((*otherMineral)->points, otherPoint)
                    {
                        TEST(cout << "\t\t\tagainst point: " << *otherPoint 
                             << endl);
                        if (pointToTest->distance(*otherPoint) < 8)
                        {
                            TEST(cout << "\t\t\t\tp: " << *pointToTest 
                                 << " too close to p: " << *otherPoint << endl);
                            clashes = true;
                            pointToTest = (*mineralToTest)->points.erase(pointToTest);
                            break;
                        }
                    }
                    if (clashes) break;
                }
                if (clashes) continue;
                ++pointToTest;
            }
        }
        TEST(cout <<  "end of cluster" << endl);
        TEST(cout << minerals.size() << " " << mineralsInCluster.size() 
             << " " << mineralClusters.size() << endl);
        Vector<boost::shared_ptr<Mineral> >::iterator mit = minerals.begin();
        FORALL(mineralsInCluster, mrit)
        {
            FORALL((*mit)->points, p)
            {
                TEST(cout << "adding Route at: " << *p << " for mineral " 
                     << (*mit)->centre << endl);
                
                RoutePtr route(new Route(this,
                                         *mrit,
                                         *p,
                                         controlCenters[0]));
                routeAllocator->addRoute(route);
            }
            ++mit;
        }
    }
    TEST(cout << "......finished computing routes " << routeAllocator->getNumAvailableRoutes() << endl);
}
/*
sint4 GatherCommander::computeNumberOfWorkersOnRoute(AccessPoint ap) {
    ap.setReturnPoint(Loc(4,4));
    return 1;//((ap.loc.distance(ap.returnPoint) / 4) * 2 + 50) / 50;
}

*/
