#ifndef MineralCluster_H
#define MineralCluster_H

#include "TerrainBase.H"
#include "History.H"

/**
 * A mineral cluster contains a set of minerals that are near each other.
 */
class MineralCluster
{
private:
    std::set<const History::Record *> myMinerals;

    TerrainBase::Loc centroid;
    
public:
    /**
     * creates a mineral cluster with no minerals
     */
    MineralCluster();

    /**
     * creates a mineral cluster with a set of minerals
     * @param minerals the minerals of this mineral cluster
     */
    MineralCluster(std::set<const History::Record*> &minerals);
    
    /**
     * adds a mineral
     */
    void addMineral(const History::Record *mineral);

    /**
     * calculate the centre of the cluster.
     */
    void calculateCentroid();

    /**
     * returns the centre of the mineral cluster
     */
    TerrainBase::Loc getCentroid() const;

    /**
     * returns the set of minerals that comprise this cluster
     */
    void getMinerals(std::set<const History::Record *> &minerals) const;

    /**
     * returns the number of minerals in this cluster
     */
    sint4 getNumMinerals() const;

    /**
     * sets the minerals for this cluster
     */
    void setMinerals(std::set<const History::Record *> &minerals);
};

#endif //MineralCluster_H
