// $Id: ClusterSquadsWidget.C 5279 2007-06-23 02:49:08Z mburo $

// ClusterSquads Widget Implementation

#include "ClusterSquadsWidget.H"
#include "ClusterSquad.H"
#include "GfxGlobal.H"
#include "GfxModule.H"
#include "GameStateModule.H"
#include "Game.H"

using namespace std;

//===================================================================

ClusterSquadsWidget::ClusterSquadsWidget(const ClusterSquads &s)
{
  num_players = -1;

  set_name("Clustering");

  squads = &s;
}

//===================================================================

void ClusterSquadsWidget::init()
{
  num_players = squads->squads.size();

  colors.resize(num_players);
  FORU (i, num_players) { // fixme: use better colors

    colors[i].resize(squads->squads[i]->MAX_K);
    FORU (j, squads->squads[i]->MAX_K) {

      Vec4<real4> v;
      FORU (k, 3) {
        v[k] = 0.4 + drand48()*0.6;
      }
      v[3] = 1.0;
      colors[i][j] = v;
    }
  }

  show_player.resize(num_players);
  FORU (i, num_players) {
    show_player[i] = 0;
  }
  if (num_players > 0) show_player[0] = 1;
}

//===================================================================

stype ClusterSquadsWidget::event_key_down(stype key, stype /*mod*/, stype x, stype y)
{
  if (is_outside(x,y)) return stype();
  cout << "event_key_down" << endl;
  
  int index = key.to_int();
  
  if (index >= '1' && index <= '9') {
    index -= '1';
    if (index < num_players) {
      show_player[index] = !show_player[index];
      return stype(1);
    }
  }

  return stype();
}

//===================================================================

stype ClusterSquadsWidget::draw()
{
  if (!!off) return stype();
  if (num_players < 0) init();
  if (num_players <= 0) return stype();

  wwidth  = squads->gsm.get_game().get_map().get_width();
  wheight = squads->gsm.get_game().get_map().get_height();
  sint4 tp = squads->gsm.get_game().get_tile_points();
  
  sint4 x = this->x.to_int();
  sint4 y = this->y.to_int();
  sint4 w = this->w.to_int();
  sint4 h = this->h.to_int();

  real8 xd = (real8)w/(real8)(wwidth*tp);
  real8 yd = (real8)h/(real8)(wheight*tp);

  draw_grid(wwidth, wheight);

  sint4 start_x = x + 16;
  sint4 start_y = y+h;

#if 1
  FORU (i, num_players) {
    if (!show_player[i]) continue;

    const ClusterSquad *cs = squads->squads[i];

    start_y -= 16;
    ostringstream oss;
    oss << cs->kmeans.size();
    glColor4f(1.0, 1.0, 1.0, 1.0);
    render_bitmap_string(start_x, start_y, GLUT_BITMAP_8_BY_13, oss.str().c_str());

    // objects
    FORALL (cs->obj2dat, it) {
      const GameObj *g = it->first;
      const ClusterSquad::ObjData &data = it->second;

      sint4 id = data.cluster_id;

      glColor4fv(colors[i][id].ptr());

      sint4 ux, uy;
      g->get_center(ux, uy);
      real8 wux = ux * xd;
      real8 wuy = uy * yd;

      real8 dx = xd * 5;
      real8 dy = yd * 5;

      glBegin(GL_QUADS);
      glVertex2f(x+wux-dx, y+h-wuy-dy);
      glVertex2f(x+wux+dx, y+h-wuy-dy);
      glVertex2f(x+wux+dx, y+h-wuy+dy);
      glVertex2f(x+wux-dx, y+h-wuy+dy);
      glEnd();
    }

    // cluster centers
    glEnable(GL_POINT_SMOOTH);
    real4 ps = ::max(xd, yd) * 20;
    glPointSize(ps);
    glBegin(GL_POINTS);
    sint4 id = 0;
    FORALL (cs->kmeans, it) {
      glColor4fv(colors[i][id++].ptr());
      sint4 ux = it->x;
      sint4 uy = it->y;
      real8 wux = ux * xd;
      real8 wuy = uy * yd;
      glVertex2f(x+wux, y+h-wuy);
    }
    glEnd();
    /*
    glPointSize(ps*0.75);
    glBegin(GL_POINTS);
    id = 0;
    FORALL (cs->prev_means[cs->kmeans.size()+1], it) {
      glColor4fv(colors[i][id++].ptr());
      sint4 ux = it->x;
      sint4 uy = it->y;
      real8 wux = ux * xd;
      real8 wuy = uy * yd;
      glVertex2f(x+wux, y+h-wuy);
    }
    glEnd();
    */
    glDisable(GL_POINT_SMOOTH);
  }
#endif

  glEnable(GL_TEXTURE_2D);
  
  return stype();
}

//===================================================================
