#ifndef PURSUEATTACK_H
#define PURSUEATTACK_H

// $Id: PursueAttack.H 5279 2007-06-23 02:49:08Z mburo $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "Global.H"
#include "EventHandler.H"
#include "ClientModule.H"
#include "LowAI.H"
#include "GameObj.H"
#include "GameChanges.H"

class GameStateModule;
//===================================================================

struct PursueData{ 
  GameObj *targ;
  enum State { MOVE_TO, ATTACK } state;
  sint4 move_failure_count;
  PursueData() : targ(0), move_failure_count(0) {}
}; 

class PursueAttack : public LowAI<PursueData>, public EventHandler, public ClientModule
{
public:
  static const sint4 FROM;
  static const sint4 PURSUE_START;
  static const sint4 PURSUE_STOP;
  static const std::string bp_name;
  static const sint4 FAILURE_CUTOFF;
  static const sint4 ATTACK_PRIORITY; ///<The priority of attack commands

  PursueAttack(GameStateModule &gsm);
  ~PursueAttack();
  /// \name LowAI methods
  //@{
  virtual void act(GameStateModule &gsm);
  virtual void add(GameObj*) { ERR("?"); }
  virtual void del(GameObj* pursuer);
  //@}

  void add(GameObj* pursuer, GameObj* targ);
  ///\name EventHandler Methods
  virtual bool handle_event(const Event &e);

protected:
  Vector<GameObj*> need_act;
  GameStateModule &gsm;
  /** returns the range of the object's weapon when firing at units on the given level */
  sint4 get_range(GameObj *obj, GameObj::ZCat zcat);
  void update(const GameChanges &changes);

};

class PursueEvent : public Event
{
public:
  PursueEvent(sint4 who, sint4 what, GameObj *pursuer_, GameObj *targ_) :
    Event(who, what), pursuer(pursuer_), targ(targ_) {}
  ~PursueEvent() {}

  GameObj* pursuer;
  GameObj* targ;
};

typedef std::map<GameObj*, PursueData> PursueMap;

REGISTER_TYPEOF(496, PursueMap::iterator);
REGISTER_TYPEOF(497, PursueMap::const_iterator);
#endif
