#include "FactorPath.H"
#include "ST_PFEngine.H"

namespace Movement
{
	class LocationGoal;

	void FactorTask::planTask(uint4 stopTime)
	{
		planCount++;
		pending = false;

		if( !goal->isValid() )
		{
			noPathFound = true;
			return;
		}
		
		Vec2D start = exec->getStartingPoint(units, goal);

		if(airOnly)
		{
			Vector<Vec2D> waypoints;
			waypoints.push_back(start);
			waypoints.push_back(goal->getCenter());
			path = Movement::MakeWaypointPath(waypoints, goal);
		}
		else
		{		
			sint4 radius = (sint4)exec->getPathRadius(units);
			path = pf->findPath(start, radius, goal, stopTime);
		}
		
		if( path ) 
		{
			guide = exec->getGuide(units, path);
			failed = false;
		}
		else 
		{
			noPathFound = true;
		}
	}

	bool FactorTask::isInvalidatedBySegment(const TerrainBase::Segment & segment) const
	{
		assert( isValid() );

		if(airOnly) return false;
		
		Vec2D a( segment.l1.x, segment.l1.y );
		Vec2D b( segment.l2.x, segment.l2.y );

		// Check if segment invalidates path
		return path->intersectsSegment(a, b);
	}

	void FactorTask::cancel()
	{
		boost::shared_ptr<FactorPath> fp = manager.lock();
		boost::shared_ptr<FactorTask> me = self.lock();
		fp->remove_task(me);
	}
}
