#include "Movement.H"

#include "GameObj.H"
#include "Delegate.H"

REGISTER_TYPEOF(400101, std::set<Movement::Task::ptr>::iterator);
REGISTER_TYPEOF(400102, std::set<Movement::Task::const_ptr>::const_iterator);

namespace Movement
{
	Context::Context(Movement::Module & mm, const std::string & pathfinder, const std::string & pathExec) :
		mm(mm), stopTime(0), pathfinder(pathfinder), pathExec(pathExec)
	{

	}
	
	Context::~Context()
	{
		FORALL(activeTasks, t)
		{
			Task & task(**t);
			task.cancel();
		}
	}

	void Context::handleStatusUpdate(const Task & task)
	{
		// TODO: This could be optimised
		if(task.isFinished())
		{
			// Remove task from list of activeTasks
			FORALL(activeTasks, t)
			{
				Task::ptr taskPtr(*t);
				if(taskPtr.get() == &task)
				{
					activeTasks.erase(taskPtr);
					break;
				}
			}
		}

		if(statProc)
		{
			statProc(task);
		}
	}

	Path::ptr Context::findPath(GameObj * unit, Goal::const_ptr goal) const
	{
		Vec2D start; unit->get_center(start.x, start.y);
		return findPath(start, unit->get_radius(), goal);
	}

	Path::ptr Context::findPath(const Vec2D & start, sint4 radius, Goal::const_ptr goal) const
	{
		return mm.findPath(pathfinder, start, radius, goal, stopTime);
	}

	Task::ptr Context::moveUnit(GameObj * unit, Goal::const_ptr goal)
	{
		std::set<GameObj *> units;
		units.insert(unit);
		return moveUnits(units, goal);
	}

	Task::ptr Context::moveUnits(const std::set<GameObj*> & units, Goal::const_ptr goal)
	{
		Task::ptr task( mm.moveUnits(pathfinder, pathExec, units, goal, make_delegate(this, &Context::handleStatusUpdate)) );
		activeTasks.insert( task );
		return task;
	}

	Task::ptr Context::moveUnitViaPath(GameObj * unit, Path::ptr path)
	{
		std::set<GameObj *> units;
		units.insert(unit);
		return moveUnitsViaPath(units, path);
	}

	Task::ptr Context::moveUnitsViaPath(const std::set<GameObj*> & units, Path::ptr path)
	{
		Task::ptr task( mm.moveUnitsViaPath(pathExec, units, path, make_delegate(this, &Context::handleStatusUpdate)) );
		activeTasks.insert( task );
		return task;
	}

	void Context::setPathfinderStopTime(uint4 milliseconds)
	{
		stopTime = milliseconds;
	}

	void Context::setPathfinderName(const std::string & pathfinder)
	{
		this->pathfinder = pathfinder;
	}

	void Context::setPathExecName(const std::string & pathExec)
	{
		this->pathExec = pathExec;
	}

	void Context::setStatusProcedure(const Task::StatusProc & statProc)
	{
		this->statProc = statProc;
	}
}
