
// This is an ORTS file (c) Kai Chen, licensed under the GPL

#if 0


#include "Global.H"
#include "Options.H"
#include "Movement.H"

#include "GameObj.H"

#include "TerrainBasicImp.H"
#include "DCDTLite.h"
#include "DCDTPath.h"
#include "Vertex.h"
#include "HalfEdge.h"
#include "PathFace.h"
#include "Point.h"


using namespace std;

#define EPSILON 0.00001f
#define NO_INFLUENCE 0.01f
#define SIDES 8
#define TRIES 10
//a float definition for pi
#define PI 3.1415926535897932384626433832795f

 



struct XTR_Object
{
        //the ID of the object being monitored
         ObjId id;
        //the triangle in which the object lies
         Face *face;
        //the influence of the object
         float influence;

        //constructor - initializes member variables
         XTR_Object(ScriptObj *_obj, Face *_face = NULL) : id (id = GameObj::get_obj_id(_obj)), face (_face)
         {
                  CalculateInfluence(_obj);
         }

        //calculates the object's influence based on its attributes
         void CalculateInfluence(ScriptObj *_obj)
         {
                //get the game object associated with the object
                  GameObj *gob = _obj->get_GameObj();
                //get its hit points and weapon
                  const sintptr *hp = gob->get_int_ptr("hp");
                  ScriptObj *weapon = gob->component("weapon");
                //if it has no weapon, it has no influence
                  if (weapon == NULL)
                  {
                           influence = 0.0f;
                           return;
                  }
                //get the minimum and maximum damage and cooldown for the weapon
                  const sintptr *min = weapon->get_int_ptr("min_damage");
                  const sintptr *max = weapon->get_int_ptr("max_damage");
                  const sintptr *cooldown = weapon->get_int_ptr("cooldown");
                //if any value failed, it has no influence
                  if ((hp == NULL) || (min == NULL) || (max == NULL) || (cooldown == NULL))
                  {
                           influence = 0.0f;
                  }
                  else
                  {
                        //otherwise the influence is calculated as the average damage of its weapon, divided by
                        //the number of turns between its attacks, times its hit points
                           influence = ((float)(*min) + (float)(*max)) / 2.0f * (float)(*hp) / (float)((*cooldown) + 1);
                  }
         }
};

struct XTR_Obstacle
{
        //the ID of the obstacle within ORTS
         ObjId ortsId;
        //the ID of the obstacle within the triangulation
         int dcdtId;

        //constructor - initializes the member variables
         XTR_Obstacle(ObjId _ortsId, int _dcdtId) : ortsId (_ortsId), dcdtId (_dcdtId) {}

        //returns whether or not the given object is an obstacle
         static bool IsObstacle(GameObj *gob)
         {
                // Harvester units sometimes become immobile while harvesting.
                // Do NOT consider them as obstacles or they will be trapped.
                  if(gob->bp_name() == "worker") return false;
                  if(gob->bp_name() == "dactyl") return false;
                  if(gob->bp_name() == "vector") return false;

                // Otherwise consider immobile land objects to be obstacles
                  return ((gob->has_attr("is_mobile")) && (gob->get_int("is_mobile") == 0) &&
                                    (gob->get_zcat(NULL) == Object::ON_LAND));
         }
};

class MyMeshObjFactory : public MeshObjFactory
{
         public:
                  MyMeshObjFactory(){}
                  ~MyMeshObjFactory(){}
         
         Vertex*   newVertex()   const { return new Vertex();   }
         HalfEdge* newHalfEdge() const { return new HalfEdge(); }
         Face*     newFace()     const { return new PathFace(); }
};


static sint4 myRound(real8 f)
{
         return static_cast<sint4>(f+0.5f);
};



namespace Movement
{
  
  
  
  //Mesh Obj factory used in the mesh
  static MyMeshObjFactory mof;
  static Mesh     pfMesh(mof);
         
  class TRLitePathfinder : public Pathfinder
  {
     

    std::auto_ptr<DCDT> pfDcdt;
    std::auto_ptr<DCDTPath> pfPath;
    Vector<XTR_Object> objects;
    Vector<XTR_Obstacle> obstacles;
    sint4 client, neutral;
    
   
    
    public:
      TRLitePathfinder(std::auto_ptr<DCDT> & pfDcdt);
      virtual ~TRLitePathfinder();
      void initialize(sint4 mapWidth, sint4 mapHeight,
                      sint4 tilePoints, sint4 clientPID,
                      sint4 neutralPID);
      void addObject(const Object * object);
      void updateObject(const Object * object);
      void removeObject(const Object * object);
      void addBoundary(const Vec2D & a, const Vec2D & b);
      
      void AddObstacle(GameObj *gob);
      void RemoveObstacle(GameObj *gob);
      void UpdateAbstraction();
      


      Path::ptr findPath(const Vec2D & start,
                         sint4 radius,
                         Goal::const_ptr goal,
                         uint4 stopTime = 0) const;
      Path::ptr FindPathToPoint(const Vec2D & start, const Vec2D & point, sint4 radius, const Goal::const_ptr & goal) const;
      
      void updateData() {}

      
  };

  TRLitePathfinder::TRLitePathfinder(std::auto_ptr<DCDT> & _pfDcdt) : pfDcdt(_pfDcdt)
  {
                 
           pfPath = std::auto_ptr<DCDTPath>(new DCDTPath(*pfDcdt));
         

  }

  TRLitePathfinder::~TRLitePathfinder()
  {
        
  }

  void TRLitePathfinder::initialize(sint4 mapWidth, sint4 mapHeight,
			sint4 tilePoints, sint4 clientPID,
			sint4 neutralPID)
  {
	  client = clientPID;
	  neutral = neutralPID;

	  float width = (float)(mapWidth * tilePoints);
	  float height = (float)(mapHeight * tilePoints);

	  IPolygon boundary;
          boundary.push_back(IPoint(0.0f,0.0f));
          boundary.push_back(IPoint(width,0.0f));
          boundary.push_back(IPoint(width, height));
          boundary.push_back(IPoint(0.0f,height));
          
          boundary.setClosed(true);

         pfDcdt->init(boundary);
         pfDcdt->initializeSectors();
         
  }
  
  
  void TRLitePathfinder::addObject(const Object * obj)
  {
           GameObj *gob = (GameObj *)obj;
           if (XTR_Obstacle::IsObstacle(gob))
           {
                    AddObstacle(gob);
                    UpdateAbstraction();
                    return;
           }
                
                // Otherwise, if unit is an enemy
           const sintptr *owner = gob->get_int_ptr("owner");
           if ((*owner == client) || (*owner == neutral))
           {
                    return;
           }
                //locates the triangle in which the object's center lies
           
           Face*     resultFace;
           HalfEdge* resultEdge;
           Vertex*   resultVertex;
          
           sint4 x, y;
           obj->get_center(x, y);
           
           Mesh::C cpx = x;
           Mesh::C  cpy = y;
           
           DCDT::LocateResult resType = pfDcdt->locatePoint(NULL, cpx, cpy, &resultFace, &resultEdge, &resultVertex);
           
           
                //sets the object's triangle to null if not found
           if ( resType == DCDT::NotFound)
           {
                    objects.push_back(XTR_Object(gob));
           }
           else if (resType == DCDT::VertexFound) 
           {
                    Face *face = resultVertex->getOutEdge()->getFace();
                    XTR_Object trobj(gob, face);
                    objects.push_back(trobj);
                    
                    //Kai: function UpdateInfluence is not implemented
                        //and updates the unit's influence in the triangulation
                    //pfDcdt->UpdateInfluence(face, trobj.influence);
                  

           } 
           else if (resType == DCDT::EdgeFound) 
           {

                    Face *face = resultEdge->getFace();
                    XTR_Object trobj(gob, face);
                    objects.push_back(trobj);
                    
                    //Kai: function UpdateInfluence is not implemented
                        //and updates the unit's influence in the triangulation
                    //pfDcdt->UpdateInfluence(face, trobj.influence);
           }
           else 
           {
                        //otherwise sets it to the triangle face found
                    Face *face = resultFace;
                    XTR_Object trobj(gob, face);
                    objects.push_back(trobj);
                    
                    //Kai: function UpdateInfluence is not implemented 
                        //and updates the unit's influence in the triangulation
                    //pfDcdt->UpdateInfluence(face, trobj.influence);
           }
  }
  void TRLitePathfinder::updateObject(const Object * obj)
  {
           GameObj *gob = (GameObj *)obj;
           if (XTR_Obstacle::IsObstacle(gob))
           {
                    RemoveObstacle(gob);
                    AddObstacle(gob);
                    UpdateAbstraction();
                    return;
           }       
        
                //only processes enemy units
           const sintptr *owner = gob->get_int_ptr("owner");
           if ((*owner == client) || (*owner == neutral))
           {
                    return;
           }
           ObjId id = GameObj::get_obj_id(gob);
                //goes through the objects
           for (uint4 i = 0; i < objects.size(); i++)
           {
                        //until it finds the desired object
                    if (objects[i].id == id)
                    {
                                //record the object's old triangle and influence
                             Face *oldFace = objects[i].face;
                             float oldInfluence = objects[i].influence;
                                //calculate the object's new influence
                             objects[i].CalculateInfluence((GameObj *)obj);
                                //remove the object's influence from it last triangle
                             if (objects[i].face != NULL)
                             {
                                      //Kai: function UpdateInfluence is not implemented
                                      //pfDcdt->UpdateInfluence(objects[i].face, -(objects[i].influence));
                             }
                             
                             
                             Face*     resultFace;
                             HalfEdge* resultEdge;
                             Vertex*   resultVertex;
          
                             sint4 x, y;
                             obj->get_center(x, y);
           
                             Mesh::C cpx = x;
                             Mesh::C  cpy = y;
           
                             DCDT::LocateResult result ;
                             
                             //if possible, uses the last triangle as the starting point
                             //otherwise, uses the triangle around the nearest sector midpoint
                             //then finds the triangle in which the object is currently centered
                             if (objects[i].face == NULL)
                             {
                                    result = pfDcdt->locatePoint(NULL, cpx, cpy, &resultFace, &resultEdge, &resultVertex);
                             }
                             else
                             {
                                    result = pfDcdt->locatePoint(objects[i].face, cpx, cpy, &resultFace, &resultEdge, &resultVertex);
                           
                             }
                                //sets the object's face to NULL if it wasn't found
                             if (result == DCDT::NotFound)
                             {
                                      objects[i].face = NULL;
                             }//otherwise, sets it appropriately
                             else if (result == DCDT::VertexFound) 
                             {
                                      objects[i].face = resultVertex->getOutEdge()->getFace();

                             } 
                             else if (result == DCDT::EdgeFound) 
                             {

                                      objects[i].face = resultEdge->getFace();
   
                             }
                             else 
                             {     
                                      objects[i].face = resultFace;
                             }
                             
                            
                                //checks if the object has moved triangles or if it has changed influence
                             if ((oldFace != objects[i].face) || (fabs(oldInfluence - objects[i].influence) > NO_INFLUENCE))
                             {
                                        //if it has and the old triangle was valid, subtract its influence from the old triangle
                                      if(oldFace != NULL)
                                      {      
                                               //Kai: function UpdateInfluence is not implemented 
                                              // pfDcdt->UpdateInfluence(oldFace, -oldInfluence);
                                      }
                                        //if the new triangle is valid, add the object's influence to the new triangle
                                      if (objects[i].face != NULL)
                                      {
                                               //Kai: function UpdateInfluence is not implemented
                                                //and updates the object's new influence on the map
                                               //pfDcdt->UpdateInfluence(objects[i].face, objects[i].influence);
                                      }
                             }
                             break;
                    }
           }
  }
  void TRLitePathfinder::removeObject(const Object * obj)
  {
           GameObj *gob = (GameObj *)obj;
           if (XTR_Obstacle::IsObstacle(gob))
           {
                    RemoveObstacle(gob);
                    UpdateAbstraction();
                    return;
           }       

                //only processes enemy units
           const sintptr *owner = gob->get_int_ptr("owner");
           if ((*owner == client) || (*owner == neutral))
           {
                    return;
           }
           ObjId id = GameObj::get_obj_id(gob);
                //goes through the list of objects
           for (std::vector<XTR_Object>::iterator i = objects.begin(); i != objects.end(); i++)
           {
                        //upon finding the matching object
                    if (i->id == id)
                    {
                             //Kai: function UpdateInfluence is not implemented
                             //pfDcdt->UpdateInfluence(i->face, -(i->influence));
                                //removes it from the list
                             objects.erase(i);
                             break;
                    }
           }
  }
  void TRLitePathfinder::addBoundary(const Vec2D & a, const Vec2D & b)
  {
  }
  
  void TRLitePathfinder::AddObstacle(GameObj *gob)
  {
           
           
           //create a closed polygon for the obstacle
           IPolygon poly;
           poly.setClosed(true);
           
                //check if the obstacle is rectangular
           if (gob->get_int("shape") == Object::RECTANGLE)
           {
                        //get its corners
                    sint4 x1, y1, x2, y2;
                    gob->get_p1(x1, y1);
                    gob->get_p2(x2, y2);
                        //create a polygon using them
                    poly.push_back(IPoint(x1,y1));
                    poly.push_back(IPoint(x2,y1));
                    poly.push_back(IPoint(x2,y2));
                    poly.push_back(IPoint(x1,y2));
                 
           }
                //check if the obstacle is circular
           else if (gob->get_int("shape") == Object::CIRCLE)
           {
                        //get its center and radius
                    sint4 cx, cy;
                    gob->get_center(cx, cy);
                    float r = (float)gob->get_radius() - 0.5f;
                    float x, y;
                    float angle = 0.0f;
                    float inc = 2.0f * PI / (float)SIDES;
                        //create a certain number of vertices to approximate the circle
                    for (int i = 0; i < SIDES; i++)
                    {
                                //calculate the position of the vertex
                             x = (float)cx + cos(angle) * r;
                             y = (float)cy + sin(angle) * r;
                                //add it to the polygon
                             poly.push_back(IPoint(x,y));
                                //and increment the angle to the next vertex
                             angle += inc;
                    }
           }
           else
           {
                        //we only deal with circular and rectangular obstacles
                    return;
           }
                //get the obstacle's ID within ORTS
           ObjId ortsId = GameObj::get_obj_id(gob);
               
                //Kai: Id issue ?? check if passing along ortsId is sufficient  
               //insert the obstacle's polygon into the triangulation and record its ID there
           pfDcdt->insertPolygon(poly,ortsId);
                //add this to our list of obstacles
           obstacles.push_back(XTR_Obstacle(ortsId, ortsId));
  }
  void TRLitePathfinder::RemoveObstacle(GameObj *gob)
  {
           //get the ID of the obstacle within ORTS
           ObjId ortsId = GameObj::get_obj_id(gob);
                //go through our list of obstacles
           for (std::vector<XTR_Obstacle>::iterator i = obstacles.begin(); i != obstacles.end(); i++)
           {
                        //when we find one with a matching ID,
                    if (ortsId == i->ortsId)
                    {
                                //if the object was successfully added to the triangulation,
                             if (i->dcdtId != -1)
                             {
                                        //remove it
                                      pfDcdt->removePolygon(i->dcdtId);
                             }
                                //remove it from the list of obstacles
                             obstacles.erase(i);
                             break;
                    }
           }
  
  }
  void TRLitePathfinder::UpdateAbstraction()
  {
           //delete the abstraction
           pfPath->deleteAbstraction();
           //rebuild it
           pfPath->abstract();
         
           //and record that the new faces have been processed
           //TraFace::Clear();
  }
      

  struct CloserTo
  {
           Vec2D point;
           CloserTo(const Vec2D & point) : point(point) {}
           bool operator()(const Vec2D & a, const Vec2D & b)
           {
                    return (point-a).lengthSq() < (point-b).lengthSq();
           }
  };

  Path::ptr TRLitePathfinder::findPath(const Vec2D & start,
                     sint4 radius,
                     Goal::const_ptr goal,
                     uint4 stopTime) const
  {
           const Goal & g(*goal);
           if(!g.isObstacle())
           {
                    return FindPathToPoint(start, g.getCenter(), radius, goal);
           }
           else
           {
                    std::vector<Vec2D> bounds;
                    g.getBoundaries(bounds, radius + 1);
                    std::sort(bounds.begin(), bounds.end(), CloserTo(start));
                    for(size_t i=0; i<bounds.size(); i++)
                    {
                             Path::ptr path = FindPathToPoint(start, bounds[i], radius, goal);
                             if(path || SDL_GetTicks() > stopTime)
                             {
                                    return path;
                             }
                    }
                    return Path::ptr();
           }
  }
  
  Path::ptr TRLitePathfinder::FindPathToPoint(const Vec2D & start, const Vec2D & point, sint4 radius, const Goal::const_ptr & goal) const
  {
           Uint32 time = SDL_GetTicks();
                     
           //Kai: time limit is not implemented
           /*if (!pfPath->searchPathBaseFast((float)start.x, (float)start.y, (float)point.x, (float)point.y,
                (float)radius))
           {
                    return Path::ptr();
           }*/
           
           if (!pfPath->searchPathBaseFast((float)start.x, (float)start.y, (float)point.x, (float)point.y,
                (float)radius))
           {
                    return Path::ptr();
           }

           Vector<Vec2D> waypoints;
        
                //get the path and record the first waypoint
           FPolygon CurPath = pfPath->getPath();
           waypoints.push_back( Vec2D(myRound(CurPath[0].x), myRound(CurPath[0].y)) );
                //go through the straight segments of the path
           for (int i = 0; i < CurPath.size() - 1; i += 2)
           {
                        //if another follows it, construct the curved segment in between
                    if (i +3 < CurPath.size() )
                    {
                             float p0x(CurPath[i+0].x), p0y(CurPath[i+0].y);
                             float p1x(CurPath[i+1].x), p1y(CurPath[i+1].y);
                             float p2x(CurPath[i+2].x), p2y(CurPath[i+2].y);
                             float p3x(CurPath[i+3].x), p3y(CurPath[i+3].y);

                             float nx(p2x - p3x), ny(p3y - p2x);
                             float vx(p1x - p0x), vy(p1y - p0y);

                             real8 denom = vx*nx + vy*ny; 
                             if(denom != 0.0f)
                             {
                                      real8 dx(p2x - p0x), dy(p2y - p0y);
                                      real8 t = (dx*nx + dy*ny) / denom;
                                      vx *= t; vy *= t; vx += p0x; vy += p0y;
                                      waypoints.push_back(Vec2D(myRound(vx), myRound(vy)));
                             }
                    }
           }

                //calculate and place the last vertex on the curve
           Vec2D l(myRound(CurPath[CurPath.size() - 1].x), myRound(CurPath[CurPath.size() - 1].y));
           if ((l.x != waypoints.back().x) && (l.y != waypoints.back().y))
           {
                    waypoints.push_back(l);
           }

                //reverse order
           for (uint4 i = 0; i <= (waypoints.size()-1)/2; i++)
           {
                    std::swap(waypoints[i], waypoints[waypoints.size()-1-i]);
           }

           return Movement::MakeWaypointPath(waypoints, goal);
  }

  Pathfinder::ptr MakeTriangulationPathfinder(DCDT ** pPF)
  {
           
     
           std::auto_ptr<DCDT> pfDcdt(new DCDT(pfMesh,10,10));
           if(pPF) (*pPF) = pfDcdt.get();
           return Pathfinder::ptr(new TRLitePathfinder(pfDcdt));
  }
   
}

#endif
