#ifndef _MOVEMENT_VEC_2D_H_
#define _MOVEMENT_VEC_2D_H_

#include "TerrainBase.H"

namespace Movement
{
	struct Vec2D
	{
		sint4 x; 
		sint4 y; 

		Vec2D() : x(0), y(0) {}
		Vec2D(sint4 x, sint4 y) : x(x), y(y) {}
		Vec2D(const Vec2D & v) : x(v.x), y(v.y) {}
		Vec2D(const TerrainBase::Loc & l) : x(l.x), y(l.y) {}

		Vec2D(const TerrainBase::Loc &a, const TerrainBase::Loc &b) : 
			x(b.x - a.x), y(b.y - a.y) {}

		inline const Vec2D & operator += (const Vec2D & v);
		inline const Vec2D & operator -= (const Vec2D & v);

		Vec2D operator+(const Vec2D & v) const { return Vec2D(x+v.x, y+v.y); }
		Vec2D operator-(const Vec2D & v) const { return Vec2D(x-v.x, y-v.y); }

		Vec2D project_onto(const Vec2D &other) const
		{
			real8 k = (real8)dot(*this, other) / dot(other, other);
			return other.multiply(k);
		}

		sint4 lengthSq() const { return x * x + y * y; }
		real8 length() const { return sqrt( (real8)(x*x + y*y) ); }

		Vec2D multiply(real8 k) const { return Vec2D(round(k * x), round(k * y)); }

		bool operator==(const Vec2D & v) const
		{
			return x == v.x && y == v.y;
		}

		bool operator!=(const Vec2D & v) const
		{
			return x != v.x || y != v.y;
		}

		inline static sint4 round(real8 real) 
		{
			return (sint4)floor(real + 0.5);
		}

		static sint4 dot(const Vec2D & a, const Vec2D & b)
		{
			return a.x * b.x + a.y * b.y;
		}

		static sint4 cross(const Vec2D & a, const Vec2D & b)
		{
			return a.x * b.y - a.y * b.x;
		}
    };

	const Vec2D & Vec2D::operator += (const Vec2D & v)
	{
		x += v.x;
		y += v.y;
		return *this;
	}

	const Vec2D & Vec2D::operator -= (const Vec2D & v)
	{
		x -= v.x;
		y -= v.y;
		return *this;
	}

	struct Seg2D
	{
		Vec2D a;
		Vec2D b;

		Seg2D() {}
		Seg2D(const Vec2D & a, const Vec2D & b) : a(a), b(b) {}
		Seg2D(const Seg2D & s) : a(s.a), b(s.b) {}

		sint4 lengthSq() const { return (b-a).lengthSq(); }
		real8 length() const { return (b-a).length(); }

		Seg2D getReversed() const { return Seg2D(b,a); }
	};
}

#endif
