// $Id: SuicideAttackAI.C 5279 2007-06-23 02:49:08Z mburo $

// This is an ORTS file (c) Tim Furtak, licensed under the GPL

#include "SuicideAttackAI.H"
#include "GameStateModule.H"
#include "Game.H"
#include "GameChanges.H"
#include "GameObj.H"
#include "GfxModule.H"
#include "TerrainModule.H"
#include "TerrainBase.H"

using namespace std;

const sint4 SuicideAttackAI::FROM = EventFactory::new_who();

//const std::string SuicideAttackAI::bp_name = "suicide_attack";

//===================================================================

SuicideAttackAI::SuicideAttackAI(GameStateModule &gsm_) : gsm(gsm_)
{
}

//===================================================================

SuicideAttackAI::~SuicideAttackAI()
{
}

//===================================================================

void SuicideAttackAI::stop(GameObj *obj)
{

  cout << "stopping object\n";
  FIND (obj2dat, it, obj);
  if (it != obj2dat.end()) obj2dat.erase(it);
}

//===================================================================

void SuicideAttackAI::act(GameStateModule &/*gsm*/)
{

  FORALL (need_act, i) {
    FIND (obj2dat, it, *i);
    if (it == obj2dat.end()) continue;

    SuicideData &s_data = it->second;
   
    switch(s_data.state) {

    case SuicideData::MOVE_TO:
      {
	TerrainBase::Task task(TerrainBase::Task::OBJ, TerrainBase::Task::ATTACK,
			       TerrainBase::Task::INDIVIDUAL);
	task.goal_obj = s_data.base;
	task.objs.insert(*i);
	cout << "Send Pathfinding Message\n";
        call_handlers(TerrainModule::PathEvent(FROM, TerrainModule::FIND_PATH_MSG, task));
      }
      continue;

    case SuicideData::ATTACK:
      {
        cout << "suicide attack ai -> ATTACK!!!" << endl;
	//         string bp = gd.res->bp_name();
	//         sint4 code = gd.res->code();
	//         ResBase &rb = res_map[bp];
	//         if (rb.resources.find(gd.res) == rb.resources.end()) {
	//           stop(*i);
	//           cout << "no resource?" << endl;
	//           continue;
	//         }

	//Shooter Ai should handle target selection:

// 	GameObj *targ = s_data.base;
//  	Vector<sint4> args;
//  	args.push_back(gsm.get_game().get_cplayer_info().get_id(targ));
//  	(*i)->component("weapon")->set_action("attack", args);
      }
      continue;

    default:
      ERR("??");
    }
  }

  need_act.clear();
}

//===================================================================

/**
 * Returns true if event was handled.
 */
bool SuicideAttackAI::handle_event(const Event &e)
{
  if (e.get_what() == TerrainModule::PATH_SUCCESS) {

    const TerrainModule::PathStatusEvent *p =
      dynamic_cast<const TerrainModule::PathStatusEvent*>(&e);
    assert(p && "SUCCESS: not a PathStatusEvent");

    FIND (obj2dat, it, p->get_obj());
    if (it == obj2dat.end()) return false;

    SuicideData::State &s = it->second.state;
    if (s == SuicideData::MOVE_TO) s = SuicideData::ATTACK;
    //if (s == GatherData::MOVE_BACK) s = GatherData::RETURN;
    need_act.push_back(p->get_obj());
    act(gsm);
    return true;
  }

  if (e.get_what() == TerrainModule::PATH_FAILURE) {
    cout << "Suicide ai got path failure" << endl;

    const TerrainModule::PathStatusEvent *p =
      dynamic_cast<const TerrainModule::PathStatusEvent*>(&e);
    assert(p && "FAILURE: not a PathStatusEvent");
    
    //find a new target that is different from the old one
    FIND (obj2dat, it, p->get_obj());
    if (it != obj2dat.end()){
      it->second.base = find_target(p->get_obj(), it->second.base);
      if (it->second.base){
	need_act.push_back(p->get_obj());
	act(gsm);
      }
      else{
	cout << "unable to find another target";
	//stop(p->get_obj());
      }
    }
    //stop(p->get_obj());
    return true;
  }



  //if (e.get_who() == GfxModule::FROM) {
    /*
    if (e.get_what() == GfxModule::ADD_CWIDGETS_MSG) {
      init_widget();
      return true;
    }
    */

//     if (e.get_what() == GatherAI::GATHER_START) {
//       REM("GATHER EVENT");
//       const GatherEvent &ge = static_cast<const GatherEvent&>(e);
//       if (ge.obj && ge.targ) {
//         GatherData gd;
//         gd.res = ge.targ;
//         gd.base = 0; // compute later
//         gd.state = GatherData::MOVE_TO;
//         obj2dat[ge.obj] = gd;
//         need_act.push_back(ge.obj);
//       }
//       return true;
//     }

//     if (e.get_what() == GatherAI::GATHER_STOP) {
//       REM("STOP GATHER EVENT");
//       const GatherEvent &ge = static_cast<const GatherEvent&>(e);
//       stop(ge.obj);
//       return true;
//     }
//  }

  if (e.get_who() == GameStateModule::FROM) {

    if (e.get_what() == GameStateModule::VIEW_MSG) {

      const sint4 me = gsm.get_game().get_client_player();
      
      // add new objects
      FORALL (gsm.get_changes().new_objs, i) {
        GameObj *gob = (*i)->get_GameObj();
        if (gob == 0) continue;
        if (*gob->sod.owner == me){
	  //fixme: Add only units capable of moving and attacking...
	  //Right now it adds
	  //everything that can move
	  if (gob->get_max_speed() >0){
	    SuicideData sd;
	    sd.base = find_target(gob);
	    sd.state = SuicideData::MOVE_TO;
	    obj2dat[gob] = sd;
	    if (sd.base){
	      need_act.push_back(gob);
	    }
	    else{
	      //waiting.push_back(gob);
	    }
	  }
	}
	else{
	  if (gob->bp_name() == "controlCenter") add_base(gob);
	}
      }
      
      // remove dead objects
      FORALL (gsm.get_changes().dead_objs, i) {
        GameObj *gob = (*i)->get_GameObj();
        if (gob == 0) continue;
        if (*gob->sod.owner == me){
	  stop(gob);
	}
	else if (gob->bp_name() == "controlCenter") del_base(gob);
      }
      
      // compute actions
      act(gsm);

      return true;
    }
  }

  return false;
}

//===================================================================

void SuicideAttackAI::add_base(GameObj *gob) {
  bases.insert(gob);
  
  FORALL(obj2dat, it){
    SuicideData &s_data = it->second;
    GameObj *target = find_target(it->first);
    if (s_data.base != target){
      s_data.base = target;
      s_data.state = SuicideData::MOVE_TO;
      need_act.push_back(it->first);
    }
  }

//   FORALL(waiting, i){
//     FIND (obj2dat, it, *i);
//     if (it != obj2dat.end()) {
//       SuicideData &s_data =  it->second;
//       s_data.base = gob;
//       s_data.state = SuicideData::MOVE_TO;
//       need_act.push_back(*i);
//     }
//     waiting.clear();
//   }
}

//===================================================================
void SuicideAttackAI::del_base(GameObj *gob) {
  bases.erase(gob);
  FORALL(obj2dat, i){
    SuicideData &s_data = i->second;
    if (s_data.base == gob){
      //find a new target
      GameObj *target = find_target(i->first);
      s_data.base = target;
      if (target){
	s_data.state = SuicideData::MOVE_TO;
	need_act.push_back(i->first);
      }
      s_data.base = target;
      //waiting.push_back(i->first);
      
    }
  }
}


//===================================================================

/**
 * Find and set a target for the targeter. If there are no more 
 * enemy command centers left, remove the targeter from the 
 */
GameObj* SuicideAttackAI::find_target(GameObj *targeter, GameObj *baseToAvoid){
  Game &game = gsm.get_game();
  GameObj *target = 0, *baseObj;
  real8 min_distance = -1; //set to -1 to avoid warning... still safe, I think
  real8 temp;
  FORALL(bases, i){
    baseObj = *i;
    if (baseObj != baseToAvoid){
      temp = game.distance(*targeter, *baseObj);
      if (!target || (temp < min_distance && temp >= 0)){
	min_distance = temp;
	target = baseObj;
      }
    }
  }
  
  if (target){
    cout << "new target found.\n";
    return target;
  }
  cout << "no more targets found.\n";
  return 0;

}

