#ifndef __BUILDING_PLACEMENT_H__
#define __BUILDING_PLACEMENT_H__

#include "TerrainBase.H"
#include "EventHandler.H"

class ModuleSet;
class GameStateModule;
class Game;
class GameObj;
namespace History { class Record; }

namespace TerrainAnalysis
{
    struct Size
    {
        sint4 x;
        sint4 y;

        Size() : x(0), y(0) {}
        Size(sint4 x, sint4 y) : x(x), y(y) {}
    };

    struct Rect
    {
        sint4 x0; ///< Inclusive index of left border
        sint4 y0; ///< Inclusive index of top border
        sint4 x1; ///< Exclusive index of right border
        sint4 y1; ///< Exclusive index of bottom border

        Rect() {}
        Rect(sint4 x0, sint4 y0, sint4 x1, sint4 y1) :
            x0(x0), y0(y0), x1(x1), y1(y1) {}

        bool contains(sint4 x, sint4 y) const 
        {
            return (x >= x0 && y >= y0 && x < x1 && y < y1);
        }

        Size size() const 
        { 
            return Size(x1-x0, y1-y0); 
        }
    };

    class GridAbstraction
    {
        ModuleSet & ms;
        GameStateModule & gsm;
        Game & game;
        const sint4 ppt;

        std::map<std::string, sint4> borders;

        Rect   area;   ///< Area of abstraction
        Size   dim;    ///< Cached dimensions of area
        sint4  height; ///< Height at which to build the abstraction

        /// Interesting objects
        mutable std::vector<const History::Record*> records; 
        mutable uint1* obs;    ///< Obstruction data
        mutable bool   valid;  ///< True if up-to-date

        bool get_building_size(const std::string &bp_name,
                               Size & result) const;

        void block_spot(sint4 x, sint4 y) const;
        void block_terrain() const;
        void block_obj(GameObj * obj) const;
        void update() const;

        bool is_blocked(sint4 x, sint4 y) const;
        bool is_blocked(const Rect & rect) const;

        sint4 eval_nearest(const TerrainBase::Loc & candidate,
                           const TerrainBase::Loc & near) const;
        sint4 eval_prox_min(const TerrainBase::Loc & candidate) const;
    public:
        GridAbstraction(ModuleSet & ms, 
                        const Rect & area);
        GridAbstraction(ModuleSet & ms, 
                        sint4 center_x, 
                        sint4 center_y, 
                        sint4 radius);
        ~GridAbstraction();

        /// Set the border around an object
        void set_object_border(const std::string & bp_name, sint4 border);

        /// Get the border around an object
        sint4 get_object_border(const std::string & bp_name) const;

        /// Set the height of terrain to consider "passable"
        void set_passable_height(sint4 passHeight);

        /// Sample the height of terrain at a given point
        sint4 sample_height(const TerrainBase::Loc & point) const;

        /// Locate a spot free from obstructions
        bool find_free_spot(const TerrainBase::Loc & near,
                            real8 maxDistance,
                            TerrainBase::Loc & result) const;

        /// Generate several candidate locations to build at
	size_t place_building(const std::string & building,
			      const TerrainBase::Loc & near,
                              Vector<TerrainBase::Loc> & results) const;
    };
}

#endif







