#ifndef __WATCHER_AUX_H__
#define __WATCHER_AUX_H__

#include "Watcher.H" // For UnitQuery, UnitPredicate, SetPredicate



namespace Watcher
{
    // Unit name strings
    extern const std::string WORKER;
    extern const std::string MARINE;
    extern const std::string TANK;

    extern const std::string CONTROL_CENTER;
    extern const std::string BARRACKS;
    extern const std::string FACTORY;

    // Comparators
    enum Comparator
    {
        LESS_THAN          = 0,
        GREATER_THAN       = 1,
        LESS_THAN_EQUAL    = 2,
        GREATER_THAN_EQUAL = 3,
        EQUAL_TO           = 4,
        NOT_EQUAL_TO       = 5,
    };

    // Unit Predicates
    UnitPredPtr Not(const UnitPredPtr & p0);

    UnitPredPtr And(const UnitPredPtr & p0, const UnitPredPtr & p1);
    UnitPredPtr And(const UnitPredPtr & p0, const UnitPredPtr & p1, 
                    const UnitPredPtr & p2);
    UnitPredPtr And(const UnitPredPtr & p0, const UnitPredPtr & p1, 
                    const UnitPredPtr & p2, const UnitPredPtr & p3);

    UnitPredPtr Or(const UnitPredPtr & p0, const UnitPredPtr & p1);
    UnitPredPtr Or(const UnitPredPtr & p0, const UnitPredPtr & p1, 
                   const UnitPredPtr & p2);
    UnitPredPtr Or(const UnitPredPtr & p0, const UnitPredPtr & p1, 
                   const UnitPredPtr & p2, const UnitPredPtr & p3);

    UnitPredPtr TypeIs(const std::string & type);
    UnitPredPtr IsBuilding();
    UnitPredPtr IsWorker();
    UnitPredPtr IsCombat();
    UnitPredPtr BuildingDone();

    UnitPredPtr OwnerIs(sint4 owner);

    UnitPredPtr HealthIs(Comparator comp, sint4 value);
    UnitPredPtr HealthPctIs(Comparator comp, sint4 value);
    UnitPredPtr StrengthIs(Comparator comp, sint4 value);

    UnitPredPtr InCircle(sint4 x, sint4 y, sint4 r);
    UnitPredPtr InRectangle(sint4 x0, sint4 y0, sint4 x1, sint4 y1);

    // Useful functions
    sint4 StrengthOf(const GameObj * unit);

    // Set Predicates
    SetPredPtr TotalCountIs(Comparator comp, sint4 value);
    SetPredPtr TotalStrengthIs(Comparator comp, sint4 value);
};

#endif // __WATCHER_AUX_H__
