// $Id: CWidget.C 5279 2007-06-23 02:49:08Z mburo $
 
// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "CWidget.H"

using namespace std;

//===================================================================

CWidget::CWidget()
{
  off = stype();

  x = y = stype(100);
  w = h = stype(400);
  
  layout = 0;
  layout_wrt = 0;
  name = "cwidget";

  add("event_mouse",  &CWidget::event_mouse,    0, 4, 0);
  add("event_key",    &CWidget::event_key,      0, 5, 0);

  add("set_size",     &CWidget::set_size,       0, 2, 0);
  add("set_position", &CWidget::set_position,   0, 2, 0);
  add("draw",         &CWidget::draw,           0, 0, 0);
  add("is_outside",   &CWidget::is_outside,     0, 2, 0);
  add("show",         &CWidget::show,           0, 0, 0);
  add("hide",         &CWidget::hide,           0, 0, 0);
}

//-------------------------------------------------------------------

CWidget::~CWidget() {}

//-------------------------------------------------------------------

void CWidget::add(const std::string &name, myfunc f, sint4 objs, sint4 p, sint4 p2)
{
  ObjActionDefinition def(name, objs, p, p2);
  ObjAction *a = new ObjAction(static_cast<ObjFunction>(f), def); // fixme: danger
  actions.push_back(a);
  act_map[name] = a;
}

//-------------------------------------------------------------------

stype CWidget::event_mouse(const SActParam &p) {
  VALN_CHECK(4);
  const sint4 MOUSE_DOWN    = 1;
  const sint4 MOUSE_UP      = 2;
  const sint4 MOUSE_MOVE    = 4;
  const sint4 MOUSE_HOVER   = 8;

  switch (p.val[0].to_int()) {
  case MOUSE_DOWN:  return event_mouse_down  (p.val[1], p.val[2], p.val[3]);
  case MOUSE_UP:    return event_mouse_up    (p.val[1], p.val[2], p.val[3]);
  case MOUSE_MOVE:  return event_mouse_motion(p.val[1], p.val[2], p.val[3]);
  case MOUSE_HOVER: return event_mouse_hover (p.val[1], p.val[2], p.val[3]);
  default: ERR("unhandled mouse event");
  }
  return stype();
}

stype CWidget::event_key(const SActParam &p) {
  VALN_CHECK(5);  
  const sint4 KEY_DOWN_MASK = 16;
  const sint4 KEY_UP_MASK   = 32;

  switch (p.val[0].to_int()) {
  case KEY_DOWN_MASK: return event_key_down (p.val[1], p.val[2], p.val[3], p.val[4]);
  case KEY_UP_MASK:   return event_key_up   (p.val[1], p.val[2], p.val[3], p.val[4]);
  default: break;
  }
  return stype();
}

stype CWidget::set_size(const SActParam &p) {
  VALN_CHECK(2);  
  return script_set_size(p.val[0], p.val[1]);
}

stype CWidget::set_position(const SActParam &p) {
  VALN_CHECK(2);  
  return script_set_position(p.val[0], p.val[1]);
}

stype CWidget::draw(const SActParam &) {
  return draw();
}

stype CWidget::is_outside(const SActParam &p) {
  VALN_CHECK(2);  
  return stype(is_outside(p.val[0], p.val[1]));
}

stype CWidget::show(const SActParam &) {
  return set_enabled(stype(1));
}

stype CWidget::hide(const SActParam &) {
  return set_enabled(stype(0));
}

//-------------------------------------------------------------------

ObjAction* CWidget::action(const string &name) const
{
  FIND (act_map, it, name);
  if (it != act_map.end()) return it->second;
  cout << "action = " << name << " not found" << endl;
  return 0;
}

//-------------------------------------------------------------------

stype CWidget::script_set_position(stype x, stype y)
{
  set_position(x.to_int(), y.to_int());
  return stype();
}

//-------------------------------------------------------------------

stype CWidget::script_set_size(stype w, stype h)
{
  set_size(w.to_int(), h.to_int());
  return stype();
}

//-------------------------------------------------------------------

stype CWidget::set_enabled(stype e)
{
  if (e == off) cout << "enabled = " << e.to_int() << endl;
  off = stype(!e);
  return stype();
}

//===================================================================

sintptr* CWidget::get_int_ptr(const string &a) {
  if (a == "x") return &x.val.i;
  if (a == "y") return &y.val.i;
  if (a == "width") return &w.val.i;
  if (a == "height") return &h.val.i;
  return 0;
}

const sintptr* CWidget::get_int_ptr(const string &a) const {
  if (a == "x") return &x.val.i;
  if (a == "y") return &y.val.i;
  if (a == "width") return &w.val.i;
  if (a == "height") return &h.val.i;
  return 0;
}

//===================================================================

void CWidget::set_position(sint4 x, sint4 y)
{
  this->x = stype(x);
  this->y = stype(y);
}

void CWidget::set_size(sint4 w, sint4 h)
{
  if (w < 0) w = 0;
  if (h < 0) h = 0;
  this->w = stype(w);
  this->h = stype(h);
}

bool CWidget::is_outside(stype X, stype Y) {
  return
    X.to_int() < x.to_int() || Y.to_int() < y.to_int() ||
    X.to_int() >= x.to_int() + w.to_int() || Y.to_int() >= y.to_int() + h.to_int();
}

//===================================================================


