// $Id: Gfx2d.C 7357 2008-08-07 02:38:57Z furtak $
 
// This is an ORTS file 
// (c) Michael Buro  
// (c) Tim Furtak    
// (c) Keith Yerex   
// licensed under the GPL

// 2d stuff and script gfx interface

#include "GfxGlobal.H"
#include "Global.H"
#include "GameStateModule.H"
#include "GameObj.H"
#include "GfxModule.H"
#include "Game.H"
#include "PlayerInfo.H"
#include "GfxObject.H"
#include "GfxTerrain.H"
#include "GfxAnimesh.H"
#include "Minimap.H"
#include "GameTile.H"
#include "Token.H"
#include "LayoutHV.H"
#include "Options.H"
#include "Image.H"
#include "GfxModule.H"
#include "ScriptModel.H"
#include "ScriptVector.H"
#include "Profiler.H"
#include "TextureStore.H"

using namespace std;


#define PARSE_ERR(s) {\
  errstr << endl << ": parse error at line " << tokenizer.cur_line();\
  ERR(s);\
}

//===================================================================

stype GfxModule::GUI_CreateView(const SActParam &)
{
  sint4 id;
  for (id = 1; id <= sint4(views.size()) + 1; ++id) {
    bool found = true;
    FORALL (views, i) {
      if ((*i).id == id) {
        found = false;
        break;
      }
    }
    if (found) break;
  }

  SDL_Rect r;
  r.x = 0;
  r.y = 0;
  r.w = 100;
  r.h = 100;

  GfxView v(r);
  v.id = id;
  
  if (get_game().is_rotated()) { v.x_angle = 65; v.y_angle = 45; }
  else                         { v.x_angle = 65; v.y_angle = 0;  }
  
  //  v.frame_color = view_colors[views.size()+1];
  v.frame_color = view_colors[1];
  
  v.vp_scale = max(v.viewport.w, v.viewport.h);
  
  cout << "NEW VIEW: "; v.write(); 
  views.push_back(v);

  update_active_view();
  return stype(id);
}

//-------------------------------------------------------------------

stype GfxModule::GUI_DeleteView(const SActParam &p)
{
  VALN_CHECK(1);  
  sint4 id = p.val[0].to_int();

  cout << "DELETING VIEW: " << id << endl;

  FORALL (views, i) {
    if ((*i).id == id) {
      *i = views[views.size()-1];
      views.resize(views.size()-1);
      update_active_view();
      return stype();
    }
  }
  return stype(1);
}

//-------------------------------------------------------------------

stype GfxModule::GUI_ResizeView(const SActParam &p)
{
  VALN_CHECK(5);  
  
  sint4 k = p.val[0].to_int();
  sint4 id = -1;

  FORALL (views, i) {
    if ((*i).id == k) {
      id = i-views.begin();
      break;
    }
  }

  if (id == -1) return stype(1);

  GfxView &v = views[id];

  v.viewport.x = p.val[1].to_int();
  v.viewport.y = p.val[2].to_int();
  v.viewport.w = p.val[3].to_int();
  v.viewport.h = p.val[4].to_int();

  v.vp_scale = max(v.viewport.w, v.viewport.h);
  update_active_view();
  return stype();
}

//===================================================================

void render_bitmap_string(GLfloat x, GLfloat y, void *font, const char *text)
{
  const sint4 lineHeight = 13;
    
  const char *c;
  glRasterPos2f(x,y);
  for (c=text; *c != '\0'; ++c) {
    if (*c == '\n') {
      y -= lineHeight;
      glRasterPos2f(x,y);
      continue;
    }
    glutBitmapCharacter(font, *c);
  }
}

//===================================================================

/** Negate MODELVIEW rotation and scaling so that objects are drawn facing the camera.
    Used for things like health bars.
*/

void GfxModule::overlay_mode()
{
  const GfxView &v = *cur_view;

  real4 modelview[16];
  glGetFloatv(GL_PROJECTION_MATRIX, modelview);

  real4 s1, s2, s3;

  if (opt.perspective_mode) {
    s1 = 8.0;
    s2 = 8.0;
    s3 = 8.0;
  } else {
    real8 m = min(modelview[0], modelview[5]);
    const real8 cap = 0.08;

    if (m > cap) m = cap;
    m = cap / m;
    s1 = s2 = s3 = 8.0 * m;
  }

  glGetFloatv(GL_MODELVIEW_MATRIX, modelview);

  // remove rotation, negate projection matrix scaling by adjusting modelview
  modelview[0] = s1;   modelview[4] = 0.0;  modelview[8]  = 0.0;
  modelview[1] = 0.0;  modelview[5] = s2;   modelview[9]  = 0.0;
  modelview[2] = 0.0;  modelview[6] = 0.0;  modelview[10] = s3;

  glLoadMatrixf(modelview);

  real4 s =  1.0 / v.vp_scale * 3;
  
  glScalef(s,s,1.0);
  
#if 0
  if (!perspective_mode) {
    sint4 d = v.viewport.w - v.viewport.h;
    if      (d > 0) glScalef(1.0, real4(v.viewport.w)/v.viewport.h, 1.0);
    else if (d < 0) glScalef(real4(v.viewport.h)/v.viewport.w, 1.0, 1.0);
  }
#endif
}

//===================================================================

/** Creates textures from a list of tga file names terminated by \</BITMAPS\> and,
    within the tokenizer, aliases each quoted filename as its texture id number.
*/

void GfxModule::load_bitmap_list(Tokenizer &tokenizer, const string &path)
{
  PROFILE("load bitmap list");

  string token;
  Image image;
  GLuint id;

  FOREVER {

    token = tokenizer.next_tag();
    if (token == "</BITMAPS>") break;
    if (token != "") ERR ("</BITMAPS> expected");
    token = tokenizer.next_token();

    bool rel_path = false;
    if (token == "!") {
      rel_path = true;
      token = tokenizer.next_token();
    }

    // fixme: ugly hack - just use strings now that we have script support
    if (tokenizer.token_type() == Tokenizer::NUMBER) continue; // probably already parsed

    if (tokenizer.token_type() != Tokenizer::STRING) {
      cout << endl << "found " << token << endl;
      PARSE_ERR("filename expected");
    }

    string t = token; // the token as it's used in the current file

    token = token.substr(1, token.size()-2);
    if (rel_path) {
      token = path + token;
    } else {
      token = opt.gfx_path + "/" + token;
    }

    id = TextureStore::load(token, Image::LINEAR);
    stringstream ss;
    ss << id;
    tokenizer.define_var(t, ss.str());
  }
}

//===================================================================

// just testing: output terrain heightfield as pgm.

void GfxModule::save_terrain()
{
  sint4 tp = get_game().get_tile_points();
  sint4 w = 128*tp+1;
  sint4 h = 128*tp+1;
  FILE *f = fopen("height.pgm", "w");
  fprintf(f, "P2\n%d %d\n%d\n", static_cast<int>(w), static_cast<int>(h), 65535);

  FORS (y, h) {
    FORS (x, w) {
      sint4 ti = get_game().get_map().xy2ind(x/tp, y/tp);
      real4 height = get_game().get_map()(ti).height_at(real4(x % tp)/tp, real4(y % tp)/tp);
      fprintf(f, "%d ", static_cast<int>(height*10000));
    }
  }
  
  fclose(f);

  f=fopen("texture.ppm","w");
  fprintf(f,"P6\n%d %d\n%d\n", int(minimap->map_width), int(minimap->map_height), 255);

  FORS (y, minimap->map_height)
    fwrite(&minimap->color_texture[(minimap->texture_width*y)*3], 3*minimap->map_width, 1, f);
  
  fclose(f);
}

//===================================================================

/** Convert minimap coordinates into real, centered playfield coordinates.
    Minimap midpoint is (0,0).
*/

void GfxModule::mm_to_playfield(sint4 mm_x, sint4 mm_y, real4 &pf_x, real4 &pf_y)
{
  if (mm_rect.w != mm_rect.h) ERR("mm w != h");
  
  real4 mx = real4(mm_x - mm_rect.x)/mm_rect.w-0.5;
  real4 my = 0.5-real4(mm_y - mm_rect.y)/mm_rect.h;
  real4 gtx = get_game().get_gtiles_x();
  real4 gty = get_game().get_gtiles_y();
  real4 m = gty / (gtx + gty);
  real4 scale_x, scale_y;
  real4 x=mx,y=my;
  
  if (get_game().is_rotated()) {
    scale_x = (2*sqrt(2.0)*::max(real8(m), 1.0-m))*(gtx + gty)*0.5;
    scale_y = scale_x;
  } else {
    scale_x=max(gtx,gty);
    scale_y=-scale_x;
  }

  if (opt.minimap_rot) {
    real4 angle;
    if (get_game().is_rotated())angle=(-views[active_view].y_angle+45)*M_PI/180.0;
    else angle = (-views[active_view].y_angle)*M_PI/180.0;
    real4 a = max(gtx, gty);
    real4 b = min(gtx, gty);
    real4 c = sqrt(1.0+(b*b)/(a*a));
    scale_x *= c;
    scale_y *= c;
    
    y = mx*sin(angle) - my*cos(angle);
    x = mx*cos(angle) + my*sin(angle);

    if (get_game().is_rotated()) {
      real4 angle=-0.25*M_PI;
      
      mx = -x*sin(angle) + y*cos(angle);
      my = x*cos(angle) + y*sin(angle);
      x = mx;
      y = my;
    }
  } else if (get_game().is_rotated()) {
    real4 angle = 0.25*M_PI;
    
    mx = -x*sin(angle) + y*cos(angle);
    my = x*cos(angle) + y*sin(angle);
    x = -mx;
    y = my;
  }

  if (!get_game().is_rotated() && !opt.minimap_rot) y = -y;

  pf_x = x*scale_x;
  pf_y = y*scale_y;
}

//===================================================================

void GfxModule::draw_minimap()
{
  real4 mmw, mmh;
  mmw = mm_rect.w;
  mmh = mm_rect.h;

  if (mmw != mmh) ERR("mm w != h");

  const Game &g = get_game();
  
  real4 gtx = g.get_gtiles_x();
  real4 gty = g.get_gtiles_y();

  glViewport(mm_rect.x, mm_rect.y, sint4(mmw-1), sint4(mmh));

  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();

  glOrtho(0, 1, 0, 1, -1, 1);

  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();

  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);

  // dim the mm rectangle
  glColor4f(0, 0, 0, 0.2);
  glBegin(GL_QUADS);
  glVertex2f(0,1);
  glVertex2f(0,0);
  glVertex2f(1,0);
  glVertex2f(1,1);
  glEnd();


  if (opt.minimap_rot) {
    // rotate to match view direction and scale to fit
    glTranslatef(0.5, 0.5, 0);
    if (g.is_rotated())
      glRotatef(views[active_view].y_angle-45, 0, 0, 1);
    else
      glRotatef(views[active_view].y_angle, 0, 0, 1);
    real4 a = max(gtx,gty);
    real4 b = min(gtx,gty);
    real4 c = sqrt(1.0/(1.0+(b*b)/(a*a)));
    glScalef(c, c, c);
    glTranslatef(-0.5, -0.5, 0);
  }

 
  GLfloat tcx[4], tcy[4];
  GLfloat m = gty / (gtx + gty);

  if (g.is_rotated()) {

    tcx[0] = m*minimap->scale_w;     tcy[0] = 0;
    tcx[1] = 0;                      tcy[1] = m*minimap->scale_h;
    tcx[2] = (1-m)*minimap->scale_w; tcy[2] = minimap->scale_h;
    tcx[3] = minimap->scale_w;       tcy[3] = (1-m)*minimap->scale_h;

    if (gtx > gty) {
      glTranslatef(0, 0.5*(1-real4(gty)/real4(gtx)), 0);
      glScalef(1, real4(gty)/real4(gtx), 1);
    } else {
      glTranslatef(0.5*(1-real4(gtx)/real4(gty)), 0, 0);
      glScalef(real4(gtx)/real4(gty), 1, 1);
    }

  } else {

    tcx[0] = 0;                tcy[0] = 0;
    tcx[1] = 0;                tcy[1] = minimap->scale_h;
    tcx[2] = minimap->scale_w; tcy[2] = minimap->scale_h;
    tcx[3] = minimap->scale_w; tcy[3] = 0;

    if (gtx > gty) {
      glTranslatef(0, 0.5*(1-real4(gty)/real4(gtx)), 0);
      glScalef(1, real4(gty)/real4(gtx), 1);
    } else {
      glTranslatef(0.5*(1-real4(gtx)/real4(gty)), 0, 0);
      glScalef(real4(gtx)/real4(gty), 1, 1);
    }
  }
 
  real4 fx = 1.0;
  real4 fy = 1.0;
  real4 mmw2 = mmw * 0.5;
  real4 mmh2 = mmh * 0.5;

  if (gtx > gty) {
    fy = real4(gty) / gtx;
    mmh2 *= fy;
  }
  if (gtx < gty) {
    fx = real4(gtx) / gty;
    mmw2 *= fx;
  }
  // minimap: blend textures

  glActiveTextureARB(GL_TEXTURE0);
  glDisable(GL_TEXTURE_2D);
  glActiveTextureARB(GL_TEXTURE1);
  glDisable(GL_TEXTURE_2D);

  glColor4f(0,0,0,1);

  glBegin(GL_QUADS);
  glVertex2f(0,1);
  glVertex2f(0,0);
  glVertex2f(1,0);
  glVertex2f(1,1);
  glEnd();

  
  glColor4f(1,1,1,1);
  glBlendFunc(GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA);
  
  glActiveTextureARB(GL_TEXTURE0);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, minimap->get_visible_tex_id());
  
  glActiveTextureARB(GL_TEXTURE1);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, minimap->get_color_tex_id());

  glBegin(GL_QUADS);
  glMultiTexCoord2fARB(GL_TEXTURE0,tcx[0], tcy[0]);
  glMultiTexCoord2fARB(GL_TEXTURE1,tcx[0], tcy[0]);
  glVertex2f(0,1);
  glMultiTexCoord2fARB(GL_TEXTURE0,tcx[1], tcy[1]);
  glMultiTexCoord2fARB(GL_TEXTURE1,tcx[1], tcy[1]);
  glVertex2f(0,0);
  glMultiTexCoord2fARB(GL_TEXTURE0,tcx[2], tcy[2]);
  glMultiTexCoord2fARB(GL_TEXTURE1,tcx[2], tcy[2]);
  glVertex2f(1,0);
  glMultiTexCoord2fARB(GL_TEXTURE0,tcx[3], tcy[3]);
  glMultiTexCoord2fARB(GL_TEXTURE1,tcx[3], tcy[3]);
  glVertex2f(1,1);
  glEnd();
  
  glDisable(GL_TEXTURE_2D);
  glActiveTextureARB(GL_TEXTURE0);
  glDisable(GL_TEXTURE_2D);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  //  minimap->draw_overlays();

  // draw units

  if (g.is_rotated()) {
    glScalef(1.0/(gtx*sqrt(2.0)), -1.0/(gty*sqrt(2.0)), 1);
    glTranslatef(-sqrt(2.0)*gty/2.0, -sqrt(2.0)*gty/2.0, 0);
    glRotatef(-45, 0, 0, 1);
  } else {
    glTranslatef(0, 1, 0);
    glScalef(1.0/gtx, -1.0/gty, 1);
  }

  const real4 inv_tp = g.get_inv_tile_points();

  glEnable(GL_POINT_SMOOTH);
  FORS (q, g.get_player_num()+1) {

    glColor3f(player_colors[q].r,
              player_colors[q].g,
              player_colors[q].b);

    FORALL (g.get_objs(q), i) {
      ScriptObj *s = *i;
      GameObj *gob = s->get_GameObj();
      if (gob == 0) continue;
      if (!gob->sod.in_game) continue;

      sint4 x = *gob->sod.x;
      if (x < 0) continue;

      if (*gob->sod.shape == Object::CIRCLE) {
        if (*gob->sod.radius == 0) continue;
        real4 r = *gob->sod.radius/3.0;
        if (r < 2) r = 2;
        glPointSize(r);
        sint4 y = *gob->sod.y;
        glBegin(GL_POINTS);
        glVertex2f(x * inv_tp, y * inv_tp);
        glEnd();
      } else if (*gob->sod.shape == Object::RECTANGLE) {
        sint4 x1 = *gob->sod.x1;
        sint4 x2 = *gob->sod.x2;
        sint4 y1 = *gob->sod.y1;
        sint4 y2 = *gob->sod.y2;
        glBegin(GL_LINE_LOOP);
        glVertex2f(x1 * inv_tp, y1 * inv_tp);
        glVertex2f(x2 * inv_tp, y1 * inv_tp);
        glVertex2f(x2 * inv_tp, y2 * inv_tp);
        glVertex2f(x1 * inv_tp, y2 * inv_tp);
        glEnd();
      }
    }
  }

  glColor4f(1, 1, 1, 0.8);
  {
    FORALL (sel_objs, i) {
      GameObj *obj = (*i)->gob;
      if (obj == 0) continue;
      if (*obj->sod.shape == Object::CIRCLE) {
        if (*obj->sod.radius == 0) continue;
        real4 r = *obj->sod.radius/3.0;
        if (r < 2) r = 2;
        glPointSize(r);
        sint4 x = *obj->sod.x;
        sint4 y = *obj->sod.y;
        glBegin(GL_POINTS);
        glVertex2f(x * inv_tp, y * inv_tp);
        glEnd();
      } else if (*obj->sod.shape == Object::RECTANGLE) {
        sint4 x1 = *obj->sod.x1;
        sint4 x2 = *obj->sod.x2;
        sint4 y1 = *obj->sod.y1;
        sint4 y2 = *obj->sod.y2;
        glBegin(GL_LINE_LOOP);
        glVertex2f(x1 * inv_tp, y1 * inv_tp);
        glVertex2f(x2 * inv_tp, y1 * inv_tp);
        glVertex2f(x2 * inv_tp, y2 * inv_tp);
        glVertex2f(x1 * inv_tp, y2 * inv_tp);
        glEnd();
      }
    }
  }
  glDisable(GL_POINT_SMOOTH);

  // draw active view quadrangle
  
  Col3 &col = view_colors[0];

  glColor3f(col.r, col.g, col.b);

  Coor3 c1 = views[active_view].c1;
  Coor3 c2 = views[active_view].c2;
  Coor3 c3 = views[active_view].c3;
  Coor3 c4 = views[active_view].c4;

  glEnable(GL_LINE_SMOOTH);

  glLineWidth(2.0);
  glColor3f(0, 0, 0); // draw the region visible from the active view
  glBegin(GL_LINE_STRIP);
  glVertex2f(c3.x, c3.y);
  glVertex2f(c4.x, c4.y);
  glVertex2f(c1.x, c1.y);
  glVertex2f(c2.x, c2.y);
  glEnd();

  glColor3f(1,1,1); // highlight top edge
  glBegin(GL_LINES);  
  glVertex2f(c2.x, c2.y);
  glVertex2f(c3.x, c3.y);
  glEnd();

  glLineWidth(1.0);
  glColor3f(1, 1, 1); // draw the region visible from the active view
  glBegin(GL_LINE_STRIP);
  glVertex2f(c3.x, c3.y);
  glVertex2f(c4.x, c4.y);
  glVertex2f(c1.x, c1.y);
  glVertex2f(c2.x, c2.y);
  glEnd();

  glDisable(GL_LINE_SMOOTH);

  //------------------------------

  glEnable(GL_DEPTH_TEST);
  //  glEnable(GL_LIGHTING);

  glPopMatrix();
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();

  glDisable(GL_SCISSOR_TEST);

  glerror("draw_minimap");
}

//===================================================================

real4 GfxModule::ground_height(real4 fx, real4 fy)
{
  const Map<GameTile> &map = game->get_map();
  sint4 mw = map.get_width();
  sint4 mh = map.get_height();  
  
  sint4 tx = (sint4)fx;
  sint4 ty = (sint4)fy;

  if (tx < 0) tx = 0;
  else if (tx >= mw) tx = mw - 1;

  if (ty < 0) ty = 0;
  else if (ty >= mh) ty = mh - 1;

  const GameTile &tile = map(tx, ty);
  // Determine the terrain height
  real4 dx = std::min(std::max(fx - tx, 0.0f), 1.0f);
  real4 dy = std::min(std::max(fy - ty, 0.0f), 1.0f);
  real4 h = tile.height_at(dx, dy) / GameConst::HEIGHT_MULT;
  
  return h;
}

//===================================================================

