#include "GfxModule.H"
#include "Options.H"

using namespace std;

GfxModule::Options::Options()
{
  ::Options::get("-title",       title);  
  ::Options::get("-w",           win_w);
  ::Options::get("-h",           win_h);
  ::Options::get("-x",           win_x);
  ::Options::get("-y",           win_y);
  ::Options::get("-perspective", perspective_mode);
  ::Options::get("-minimap",     minimap_on);
  ::Options::get("-minimap_rot", minimap_rot);
  ::Options::get("-shadowres",   shadow_res);
  ::Options::get("-blur",        blur);
  ::Options::get("-blurres",     blur_res);
  ::Options::get("-nosound",     no_sound);
  ::Options::get("-splash",      splash);
  ::Options::get("-cheapshadows",  cheap_shadows);  
  ::Options::get("-full",        full_screen);
  ::Options::get("-occ",         draw_occluded);
  ::Options::get("-clearfast",   clear_fast);

  ::Options::get("-modelmode",   model_mode);
  ::Options::get("-texlevel",    texture_level);
  ::Options::get("-texmode",     texture_mode);
  ::Options::get("-gt",          terrain_gamma);
  ::Options::get("-gm",          model_gamma);
  ::Options::get("-n1",          terrain_noise1);
  ::Options::get("-n2",          terrain_noise2);
  ::Options::get("-la",          ambient_light);
  ::Options::get("-ld",          diffuse_light);
  ::Options::get("-ls",          specular_light);
  ::Options::get("-gui",         gui_file);
  ::Options::get("-p",           path);
  ::Options::get("-gfx",         gfx_path);
  ::Options::get("-db",          db_file);
  ::Options::get("-fps",         fps);  
  ::Options::get("-seed",        seed);
  ::Options::get("-refresh",     refresh);
  ::Options::get("-tileset",     tileset);
  ::Options::get("-tile_vert",   terrain_vertex_shader);
  ::Options::get("-tile_frag",   terrain_fragment_shader);
  ::Options::get("-water_vert",   water_vertex_shader);
  ::Options::get("-water_frag",   water_fragment_shader);
  ::Options::get("-model_vert",   model_vertex_shader);
  ::Options::get("-model_frag",   model_fragment_shader);
  ::Options::get("-verbose",     verbose_load);
}


void GfxModule::Options::add()
{
  ::Options o("GfxModule");

  o.put("-title", string("ORTS GL Client"), "window title");  
  o.put("-w", (sint4)1000, "playscreen width");
  o.put("-h", (sint4)800,  "playscreen height");
  o.put("-x", (sint4)30,   "playscreen x left");
  o.put("-y", (sint4)30,   "playscreen y top");
  o.put("-perspective", true, "render views in perspective");
  o.put("-minimap",     true, "display minimap");
  o.put("-minimap_rot",  "rotate minimtap");

  o.put("-shadowres", (sint4)512,  "shadow map resolution");
  o.put("-blur",    (sint4)0,    "effects blur (0: disable)");
  o.put("-blurres", (sint4)512,  "effects blur resolution");

  o.put("-nosound",      "switch off sound");
  o.put("-splash",       "start the client with a \"loading\" splash screen");
  o.put("-cheapshadows", "fast and simple shadows");
  o.put("-full",         "fullscreen mode");
  o.put("-occ",          "draw occluded models");
  o.put("-clearfast",    "recompute .fast files");
  o.put("-verbose",      "print md2/md3 info when loading");

  o.put("-modelmode", (sint4)0, "0: display lists, 1: immediate mode with vertex interpolation, 2: vertex buffers");
  
  o.put("-texlevel", (sint4)0, "model texture scale (0 = full, 1 = half, 2 = quarter, etc)");
  o.put("-texmode",  (sint4)2, "0: Nearest, 1: Linear, 2: Bilinear, 3: Trilinear");

  // terrain
#if 0
  o.put("-tile_vert", string("shaders/tile.vert"), "terrain vertex shader");
  o.put("-tile_frag", string("shaders/tile.frag"), "terrain fragment shader");
  o.put("-water_vert", string("shaders/water.vert"), "water vertex shader");
  o.put("-water_frag", string("shaders/water.frag"), "water fragment shader");
  o.put("-model_vert", string("shaders/model.vert"), "model vertex shader");
  o.put("-model_frag", string("shaders/model.frag"), "model fragment shader");
#else
  o.put("-tile_vert", string(""), "terrain vertex shader");
  o.put("-tile_frag", string(""), "terrain fragment shader");
  o.put("-water_vert", string(""), "water vertex shader");
  o.put("-water_frag", string(""), "water fragment shader");
  o.put("-model_vert", string(""), "model vertex shader");
  o.put("-model_frag", string(""), "model fragment shader");
#endif

  // gamma
  o.put("-gt", (real8)1.2, "gamma: terrain");
  o.put("-gm", (real8)1.0, "gamma: models");

  // terrain noise
  o.put("-n1", (real8)0.003,  "terrain noise 1");
  o.put("-n2", (real8)0.0025, "terrain noise 2"); 

  // lighting
  o.put("-la", (real8)0.5,  "light: ambient");
  o.put("-ld", (real8)0.8,  "light: diffuse");
  o.put("-ls", (real8)0.5,  "light: specular");

  o.put("-gui",  string("gui.bp"),         "interface layout file");
  o.put("-p",    string("testgame/ortsg"), "data path");
  o.put("-gfx",  string("../../orts_data/trunk"), "3D model and bitmap path");
  o.put("-db",   string(""),               "script database file");

  o.put("-fps", true, "display fps");

  o.put("-seed", (sint4)0, "RNG seed (0:time)");

  o.put("-refresh", true, "maximum refresh rate");

  o.put("-tileset", string("tileset.txt"), "tileset file");
}

