#ifndef GfxView_H
#define GfxView_H

// $Id: GfxView.H 5279 2007-06-23 02:49:08Z mburo $

// This is an ORTS file
// (c) Michael Buro
// (c) Tim Furtak
// (c) Keith Yerex
// licensed under the GPL

#include "GfxGlobal.H"
#include "GfxObject.H"

// view container class, opengl coordinates!

class GfxView {
  
public:

  GfxView()
  {
    x_angle = 0;
    y_angle = 0;
    x_off   = y_off = z_off = 0;
    scale   = 1.0;
    viewport.x = 0;
    viewport.y = 0;
    viewport.w = 0;
    viewport.h = 0;
  }

  GfxView(const SDL_Rect &vp) : viewport(vp)
  {
    x_angle = 45;
    y_angle = 45;
    x_off   = y_off = z_off = 0;
    scale   = 1.0;
  }

  bool equal(const GfxView &x)
  {
    return
      y_angle == x.y_angle &&
      x_angle == x.x_angle &&
      scale   == x.scale   &&
      x_off   == x.x_off   &&
      y_off   == x.y_off   &&
      z_off   == x.z_off;
  }

  // screen point in viewport? (opengl coordinates!)
  bool inside(sint4 x, sint4 y) const
  {
    return
      x >= viewport.x && x < viewport.x + viewport.w &&
      y >= viewport.y && y < viewport.y + viewport.h;
  }

  static sint4 clamp_int(sint4 v, sint4 b, sint4 t) {
    if (v < b) return b;
    if (v > t) return t;
    return v;
  }

  SDL_Rect viewport;
  Col3 frame_color;

  sint4 id;
  
  real4 y_angle;
  real4 x_angle;
  real4 scale;
  real4 vp_scale; // size of the viewport
  real4 x_off;
  real4 y_off;
  real4 z_off;
  Vector<Coor3> corners;
  Coor3 c1, c2, c3, c4; // view corners on playfield (used in minimap)

  void move_x  (real4 amount) { change_offset(amount/vp_scale*scale, 0); }
  void move_y  (real4 amount) { change_offset(0, amount/vp_scale*scale); }
  void zoom    (real4 amount) {
    scale += amount*scale;
    if (scale > 2) scale = 2;
    if (scale < 0.01) scale = 0.01;
  }
  void rotate_x(real4 amount) { change_angle(x_angle, amount); }
  void rotate_y(real4 amount) { change_angle(y_angle, amount); }

  static void change_angle(real4 &angle, real4 amount) {
    assert(-360 < amount && amount < 360);
    angle += amount;
    if (angle >= 360) angle -= 360;
    if (angle <=-360) angle += 360;
  }
  
  // find delta in world coordinates that lead to window coordinate change (dx,dy)

  void change_offset(real4 dx, real4 dy)
  {
    const double limit = 0.1;

    // dy correction due to x_angle
    double f = sin((x_angle/180)*M_PI);
    if (f >= 0 && f < limit) f = limit;    // limit correction
    else if (f >= -limit && f < 0) f = -limit;
    dy /= f;
    
    // inverse rotation
    double phi = (y_angle/180)*M_PI;
    //    double psi = (x_angle/180)*M_PI;
    double c = cos(phi);
    double s = sin(phi);
    x_off -= dx*c+dy*s;
    z_off += (-dx*s+dy*c);
  }

  void write() const
  {
    std::cout << "view: " << viewport.x << " " << viewport.y
	      << " " << viewport.w << "x" << viewport.h
	      << " ya=" << y_angle
	      << " xa=" << x_angle
	      << " s=" << scale
	      << " xo=" << x_off
	      << " yo=" << y_off
	      << std::endl;
  }
};

#endif
