// $Id: ProfileWidget.C 6725 2008-01-28 09:12:35Z furtak $

// This is an ORTS file (c) Tim Furtak, licensed under the GPL

#include "ProfileWidget.H"
#include "GfxGlobal.H"
#include "GfxModule.H"

using namespace std;

const static sint4 line_height = 16;
const static sint4 start_off_y = -14;
const static real4 size = 7; // +/- size
const static sint4 indent = 14;

//===================================================================

ProfileWidget::ProfileWidget()
{
  set_name("Profiler");
  write_avg = true;
  
  stringstream avg;
  avg << "  AVG (" << ProfileLevel::window << " frames)";
  avg_text = avg.str();
}

//===================================================================

stype ProfileWidget::event_key_down(stype key, stype /*mod*/, stype x, stype y)
{
  if (is_outside(x,y)) return stype();

  if (key.to_int() == '\t') {
    write_avg = !write_avg;
    return stype(1);
  }
  return stype();
}

//===================================================================

stype ProfileWidget::event_mouse_down(stype /*mx*/, stype my_, stype /*mb*/)
{
  if (is_outside(x,y)) return stype();

  sint4 my = my_.to_int();

  sint4 dy = h.to_int() - (my - y.to_int());
  dy += start_off_y;
  dy -= 5; // magic offset
  if (dy <= 0) return stype(1);

  //  cout << "dy = " << dy << endl;
  dy /= line_height;
  
  if (dy < 0 || dy >= static_cast<sint4>(pl_at_line.size())) return stype(1);

  ProfileLevel *pl = pl_at_line[dy];
  if (pl->children.empty()) return stype(1);

  FIND (tree_state, it, pl);
  if (it != tree_state.end()) {
    LevelState ls = it->second;
    tree_state[pl] = ls != OPEN ? OPEN : EMPTY; // empty -> closed if appropriate
  }

  return stype(1);
}

//===================================================================

stype ProfileWidget::draw()
{
  PROFILE("draw me");

  glDisable(GL_TEXTURE_2D);

  sint4 X = x.to_int();
  sint4 Y = y.to_int();
  sint4 W = w.to_int();
  sint4 H = h.to_int();

  // black background
  glColor4f(0, 0, 0, 0.5);
  glBegin(GL_QUADS);
  glVertex2f(X,   Y);
  glVertex2f(X+W, Y);
  glVertex2f(X+W, Y+H);
  glVertex2f(X,   Y+H);
  glEnd();

  glEnable(GL_SCISSOR_TEST);
  glScissor(X, Y, W, H);

  sint4 start_x = X + 10;
  sint4 start_y = Y + H + start_off_y;

  glColor3f(1, 1, 1);

  if (write_avg) {
    render_bitmap_string(X, start_y, GLUT_BITMAP_8_BY_13, avg_text.c_str());
  } else {
    render_bitmap_string(X, start_y, GLUT_BITMAP_8_BY_13, "  TOTAL");
  }
  start_y -= line_height;

  pl_at_line.clear();
  draw_level(Profiler::get_root(), start_x, start_y);

  // white border
  glColor4f(0.5, 0.5, 0.5, 1);
  glBegin(GL_LINE_LOOP);
  glVertex2f(X    , Y);
  glVertex2f(X+W-1, Y);
  glVertex2f(X+W-1, Y+H-1);
  glVertex2f(X    , Y+H-1);
  glEnd();

  glEnable(GL_TEXTURE_2D);
  glDisable(GL_SCISSOR_TEST);

  return stype();
}

//===================================================================

void ProfileWidget::draw_level(ProfileLevel *pl, sint4 &x, sint4 &y)
{
  FIND (tree_state, it, pl);
  LevelState ls;

  bool empty = pl->children.empty();

  if (it == tree_state.end()) {
    ls = empty ? EMPTY : CLOSED;
    tree_state[pl] = ls;
  } else {
    ls = it->second;
    if (!empty && ls == EMPTY) {
      ls = CLOSED;
      tree_state[pl] = ls;
    }
  }

  const real4 s2 = size / 2;
  const real4 dx = -4.5;
  const real4 dy = +4.5;

  if (ls == CLOSED) {
    glBegin(GL_LINES);
    glColor3f(1,1,1);
    glVertex2f(x-s2+dx, y+dy);
    glVertex2f(x+s2+dx, y+dy);
    glVertex2f(x+dx,    y-s2+dy);
    glVertex2f(x+dx,    y+s2+dy);
    glEnd();
  } else if (ls == OPEN) {
    glBegin(GL_LINES);
    glColor3f(1,1,1);
    glVertex2f(x-s2+dx, y+dy);
    glVertex2f(x+s2+dx, y+dy);
    glEnd();
  }

  string text = pl->my_name + " ";
  real8 t = write_avg ? pl->avg_time : pl->total_time;
  char buf[64];
  sprintf(buf, "%.1f", static_cast<real8>(t));
  text += buf;
  
  pl_at_line.push_back(pl);
  render_bitmap_string(x, y, GLUT_BITMAP_8_BY_13, text.c_str());

  y -= line_height;

  if (ls == CLOSED) return;

  x += indent;
  FORALL (pl->children, it) draw_level(it->second, x, y);
  x -= indent;
}

//===================================================================

