#ifndef ScriptEmitter_H
#define ScriptEmitter_H

/** @file ScriptEmitter.H
    Particle emitter.

    $Id: ScriptEmitter.H 5279 2007-06-23 02:49:08Z mburo $
    $Source$
*/

// This is an ORTS file
// (c) Keith Yerex
// (c) Tim Furtak
// (c) Erik Benner

#include "GfxGlobal.H"
#include "GfxObject.H"
#include "GameObj.H"
#include "Global.H"
#include "Action.H"

class GfxEmitter;

//===================================================================

class ScriptEmitter : public ScriptObj
{
private:
  Vector<ObjAction*> actions;
  std::map<std::string, ObjAction*> act_map;
  GfxEmitter *emitter;

public:

  ScriptEmitter();
  ~ScriptEmitter();
  static ScriptObj* create() { return new ScriptEmitter; }

  std::string bp_name() const { return "ScriptEmitter"; }

  ObjAction* action(const std::string &name) const;

  GfxEmitter* get_emitter() { return emitter; }

  void destruct();

private:

  stype init_box       (const SActParam&);
  stype init_spray     (const SActParam&);
  stype init_circle    (const SActParam&);
  stype init_corona    (const SActParam&);
  stype init_fire      (const SActParam&);
  stype init_shell     (const SActParam&);
  stype init_lightning (const SActParam&);
  stype set_rate       (const SActParam&);
  stype position       (const SActParam&);
  stype rotate         (const SActParam&);
  stype scale          (const SActParam&);
  //  stype draw_2d        (const SActParam&);
  //  stype draw_overlay   (const SActParam&);
  stype set_parent     (const SActParam&);
  stype enable         (const SActParam&);

  typedef stype (ScriptEmitter::*myfunc)(const SActParam&);
  void add(const std::string &name, myfunc f, sint4 objs, sint4 p, sint4 p2);

  static const sint4 STYPE_GFXEMITTER_ID;
  
  GfxEmitter *stype_to_GfxEmitter(const stype &s) {
    return static_cast<GfxEmitter*>(s.to_special(STYPE_GFXEMITTER_ID));
  }

  stype stype_from_Emitter(GfxEmitter *e) {
    return stype(e, STYPE_GFXEMITTER_ID);
  }
  
};

#endif
