/** @file ScriptObjSet.C
    @see ScriptObjSet.H

    $Id: ScriptObjSet.C 5279 2007-06-23 02:49:08Z mburo $
    $Source$
*/

// This is an ORTS file (c) Tim Furtak

#include "Global.H"
#include "ScriptObjSet.H"
#include "GfxModule.H"
#include "Game.H"

using namespace std;

GfxModule* ScriptObjSet::gfxm;
Vector<ScriptObjSet*> ScriptObjSet::instances;

REGISTER_TYPEOF(260, Vector<ScriptObjSet*>::iterator);

//===================================================================

ScriptObjSet::ScriptObjSet()
{
  instances.push_back(this);

  add("select",     &ScriptObjSet::select,      0, 0, 0);
  add("select_add", &ScriptObjSet::select_add,  0, 0, 0);
  add("deselect",   &ScriptObjSet::deselect,    0, 0, 0);
}

ScriptObjSet::~ScriptObjSet()
{
  FORU (i, instances.size()) {
    if (instances[i] == this) {
      instances.erase(instances.begin() + i);
      break;
    }
  }
}

//-------------------------------------------------------------------

void ScriptObjSet::add(const std::string &name, myfunc f, sint4 objs, sint4 p, sint4 p2)
{
  ObjActionDefinition def(name, objs, p, p2);
  ObjAction *a = new ObjAction(static_cast<ObjFunction>(f), def); // fixme: nonportable
  actions.push_back(a);
  act_map[name] = a;
}

//===================================================================

stype ScriptObjSet::select(const SActParam &p)
{
  gfxm->sel_objs.clear();
  gfxm->script_sel_objs->clear();

  return select_add(p);
}

//-------------------------------------------------------------------

stype ScriptObjSet::select_add(const SActParam&)
{
  FORALL (data, it) {
    ScriptObj *x = it->to_obj();
    GameObj *y = x->get_GameObj();
    if (y == 0) continue;
    gfxm->sel_objs.insert(gfxm->game2gfx[y]);
  }

  FORALL (data, it) {
    ScriptObj *x = it->to_obj();
    gfxm->script_sel_objs->push_back(stype(x)); // fixme: use a set
  }

  gfxm->gui_obj->set_int("sel_change", 1, false);

  return stype();
}

//-------------------------------------------------------------------

stype ScriptObjSet::deselect(const SActParam&)
{
  return stype();
}

//===================================================================

void ScriptObjSet::invalidate_obj(ScriptObj *obj)
{
  FORALL (instances, it) {
    (*it)->invalidate(obj);
  }
}

//-------------------------------------------------------------------

void ScriptObjSet::invalidate(ScriptObj *obj)
{
  stype s(obj);
  FIND (data, it, s);
  if (it != data.end()) data.erase(it);
}

//===================================================================
