#ifndef Shader_H
#define Shader_H

// $Id: Shader.H 7357 2008-08-07 02:38:57Z furtak $

// This is an ORTS file
// (c) Tim Furtak
// licensed under the GPL

#include "Global.H"
#include "GfxGlobal.H"

//===================================================================

class Shader
{
public:
  GLuint shader;

  void clear();
  void compile();

protected:
  static char* read_source(const std::string &filename);
};

//-------------------------------------------------------------------

class VertexShader : public Shader
{
public:
  VertexShader() { shader = 0; }
  VertexShader(const std::string &filename) { load(filename); }

  bool load(const std::string &filename);

  ~VertexShader() {
    if (shader != 0) {
      glDeleteObjectARB(shader);
      shader = 0;
    }
  }
};

//-------------------------------------------------------------------

class FragmentShader : public Shader
{
public:
  FragmentShader() { shader = 0; }
  FragmentShader(const std::string &filename) { load(filename); }

  bool load(const std::string &filename);

  ~FragmentShader() {
    if (shader != 0) {
      glDeleteObjectARB(shader);
      shader = 0;
    }
  }
};

//-------------------------------------------------------------------

class ShaderProgram
{
public:
  ShaderProgram();
  ~ShaderProgram();

  GLuint id() const { return prog; }
  void fragment_shader(const std::string &filename);
  void vertex_shader(const std::string &filename);
  void reload();
  static void reload_all();

  bool begin() const {
    if (!glUseProgram || !prog) return false;
    glUseProgram(prog);
    return true;
  }

  void end() const {
    if (glUseProgram) glUseProgram(0);
  }

  static void cleanup();

protected:
  GLuint prog;
  FragmentShader *fs;
  VertexShader *vs;

  std::string fname, vname;

  static std::set<ShaderProgram*> shaders;

  void link();
};

//-------------------------------------------------------------------

/** Support for multiple passes, arbitrary textures. */

class CustomShaderProgram
{
public:

  CustomShaderProgram();
  ~CustomShaderProgram();

  bool load(const std::string &filename);
  bool load(ShaderProgram *sp);

  void reload();
  static void reload_all();

  struct Pass {
    ShaderProgram *prog;
    Vector<GLint> var_loc; ///< Variable location in the shader.
    Vector<std::string> var_name; ///< Variable name in the shader.
    Vector<GLuint> var_tex; ///< Texture to load into location.

    Pass();
    ~Pass();
    void setup() const;
  };

  sint4 num_passes() const { return passes.size(); }
  const Pass& get_pass(sint4 i) const { return passes[i]; }

  static void cleanup();

protected:

  std::string fname;
  Vector<Pass> passes;
  static std::set<CustomShaderProgram*> shaders;
};

//===================================================================

REGISTER_TYPEOF(660, std::set<ShaderProgram*>::iterator);
REGISTER_TYPEOF(661, std::set<CustomShaderProgram*>::iterator);
//REGISTER_TYPEOF(661, std::set<ShaderProgram*>::const_iterator);


#endif
