// $Id: TextureStore.C 7358 2008-08-07 02:54:21Z furtak $
 
// This is an ORTS file 
// (c) Tim Furtak    
// licensed under the GPL

#include "Global.H"
#include "TextureStore.H"

using namespace std;

map<string, GLuint> TextureStore::file2tex;
map<GLuint, string> TextureStore::tex2file;

//===================================================================

GLuint TextureStore::load(const string &filename, Image::FILTER filter, GLuint id)
{
  string fn = sanitize_path(filename);

  FIND (file2tex, it, fn);
  if (it != file2tex.end()) return it->second;
  
  Image image;

  if (fn.substr(fn.size()-4, 4) == ".tga") {
    image.loadTGA(fn.c_str());
  } else {
    cout << "unknown bitmap type " << fn.substr(fn.size()-4, 4) << endl;
  }

  id = image.createTexture(filter, 0, id);

  file2tex[fn] = id;
  tex2file[id] = fn;
  return id;
}

//-------------------------------------------------------------------

void TextureStore::reload(const string &filename)
{
  string fn = sanitize_path(filename);

  FIND (file2tex, it, fn);
  if (it != file2tex.end()) { reload(it->second); return; }
  REM2("file not loaded:", fn.c_str());
}

//-------------------------------------------------------------------

void TextureStore::unload(const string &filename)
{
  string fn = sanitize_path(filename);

  FIND (file2tex, it, fn);
  if (it != file2tex.end()) { unload(it->second); return; }
}

//-------------------------------------------------------------------

void TextureStore::reload(GLuint id)
{
  string file = tex2file[id];
  file2tex.erase(file);
  load(file, Image::LINEAR, id);
}

//-------------------------------------------------------------------

void TextureStore::unload(GLuint id)
{
  string file = tex2file[id];

  if (glIsTexture(id)) {
    glDeleteTextures(1, &id);
  }

  tex2file.erase(id);
  file2tex.erase(file);
}

//===================================================================
