#ifndef ByteData_H
#define ByteData_H

/** @file ByteData.H
    Binary encoding of integers.

    $Id: ByteData.H 6531 2008-01-07 11:00:55Z furtak $
    $Source$
*/

// This is an ORTS file (c) Tim Furtak, licensed under the GPL

#include "Global.H"

//===================================================================

/*
inline void write_b_int4(char* &s, sint4 num)
{
  ERR("foo");
  char x;

  if (num < 0) { x = 0x40; num = -num; }
  else         { x = 0x00; }

  x += num & 0x3F; num >>= 6;

  while (num) {
    x += 0x80; *s++ = x;
    x = num & 0x7F; num >>= 7;
  }
  *s++ = x;
}

//-------------------------------------------------------------------

inline void write_b_int8(char* &s, sint8 num)
{
  ERR("foo");
  char x;

  if (num < 0) { x = 0x40; num = -num; }
  else         { x = 0x00; }

  x |= num & 0x3F; num >>= 6;

  while (num) {
    x |= 0x80; *s++ = x;
    x = num & 0x7F; num >>= 7;
  }
  *s++ = x;
}

//-------------------------------------------------------------------

inline void write_b_flat4(char* &s, uint4 num)
{
  ERR("foo");
  FORU(i, 4) {
    s[3-i] = num & 255;
    num >>= 8;
  }
  s+=4;
}
*/
//-------------------------------------------------------------------

inline void write_b_int4(std::string &s, sint4 num)
{
  char x;
  uint4 n;

  if (num < 0) { x = 0x40; n = uint4(-(num+1))+1; }
  else         { x = 0x00; n = num; }

  x += n & 0x3F; n >>= 6;

  while (n) {
    x += 0x80; s += x;
    x = n & 0x7F; n >>= 7;
  }
  s += x;
}

//-------------------------------------------------------------------

inline void write_b_int8(std::string &s, sint8 num)
{
  char x;
  uint8 n;

  if (num < 0) { x = 0x40; n = uint8(-(num+1))+1; }
  else         { x = 0x00; n = num; }

  x |= n & 0x3F; n >>= 6;

  while (n) {
    x |= 0x80; s += x;
    x = n & 0x7F; n >>= 7;
  }
  s += x;
}

//-------------------------------------------------------------------

inline void write_b_flat4(std::string &s, uint4 num)
{
  char u[4];
  FORU (i, 4) {
    u[3-i] = num & 255;
    num >>= 8;
  }
  s.append(u, 4);
}

//===================================================================

inline char read_b_char(const char* &s, const char *end)
{
  if (s >= end) { std::cerr << "past end of stream" << std::endl; return 0; }
  return *s++;
}

//-------------------------------------------------------------------

/** Read an integer from a byte string and advance the pointer. */

inline sint4 read_b_int4(const char* &s, const char *end)
{
  if (s >= end) { std::cerr << "past end of stream" << std::endl; return 0; }
  bool negate = *s & 0x40;
  sint4 shift = 0;

  uint4 v = *s & 0x3F; shift += 6;

  while (*s & 0x80) {
    s++;
    if (s >= end) { std::cerr << "past end of stream" << std::endl; break; }
    if (shift > 32) ERR("danger");
    
    v += sint4(*s & 0x7F) << shift;
    shift += 7;
  }
  s++;

  if (negate) return -(sint4)v;
  return v;
}

//-------------------------------------------------------------------

inline sint8 read_b_int8(const char* &s, const char *end)
{
  if (s >= end) { std::cerr << "past end of stream" << std::endl; return 0; }
  bool negate = *s & 0x40;
  sint4 shift = 0;

  uint8 v = *s & 0x3F; shift += 6;

  while (*s & 0x80) {
    s++;
    if (s >= end) { std::cerr << "past end of stream" << std::endl; break; }
    if (shift > 57) ERR("more danger");

    v += sint8(*s & 0x7F) << shift;
    shift += 7;
  }
  s++;

  if (negate) return -(sint4)v;
  return v;
}

//-------------------------------------------------------------------

inline uint4 read_b_flat4(const char* &s, const char *end)
{
  if (s >= end) { std::cerr << "past end of stream" << std::endl; return 0; }
  uint4 x = uint1(*s++);
  FORU (i, 3) {
    if (s >= end) { std::cerr << "past end of stream" << std::endl; break; }
    x <<= 8;
    x += uint1(*s++);
  }
  return x;
}

//===================================================================

class ByteStream
{
public:

  ByteStream(const char *str, int len) {
    start = str;
    s = str;
    end = s + len;
    alloc = false;
  }

  ByteStream(const char *str, const char *_end) {
    start = str;
    s = str;
    end = _end;
    alloc = false;
  }

  ByteStream(const std::string &str) {
    char *r = new char[str.length()+1];
    if (!r) ERR ("couldn't allocate buffer");
    memcpy(r, str.c_str(), str.length()+1);
    start = r;
    s = r;
    end = start + str.length();
    alloc = true;
  }

  ~ByteStream() {
    if (alloc) delete [] start;
  }

  inline char read_b_char() { return ::read_b_char(s, end); }
  inline char peek_b_char() const { return *s; }
  inline void advance(int a=1) { s += a; }
  inline sint4 read_b_int4() { return ::read_b_int4(s, end); }
  inline sint4 read_b_int8() { return ::read_b_int8(s, end); }
  inline sint4 read_b_flat4() { return ::read_b_flat4(s, end); }

  inline sint4 read_c_int4() {
    char *e;
    int r = strtol(s, &e, 10);
    s = e;
    if (e > end) { std::cerr << "past end of stream" << std::endl; }
    return r;
  }

  inline sint4 length() const { return end - s; }

  inline const char* ptr() const { return s; }

  inline void set_ptr(const char *ptr) { s = ptr; }

  inline bool good() const { return s < end; }

private:

  bool alloc;
  const char *start;
  const char *s;
  const char *end;
};

//===================================================================

#endif
