#ifndef Collision2D_H
#define Collision2D_H

/** @file Collision2D.H
    Computes collision times for simple primitives.

    $Id: Collision2D.H 5279 2007-06-23 02:49:08Z mburo $
    $Source$
*/

// This is an ORTS file 
// (c) Tim Furtak
// (c) Michael Buro
// licensed under the GPL

#include "Global.H"
#include "Coord.H"

//===================================================================

namespace collision2d
{
  // internal functions

  bool solve_quadratic(real8 a, real8 b, real8 c, real8 &r1, real8 &r2);
 
  bool collision_outside(real8 a, const ScalarPoint &center, const ScalarPoint &h,
                         real8 rr, real8 u, real8 v,
                         const Scalar &lx, const Scalar &ly,
                         real8 &t);
  
  bool collision_inside_x(const ScalarPoint &c, const ScalarPoint &h,
                          const Scalar &r, real8 u, real8 v,
                          const Scalar &lx, real8 &t);
  
  bool collision_inside_y(const ScalarPoint &c, const ScalarPoint &h,
                          const Scalar &r, real8 u, real8 v,
                          const Scalar &ly, real8 &t);

  //===================================================================
  // more accessible functions
  
  real8 collision_time_circle_point(const ScalarPoint &c, real8 rr,
                                    const ScalarPoint &p,
                                    const ScalarPoint &h, real8 a,
                                    real8 time = 0);

  real8 collision_time_circle_point(const ScalarPoint &c, real8 rr, const ScalarPoint &h1,
                                    const ScalarPoint &p, const ScalarPoint &h2,
                                    real8 time = 0);

  real8 collision_time_circle_hvline(const ScalarPoint &c, real8 r, const ScalarPoint &h1,
                                     const ScalarPoint &a1, const ScalarPoint &a2, const ScalarPoint &h2,
                                     sint4 &coll_type, real8 time = 0);

  real8 collision_time_circle_line(const ScalarPoint &c, real8 r, const ScalarPoint &h1,
                                   const ScalarPoint &a1, const ScalarPoint &a2, const ScalarPoint &h2,
                                   sint4 &coll_type, real8 time = 0);

  real8 collision_time_circle_rect(const ScalarPoint &c, real8 r, const ScalarPoint &h1,
                                   const ScalarPoint &a1, const ScalarPoint &a2, const ScalarPoint &h2,
                                   sint4 &coll_type, real8 time = 0);

  real8 collision_time_line_point(const ScalarPoint &a1, const ScalarPoint &a2, const ScalarPoint &h1,
                                  const ScalarPoint &p, const ScalarPoint &h2,
                                  sint4 &coll_type, real8 time = 0);

  real8 collision_time_line_line(const ScalarPoint &a1, const ScalarPoint &a2, const ScalarPoint &h1,
                                 const ScalarPoint &b1, const ScalarPoint &b2, const ScalarPoint &h2,
                                 sint4 &coll_type, real8 time = 0);

  real8 collision_time_line_rect(const ScalarPoint &a1, const ScalarPoint &a2, const ScalarPoint &h1,
                                 const ScalarPoint &b1, const ScalarPoint &b2, const ScalarPoint &h2,
                                 sint4 &coll_type, real8 time = 0);

  real8 collision_time_rect_rect(const ScalarPoint &a1, const ScalarPoint &a2, const ScalarPoint &h1,
                                 const ScalarPoint &b1, const ScalarPoint &b2, const ScalarPoint &h2,
                                 sint4 &coll_type, real8 time = 0);

  //===================================================================

  sint4 resolve_collision_circle_point(const ScalarPoint &c, real8 rr,
                                       const ScalarPoint &h1,
                                       const ScalarPoint &p,
                                       const ScalarPoint &h2,
                                       real8 time);

  sint4 resolve_collision_circle_circle(const ScalarPoint &c1, real8 rr1,
                                        const ScalarPoint &h1,
                                        const ScalarPoint &c2, real8 rr2,
                                        const ScalarPoint &h2,
                                        real8 time);

  sint4 resolve_collision_circle_line(const ScalarPoint &c, real8 r,
                                      const ScalarPoint &h1,
                                      const ScalarPoint &a1,
                                      const ScalarPoint &a2,
                                      const ScalarPoint &h2,
                                      sint4 coll_type, real8 time);

  sint4 resolve_collision_circle_rect(const ScalarPoint &c, real8 r,
                                      const ScalarPoint &h1,
                                      const ScalarPoint &a1,
                                      const ScalarPoint &a2,
                                      const ScalarPoint &h2,
                                      sint4 coll_type, real8 time);

  //===================================================================

  extern real8 no_collision;
  extern real8 now;
}

//===================================================================

#endif
