// $Id: Compress.C 5279 2007-06-23 02:49:08Z mburo $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "Compress.H"
#include <zlib.h>

using namespace std;

static voidpf zmalloc(voidpf, uInt items, uInt size)
{
  voidpf r = malloc(items*size);
  if (r) return r;
  return Z_NULL;
}

static void zfree(voidpf, voidpf address)
{
  free(address);
}

Compress::Compress(sint4 level_) {

  out_buffer = new sint1[N];
  if (!out_buffer) ERR("out of memory");

  pzs = new z_stream_s;
  if (!pzs) ERR("out of memory");

  pzs->zalloc = zmalloc;
  pzs->zfree  = zfree;

  init(level_);
}


void Compress::init()
{
  if (deflateInit(pzs, level) != Z_OK) ERR("deflateInit problem");
}


void Compress::init(sint4 level_)
{
  level = level_;
  if (level > 9 || level < 0) ERR("illegal compression level");
  init();
}

Compress::~Compress() {
  if (pzs) {
    deflateEnd(pzs);
    delete pzs;
    pzs = 0;
  }
  if (out_buffer) { delete [] out_buffer; out_buffer = 0; }
}

void Compress::doit(const string &input, string &output) {

  const char *s = input.c_str();

  pzs->next_in = const_cast<Bytef*>(reinterpret_cast<const Bytef*>(s));
  pzs->avail_in = input.size();
  pzs->next_out = reinterpret_cast<Bytef*>(out_buffer);
  pzs->avail_out = N;
  
  output = "";
  
  while (pzs->avail_in) {
    
    if (pzs->avail_out == 0) {
      string s(reinterpret_cast<const char*>(out_buffer), N);
      output += s;
      pzs->next_out = reinterpret_cast<Bytef*>(out_buffer);
      pzs->avail_out = N;
    }

    deflate(pzs, Z_NO_FLUSH);
  }

  deflate(pzs, Z_SYNC_FLUSH);

  if (pzs->avail_out != N) {
    string s(reinterpret_cast<const char*>(out_buffer), N-pzs->avail_out);
    output += s;
  }
}

Decompress::Decompress() {

  out_buffer = new sint1[N];
  if (!out_buffer) ERR("out of memory");
  
  pzs = new z_stream_s;
  if (!pzs) ERR("out of memory");

  pzs->zalloc = zmalloc;
  pzs->zfree  = zfree;

  init();
}

void Decompress::init()
{
  if (inflateInit(pzs) != Z_OK) ERR("inflateInit problem");
}

Decompress::~Decompress() {
  if (pzs) {
    inflateEnd(pzs);
    delete pzs;
    pzs = 0;
  }
  if (out_buffer) { delete [] out_buffer; out_buffer = 0; }
}

bool Decompress::doit(const string &input, string &output, sint4 expected_length) {

  const char *s = input.c_str();
  sint4 zres;
    
  pzs->next_in = const_cast<Bytef*>(reinterpret_cast<const Bytef*>(s));
  pzs->avail_in = input.size();
  pzs->next_out = reinterpret_cast<Bytef*>(out_buffer);
  pzs->avail_out = N;

  sint4 length = 0;
  
  while (pzs->avail_in) {

    if (pzs->avail_out == 0) {
      string s(reinterpret_cast<const char*>(out_buffer), N);
      output += s;
      pzs->next_out = reinterpret_cast<Bytef*>(out_buffer);
      pzs->avail_out = N;
      length += N;
      // inflated message too long?
      if (expected_length >= 0 && length > expected_length) {
	cout << "FOO1" << endl;
	return true;
      }
    }

    zres = inflate(pzs, Z_SYNC_FLUSH);
    if (zres != Z_OK && zres != Z_STREAM_END) {
      cout << "FOO2" << endl; return true;
    }
  }

  if (pzs->avail_out != N) {
    string s(reinterpret_cast<const char*>(out_buffer), N-pzs->avail_out);
    output += s;
    length += N-pzs->avail_out;
    if (expected_length >= 0 && length != expected_length) {
      cout << "FOO3 " << expected_length << " " << length << endl;
      return true;
    }
  }

  return false;
}
