#ifndef FileArchive_H
#define FileArchive_H

/** @file FileArchive.H

    Caches file data to prevent rereading.  This may best be left to the OS.

    Also has the option to load files by reading from a compressed
    archive.  The speed gain from this was never great enough to
    bother with, although it is still possibly useful for very slow
    or very small disks.

    $Id: FileArchive.H 8219 2009-04-07 00:18:55Z furtak $
    $Source$
*/

// This is an ORTS file (c) Tim Furtak, licensed under the GPL

#include "Global.H"

//===================================================================

class FileArchive
{
private:
  static std::map<std::string, std::string> name2file;

public:
  static void read_archive(const std::string &filename);
  static void make_archive(const std::string &filename,
                           const Vector<std::string> &files,
                           sint4 compression_level=9);

  static bool read_file(const std::string &filename, std::string &data);
};

//===================================================================

#endif
