
#include "Global.H"
#include "FileLoader.H"
#include "Options.H"

#include <boost/algorithm/string.hpp>
#include <boost/algorithm/string/split.hpp>
#include <fstream>
#include <sstream>

// This class is not used by ORTS.
// The filename prefixes are hard-coded and can be changed using options.  

using namespace std;

bool FileLoader::initialized = false;
std::string FileLoader::pathSeparator;
std::vector<std::string> FileLoader::defaultFilePaths; 
std::vector<std::string> FileLoader::optionsFilePaths; 

void FileLoader::init()
{
  if (defaultFilePaths.size() == 0) {
    defaultFilePaths.push_back(".");
    //defaultFilePaths.push_back("..");
    defaultFilePaths.push_back("../..");
    defaultFilePaths.push_back("/opt/orts");
  }
  
  pathSeparator = PATH_SEP;
  
  // parse the options paths
  string filepaths;
  Options::get("-filepaths", filepaths);
  if (filepaths != "")
    boost::split(optionsFilePaths, filepaths, boost::is_any_of(":"));
  
  initialized = true;
}

/* Removed things like double slashes, and sanitizes */
std::string FileLoader::fixPathString(const std::string& pathstr)
{
  ostringstream oss; 
  
  FORU(i, pathstr.size()) {
    unsigned int j = i+1;
    
    if (j < pathstr.size()) {
      if (pathstr[i] == '/' && pathstr[j] == '/') {  
        oss << '/';
        i++;
      }
      else if (pathstr[i] == '\\' && pathstr[j] == '\\') {
        oss << '\\';
        i++;
      }
      else
        oss << pathstr[i];      
    }
    else
      oss << pathstr[i];
  }
  
  return sanitize_path(oss.str());
}


bool FileLoader::load_file(const std::string& filename, const std::string& prefix, 
                           std::ifstream& stream, ios::openmode openmode)
{
  ostringstream oss; 
  
  string fixedFileName = fixPathString(filename);
  oss << prefix << pathSeparator << fixedFileName; 
  
  stream.open(oss.str().c_str(), openmode);
  
  if (!stream || !stream.is_open()) 
    return false;
  
  return true;
}

FILE* FileLoader::fopen(const std::string& prefix, const std::string& file, const char* mode)
{
  string fixedFileName = fixPathString(file);
  
  ostringstream oss; 
  oss << prefix << pathSeparator << fixedFileName; 
  
  //cout << "fopening " << oss.str() << endl;
  
  return std::fopen(oss.str().c_str(), mode);
}

// ---
// Public methods
// ---

bool FileLoader::load_file(const std::string& filename, std::ifstream& stream)
{
  return load_file(filename, stream, ios::in); 
}

bool FileLoader::load_file(const std::string& filename, std::ifstream& stream, ios::openmode openmode)
{
  if (!initialized) 
    init(); 

  FORALL(optionsFilePaths, iter) {
    string prefix = *iter;
    
    if (load_file(filename, prefix, stream, openmode)) 
      return true;    
  }  
  
  FORALL(defaultFilePaths, iter) {
    string prefix = *iter;
    
    if (load_file(filename, prefix, stream, openmode)) 
      return true;    
  }
  
  return false;
}

FILE* FileLoader::fopen(const std::string& file, const char *mode)
{
  if (!initialized) 
    init(); 

  FORALL(optionsFilePaths, iter) {
    string prefix = *iter;

    FILE* fp = FileLoader::fopen(prefix, file, mode);
    if (fp)
      return fp;
  }  
  
  FORALL(defaultFilePaths, iter) {
    string prefix = *iter;
    
    FILE* fp = FileLoader::fopen(prefix, file, mode);
    if (fp)
      return fp;
  }
  
  return 0;  
}



