// $Id: Global.C 7357 2008-08-07 02:38:57Z furtak $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "Global.H"

using namespace std;

bool gdb;

/** 
 * Fix directory separators to match system.
 * Canonizes by collapsing extra slashes and backpaths.
 */
string sanitize_path(const string &fp)
{
  // adjust path separators
  string p = fp;
  FORU (i, p.size()) {
    if (p[i] == '/' || p[i] == '\\') p[i] = PATH_SEP;
  }

  // try to canonize by dealing with extra slashes and backpaths (../)
  Vector<string> subpath;
  size_t i, j;
  i = 0;
  j = p.find(PATH_SEP);
  while (j != string::npos) {
    string s = p.substr(i, j-i);
    if (s == ".." && subpath.size() > 0 && subpath[subpath.size()-1] != "..") {
      subpath.pop_back();
    } else {
      if (s != "" && s != ".") subpath.push_back(s);
    }
    i = j + 1;
    j = p.find(PATH_SEP, j+1);
  }
  string fname = p.substr(i, p.length()-i); // file name

  string cp;
  FORALL (subpath, i) { cp += *i + PATH_SEP; }
  cp += fname;

  return cp;
}

// create string from format and arguments

char* alloc_sprintf(const char *fmt, ...) {
  va_list ap;
  va_start(ap, fmt);
  char *msg = alloc_sprintf(fmt, ap);
  va_end(ap);
  return msg;
}


char* alloc_sprintf(const char *fmt, va_list ap) {

  sint4 size = 100;
  char *p;

  if ((p = static_cast<char*>(malloc (size))) == 0) return 0;

#ifndef GCC

  // fixme: can't find va_copy
  size = 200;
  vsnprintf(p, size, fmt, ap);
  return p;

#else

  va_list ap2;
  for (;;) {
    /* Try to print in the allocated space. */
    va_copy(ap2, ap);
    sint4 n = vsnprintf (p, size, fmt, ap2);
    va_end(ap2);
    /* If that worked, return the string. */
    if (n > -1 && n < size)
      return p;
    /* Else try again with more space. */
    if (n > -1)    /* glibc 2.1 */
      size = n+1; /* precisely what is needed */
    else           /* glibc 2.0 */
      size *= 2;  /* twice the old size */

    if ((p = static_cast<char*>(realloc (p, size))) == 0) return 0;
  }

#endif
}

ostream& form(ostream &os, const char *fmt, ...)
{
  va_list ap;
  va_start(ap, fmt);
  form(os, fmt, ap);
  va_end(ap);
  return os;
}

ostream& form(ostream &os, const char *fmt, va_list ap)
{
  char *msg = alloc_sprintf(fmt, ap);
  if (msg == 0) ERR("out of memory");
  os << msg;
  free(msg);
  return os;
}


void tokenize(const string& str,
	      Vector<string>& tokens,
	      const string& delimiters)
{
  string::size_type pos=0;

  tokens.clear();
  
  FOREVER {

    string::size_type new_pos = str.find_first_of(delimiters, pos);

    if (new_pos == string::npos) {
      tokens.push_back(str.substr(pos));
      return;
    }

    tokens.push_back(str.substr(pos, new_pos - pos));
    pos = new_pos + 1; // skip delimiter
  }
}


//#endif

