/** @file Object.C
    Just shape rasterization functions at this point.

    @see Object.H

    $Id: View.C,v 1.29 2007/03/21 17:16:02 orts_mburo Exp $
    $Source: /usr/bodo1/cvs/cvs/orts3/libs/kernel/src/View.C,v $
*/

// This is an ORTS file (c) Tim Furtak, licensed under the GPL

#include "Object.H"

using namespace std;

//===================================================================

void Object::circle_footprint(sint4 x, sint4 y, sint4 r, Vector< Vec2<sint4> > &tiles, sint4 resolution)
{
  tiles.clear();

  sint4 rr = square(r);
  
  sint4 tx0 = (x - r) / resolution;
  sint4 tx1 = (x + r) / resolution;
  sint4 ty0 = (y - r) / resolution;
  sint4 ty1 = (y + r) / resolution;
  
  sint4 tx = x / resolution;
  sint4 ty = y / resolution;
  sint4 dx = x % resolution;
  sint4 dy = y % resolution;

  sint4 ry = ty0 - ty;
  for (sint4 y = ty0; y <= ty1; ++y, ++ry) {
    sint4 rx = tx0 - tx;
    for (sint4 x = tx0; x <= tx1; ++x, ++rx) {
      if (rx == 0 || ry == 0) tiles.push_back(Vec2<sint4>(x,y));
      else {
        sint4 mx = resolution * rx - dx;
        sint4 my = resolution * ry - dy;
        if (rx < 0) mx += resolution;
        if (ry < 0) my += resolution;
        sint4 dist = square(mx) + square(my);
        if (dist < rr) tiles.push_back(Vec2<sint4>(x,y));
      }
    }
  }
}

//-------------------------------------------------------------------

void Object::rect_footprint(sint4 x1, sint4 y1, sint4 x2, sint4 y2,
                            Vector< Vec2<sint4> > &tiles, sint4 resolution)
{
  tiles.clear();

  sint4 tx0 = x1 / resolution;
  sint4 tx1 = x2 / resolution;
  sint4 ty0 = y1 / resolution;
  sint4 ty1 = y2 / resolution;
  
  for (sint4 ty = ty0; ty <= ty1; ++ty) {
    for (sint4 tx = tx0; tx <= tx1; ++tx) {
      tiles.push_back(Vec2<sint4>(tx,ty));
    }
  }
}

//-------------------------------------------------------------------

void Object::line_footprint(sint4 x1, sint4 y1, sint4 x2, sint4 y2,
                            Vector< Vec2<sint4> > &tiles, sint4 resolution)
{
  tiles.clear();

  int tx1 = x1 / resolution;
  int tx2 = x2 / resolution;
  int ty1 = y1 / resolution;
  int ty2 = y2 / resolution;

  if (tx1 == tx2) {
    if (ty1 > ty2) swap (ty1, ty2);
    for (int y = ty1; y <= ty2; ++y) {
      tiles.push_back(Vec2<sint4>(tx1,y));
    }
    return;
  }

  if (ty1 == ty2) {
    if (tx1 > tx2) swap (tx1, tx2);
    for (int x = tx1; x <= tx2; ++x) {
      tiles.push_back(Vec2<sint4>(x,ty1));
    }
    return;
  }

  int dx = x2 - x1;
  int dy = y2 - y1;
  int dir_x = dx < 0 ? -1 : 1;
  int dir_y = dy < 0 ? -1 : 1;
  
  int tx = tx1;
  int ty = ty1;

  if (abs(dx) >= abs(dy)) { // line is horizontal-ish

    int next_x = (tx + dir_x) * resolution;
    int trans = (ty + dir_y) * resolution;

    int count = abs(tx2 - tx1);
    FORU (i, count) {
      tiles.push_back(Vec2<sint4>(tx, ty));

      sint4 a = dir_y * dy * (next_x - x1);
      sint4 b = dir_y * dx * (trans - y1);

      // the y coordinate at the tile transition has changed
      if (a >= b) {
        ty += dir_y;
        trans += dir_y * resolution;
      }
      if (a > b) {
        tiles.push_back(Vec2<sint4>(tx, ty));
      }
      tx += dir_x;
      next_x += dir_x * resolution;
    }

    // last tile(s)
    assert(tx == tx2);
    tiles.push_back(Vec2<sint4>(tx, ty));
    if (ty != ty2) tiles.push_back(Vec2<sint4>(tx, ty2));

  } else {

    int next_y = (ty + dir_y) * resolution;
    int trans = (tx + dir_x) * resolution;

    int count = abs(ty2 - ty1);
    FORU (i, count) {
      tiles.push_back(Vec2<sint4>(tx, ty));

      sint4 a = dir_x * dx * (next_y - y1);
      sint4 b = dir_x * dy * (trans - x1);

      // the x coordinate at the tile transition has changed
      if (a >= b) {
        tx += dir_x;
        trans += dir_x * resolution;
      }
      if (a > b) {
        tiles.push_back(Vec2<sint4>(tx, ty));
      }
      ty += dir_y;
      next_y += dir_y * resolution;
    }

    // last tile(s)
    assert(ty == ty2);
    tiles.push_back(Vec2<sint4>(tx, ty));
    if (tx != tx2) tiles.push_back(Vec2<sint4>(tx2, ty));
  }
}

//===================================================================

