#ifndef Pathfind_H
#define Pathfind_H

// $Id: Pathfind.H 5279 2007-06-23 02:49:08Z mburo $
 
// This is an ORTS file (c) Michael Buro, licensed under the GPL

// abstract pathfinding class

#include "Global.H"
#include "Map.H"

// template parameters:  Tile type and Object type
// (base classes in Tile.H and Object.H)

template <typename Tile, typename Object> class Pathfind {

public:

  typedef pair<sint4,sint4> Location;  // location on object center grid
  typedef Map<Tile> PlayField;
  typedef Vector<Object*> ObjCont;

  Pathfind() {
    center_grid_res = 0;
    pf = 0; 
  }
  
  // initialize with pointers to game state
  // ocs = array of player_n pointers to ObjCont
  // center_grid_res = resolution of object center grid

  virtual void init(sint4 player_n, ObjCont *ocs,
		    sint4 center_grid_res_, PlayField *pf_)
  {
    center_grid_res = center_grid_res_;
    pf = pf_;
    objs.clear();
    FORS (i, player_n) objs.push_back(ocs[i]);
  }

  // incremental terrain update; called whenever pf(index) is updated

  virtual void tile_update(sint4 index) = 0;

  // compute a path (sequence of waypoints) from start to end
  // return true iff path found

  virtual bool single_path(const Location &start, const Location &end,
			   Vector<Location> &path) = 0;

  virtual ~Pathfind() {}

protected:

  sint4 center_grid_res; // resolution of grid for object centers
  PlayField *pf;         // playfield
  Vector<ObjCont*> objs; // object containers (for each player)
};

#endif
