#ifndef PlayerInfo_H
#define PlayerInfo_H

// $Id: PlayerInfo.H 6681 2008-01-25 06:27:19Z furtak $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "Global.H"
#include "IdFactory.H"
#include "Vision.H"
#include "GameObjFactory.H"
#include "SType.H"
#include "SaveLoad.H"
#include "ScriptObj.H"

//===================================================================

class PlayerInfo : public SaveLoad {

public:

  MapView visible_W;    ///< Currently visible tiles (the West side if split).
  MapView visible_E;    ///< Split tiles where the East side is visible.

  MapView explored;     ///< Tiles which are known this cycle.
  MapView p_explored;   ///< Tiles which were known last cycle.

  MapView detected_W;   ///< Tiles (the West side if split) seen by a detector.
  MapView detected_E;   ///< Split tiles where the East side is detected.


  GameObjFactory gof;   ///< Creates objects for this player from its current set of blueprints.

  Vector<ScriptObj::Ptr> *my_objs; ///< Used by the action scripts to quickly access all player-owned objects.

  class SharedInfo;
  PlayerInfo(SharedInfo *si, uint4 TX, uint4 TY, bool server);
  ~PlayerInfo() {}

  void save(std::ostream &os) const;
  void load(std::istream &is);

  void new_global_obj(const std::string &name, ScriptObj **obj);
  void new_shared_obj(const std::string &name, ScriptObj **obj);
  void new_global_var(const std::string &name, stype val=stype(0));
  
  ScriptObj* global_obj(const std::string &name);
  stype* global_var(const std::string &name);

  ScriptObj* shared_obj(const std::string &name);


  ObjId new_id(ScriptObj *obj);
  ObjId get_id(const ScriptObj *obj) const;  // 0 iff not found
  ScriptObj* get_obj(ObjId cid) const; // 0 iff not found
  bool valid_id(ObjId cid) const;      // whether it's still valid
  void delete_obj(ScriptObj *obj, sint4 generation);


  // returns all addresses in unique order
  void objs(Vector<ScriptObj::Ptr> &v);
  
  Vector<ScriptObj*> global_shared; ///< Hack for the watcher.

  struct SharedInfo
  {
    Vector<ScriptObj**> objects;
    Vector<std::string> object_names;
    NameObjMap object_map;
    ~SharedInfo();
  };

protected:
  
  IdFactory cidf;       ///< Generates object ids local to the player.
  IdObjMap id2obj;      ///< id -> object address
  IdObjMap full_id2obj; ///< All objects with an id (even dead ones)
  ObjIdMap obj2id;      ///< object address -> id

  /** Player-owned objects which are globally referencable from actions.  They don't exist on the
      board and can't be killed.  Each player has the same objects, but unique instances of each. */
  Vector<ScriptObj**> global_objects;
  Vector<std::string> global_object_names;
  NameObjMap global_object_map;

  /** Like global_objects, except there is only one instance
      of each object shared by all players. */
  SharedInfo *shared;

  /** Global variables for scripts.  Points to data stored in global_var. */
  NameVarMap global_sym_table;
  std::list<stype> global_var_list;


  bool server; ///< fixme: hack for object deletion

  std::list<ObjId> to_free_ids;
  std::list<sint4> to_free_times;
};


#endif
