/** @file Profiler.C
    @see Profiler.H

    $Id: Profiler.C 6532 2008-01-07 11:02:43Z furtak $
    $Source$
*/

// This is an ORTS file
// (c) Tim Furtak
// licensed under the GPL

#include "Global.H"
#include "Profiler.H"

using namespace std;

ProfileLevel Profiler::root;
ProfileLevel *Profiler::cur = &Profiler::root;

//===================================================================

ProfileLevel::ProfileLevel(const string &n)
{
  my_name = n;
  total_time = avg_time = 0.0;
  up = 0;
  FORU (i, window) win_time[i] = 0.0;
  window_ind = 0;
}

Profiler::Profiler(const string &name)
{
  pl = cur->children[name];
  if (pl == 0) {
    pl = new ProfileLevel(name);
    cur->children[name] = pl;
  }
  pl->up = cur;
  cur = pl;
}


Profiler::~Profiler()
{
  CycleCounter end;
  real8 dt = (end.count() - cc.count()) * (1.0 / 1000000); // results in mega-cycles (1M cycles)

  ProfileLevel *p = pl;

  p->total_time += dt;
  p->win_time[p->window_ind] += dt;
  p = p->up;
  if (p == &root) {
    p->total_time += dt;
    p->win_time[p->window_ind] += dt;
  }
  cur = cur->up;
}

void Profiler::tick()
{
  root.tick();
}

void ProfileLevel::tick()
{
  window_ind = (window_ind+1) % window;
  win_time[window_ind] = 0.0;
  avg_time = 0;
  FORU (i, window) avg_time += win_time[i];
  avg_time /= window;

  FORALL (children, it) it->second->tick();
}
