#ifndef Profiler_H
#define Profiler_H

/** @file Profiler.H
    Hierarchical profiler.

    $Id: Profiler.H 8322 2009-06-01 21:39:51Z furtak $
    $Source$
*/

// This is an ORTS file (c) Tim Furtak, licensed under the GPL

#include "Global.H"
#include "CycleCounter.H"

#define PROFILE(n) Profiler _prof_##__LINE__(n);

//===================================================================

class ProfileLevel {

private:
  void tick();

public:
  static const sint4 window = 5;

  std::string my_name;
  real8 total_time;
  real8 avg_time;         ///< Time averaged over 'window' steps.
  real8 win_time[window]; ///< Buckets for storing the last few times.
  sint4 window_ind;       ///< Index of the current bucket.
  std::map<std::string, ProfileLevel*> children;
  ProfileLevel *up;

  ProfileLevel(const std::string &n = "ROOT");
  // fixme: delete old profile levels? certainly as a destructor step

  friend class Profiler;
};

//===================================================================

/** Measures the amount of time that each Profiler object exists
    and stores the results hierarchically.
    Usage: enclose the area of concern within scope braces and
           use the PROFILE macro to label the region.
*/

class Profiler {

private:

  static ProfileLevel root;
  static ProfileLevel *cur;
  CycleCounter cc;
  ProfileLevel *pl;

public:

  Profiler(const std::string &name);
  ~Profiler();
  static ProfileLevel* get_root() { return &root; }
  static void tick();
};

//===================================================================

REGISTER_TYPEOF2(5001, std::map<std::string,ProfileLevel*>::iterator);

#endif
