// $Id: Random.C 5279 2007-06-23 02:49:08Z mburo $

// This is an ORTS file 

/* 
   A C-program for MT19937, with initialization improved 2002/1/26.
   Coded by Takuji Nishimura and Makoto Matsumoto.

   Before using, initialize the state by using init_genrand(seed)  
   or init_by_array(init_key, key_length).

   Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura,
   All rights reserved.                          

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. The names of its contributors may not be used to endorse or promote 
        products derived from this software without specific prior written 
        permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


   Any feedback is very welcome.
   http://www.math.keio.ac.jp/matumoto/emt.html
   email: matumoto@math.keio.ac.jp
*/


/* 7-2-03 Nathan Sturtevant
 * Turned into C++ class & combined all code from int/float generation
 * into a single class.
 */

#include "Global.H"
#include "Random.H"

using namespace std;

/* Initializing the array with a seed */

void Random::srand(uint4 s)
{
  mt[0]= s & max_uint4;
  for (mti=1; mti<N; mti++) {
    mt[mti] = 
      (1812433253UL * (mt[mti-1] ^ (mt[mti-1] >> 30)) + mti); 
    /* See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier.
       In the previous versions, MSBs of the seed affect  
       only MSBs of the array mt[].                       
       2002/01/09 modified by Makoto Matsumoto
    */
    mt[mti] &= max_uint4;
    // for >32 bit machines
  }
}

/*
 Initialization by "sgenrand()" is an example. Theoretically,      
 there are 2^19937-1 possible states as an intial state.           
 This function allows to choose any of 2^19937-1 ones.             
 Essential bits in "seed_array[]" is following 19937 bits:         
  (seed_array[0]&UPPER_MASK), seed_array[1], ..., seed_array[N-1]. 
 (seed_array[0]&LOWER_MASK) is discarded.                           
 Theoretically,                                                    
  (seed_array[0]&UPPER_MASK), seed_array[1], ..., seed_array[N-1]  
 can take any values except all zeros.                             
*/
   
void Random::lsrand(uint4 init_key[], sint4 key_length)
  // the length of seed_array[] must be at least N
{
  cerr << "messes up rand seed" << endl;
  exit(0);
  sint4 i, j, k;
  srand(19650218UL);
  i=1; j=0;
  k = (N>key_length ? N : key_length);
  for (; k; k--) {
    mt[i] = (mt[i] ^ ((mt[i-1] ^ (mt[i-1] >> 30)) * 1664525UL))
      + init_key[j] + j; // non-linear
    mt[i] &= max_uint4;  // for WORDSIZE > 32 machines
    i++; j++;
    if (i>=N) { mt[0] = mt[N-1]; i=1; }
    if (j>=key_length) j=0;
  }
  for (k=N-1; k; k--) {
    mt[i] = (mt[i] ^ ((mt[i-1] ^ (mt[i-1] >> 30)) * 1566083941UL))
      - i; // non-linear
    mt[i] &= max_uint4; // for WORDSIZE > 32 machines
    i++;
    if (i>=N) { mt[0] = mt[N-1]; i=1; }
  }

  mt[0] = max_sint4; //0x80000000UL; /* MSB is 1; assuring non-zero initial array */ 
}

uint4 Random::rand_uint4()
{
  uint4 y;
  static uint4 mag01[2]={0x0UL, MATRIX_A};
  // mag01[x] = x * MATRIX_A  for x=0,1

  if (mti >= N) { // generate N words at one time
    int kk;

    if (mti == N+1) {  // if init_genrand() has not been called,
      ERR("Rand has not been seeded");
      srand(time(0));  // a default initial seed is used
      //srand(5489UL); /* a default initial seed is used
    }
    
    for (kk=0;kk<N-M;kk++) {
      y = (mt[kk]&UPPER_MASK)|(mt[kk+1]&LOWER_MASK);
      mt[kk] = mt[kk+M] ^ (y >> 1) ^ mag01[y & 0x1UL];
    }
    for (;kk<N-1;kk++) {
      y = (mt[kk]&UPPER_MASK)|(mt[kk+1]&LOWER_MASK);
      mt[kk] = mt[kk+(M-N)] ^ (y >> 1) ^ mag01[y & 0x1UL];
    }
    y = (mt[N-1]&UPPER_MASK)|(mt[0]&LOWER_MASK);
    mt[N-1] = mt[M-1] ^ (y >> 1) ^ mag01[y & 0x1UL];

    mti = 0;
  }
  
  y = mt[mti++];

  // Tempering
  y ^= (y >> 11);
  y ^= (y << 7) & 0x9d2c5680UL;
  y ^= (y << 15) & 0xefc60000UL;
  y ^= (y >> 18);

  return y;
}

real8 Random::rand_real8()
{
  // reals: [0,1)-interval
  return rand_uint4()*(1.0/4294967296.0);
}

uint4 Random::ranged_uint4(uint4 minimum, uint4 maximum)
{
  //if (minimum <= maximum) cerr << "bad range for ranged_long" << endl;
	
  if (minimum >= maximum) return minimum;
  else {
    uint8 range = uint8(maximum)+1-minimum;
    if (range >= max_uint4) ERR("range too big");
    return minimum + rand_uint4() % range;
  }
}

uint4 Random::ranged_sint4(sint4 minimum, sint4 maximum)
{
  //if (minimum <= maximum) cerr << "bad range for ranged_long" << endl;
	
  if (minimum >= maximum) return minimum;
  else {
    uint8 range = uint8(maximum)+1-minimum;
    if (range >= max_uint4) ERR("range too big");
    return minimum + rand_uint4() % range;
  }
}


sint4 UniRandSrc::operator()() 
{
  sint4 r = rand.ranged_sint4(min, max);
  //  std::cout << "R=" << r << std::endl;
  return r;
}
