#ifndef Replayer_H
#define Replayer_H

// $Id: Replayer.H 5279 2007-06-23 02:49:08Z mburo $

// This is an ORTS file (c) Tim Furtak, licensed under the GPL

#include "Global.H"
#include "GameChanges.H"

class Game;
class Options;

//===================================================================

class Replayer
{
public:

  // see .C file for option comments
  
  struct Options {
    sint4 seed; // game rng seed
    std::string load_replay;

    std::string ids; // client id:pw pairs
    std::string defines;

    bool nfog;

    Options();
    static void add();
  };
  
  struct ClientData {
    Game *game;
    GameChanges changes;
    ActionChanges action_changes;
  };

  Replayer();
  ~Replayer();

  void init(const std::string &world, Options &opt, Vector<std::string> &views);
  void tick();
  void tick(Vector<std::string> &views);
  void finish();

  bool is_good();

  // get access to the game
  Game &get_game() const { return *server; }
  ClientData &get_client(sint4 i) { return clients[i]; }
  sint4 get_nplayers() const { return clients.size(); }
  
protected:

  Game *server;
  Vector<ClientData> clients;

  sint4 frame;
  FILE *load_file;

  Vector<std::string> msgs;
  
  // |msgs| = #players
  void send_messages(const Vector<std::string> &msgs);

  void load_messages(Vector<std::string> &msgs);

private:

  // forbid copy/assignment
  Replayer(const Replayer &) { ERR("what"); }
  Replayer &operator=(const Replayer &) { ERR("what"); }  
};

//===================================================================

class ViewReplayer
{
public:

  // see .C file for option comments
  
  struct Options {
    std::string load_replay;
    std::string defines;
    bool nfog;

    Options();
    static void add();
  };
  
  ViewReplayer();
  ~ViewReplayer();

  void init(Options &opt);
  void tick();
  void finish();
  bool is_good();

  // get access to the game
  Game &get_game() const { return *game; }
  
protected:

  Game *game;

  sint4 frame;
  FILE *load_file;

private:

  // forbid copy/assignment
  ViewReplayer(const ViewReplayer &) { ERR("what"); }
  ViewReplayer &operator=(const ViewReplayer &) { ERR("what"); }  
};

//===================================================================

#endif
