#ifndef SType_H
#define SType_H

// $Id: SType.H 5279 2007-06-23 02:49:08Z mburo $
 
// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "Global.H"
#include "SaveLoad.H"
#include <boost/format.hpp>
#include "ScriptErr.H"

#ifdef VOID //Windows headers define VOID to void, causing errors. Matt
#undef VOID
#endif

class ScriptObj;
class GfxObject;
class PlayerInfo;
class ByteStream;

class stype;
REGISTER_TYPEOF(1000, Vector<stype>::iterator);
REGISTER_TYPEOF(1001, Vector<stype>::const_iterator);
REGISTER_TYPEOF(1002, std::set<stype>::iterator);
REGISTER_TYPEOF(1003, std::set<stype>::const_iterator);

class stype
{
public:

  union {
    sintptr     i;  // int needs to be as big as pointer!
    sint4       h;  // heading
    ScriptObj  *o;
    const char *c;
    stype      *a;
    void       *v;  // special types
  } val;

  enum { MAX_INT = 1000000000, UNDEF_INT = 2000000000 };

  enum Type {
    INT,   // integer
    DIR,   // heading
    OBJ,   // ScriptObj*
    CSTR,  // const char*
    ADDR,  // stype*
    NONE
    // ... special
  };

  // for "special" types
  inline static sint4 next_id() { static sint4 id = NONE+1; return ++id; }
  
  Type type;

  void save(std::ostream &os) const {
    OS_WRITE(val);
    OS_WRITE(type);
  }

  void load(std::istream &is) {
    IS_READ(val);
    IS_READ(type);
  }

  sint4 to_int() const {
    SCRIPT_ASSERT(type == INT && "type not INT"); return val.i;
  }
  ScriptObj* to_obj() const {
    if (val.i == 0) return 0;
    SCRIPT_ASSERT(type == OBJ && "type not OBJ"); return val.o;
  }
  const char* to_cstr() const {
    SCRIPT_ASSERT(type == CSTR && "type not CSTR"); return val.c;
  }
  stype* to_addr() const {
    SCRIPT_ASSERT(type == ADDR && "type not ADDR"); return val.a;
  }
  void* to_special(sint4 t) const {
    SCRIPT_ASSERT(type == t && "type mismatch"); return val.v;
  }

  inline stype(int i=0) {
    SCRIPT_ASSERT(abs(i) <= MAX_INT && "too big");
    val.i = i; type = INT;
  }

  inline explicit stype(ScriptObj  *p)    { val.o = p; type = OBJ;  }
  inline explicit stype(const char *p)    { val.c = p; type = CSTR; }
  inline explicit stype(stype      *p)    { val.a = p; type = ADDR; }

  inline explicit stype(void *p, sint4 t) {
    val.v = p; type = static_cast<Type>(t);
  }
  
  ~stype() {}
  
  inline bool  operator! () const { return val.i == 0; }

  inline stype operator~ () const {
    SCRIPT_ASSERT(type == INT && "no int for op ~");
    return stype(~val.i);
  }

  inline stype operator- () const {
    SCRIPT_ASSERT(type == INT && "no int for op -");
    return stype(-val.i);
  }

  inline bool operator != (const stype &s) const {
    return val.i != s.val.i;
  }

  inline bool operator == (const stype &s) const {
    return val.i == s.val.i;
  }

#define STYPE_INT_OP(RET, OP)						\
  inline RET operator OP (const stype &s) const {                       \
    SCRIPT_ASSERT(((type == INT && s.type == INT)                       \
  || (type == OBJ && s.type == OBJ))  && "no 2 int args for op "#OP)	\
    return val.i OP s.val.i;                                            \
  }

  STYPE_INT_OP(bool, <);

  //===================================================================

  // skip n values
  static std::string print_va_list(const char *format, const Vector<stype> &v, sint4 n);
  static std::string my_printf(const char *format, const Vector<stype> &v, sint4 n);
  
  //-------------------------------------------------------------------

  struct SortComp {
    bool operator()(const stype &s1, const stype &s2) const { return s1.val.i < s2.val.i; }
  };

  //===================================================================

  void gen_diff(const stype *orig, bool allied, std::string &u, PlayerInfo *pi) const;
  bool apply_diff(bool first, ByteStream &s, PlayerInfo *pi);

  //===================================================================
};

#endif
