/** @file ScriptSet.H
    @see ScriptSet.H

    $Id: ScriptSet.C 5279 2007-06-23 02:49:08Z mburo $
    $Source$
*/

// This is an ORTS file (c) Tim Furtak

#include "Global.H"
#include "ScriptSet.H"
#include "ByteData.H"
#include "Script.H"

using namespace std;

//===================================================================

ScriptSet::ScriptSet()
{
  add("op",        &ScriptSet::op,        0, 1, 0);
  // add("op_addr",   &ScriptSet::op_addr,   0, 1, 0); // doesn't work (must be const)
  add("size",      &ScriptSet::size,      0, 0, 0);
  add("clear",     &ScriptSet::clear,     0, 0, 0);
  add("remove",    &ScriptSet::remove,    0, 1, 0);
  add("insert",    &ScriptSet::insert,    0, 1, 0);
  add("intersect", &ScriptSet::intersect, 1, 0, 0);
  add("subtract",  &ScriptSet::subtract,  1, 0, 0);
  add("union",     &ScriptSet::set_union, 1, 0, 0);
}

void ScriptSet::add(const std::string &name, myfunc f, sint4 objs, sint4 p, sint4 p2)
{
  ObjActionDefinition def(name, objs, p, p2);
  ObjAction *a = new ObjAction(static_cast<ObjFunction>(f), def);
  actions.push_back(a);
  act_map[name] = a;
}

stype ScriptSet::op(const SActParam &p)
{
  VALN_CHECK(1);
  sint4 i = p.val[0].to_int();
  sint4 s = data.size();
  if (i < 0) return stype(s);
  SCRIPT_ASSERT(i < s &&  "index out of range");
  set<stype>::const_iterator it = data.begin();
  FORU (j, i) ++it;
  return *it;
}

#if 0

/** Address of an element. */

// can't change element in set anyway!

stype ScriptSet::op_addr(const SActParam &p)
{
  VALN_CHECK(1);  
  sint4 i = p.val[0].to_int();
  sint4 s = data.size();
  SCRIPT_ASSERT(i >= 0 && i < s && "index out of range");
  set<stype>::iterator it = data.begin();
  FORU (j, i) ++it;
  return stype(&(*it)); // fixme: REVERT BACK TO STYPE VOID POINTER
}
#endif

stype ScriptSet::clear(const SActParam &)
{
  data.clear();
  return stype();
}

stype ScriptSet::size(const SActParam &)
{
  return stype(data.size());
}

/// Erases element with value p.val[0]
stype ScriptSet::remove(const SActParam &p)
{
  VALN_CHECK(1);
  FIND (data, i, p.val[0]);
  if (i != data.end()) data.erase(i);
  return stype(data.size());
}

stype ScriptSet::insert(const SActParam &p)
{
  VALN_CHECK(1);  
  data.insert(p.val[0]);
  return stype(data.size());
}

stype ScriptSet::intersect(const SActParam &p)
{
  OBJN_CHECK(1);  
  ScriptSet *sp = dynamic_cast<ScriptSet*>((ScriptObj*)p.obj[0]);
  SCRIPT_ASSERT(sp && "not a ScriptSet");
  set<stype> &s = sp->data;

  if (s.empty() || data.empty()) {
    data.clear();
    return stype();
  }

  set<stype> x;
  insert_iterator<set<stype> > ii(x, x.begin());
  set_intersection(s.begin(), s.end(), data.begin(), data.end(), ii);

  data = x;
  return stype(data.size());
}

stype ScriptSet::subtract(const SActParam &p)
{
  OBJN_CHECK(1);  
  ScriptSet *sp = dynamic_cast<ScriptSet*>((ScriptObj*)p.obj[0]);
  SCRIPT_ASSERT(sp && "not a ScriptSet");
  set<stype> &s = sp->data;

  if (s.empty())    return stype(data.size());
  if (data.empty()) return stype();

  set<stype> x;
  insert_iterator<set<stype> > ii(x, x.begin());
  set_difference(data.begin(), data.end(), s.begin(), s.end(), ii);

  data = x;
  return stype(data.size());
}

stype ScriptSet::set_union(const SActParam &p)
{
  OBJN_CHECK(1);
  ScriptSet *sp = dynamic_cast<ScriptSet*>((ScriptObj*)p.obj[0]);
  SCRIPT_ASSERT(sp && "not a ScriptSet");
  set<stype> &s = sp->data;

  if (s.empty()) return stype(data.size());

  set<stype> x;
  insert_iterator<set<stype> > ii(x, x.begin());
  ::set_union(data.begin(), data.end(), s.begin(), s.end(), ii);

  data = x;
  return stype(data.size());
}

//-------------------------------------------------------------------

ObjAction* ScriptSet::action(const std::string &name) const
{
  FIND (act_map, it, name);
  if (it != act_map.end()) return it->second;
  return 0;
}

//-------------------------------------------------------------------

void ScriptSet::gen_diff(bool first, bool allied, string &u, PlayerInfo * /*pi*/) const
{
  set<stype> added;
  set<stype> removed;
  insert_iterator<set<stype> > iia(added, added.begin());
  insert_iterator<set<stype> > iir(removed, removed.begin());

  set_difference(data.begin(), data.end(), prev_data.begin(), prev_data.end(), iia);
  set_difference(prev_data.begin(), prev_data.end(), data.begin(), data.end(), iir);

  if (!allied) {
    write_b_int4(u, 2);
    return;
  }

  sint4 i = first ? 1 : 0;
  write_b_int4(u, i);

  if (first) {
    uint4 size = data.size();
    write_b_int4(u, size);
    FORALL (data, i) write_b_int4(u, i->to_int());
  } else {
    write_b_int4(u, removed.size());
    FORALL (removed, i) write_b_int4(u, i->to_int());

    write_b_int4(u, added.size());
    FORALL (added, i) write_b_int4(u, i->to_int());
  }
}

bool ScriptSet::apply_diff(ByteStream &s, PlayerInfo *)
{
  sint4 c = s.read_b_int4();
  if (c == 2) return false; // hidden
  //  bool first = c == 1;
  if (c != 0 && c != 1) ERR("neither 0 nor 1");

  sint4 num_rem = s.read_b_int4();
  FORS (i, num_rem) {
    stype r(s.read_b_int4());
    data.erase(data.find(r));
  }

  sint4 num_add = s.read_b_int4();
  FORS (i, num_add) {
    stype r(s.read_b_int4());
    data.insert(r);
  }

  bool changed = num_rem || num_add;
  return changed;
}

//-------------------------------------------------------------------

void ScriptSet::copy_to_prev() {
  prev_data = data;
  //  memcpy (&prev_attr_vals[0], &attr_vals[0], sizeof(AttrVal) * prev_attr_vals.size());
  //  prev_attr_vals = attr_vals;
  //  FORALL(components, it) it->second->copy_to_prev();
}
