/** @file Game.C
    @see Game.H

    $Id: ServerObjData.C 5279 2007-06-23 02:49:08Z mburo $
    $Source$
*/

// This is an ORTS file
// (c) Tim Furtak
// licensed under the GPL

#include "Global.H"
#include "ServerObjData.H"

using namespace std;

//===================================================================

void ServerObjData::save(ostream &os) const
{
  OS_WRITE(*shape);
  OS_WRITE(*x);
  OS_WRITE(*x1);
  OS_WRITE(*x2);
  OS_WRITE(*y);
  OS_WRITE(*y1);
  OS_WRITE(*y2);
  OS_WRITE(*radius);

  OS_WRITE(is_boundary);
  OS_WRITE(*owner);
  OS_WRITE(*max_speed);
  OS_WRITE(*speed);

  OS_WRITE(*is_moving);
  OS_WRITE(*zcat);
  OS_WRITE(*collides);
  OS_WRITE(*ncollides);

  OS_WRITE(*sight);
  OS_WRITE(*cloaked);
  OS_WRITE(*cloaked2);
  OS_WRITE(*detector);

  OS_WRITE(in_game);

  OS_WRITE(seen_by);
  OS_WRITE(prev_seen_by);

  sint4 s = last_lost.size();
  OS_WRITE(s);
  FORU (i, s) {
    sint4 v = last_lost[i];
    OS_WRITE(v);
  }
}

//-------------------------------------------------------------------

void ServerObjData::load(istream &is)
{
  IS_READ(*shape);
  IS_READ(*x);
  IS_READ(*x1);
  IS_READ(*x2);
  IS_READ(*y);
  IS_READ(*y1);
  IS_READ(*y2);
  IS_READ(*radius);

  IS_READ(is_boundary);
  IS_READ(*owner);
  IS_READ(*max_speed);
  IS_READ(*speed);

  IS_READ(*is_moving);
  IS_READ(*zcat);
  IS_READ(*collides);
  IS_READ(*ncollides);

  IS_READ(*sight);
  IS_READ(*cloaked);
  IS_READ(*cloaked2);
  IS_READ(*detector);

  IS_READ(in_game);

  IS_READ(seen_by);
  IS_READ(prev_seen_by);

  sint4 s;
  IS_READ(s);
  FORU (i, s) {
    sint4 v;
    IS_READ(v);
    last_lost[i] = v;
  }
}

//-------------------------------------------------------------------

ServerObjData::PrevData::PrevData()
{
  shape = 0;
  x = 0;
  y = 0;
  x1 = 0;
  y1 = 0;
  x2 = 0;
  y2 = 0;
  radius = 0;
  
  owner = 0;
  
  max_speed = 0;
  speed = 0;
  is_moving = 0;
  zcat = 0;
  collides = 0;
  ncollides = 0;
  
  sight = 0;
  cloaked = 0;
  cloaked2 = 0;
  detector = 0;
}

//-------------------------------------------------------------------

void ServerObjData::copy_to_prev()
{
  if (!shape) return; // any pointer will do; if it's 0 they're all uninitialized

  prev.shape = *shape;
  prev.x = *x;
  prev.y = *y;
  prev.x1 = *x1;
  prev.y1 = *y1;
  prev.x2 = *x2;
  prev.y2 = *y2;
  prev.radius = *radius;
  
  prev.owner = *owner;
  
  prev.max_speed = *max_speed;
  prev.speed = *speed;
  prev.is_moving = *is_moving;
  prev.zcat = *zcat;
  prev.collides = *collides;
  prev.ncollides = *ncollides;
  
  prev.sight = *sight;
  prev.cloaked = *cloaked;
  prev.cloaked2 = *cloaked2;
  prev.detector = *detector;
}

//===================================================================

void ServerMotionData::save(ostream &os) const
{
  OS_WRITE(start_f_pos);
  OS_WRITE(target_f_pos);
  OS_WRITE(f_delta);
  OS_WRITE(inv_ttt);
  OS_WRITE(fine_pos);
  OS_WRITE(fine_heading);
  OS_WRITE(bc_fine_center);
  OS_WRITE(bc_fine_r);
  OS_WRITE(ticks_so_far);
  OS_WRITE(ticks_total);
}

//-------------------------------------------------------------------

void ServerMotionData::load(istream &is)
{
  IS_READ(start_f_pos);
  IS_READ(target_f_pos);
  IS_READ(f_delta);
  IS_READ(inv_ttt);
  IS_READ(fine_pos);
  IS_READ(fine_heading);
  IS_READ(bc_fine_center);
  IS_READ(bc_fine_r);
  IS_READ(ticks_so_far);
  IS_READ(ticks_total);
}

//===================================================================
