#ifndef Slope_H
#define Slope_H

/** @file Slope.H
    Rational slope class for precomputing vision data.
    Doesn't need to be blazingly fast.  And it isn't.

    $Id: Slope.H 5279 2007-06-23 02:49:08Z mburo $
    $Source$
*/

// This is an ORTS file (c) Tim Furtak, licensed under the GPL

#include "Global.H"

class Slope {
public:
  sint4 rise, run;
  sint4 ang;

  Slope () {}
  Slope(sint4 x, sint4 y) { set(x, y); }

  void set(sint4 x, sint4 y) {
    ang = 0;
    if (x < 0) {
      x = -x;
      y = -y;
    }
    rise = y; run = x;
    if (x*y > 0) ang = 1;
    if (x*y < 0) ang = -1;
  }

  void print() const {
    real8 ri = static_cast<real8>(rise);
    real8 ru = static_cast<real8>(run);
    std::cout << ri/ru << std::endl;
  }

  bool operator==(const Slope& sl) const {
    if (ang != sl.ang) return false;
    if (ang == 0) return true;
    sint4 x1,x2,y1,y2;
    y1 = rise;    x1 = run;
    y2 = sl.rise; x2 = sl.run;

    FOREVER {
      if (x1 < x2) {
	x1 += run;
	y1 += rise;
      } else if (x1 > x2) {
	x2 += sl.run;
	y2 += sl.rise;
      } else {
	return (y1 == y2);
      }
    }
  }

  bool operator>(const Slope& sl) const {
    if (ang > sl.ang) return true;
    if (ang < sl.ang) return false;
    if (ang == 0) return false;

    sint4 x1,x2,y1,y2;
    y1 = rise;    x1 = run;
    y2 = sl.rise; x2 = sl.run;

    FOREVER {
      if (x1 < x2) {
	x1 += run;
	y1 += rise;
      } else if (x1 > x2) {
	x2 += sl.run;
	y2 += sl.rise;
      } else {
	return (y1 > y2);
      }
    }
  }

  bool operator>=(const Slope& sl) const {
    if (ang > sl.ang) return true;
    if (ang < sl.ang) return false;
    if (ang == 0) return true;
    
    sint4 x1,x2,y1,y2;
    y1 = rise;    x1 = run;
    y2 = sl.rise; x2 = sl.run;

    FOREVER {
      if (x1 < x2) {
	x1 += run;
	y1 += rise;
      } else if (x1 > x2) {
	x2 += sl.run;
	y2 += sl.rise;
      } else {
	return (y1 >= y2);
      }
    }
  }

  bool operator<=(const Slope& sl) const {
    if (ang > sl.ang) return false;
    if (ang < sl.ang) return true;
    if (ang == 0) return true;
    
    sint4 x1,x2,y1,y2;
    y1 = rise;    x1 = run;
    y2 = sl.rise; x2 = sl.run;

    FOREVER {
      if (x1 < x2) {
	x1 += run;
	y1 += rise;
      } else if (x1 > x2) {
	x2 += sl.run;
	y2 += sl.rise;
      } else {
	return (y1 <= y2);
      }
    }
  }

  bool operator<(const Slope& sl) const {
    if (ang < sl.ang) return true;
    if (ang > sl.ang) return false;
    if (ang == 0) return false;

    sint4 x1,x2,y1,y2;
    y1 = rise;    x1 = run;
    y2 = sl.rise; x2 = sl.run;

    FOREVER {
      if (x1 < x2) {
	x1 += run;
	y1 += rise;
      } else if (x1 > x2) {
	x2 += sl.run;
	y2 += sl.rise;
      } else {
	return (y1 < y2);
      }
    }
  }

  static int compare(const void* e1, const void* e2) {
    const Slope *s1 = static_cast<const Slope*>(e1);
    const Slope *s2 = static_cast<const Slope*>(e2);    
    
    if ( *s1 > *s2 ) return +1;
    if ( *s1 < *s2 ) return -1;
    return 0;
  }
};

#endif
