#ifndef Token_H
#define Token_H

/** @file Token.H
    Tokenizer.

    $Id: Token.H 5279 2007-06-23 02:49:08Z mburo $
    $Source$
*/

// This is an ORTS file (c) Tim Furtak, licensed under the GPL

#include "Global.H"

//===================================================================

class Tokenizer {
public:

  enum TokenType {UNKNOWN, NUMBER, SPACE, PUNCT, ID, STRING, COMMENT};

protected:

  void readto(std::string &s, std::istream &ss, char C);
  char readto(std::string &s, std::istream &ss, char c1, char c2);

  TokenType type;  // the type of the last token returned
  sint4 cur_line_num;
  std::string fname; // name of the current file
  std::string fpath; // path of the current file

  std::stack<std::string> tokenStack; // for tokens put back into the tokenizer

  struct Bookmark {
    const char *ptr;
    sint4 line;
    std::stack<std::string> tStack;
  };

  class DefineTable {
    std::map<std::string, Vector<std::string> > def_table;
  public:
    void define_var(const std::string &var, const std::string &def);
    bool is_def(const std::string &var);
    void get_def(const std::string &var, Vector<std::string> &def);
  };

  std::stack<Bookmark> bookmark;
  DefineTable *define_table;
  static DefineTable cmd_def_table;

  bool my_def_table; ///< If this tokenizer is top-level, or just a copy.
  void putback_token(const std::string &data);
  
  TokenType type_of(char ch);

  bool my_data_buf;

  void cleanup();

public:

  const char *data_start;
  const char *data_ptr;
  const char *data_end;

  Tokenizer();
  Tokenizer(Tokenizer *t);
  ~Tokenizer();

  void init(const std::string &str, std::string file="", std::string path="");
  void init(const char *ptr, sint4 length, std::string file="", std::string path="");

  // returns space/comment as well as non-space tokens
  const std::string next_raw_token(bool redef=true);
  const std::string next_token(bool redef=true);
  const std::string next_tag();

  const std::string scan_line() const;
  const std::string read_line();
  const std::string file_name() const { return fname; }
  const std::string file_path() const { return fpath; }

  void putback(const std::string &data);
  void putback_tag(const std::string &data);
  inline sint4 cur_line() const { return cur_line_num; }
  inline void set_line(sint4 n) { cur_line_num = n; }
  bool good() const;
  void push_bookmark();
  void flip_bookmark();
  inline void drop_bookmark() { bookmark.pop(); }
  
  TokenType token_type() const { return type;}
  
  void define_var(const std::string &var, const std::string &def);
  static void init_cmd_def(Vector<std::string> &var, Vector<std::string> &def);
  static void init_cmd_def();
};

//===================================================================

#endif
