#ifndef HEURISTIC_MAP_WIDGET
#define HEURISTIC_MAP_WIDGET

// $Id: HeuristicMapWidget.h 5279 2007-06-23 02:49:08Z mburo $

// This is an ORTS file (c) Frano Sailer, licensed under the GPL

#include "GfxModule.H"
#include "Game.H"
#include "CWidget.H"

#include "GfxGlobal.H"
//#include "Decomp.H"
//#include "PolyGraph.H"
#include "PathProcessor.H"

struct Attribute{
  std::string attributeName;
  std::string componentName;
  bool isComponent;
  real8 maxValue;
};

struct QueryResult{
  Point2i pos; // THIS POSITION IS NOT ABSOLUTE COORDS, IT IS THE GRID COORDS
  real8 value;
  real8 distance;

  QueryResult(Point2i p, real8 v, real8 dist) { pos = p; value = v;distance = dist; }
};

class HeuristicMapWidget : public CWidget {

public:
  HeuristicMapWidget(Point2i pos, real8 mapScaleSize, real8 gridSquareWidth,
		     real8 gridSquareHeight,real8 alpha,bool overlay);
  ~HeuristicMapWidget();

  virtual stype event_mouse_motion(stype sx, stype sy, stype button);
  virtual stype event_mouse_hover(stype sx, stype sy, stype button);
  virtual stype event_mouse_up(stype sx, stype sy, stype button);
  //sint4 event_mouse_down(sint4 x, sint4 y, sint4 button);
  //sint4 event_key_down(sint4 key, sint4 mod, sint4 x, sint4 y);
  virtual stype event_key_down(stype key,stype mod,stype x,stype y);
  virtual stype event_mouse_down(stype x, stype y, stype button);

  virtual stype event_key_up(stype key, stype mod, stype x, stype y);
  //sint4 draw();

  virtual stype draw();

  void setGfxModule(GfxModule* gfxm);
  void setGameStateModule(GameStateModule* gsm);
  uint4 getRows();
  uint4 getColumns();
  real8 getGridWidth();
  real8 getGridHeight();
  real8 getGridSquareWidth();
  real8 getGridSquareHeight();


  Point2i getPos();
  uint4 getWidth();
  uint4 getHeight();

  void setAlpha(real8 alpha);

  real8 getAlpha();
  void setRows(uint4 rows);
  void setColumns(uint4 columns);

  void addAttribute(const std::string &attribute, real8 maxValue);
  Vector<Attribute> getAttributes();
  uint4 getNumAttributes();

  void initializeMap();
  void deleteMap();

  bool isOverlay();

  real8**** heuristicMap;
  real8**** oldHeuristicMap;
  real8**** immobileHeuristicMap;
  uint8** ageMap;
  real8 mapScaleSize;

  Vector<Attribute> attributes;

  void calculateGridDimensions();

  sint4 getAttributeIndex(std::string attributeName);


  Vector<QueryResult*> queryResults;

  // questions that can be asked of the map

  void getMaxValues(uint4 numValues, real8 lowerLimit, real8 upperLimit, uint4 type,
		    const std::string &attributeName);
  void getNearestValues(Point2i objectPos, uint4 numValues, real8 lowerLimit, real8 upperLimit,
			uint4 type, const std::string &attributeName);

  // temporary, only for being able to display nearest objects on mapScaleSize
  Point2i lastChangedObjectPos;

protected:


private:
  GfxModule* gfxm;
  GameStateModule* gsm;

  Point2i pos;
  uint4 width,height;

  uint4 numColumns;
  uint4 numRows;

  real8 gridWidth;
  real8 gridHeight;

  real8 gridSquareWidth;
  real8 gridSquareHeight;

  real8 alpha;
  sint4 currentAttributeIndex;
  uint4 currentType;
  bool showFriendly;
  bool showEnemy;
  bool showNeutral;

  bool overlay;

};



#endif
