#ifndef Text_IO_Buffer_H
#define Text_IO_Buffer_H

// $Id: Text_IO_Buffer.H 5279 2007-06-23 02:49:08Z mburo $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "IO_Buffer.H"

class Text_IO_Buffer //: public IO_Buffer
{
public:
  
  // Text_IO_Buffer() {}
  enum { MAX_LEN = 1000000 }; // max. bytes per message uncompressed/compressed

  /**
   * constructor
   * @param so        The socket to read from and write to.
   * @param blocking  Blocking or nonblocking IO.
   * @param rand_delay Random delay of messages iff != 0
  */

  Text_IO_Buffer(TCPsocket so_, bool blocking_);

  /**
    * returns current size of i_buf (before removing the message)
    * or -1 if read error occurred, blocks of the buffer was set to block
    */
  virtual sint4 write(const std::string &s);

  // returns current size of i_buf (before removing the message)
  // or -1 if read error occurred
  virtual sint4 read(std::string &s);

  virtual ~Text_IO_Buffer();

  bool read_is_good() const { return read_good; }
  bool write_is_good() const { return write_good; }  
  
protected:

  SDL_sem *read_sem; ///< semaphore for blocking the read call
  bool    blocking;  ///< blocking or non blocking IO
  
  /**
  * reads the message until the first newline character
  * returns the length of the messege read excluding the newline
  */
  sint4 probing_blocking_read(void *buffer);   

  TCPsocket so;
  sint4 clev;                          // compression level
  RandSrc *rand_delay;                 // for randomizing delays (=0: no delay)
  SDL_Thread *read_thread, *write_thread; // threads for blocking TCP I/O
  SDL_Thread *i_delay_thread;          // only used if delay_ms > 0

  SDL_mutex *buf_mut;           // protecting critical sections w.r.t. buf
  SDL_sem   *o_buf_sem;         // semaphore signaling msgs. ready to send
  SDL_mutex *i_delay_buf_mut;   // protecting critical sections w.r.t. i_delay_buf
  
  volatile bool done;                  // signals threads to finish
  volatile bool read_good, write_good; // true if reading/writing loop running
  
  std::list<std::string> i_buf, o_buf; // input/output message buffers
  uint1 *input_chars;                  // scratch space for socket reads
  volatile sint4 i_len, o_len;         // current string buffer length
  volatile sint8 i_bytes, o_bytes;     // bytes transferred

  sint4 blocking_write(void *buffer, sint4 len);
  sint4 blocking_read(void *buffer, sint4 len);  

  std::list<std::string> i_delay_buf, o_delay_buf;// buffers needed for delaying messages
  std::list<uint4> i_delay_times, o_delay_times;  // due times

public:

  static int write_loop(void *data);
  static int read_loop(void *data);
};

#endif
