#ifndef  CWIDGET_H
#define  CWIDGET_H

/** @file CWidget.H
    Example base for graphical script objects with C accessors.

    $Id: CWidget.H 5279 2007-06-23 02:49:08Z mburo $
    $Source$
*/

#include "Global.H"
#include "GameObj.H"

/** Implements some useful graphical widget functions. */

class CWidget : public ScriptObj {

public:
  CWidget();
  virtual ~CWidget();

  enum Layout{ 
    FLOATER    =    0,
    TOP        =    1,
    BOTTOM     =    2,
    LEFT       =    4,
    RIGHT      =   16,
    INSIDE     =   32,
    OUTSIDE    =   64,
    VERTICAL   =  128,
    HORIZONTAL =  256,
    DIAGONAL   =  512
  };

  void set_position (sint4 x, sint4 y);
  void set_size     (sint4 w, sint4 h);

  void set_name (const std::string &s) { name = s; }
  void set_name (char *s) { name = s; }
  
  std::string bp_name() const { return "CWidget"; }
  const char* get_name() const { return name.c_str(); }

  void turn_off() { off = stype(1); }
  void turn_on()  { off = stype(0); }

  bool is_turned_on() { return !off; }

  void set_layout(sint4 layout) { this->layout = layout; }
  
  void set_layout(sint4 layout, ScriptObj *wrt) {
    this->layout = layout;
    layout_wrt = wrt;
  }
  
  sint4 get_layout() const { return layout; }
  ScriptObj* get_layout_wrt() const { return layout_wrt; }

  virtual ObjAction* action(const std::string &) const;
  virtual sintptr *get_int_ptr(const std::string &);
  virtual const sintptr *get_int_ptr(const std::string &) const;

protected:
  
  stype off;
  stype x;
  stype y;
  stype h;
  stype w;
  
  sint4 layout; 
  ScriptObj *layout_wrt;

  std::string name;

  Vector<ObjAction*> actions;
  std::map<std::string, ObjAction*> act_map;

  virtual stype event_mouse_motion(stype /*x*/, stype /*y*/, stype /*b*/) { return stype(); }
  virtual stype event_mouse_hover (stype /*x*/, stype /*y*/, stype /*b*/) { return stype(); }
  virtual stype event_mouse_down  (stype /*x*/, stype /*y*/, stype /*b*/) { return stype(); }
  virtual stype event_mouse_up    (stype /*x*/, stype /*y*/, stype /*b*/) { return stype(); }
  virtual stype event_key_down    (stype /*key*/, stype /*mod*/, stype /*x*/, stype /*y*/) { return stype(); }
  virtual stype event_key_up      (stype /*key*/, stype /*mod*/, stype /*x*/, stype /*y*/) { return stype(); }
  virtual stype script_set_size     (stype w, stype h);
  virtual stype script_set_position (stype x, stype y);
  virtual stype draw() { return stype(); }
  virtual bool is_outside(stype x, stype y);
  virtual stype set_enabled(stype e);

private:

  stype event_mouse(const SActParam&);
  stype event_key  (const SActParam&);

  stype set_size          (const SActParam&);
  stype set_position      (const SActParam&);
  stype draw              (const SActParam&);
  stype is_outside        (const SActParam&);
  stype hide              (const SActParam&);
  stype show              (const SActParam&);

  typedef stype (CWidget::*myfunc)(const SActParam&);
  void add(const std::string &name, myfunc f, sint4 objs, sint4 p, sint4 p2);

};

REGISTER_TYPEOF(4601, Vector<CWidget*>::iterator);
REGISTER_TYPEOF(4602, Vector<CWidget*>::const_iterator);

#endif
