// $Id: GfxInit.C 5279 2007-06-23 02:49:08Z mburo $

// This is an ORTS file
// (c) Michael Buro
// (c) Tim Furtak
// (c) Keith Yerex
// (c) Sami Wagiaalla
// (c) Erik Benner
// licensed under the GPL

#include "GfxGlobal.H"
#include "Global.H"
#include "GameStateModule.H"
#include "Options.H"
#include "GameChanges.H"
#include "GfxModule.H"
#include "Game.H"
#include "GfxObject.H"
#include "GfxAnimesh.H"
#include "Image.H"
#include "TerrainModule.H"

#include "ProfileWidget.H"

using namespace std;

//===================================================================

void GfxModule::init_GLUT_window()
{
  std::cout<<"init_GLUT_window START"<<std::endl;
  GfxGlutAdaptor::set_client(this);

  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH | GLUT_ALPHA);
  
  // the window starts near the upper left corner of the screen
  glutInitWindowPosition(0, 0);
  glutInitWindowSize(opt.win_w, opt.win_h);

  // DON"T Open a window (let ogre do this step instead)
  //glutCreateWindow(APP_TITLE);//OGRE_NATHANIE
  
  // Register the splash as the draw routing until
  glutDisplayFunc   (GfxGlutAdaptor::splash);
  std::cout<<"init_GLUT_window END"<<std::endl;
  //if (opt.full_screen) glutFullScreen();
}

//-------------------------------------------------------------------

void GfxModule::init_GLUT_events()
{
  /*std::cout<<"init_GLUT_events START"<<std::endl;
  // Register the normal draw routine
  glutDisplayFunc   (GfxGlutAdaptor::draw);
  glutIdleFunc      (GfxGlutAdaptor::loop);
  
  // Register events
  glutReshapeFunc   (GfxGlutAdaptor::event_resize);
  glutKeyboardFunc  (GfxGlutAdaptor::event_key_down);
  glutKeyboardUpFunc(GfxGlutAdaptor::event_key_up);

  // special keys (arrows, page down, etc) are pressed
  glutSpecialFunc(GfxGlutAdaptor::event_special_key_down);
  //glutIgnoreKeyRepeat(true);//can't do with glut removed

  // mouse is pressed
  glutMouseFunc  (GfxGlutAdaptor::event_mouse_down);

  // mouse is pressed and moved
  glutMotionFunc (GfxGlutAdaptor::event_mouse_motion);

  // mouse is moved without being pressed
  glutPassiveMotionFunc(GfxGlutAdaptor::event_mouse_motion);
  std::cout<<"init_GLUT_window END"<<std::endl;*/
}

//===================================================================

/** Configure the OpenGL state and check for supported extensions. */

void GfxModule::init_GL()
{
  glerror("init_GL 0");
  std::cout<<"init_GL START"<<std::endl;
  glEnable(GL_CULL_FACE);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_COLOR_MATERIAL);
  //  glEnable(GL_NORMALIZE);
  //  glEnable(GL_SCISSOR_TEST);

  // light 0
  GLfloat ambientLight[]={0.1,0.1,0.1,1.0};         // set ambient light parameters
  glLightfv(GL_LIGHT0, GL_AMBIENT, ambientLight);
  
  GLfloat diffuseLight[]={0.8,0.8,0.8,1.0};         // set diffuse light parameters
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
  
  GLfloat specularLight[]={0.5,0.5,0.5,1.0};        // set specular light parameters
  glLightfv(GL_LIGHT0, GL_SPECULAR, specularLight);
  
  GLfloat lightPos[]={0.0,30.0,60.0,0.0};           // set light position
  glLightfv(GL_LIGHT0, GL_POSITION, lightPos);


  GLfloat specularReflection[]={1.0,1.0,1.0,1.0};   // set specularity
  glMaterialfv(GL_FRONT, GL_SPECULAR, specularReflection);
  glMateriali(GL_FRONT,GL_SHININESS,0);
    
  glEnable(GL_LIGHT0);                                  // activate light0
  glEnable(GL_LIGHTING);                                // enable lighting
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientLight); // set light model
  glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);

  // select modulate to mix texture with color for shading
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
  std::cout<<"init_GL END"<<std::endl;
  glerror("init_GL 1");
}

//===================================================================

void GfxModule::build_noise_textures()
{
  {
    const sint4 W = 16;
    GLuint id;
    
    glGenTextures(1, &id);
    glBindTexture(GL_TEXTURE_3D, id);
    
    glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    
    uint1 *data = new uint1[W*W*W];
    FORU (i, W*W*W) {
      data[i] = rand.rand_uint4() % 256;
    }
    
    gluBuild3DMipmaps(GL_TEXTURE_3D, GL_LUMINANCE, W, W, W, GL_LUMINANCE, GL_UNSIGNED_BYTE, data);
    //    glTexImage3D(GL_TEXTURE_3D, 0, GL_LUMINANCE, W, W, W, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, data);

    delete [] data;
    noise_tex = id;
  }
  {
    const sint4 W = 16;
    GLuint id;
    
    glGenTextures(1, &id);
    glBindTexture(GL_TEXTURE_3D, id);
    
    glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    
    uint1 *data = new uint1[W*W*W*3];
    FORU (i, W*W*W*3) {
      data[i] = rand.rand_uint4() % 256;
    }
    
    gluBuild3DMipmaps(GL_TEXTURE_3D, GL_RGB, W, W, W, GL_RGB, GL_UNSIGNED_BYTE, data);
    //    glTexImage3D(GL_TEXTURE_3D, 0, GL_RGB, W, W, W, 0, GL_RGB, GL_UNSIGNED_BYTE, data);
    
    delete [] data;
    noise_tex3 = id;
  }
}

//===================================================================

void GfxModule::initialize_colors()
{
  view_colors.clear();
  
  player_colors.clear();
  player_colors.push_back(Col3(1.0, 0.0, 0.0)); // red
  player_colors.push_back(Col3(0.0, 0.0, 1.0)); // blue
  player_colors.push_back(Col3(0.0, 1.0, 0.0)); // green

  player_colors.push_back(Col3(1.0, 1.0, 1.0)); // white
  player_colors.push_back(Col3(1.0, 1.0, 0.0)); // yellow
  player_colors.push_back(Col3(1.0, 0.0, 1.0)); // magenta
  player_colors.push_back(Col3(0.0, 1.0, 1.0)); // cyan
  player_colors.push_back(Col3(1.0, 0.6, 0.0)); // orange
  player_colors.push_back(Col3(1.0, 0.6, 0.6)); // pink
  player_colors.push_back(Col3(0.6, 0.4, 0.0)); // brown
  player_colors.push_back(Col3(0.6, 0.4, 1.0)); // purple
  player_colors.push_back(Col3(0.5, 0.75, 1.0)); // light blue

  player_colors.push_back(Col3(0.5, 0.5, 0.5)); // grey (no team)

  //------------------------------

  //  view_colors.push_back(Col3(1,0,0)); // active view
  view_colors.push_back(player_colors[game->get_client_player()]); // active view

#if 1
  view_colors.push_back(Col3(0.5,0.5,0.5));
  view_colors.push_back(Col3(0.5,0.5,0.5));
  view_colors.push_back(Col3(0.5,0.5,0.5));
  view_colors.push_back(Col3(0.5,0.5,0.5));
#else
  view_colors.push_back(Col3(0,1,0));
  view_colors.push_back(Col3(0,0,1));
  view_colors.push_back(Col3(1,1,0));
  view_colors.push_back(Col3(1,1,1));
#endif

  sint4 i = get_game().get_player_num();
  player_colors[i] = Col3(0.5, 0.5, 0.5); // set NPC color to grey

  Vector<Vec4<real4> > c;
  FORU (j, i+1) {
    Vec4<real4> foo(player_colors[j].r,
                    player_colors[j].g,
                    player_colors[j].b,
                    1.0);
    c.push_back(foo);
  }

  Model::set_colors(c);
}

//===================================================================

