/** @file GfxScript.C
    @see GfxClient.H.

    $Id: GfxScript.C 5279 2007-06-23 02:49:08Z mburo $
    $Source$
*/

// This is an ORTS file (c) Tim Furtak, licensed under the GPL

#include "Global.H"
#include "GfxModule.H"
#include "GfxTerrain.H"
#include "ScriptEvent.H"
#include "TerrainModule.H"
#include "GatherAI.H"
#include "Script.H"
#include "ScriptVector.H"

using namespace std;

//===================================================================

stype GfxModule::GUI_FlattenTerrain(const SActParam &p)
{
  OBJN_CHECK(1);  
  sint4 tile_points = game->get_tile_points();

  GameObj *gob = p.obj[0]->get_GameObj();
  if (gob == 0) return stype();

  gob->update_points();
  
  terrain->flatten((float)*gob->sod.x1/(float)tile_points,
                   (float)*gob->sod.y1/(float)tile_points,
                   (float)*gob->sod.x2/(float)tile_points,
                   (float)*gob->sod.y2/(float)tile_points);

  FIND (game2gfx, i, gob);
  if (i != game2gfx.end()) {
    GfxGameObject *ggo = i->second;
    ggo->game_tick();
  }

  return stype();
}

//===================================================================

stype GfxModule::GUI_UnFlattenTerrain(const SActParam &p)
{
  OBJN_CHECK(1);
  sint4 tile_points = game->get_tile_points();

  GameObj *gob = p.obj[0]->get_GameObj();
  if (gob == 0) return stype();
  
  gob->update_points();

  terrain->unflatten((float)*gob->sod.x1/(float)tile_points,
                     (float)*gob->sod.y1/(float)tile_points,
                     (float)*gob->sod.x2/(float)tile_points,
                     (float)*gob->sod.y2/(float)tile_points);

  FIND (game2gfx, i, gob);
  if (i != game2gfx.end()) {
    GfxGameObject *ggo = i->second;
    ggo->game_tick();
  }

  return stype();
}

//===================================================================

stype GfxModule::GUI_load_sound(const SActParam &p)
{
  VALN_CHECK(1);  
  Sound *s = load_sound(p.val[0].to_cstr());
  return stype(s, STYPE_SOUND_ID);
}

//-------------------------------------------------------------------

stype GfxModule::GUI_play_sound(const SActParam &p)
{
  VALGEN_CHECK(1);
  Sound *s = stype_to_Sound(p.val[0]);
  if (s) {

    if (p.val.size() == 3)
      mixer.play(*s,(p.val[1].to_int())/100.0,(p.val[2].to_int())/100.0);

    if (p.val.size() == 2)
      mixer.play(*s,(p.val[1].to_int())/100.0,(p.val[1].to_int())/100.0);

    if (p.val.size() == 1)
      mixer.play(*s,1,1);
  }
  return stype();
}

//===================================================================

stype GfxModule::GUI_MarkTiles(const SActParam &p)
{
  VALN_CHECK(4);  
  ColorTileSet *t = new ColorTileSet;
  t->r = p.val[1].to_int() / 255.0;
  t->g = p.val[2].to_int() / 255.0;
  t->b = p.val[3].to_int() / 255.0;

  Vector<stype> &v = *ScriptVector::stype_to_vecstype(p.val[0]);
  Vector<GfxTerrainTile*> &tiles = t->tiles;
#if 0
  for (sint4 i = 0; i < (sint4)v.size()-1; i+=2) {
    if (v[i] < 0 || v[i] >= game->get_map().get_width()) continue;
    if (v[i+1] < 0 || v[i+1] >= game->get_map().get_height()) continue;
    sint4 index = v[i] + v[i+1] * game->get_map().get_width();
    tiles.push_back(&terrain->tiles[index]);
  }
#else
  FORALL (v, i) {
    sint4 ival = i->to_int();
    if (ival < 0 || ival >= game->get_map().get_num()) continue;
    tiles.push_back(&terrain->tiles[ival]);
  }
#endif
  marked_tiles.push_back(t);

  // fixme: does this t get deleted eventually?
  return stype(t, STYPE_COLORTILESET_ID);
}

//-------------------------------------------------------------------

stype GfxModule::GUI_UnmarkTiles(const SActParam &p)
{
  VALN_CHECK(1);  
  ColorTileSet *t = stype_to_ColorTileSet(p.val[0]);
  marked_tiles.remove(t);
  delete t;
  return stype();
}

//-------------------------------------------------------------------

stype GfxModule::GUI_MarkRect(const SActParam &p)
{
  VALN_CHECK(7);    
  ColRect *t = new ColRect;
  t->r = p.val[0].to_int() / 255.0;
  t->g = p.val[1].to_int() / 255.0;
  t->b = p.val[2].to_int() / 255.0;

  t->x0 = p.val[3].to_int();
  t->y0 = p.val[4].to_int();
  t->x1 = p.val[5].to_int();
  t->y1 = p.val[6].to_int();

  marked_rectangles.push_back(t);

  return stype(t, STYPE_COLRECT_ID);
}

//-------------------------------------------------------------------

stype GfxModule::GUI_EditRect(const SActParam &p)
{
  VALN_CHECK(8);  
  ColRect *t = stype_to_ColRect(p.val[0]);
  t->r = p.val[1].to_int() / 255.0;
  t->g = p.val[2].to_int() / 255.0;
  t->b = p.val[3].to_int() / 255.0;

  t->x0 = p.val[4].to_int();
  t->y0 = p.val[5].to_int();
  t->x1 = p.val[6].to_int();
  t->y1 = p.val[7].to_int();

  return stype(t, STYPE_COLRECT_ID);
}

//-------------------------------------------------------------------

stype GfxModule::GUI_UnmarkRect(const SActParam &p)
{
  VALN_CHECK(1);    
  ColRect *t = stype_to_ColRect(p.val[0]);
  marked_rectangles.remove(t);
  delete t;
  return stype();
}

//===================================================================

#if 0
stype GfxModule::GUI_CreateObjList(const SActParam &)
{
  Vector<sint4> temp;
  id_lists.push_front(temp);
  Vector<sint4> *v = &(*id_lists.begin());
  return (sint4)v;
}

//-------------------------------------------------------------------

stype GfxModule::GUI_DeleteObjList(const SActParam &p)
{
  VALN_CHECK(1);    
  Vector<stype> *v = (Vector<stype>*)p.val[0].to_void();
  FORALL (id_lists, it) {
    if (v == &(*it)) {
      id_lists.erase(it);
      return stype();
    }
  }
  return 1;
}

//-------------------------------------------------------------------

stype GfxModule::GUI_SelectObjList(const SActParam &p)
{
  VALN_CHECK(1);  

  Vector<stype> *v = (Vector<stype>*)p.val[0];
  if (v == 0) {
    cerr << "vector pointer is null !!!" << endl;
    return stype();
  }
  
  sel_objs.clear();

  FORALL (*v, it) {
    ScriptObj *x = game->get_cplayer_info().get_obj(*it);
    GameObj *y = x->get_GameObj();
    if (y == 0) continue;
    sel_objs.insert(game2gfx[y]);
  }
  
  sel_ids->clear();
  FORALL (*v, it) sel_ids->push_back(*it);

  gui_obj->set_int("sel_change", 1, false);

  return stype();
}

#endif

//Sami===============================================================
stype GfxModule::GUI_Deselect_obj(const SActParam &p)
{
  VALN_CHECK(1);
  ScriptObj *x = p.val[0].to_obj();
  GameObj *y = dynamic_cast<GameObj*>(x);
  if (y == 0) return stype();

  FIND (sel_objs, i, game2gfx[y]);
  if (i != sel_objs.end()) {
    sel_objs.erase(*i);
    update_selected();
  }

  return stype();
}

//===================================================================

stype GfxModule::GUI_Get_DBVal(const SActParam &p)
{
  VALN_CHECK(1); 
  sint4 val;
  string key = p.val[0].to_cstr();
  stringstream ss(script_db[key]);
  ss >> val;
  
  if(ss.fail()) return stype();
  else return stype(val);
}

//-------------------------------------------------------------------

stype GfxModule::GUI_Set_DBVal(const SActParam &p)
{
  VALN_CHECK(2);  
  string key = p.val[0].to_cstr();

  stringstream ss;
  ss << p.val[1].to_int();
  script_db[key] = ss.str();
  return stype();
}

//-------------------------------------------------------------------

stype GfxModule::GUI_Get_DBVec(const SActParam &p)
{
  VALN_CHECK(2);  
  string key = p.val[0].to_cstr();
  Vector<stype> &v = *ScriptVector::stype_to_vecstype(p.val[1]);
  stringstream val(script_db[key]);
  
  FOREVER {
    sint4 t;
    val >> t;
    if (!val.good()) break;
    v.push_back(stype(t));
  }
  return stype();
}

//-------------------------------------------------------------------

stype GfxModule::GUI_Set_DBVec(const SActParam &p)
{
  VALN_CHECK(2);  
  string key = p.val[0].to_cstr();
  Vector<stype> &v = *ScriptVector::stype_to_vecstype(p.val[1]);
  stringstream val;
  FORALL (v, it) val << it->to_int() << " ";
  script_db[key] = val.str();
  return stype();
}

//===================================================================

stype GfxModule::GUI_LoadBP(const SActParam &p)
{
  VALN_CHECK(1);  
  string file = p.val[0].to_cstr();

  FORALL (loaded_bp_files, i) {
    if (*i == file) {
      loaded_bp_files.erase(i);
      break;
    }
  }

  load_interface(file);
  return stype();
}

//Sami===============================================================

stype GfxModule::GUI_quitGame(const SActParam &)
{
  quit = true;
  return stype();
}

//Sami===============================================================

stype GfxModule::GUI_toggle_pathfinding(const SActParam &p)
{
  VALN_CHECK(1);  
  pathfinding_on = p.val[0].to_int();
  return stype();
}

//Sami===============================================================

stype GUI_psin(const SActParam &p)
{
  VALN_CHECK(2);
  return stype((int)(sin((real8)p.val[0].to_int()*M_PI/180)*(real8)p.val[1].to_int()));
}

//Sami===============================================================

stype GUI_pcos(const SActParam &p)
{
  VALN_CHECK(2);
  return stype((int)(cos((real8)p.val[0].to_int()*M_PI/180)*(real8)p.val[1].to_int()));
}

//===================================================================

stype GUI_atan2(const SActParam &p)
{
  VALN_CHECK(2);
  real4 dx = p.val[0].to_int();
  real4 dy = p.val[1].to_int();
  real4 ang = atan2f(dx, dy);
  return stype((int)(180.0*ang/M_PI));
}

//===================================================================

stype GUI_strcmp(const SActParam &p)
{
  VALN_CHECK(2);  
  const char *a = p.val[0].to_cstr();
  const char *b = p.val[1].to_cstr();
  if (!a || !b) return stype(-1);
  return stype(strcmp(a, b));
}

//===================================================================

stype GfxModule::GUI_broadcast(const SActParam &p)
{
  VALGEN_CHECK(1);
  string msg = p.val[0].to_cstr();

  Vector<GameObj*> obj;
  FORALL (p.obj, i) {
    GameObj *g = (*i)->get_GameObj();
    if (g) obj.push_back(g);
    else { REM("gob = 0 in broadcast"); }
    // fixme: why is 0 skipped? isn't that an error
  }

  if (msg == "pathfind") {
    VALN_CHECK(3);
    TerrainBase::Task t(TerrainBase::Goal::LOCATION, TerrainBase::Goal::TOUCH);
    t.goal.loc = TerrainBase::Loc(p.val[1].to_int(), p.val[2].to_int());
    t.objs.insert(obj.begin(),obj.end());
    return stype(call_handlers(TerrainModule::PathEvent(FROM, TerrainModule::FIND_PATH_MSG, t)));
  }
  if (msg == "pathstop") {
    return stype(call_handlers(
      TerrainModule::PathStopEvent(FROM, TerrainModule::FIND_PATH_STOP, obj)));
  }
  if (msg == "gather") {
    OBJN_CHECK(2);        
    return stype(call_handlers(GatherEvent(FROM, GatherAI::GATHER_START, obj[0], obj[1])));
  }
  if (msg == "gatherstop") {
    OBJN_CHECK(1);    
    return stype(call_handlers(GatherEvent(FROM, GatherAI::GATHER_STOP, obj[0], 0)));
  }

  call_handlers(ScriptEvent(FROM, SCRIPT_MSG, p));
  return stype();
}

//===================================================================
