#ifndef ScriptModel_H
#define ScriptModel_H

/** @file ScriptModel.H

    $Id: ScriptModel.H 5279 2007-06-23 02:49:08Z mburo $
    $Source$
*/

// This is an ORTS file (c) Tim Furtak

#include "Global.H"
#include "GameObj.H"

class PlayerInfo;
class SActParam;
class GfxModule;
class GfxAnimesh;

//===================================================================

/** Scripted wrapper for a GfxAnimesh.
*/

class ScriptModel : public ScriptObj
{
private:
  Vector<ObjAction*> actions;
  std::map<std::string, ObjAction*> act_map;
  GfxAnimesh *mesh;

public:
  static GfxModule *gfxm;

  ScriptModel();
  ~ScriptModel();
  static ScriptObj* create() { return new ScriptModel; }

  std::string bp_name() const { return "ScriptModel"; }

  ObjAction* action(const std::string &name) const;

  GfxAnimesh* get_mesh() { return mesh; }

  void destruct();

private:

  stype load_model     (const SActParam&);
  stype set_shadow     (const SActParam&);
  stype enable_shadow  (const SActParam&);
  stype position       (const SActParam&);
  stype rotate         (const SActParam&);
  stype scale          (const SActParam&);
  stype draw_2d        (const SActParam&);
  stype draw_overlay   (const SActParam&);
  stype idle_animation (const SActParam&);
  stype play_animation (const SActParam&);
  stype set_frame      (const SActParam&);
  stype play_custom    (const SActParam&);
  stype set_parent     (const SActParam&);

  typedef stype (ScriptModel::*myfunc)(const SActParam&);
  void add(const std::string &name, myfunc f, sint4 objs, sint4 p, sint4 p2);

  static const sint4 STYPE_GFXANIMESH_ID;
  
  stype stype_from_GfxMesh(GfxAnimesh *mesh) {
    return stype(mesh, STYPE_GFXANIMESH_ID);
  }
};

#endif
