// $Id: Net_GGSClient.C 5302 2007-06-25 19:42:05Z lanctot $

#include "Net_GGSClient.H"
using namespace std;

static const bool debug = true;

Net_GGSClient::Net_GGSClient(const string &login_, const string &pass_, const string &server_,
                             sint4 port_, bool blocking)
{
  IPaddress ip;
  uint2 port = (uint2) port_;
  done = false;
  loginOK = false;
  listen_thread = 0;

  // Resolve the argument into an IPaddress type
  if (SDLNet_ResolveHost(&ip, (char *)server_.c_str(), port) == -1) ERR2("SDLNet_ResolveHost: ", SDLNet_GetError());

  //free(server);

  // open the server socket
  sock=SDLNet_TCP_Open(&ip);
  if (!sock) ERR2("(1)SDLNet_TCP_Open: ", SDLNet_GetError());

  io_buffer = new Text_IO_Buffer(sock, blocking);
  login(login_, pass_);
}

void Net_GGSClient::login(const string &login, const string &pass)
{
  Protocol::GGS_Error error;
  Net_Message m;

  io_buffer->write(login);
  io_buffer->write(pass);
  string s;

  while(io_buffer->read_is_good()){

    //int r = io_buffer->read(s);
    io_buffer->read(s);

    ///cout << endl << "r = " << r << ", received: " << s << endl;

    m.setMessage(s);
    if(error.parse(m)){
      loginOK = false;
      error_str = error.get_msg();
      return;
    }

    if(s == "READY"){
      loginOK = true;
      return;
    }
  }
  loginOK = false;
}

Net_GGSClient::~Net_GGSClient()
{
  REM("~Net_GGSClient called");

  delete io_buffer;

  if (listen_thread) {
    done = true;
    SDL_KillThread(listen_thread);
    listen_thread = 0;
  }

  SDLNet_TCP_Close(sock);
}


// sends the given message to the server
// returns -1 if error occures
sint4 Net_GGSClient::send(const string &s)
{
  if(!loginOK){  return -1; }
  return io_buffer->write(s);
  // SDLNet_TCP_Send(sock,(void*)"----\n",4);
}


// reads until the next READY
sint4 Net_GGSClient::recv(string &s)
{
  if(!loginOK){
    return -1;
  }

  string tmp;
  string test;
  s = "";

  if(io_buffer->read(tmp) < 0) return -1;

  while (tmp != "READY") {

    s += tmp;
    s += "\n";

    if (io_buffer->read(tmp) < 0) return -1;
  }

  return 1;
}

int Net_GGSClient_Listen_loop(void * data)
{
  Net_GGSClient &client = * static_cast<Net_GGSClient *>(data);
  REM("listen loop started");
  string message;
  int res;

  while (!client.done){
    res = client.recv(message);

    if (res < 0){
      ERR("listen loop exiting");
      return -1;
    }

    if (message.size() > 0){
      (*client.messageHandler)(message);
    }
  }

  return 1;
}

// reads exactly one line
sint4 Net_GGSClient::recvLine(string &s) {
  return io_buffer->read(s);
}

void Net_GGSClient::listenStart(void (* messageHandler_)(const string &message))
{
  messageHandler = messageHandler_;
  listen_thread = SDL_CreateThread(Net_GGSClient_Listen_loop, this);
}

void Net_GGSClient::listenStop(){
  done = true;
}


/**
    quits ggs;
    closes sockets and shuts down listen if it is runing
 */
void Net_GGSClient::close(){
  listenStop();
  send("quit");
}



void Net_GGSClient::tellClient(const string& name, const string& message)
{
  if (debug)
    cout << "Telling client (" << name << ") message: " << message << endl;

  this->send("tell " + name + " " + message);
}

void Net_GGSClient::tellGGS(const string& message)
{
  this->send(message);
}


