// $Id: Net_GGSClient.H 5302 2007-06-25 19:42:05Z lanctot $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

/** /brief Connects to the GGS server
             provides simple send functionality
             throws an event when a message is recieved
             Uses Text_IO_Buffer to avoid blocking
 */

#ifndef Net_GGSClient_H
#define Net_GGSClient_H

#include "Global.H"
#include "Text_IO_Buffer.H"
#include "Net_Protocol.H"
#include "Net_Message.H"

class Net_GGSClient {

public:

  /** connects the the given server */
  Net_GGSClient(const std::string &login_, const std::string &pass_, const std::string &server_, sint4 port_, bool blocking = false);
  virtual ~ Net_GGSClient();

  /** sends the given message to the server
     returns -1 if error occures */
  sint4 send(const std::string &s);

  /** if listen is not used... this can be used to recv one message
     returns -1 if error occures
         reads untill the next READY statement
   */
  sint4 recv(std::string &s);

  /** reads one line at a time */
  sint4 recvLine(std::string &s);

  /** starts a thread that will spin until a message is recieved
     then it calls the given function */
  void listenStart(void (* messageHandler)(const std::string &message));

  /** stops the listening loop */
  void listenStop();

  /**
     quits ggs;
     closes sockets and shuts down listen if it is runing
   */
  void close();
  bool good(){
    return loginOK;
  }
  const std::string& get_error(){
    return error_str;
  }
  bool  read_is_good(){
    return io_buffer->read_is_good();
  }
  bool write_is_good(){
    return io_buffer->write_is_good();
  }

protected:
  bool loginOK;
  std::string error_str;

  bool done;                 /** used to signal the listen loop to exit */
  Text_IO_Buffer * io_buffer; /** read write buffer */
  TCPsocket sock;            /** connection to GGS */
  SDL_Thread * listen_thread; /** the thread that will spin and probe */

  /** the function to be called when a message is recieved */
  void (*messageHandler)(const std::string &message);

  /** executes the loging in protocol */
  void login(const std::string &login, const std::string &pass);

  /** spin listener */
  friend int Net_GGSClient_Listen_loop(void * data);

  /** common teller functions */

  /** tell message to ggs */
  void tellGGS(const std::string& message);

  /** tell message to the given client*/
  void tellClient(const std::string& name, const std::string& message);

};

#endif
