// $Id: API.H 5279 2007-06-23 02:49:08Z mburo $ 

// (c) Jonathan Yip, licensed under the GPL

#ifndef API_H
#define API_H

#include "Global.H"
#include "Mem.H"

using namespace std;

// fixme: those should become constants in a class

#define FLAG_GC        0x0001
#define FLAG_ALLOCATED 0x0002


//typedef bool (*Cfunction)(AIClass &params);

// fimxe:  +2 must become a constant!
// also: how to address Chars (2 bytes)?

// The variable names and locations

class ScriptObjectType {

public:

  static const uint1 members_offset = 2;

  ScriptObjectType() : m_defined(false), m_size(0) {}

  bool get_type(const string &name, Mem::mem_types expected_type) 
  {
    if (m_types.find(name) != m_types.end()) return m_types[name] & expected_type; 
    return false;
  }

  sint4 get_loc(const string &name) 
  {
    return m_loc[name]; 
  }

  vector<Mem::mem_types> get_types() 
  {
    return m_types_by_index; 
  }

  vector<uint4> get_objs()
  {
    return m_obj_indices;
  }

  uint4 size() 
  {
    return m_size+members_offset;
  }

  friend class Script;


private:
  bool m_defined;

  uint4 m_size;

  map<string, Mem::mem_types> m_types;
  map<string, sint4> m_loc;

  map<string, string> m_objs;

  vector<Mem::mem_types> m_types_by_index;
  vector<uint4> m_obj_indices;

};


class ScriptObject {

public:

  typedef uint2 Char;

  ScriptObject(ScriptObjectType *definition) : m_definition(definition) {
    m_flags |= FLAG_ALLOCATED;
  }

  ~ScriptObject() {}

  bool set_int(const string &name, sint4 value)
  {
    if (!m_definition->get_type(name, Mem::INT)) return false;

    *(reinterpret_cast<sint4*>(this) + ScriptObjectType::members_offset +
      m_definition->get_loc(name)) = value;

    return true;
  }

  bool set_float(const string &name, real4 value)
  {
    if (!m_definition->get_type(name, Mem::FLOAT)) return false;

    *(reinterpret_cast<real4*>(this) + ScriptObjectType::members_offset +
      m_definition->get_loc(name)) = value;
    return true;
  }

  bool set_string(const string &name, const string &value)
  {
    if (!m_definition->get_type(name, Mem::STRING)) return false;

    string *s = new string;
    *s = value;

    *(reinterpret_cast<string**>(this) + ScriptObjectType::members_offset + m_definition->get_loc(name)) = s;
    return true;
  }
  
  bool set_object(const string &name, ScriptObject* value)
  {
    if (!m_definition->get_type(name, Mem::OBJECT)) return false;

    *(reinterpret_cast<ScriptObject**>(this) + ScriptObjectType::members_offset + m_definition->get_loc(name)) = value;
    return true;
  }


  void set_value(uint4, Char)
  {
    ERR("implement!"); // fixme
    //*(reinterpret_cast<sint4*>(this) + ScriptObjectType::members_offset + loc) = value;
  }
  

  void set_value(uint4 loc, sint4 value)
  {
    *(reinterpret_cast<sint4*>(this) + ScriptObjectType::members_offset + loc) = value;
  }
  
  void set_value(uint4 loc, real4 value)
  { 
    *(reinterpret_cast<real4*>(this) + ScriptObjectType::members_offset + loc) = value;
  }
  
  void set_value(uint4 loc, const string &value)
  { 
    string *s = new string;
    *s = value;

    *(reinterpret_cast<string**>(this) + ScriptObjectType::members_offset + loc) = s;
  }

  void set_value(uint4 loc, ScriptObject* value)
  {
    *(reinterpret_cast<ScriptObject**>(this) + ScriptObjectType::members_offset + loc) = value;
  }


  sint4 get_int(const string &name)
  {
    if (!m_definition->get_type(name, Mem::INT)) ERR("fixme");

    return *(reinterpret_cast<sint4*>(this) + ScriptObjectType::members_offset +
	     m_definition->get_loc(name));
  }

  real4 get_float(const string &name)
  {
    if (!m_definition->get_type(name, Mem::FLOAT)) ERR("fixme");

    return *(reinterpret_cast<real4*>(this) + ScriptObjectType::members_offset +
	     m_definition->get_loc(name));
  }

  string get_string(const string &name)
  {
    if (!m_definition->get_type(name, Mem::STRING)) ERR("fixme");

    return **(reinterpret_cast<string**>(this) + ScriptObjectType::members_offset +
	      m_definition->get_loc(name));
  }

  ScriptObject* get_object(const string &name)
  {
    if (!m_definition->get_type(name, Mem::OBJECT)) ERR("fixme");

    return *(reinterpret_cast<ScriptObject**>(this) + ScriptObjectType::members_offset +
	     m_definition->get_loc(name));
  }

  Char get_value(uint4, Char /*dummy*/) 
  {
    ERR("implement!"); // fixme
    //return *(reinterpret_cast<sint4*>(this) + ScriptObjectType::members_offset + loc);  // different address calculation: Chars need ScriptObjectType::members_offset bytes 
  }

  sint4 get_value(uint4 loc, sint4 /*dummy*/) 
  {
    return *(reinterpret_cast<sint4*>(this) + ScriptObjectType::members_offset + loc);
  }

  real4 get_value(uint4 loc, real4 /*dummy*/) 
  {
    return *(reinterpret_cast<real4*>(this) + ScriptObjectType::members_offset + loc);
  }

  string get_value(uint4 loc, const string &/*dummy*/) 
  {
    return **(reinterpret_cast<string**>(this) + ScriptObjectType::members_offset + loc);
  }

  ScriptObject* get_value(uint4 loc, ScriptObject* /*dummy*/)
  {
    return *(reinterpret_cast<ScriptObject**>(this) + ScriptObjectType::members_offset + loc);
  }

  ScriptObjectType* get_definition() { return m_definition; }

  friend class Mem;

private:
  ScriptObjectType*  m_definition;

  uint4 m_flags;

};
typedef sint4 (*Cfunction)(ScriptObject *params);

#endif
