// $Id: OSL_Heap.C 5279 2007-06-23 02:49:08Z mburo $

// (c) Jonathan Yip, jyip@cs.ualberta.ca, licensed nder the GPL

#include "OSL.H"

using namespace std;

namespace OSL {

#define DEBUG 1
#define GCOUT errstr << "GC: "

#define FLAG_GC 0x01

  Heap::Heap(Program *prog_) {
    heap = new mem_unit[init_heap_size];
    heap_size = init_heap_size;
    heap_index = 0;
    prog = prog_;
  }
  
  Heap::mem_unit* Heap::alloc(uint4 size) {
#if DEBUG
    GCOUT << "Requesting alloc of size " << size << ", index at " << heap_index
	  << ", heap size is " << heap_size << endl;
#endif
    // See if I should do a compaction
    if (heap_index + size > heap_size) {
      GC(size);
    }

    uint4 index = heap_index;
    heap_index += size;

    return &heap[index];
  }

  void Heap::GC(uint4 elements) {
#if DEBUG
    GCOUT << "Starting garbage collection" << endl;
#endif
    cout << (void*)&heap[0] << endl;
    mem_unit* old_heap = heap;
    heap = 0;

    while (heap_size < heap_index + elements)
      heap_size *= 2;

    heap = new mem_unit[heap_size];
    heap_index = 0;

#if DEBUG
    GCOUT << "New size is " << heap_size << ", old heap at "
	  << (void*)&old_heap[0] << ", new heap at " 
	  << (void*)&heap[0] << endl;
#endif

    uint4 check_from = 0;

    // got to run through all execution stacks
    for (uint4 i = 0; i<prog->contexts.size(); i++) {
      Value *stack = prog->contexts[i]->stack;
      uint4 sp = prog->contexts[i]->sp;

#if DEBUG
      GCOUT << "On context " << i << " with stack size " << sp << endl;
#endif

      prog->contexts[i]->write_stack();
      map<mem_unit*, mem_unit*> ptr_map;
      for (uint4 i = 0; i<sp; i++) {
	cout << i << endl;
	cout << stack[i].cat << endl;
	if (stack[i].cat == Type::TC_object && stack[i].v_object != 0) {
	  if (stack[i].v_object->flags & FLAG_GC) {
	    stack[i].v_object = reinterpret_cast<Object*>(ptr_map[reinterpret_cast<mem_unit*>(stack[i].v_object)]);
	    continue;
	  }

	  stack[i].v_object->flags |= FLAG_GC;
	    cout << "HERE" << endl;
	    cout << stack[i].v_object->flags << " " << FLAG_GC << endl;

	    cout << stack[i].v_object << endl;
	  mem_unit* ptr = reinterpret_cast<mem_unit*>(stack[i].v_object);
	  ptr_map[ptr] = &heap[heap_index];
	  uint4 class_index = prog->types[stack[i].v_object->tind].class_index;
	  uint4 size = prog->classes[class_index].inst_data_size;

	  memcpy(&heap[heap_index], ptr, size);
#if DEBUG
	  GCOUT << "Moving mem from " << (void*)ptr << " to " 
		<< (void*)&heap[heap_index] << endl;
#endif

	  ClassInstance *ci = reinterpret_cast<ClassInstance*>(stack[i].v_object);
	  // point stack to right spot
	  stack[i].v_object = reinterpret_cast<ClassInstance*>(&heap[heap_index]);

	  heap_index += size;
	  queue<ClassInstance*> cis;
	  cis.push(ci);

	  while (!cis.empty()) {
	    ClassInstance *ci = cis.front();
	    cis.pop();

	    uint4 class_index = prog->types[ci->tind].class_index;
	    uint4 check_to = prog->classes[class_index].inst_data.size();

	    for (uint4 j = 0; j<check_to; ++j)
	      if (ci->data[j].cat == Type::TC_object) {
		if (ci->data[j].v_object->flags & FLAG_GC) continue;
		
		ci->data[j].v_object->flags |= FLAG_GC;
		
		mem_unit* ptr = reinterpret_cast<mem_unit*>(ci->data[j].v_object);
		ptr_map[ptr] = &heap[heap_index];
		heap_index += size;

		uint4 class_index = prog->types[ci->data[j].v_object->tind].class_index;
		uint4 size = prog->classes[class_index].inst_data.size();

		memcpy(&heap[heap_index], ptr, size);
	      
		cis.push(reinterpret_cast<ClassInstance*>(ci->data[j].v_object));
	      }
	  }


	  // found object...push onto new heap, adjust pointer from stack
	  // store in map new pointer to it

	  // add onto new heap any objects that this points to
	}
      }
      cout << "HERE" << endl;
      // adjust pointers
      for (uint4 i = check_from; i<heap_index; /* no itr*/ ) {
	cout << "HI " << i << " " << heap_index << endl;
	ClassInstance* ci = reinterpret_cast<ClassInstance*>(&heap[i]);
	
	uint4 class_index = prog->types[ci->tind].class_index;
	uint4 check_to = prog->classes[class_index].inst_data.size();

	for (uint4 j=0; j<check_to; ++j)
	  if (ci->data[j].cat == Type::TC_object) {
	    if (ptr_map.find(reinterpret_cast<mem_unit*>(ci->data[j].v_object)) != ptr_map.end())
	      ci->data[j].v_object = reinterpret_cast<ClassInstance*>(ptr_map[reinterpret_cast<mem_unit*>(ci->data[j].v_object)]);
	    
	  }
	
	i+=prog->classes[class_index].inst_data_size;
      }

	// look in class description...fix pointers

      check_from = heap_index;
    }
    delete[] old_heap;
  
  }
}
