// $Id: OSL_Opcodes.H 5279 2007-06-23 02:49:08Z mburo $

// (c) Michael Buro, mburo@cs.ualberta.ca, licensed under the GPL

/* todo:

   - implement all instructions, many sint/uint8 ones are missing
   - test all instructions (wait for compiler to generate them)
   
*/       

#ifndef OSL_Opcodes_H
#define OSL_Opcodes_H

namespace OSL {

  struct Opcodes {

    enum {

      UNKNOWN = 0,

      NOP,

      JUMP,        // fetch(c) goto c
      JUMP_FALSE,  // fetch(c) a=pop() if a==0 goto c
      JUMP_TRUE,   // fetch(c) a=pop() if a!=0 goto c
      JUMP_PFALSE, // jump peek false (for &&)  like jump but no pop
      JUMP_PTRUE,  // jump peek true  (for ||)  like jump but no pop

      CALL_VIRT,   // fetch(fid) c=typeof(peek()) call c::virtual[fid]
      CALL_STAT,   // fetch(cid) fetch(fid) call cid::static[fid]
      CALL_EXT,    // fetch(eid) create obj[] and call-C-func[eid]

      RETURN,      // return to caller (no value)
      RETURN_VAL,  // return to caller with value

      NEW_CLASS,   // fetch(cid) a=allocate-memory type(a) = ref2cid
                   // call a->cid::<init> push(a)
      NEW_ARRAY,   // fetch(i) fetch(tid) a=allocate-memory
                   // clear-memory type(a) = ref2array of tid push(a) 

      // push value as-is
      LOAD_LOCAL,  // fetch(i) push(local[i])
      LOAD_PARAM,  // fetch(i) push(param[i])       
      LOAD_STATIC, // fetch(i) fetch(cid) push(cid::stat[i])
      LOAD_INST,   // fetch(i) a=pop() push(a->memb[c])
      LOAD_ARRAY,  // i=pop() a=pop() push(a[i])

      // local variable: fetch(i) local[i] = pop()
      STORE_LS1, STORE_LU1, STORE_LS2, STORE_LU2, STORE_LS4, // type must be sint4
      STORE_LU4, STORE_LS8, STORE_LU8, STORE_LR4, STORE_LR8, // type must be identical
      STORE_LREF, // type must be compatible reference

      // parameter: fetch(i) param[i] = pop()
      STORE_PS1, STORE_PU1, STORE_PS2, STORE_PU2, STORE_PS4, // type must be sint4
      STORE_PU4, STORE_PS8, STORE_PU8, STORE_PR4, STORE_PR8, // type must be identical
      STORE_PREF, // type must be compatible reference

      // instance variable: // fetch(i) a=pop() a->inst[i] = pop()
      STORE_IS1, STORE_IU1, STORE_IS2, STORE_IU2, STORE_IS4, // type must be sint4
      STORE_IU4, STORE_IS8, STORE_IU8, STORE_IR4, STORE_IR8, // type must be identical
      STORE_IREF, // type must be compatible reference

      // static variable: fetch(cid) fetch(i) cid::stat[i] = pop()
      STORE_SS1, STORE_SU1, STORE_SS2, STORE_SU2, STORE_SS4, // type must be sint4
      STORE_SU4, STORE_SS8, STORE_SU8, STORE_SR4, STORE_SR8, // type must be identical
      STORE_SREF, // type must be compatible reference

      // array: v=pop() i=pop() a=pop() a[i] = v
      STORE_AS1, STORE_AU1, STORE_AS2, STORE_AU2, STORE_AS4, // type must be sint4
      STORE_AU4, STORE_AS8, STORE_AU8, STORE_AR4, STORE_AR8, // type must be identical
      STORE_AREF, // type must be compatible reference

      DUP,         // push(top())
      POP,         // pop()
      
      LINE,        // fetch(line) (debug info)
      YIELD,       // suspend execution

      // not yet implemented ...
      ISA_CREF,    // fetch(cid) a=pop() push(a is a cid ref)
      CONV_CREF,   // fetch(cid) a=pop() if (a is a cid ref) push((cid)a) else push((cid)null)

      /*
	conversions:

	all conversions create values of type u4,s4,u8,s8,r4, or r8:
  	  s1,u1,s2,u2 -> s4
	  u4          -> u4
	  s8          -> s8
	  u8          -> u8
	  r4          -> r4
	  r8          -> r8
	
	real4 -> real8

	  round to nearest real8 value (no exception)
	
	int -> real:

	  round to nearest real value (no exception)

	real -> int:

	  round to nearest int value; exception on overflow

	sintK -> sintL:

	  K < L: copy (sign extend)
	  K > L: throw exception if out of range, copy otherwise

	uintK -> sintL:

	  K < L: copy (sign extend)
	  K > L: throw excpetion if out of range, copy otherwise

	sintK -> uintL:

	  if (sint value < 0) throw error
	  if (sint value > max_uintL) throw error
	  copy

	uintK -> uintL:

	  if (uintK value > max_uintL) throw error
	  copy
      */

      CONV_S4S1, CONV_U4S1, CONV_S8S1, CONV_U8S1, CONV_R4S1, CONV_R8S1,      

      CONV_S4U1, CONV_U4U1, CONV_S8U1, CONV_U8U1, CONV_R4U1, CONV_R8U1,      
      
      CONV_S4S2, CONV_U4S2, CONV_S8S2, CONV_U8S2, CONV_R4S2, CONV_R8S2,      

      CONV_S4U2, CONV_U4U2, CONV_S8U2, CONV_U8U2, CONV_R4U2, CONV_R8U2,      

                 CONV_U4S4, CONV_S8S4, CONV_U8S4, CONV_R4S4, CONV_R8S4,      

      CONV_S4U4,            CONV_S8U4, CONV_U8U4, CONV_R4U4, CONV_R8U4,
      
      CONV_S4S8, CONV_U4S8,            CONV_U8S8, CONV_R4S8, CONV_R8S8,      

      CONV_S4U8, CONV_U4U8, CONV_S8U8,            CONV_R4U8, CONV_R8U8,

      CONV_S4R4, CONV_U4R4, CONV_S8R4, CONV_U8R4,            CONV_R8R4,

      CONV_S4R8, CONV_U4R8, CONV_S8R8, CONV_U8R8, CONV_R4R8, 

      // push constant (also for creating local variables)

      PUSH_CS4, // fetch(c) push(c)
      PUSH_CU4,
      PUSH_CS8,
      PUSH_CU8,
      PUSH_CR4,
      PUSH_CR8,        
      PUSH_CARR,  // fetch(string) push(reference to new char[] = string)
                  // string format in bytecode: length(4), sequence of bytes (padded)

      // initialization with 0 (also for creating local variables)

      PUSH_NULL,  // push(null)
      PUSH_ZS4,   // push(0)
      PUSH_ZU4,   
      PUSH_ZS8,
      PUSH_ZU8,
      PUSH_ZR4,
      PUSH_ZR8,   // push((real8)0)

      // binary operations (promote types if necessary)

      // b=pop() a=pop() push(a + b)

      ADD_R4, ADD_R8,
      ADD_SS4, ADD_US4, ADD_SU4, ADD_UU4,
      //ADD_SS4_CHK, ADD_US4_CHK, ADD_SU4_CHK, ADD_UU4_CHK,
      ADD_SS8, ADD_US8, ADD_SU8, ADD_UU8,
      //ADD_SS8_CHK, ADD_US8_CHK, ADD_SU8_CHK, ADD_UU8_CHK,

      SUB_R4, SUB_R8,
      SUB_SS4, SUB_US4, SUB_SU4, SUB_UU4,
      //SUB_SS4_CHK, SUB_US4_CHK, SUB_SU4_CHK, SUB_UU4_CHK,
      SUB_SS8, SUB_US8, SUB_SU8, SUB_UU8,
      //SUB_SS8_CHK, SUB_US8_CHK, SUB_SU8_CHK, SUB_UU8_CHK,

      MUL_R4, MUL_R8,
      MUL_SS4, MUL_US4, MUL_SU4, MUL_UU4,
      //MUL_SS4_CHK, MUL_US4_CHK, MUL_SU4_CHK, MUL_UU4_CHK,
      MUL_SS8, MUL_US8, MUL_SU8, MUL_UU8,
      //MUL_SS8_CHK, MUL_US8_CHK, MUL_SU8_CHK, MUL_UU8_CHK,

      DIV_R4, DIV_R8,
      DIV_SS4, DIV_US4, DIV_SU4, DIV_UU4,
      //DIV_SS4_CHK, DIV_US4_CHK, DIV_SU4_CHK, DIV_UU4_CHK,
      DIV_SS8, DIV_US8, DIV_SU8, DIV_UU8,
      //DIV_SS8_CHK, DIV_US8_CHK, DIV_SU8_CHK, DIV_UU8_CHK,

      MOD_SS4, MOD_US4, MOD_SU4, MOD_UU4,
      //MOD_SS4_CHK, MOD_US4_CHK, MOD_SU4_CHK, MOD_UU4_CHK,
      MOD_SS8, MOD_US8, MOD_SU8, MOD_UU8,
      //MOD_SS8_CHK, MOD_US8_CHK, MOD_SU8_CHK, MOD_UU8_CHK,

      AND_I4, AND_I8,  // bitwise and
      OR_I4,  OR_I8,   // bitwise or
      XOR_I4, XOR_I8,  // bitwise xor     

      SHL_I4, SHL_I8,  // shift left  b=pop() a = pop() push(a << b)
      SHR_I4, SHR_I8,  // shift right b=pop() a = pop() push(a >> b)

      EQ_SS4, EQ_SU4, EQ_US4, EQ_UU4,     // b=pop() a=pop() push(a == b)
      EQ_SS8, EQ_SU8, EQ_US8, EQ_UU8,
      NEQ_SS4, NEQ_SU4, NEQ_US4, NEQ_UU4, // b=pop() a=pop() push(a != b)
      NEQ_SS8, NEQ_SU8, NEQ_US8, NEQ_UU8,

      LT_SS4, LT_SU4, LT_US4, LT_UU4,     // b=pop() a=pop() push(a < b)
      LT_SS8, LT_SU8, LT_US8, LT_UU8,
      GT_SS4, GT_SU4, GT_US4, GT_UU4,     // b=pop() a=pop() push(a > b)
      GT_SS8, GT_SU8, GT_US8, GT_UU8,
      LEQ_SS4, LEQ_SU4, LEQ_US4, LEQ_UU4, // b=pop() a=pop() push(a <= b)
      LEQ_SS8, LEQ_SU8, LEQ_US8, LEQ_UU8,
      GEQ_SS4, GEQ_SU4, GEQ_US4, GEQ_UU4, // b=pop() a=pop() push(a >= b)
      GEQ_SS8, GEQ_SU8, GEQ_US8, GEQ_UU8,

      // unary operations (promote types if necessary)
      
      NEG_S4, NEG_U4, NEG_R4, NEG_R8, // a=pop() push(-a)   
      COMPL_S4, COMPL_U4,             // a=pop() push(~a)
      INV_S4, INV_U4,                 // a=pop() push(!a)

      // Misc.

      PRINT,  // a=pop() print a to stdout
      EXIT,   // exit(0)
    };
  };
}

#endif
