// $Id: OSL_RecStream.H 5279 2007-06-23 02:49:08Z mburo $

// (c) Michael Buro, mburo@cs.ualberta.ca, licensed under the GPL

/** This class handles nested input streams which can either be
    C-FILEs or C-strings
*/

/* code inspired by psl
    PLIB - A Suite of Portable Game Libraries
    Copyright (C) 1998,2002  Steve Baker
*/

#ifndef OSL_RecStream_H
#define OSL_RecStream_H
 
#include "OSL.H"

namespace OSL {

  class Stream;
  
  class RecStream {

  public:

    /** push named read context */
    void push(FILE *fd, const std::string &name);
    void push(const uint1 *memptr, const std::string &name);

    /** remove current read context */
    void pop();

    /** get current data */
    std::string get_name() const;
    sint4       get_line_num() const;
    size_t      get_stack_size() const;

    /** read character from current stream 
	returns < 0 on EOF
	throws Exception on I/O error
    */
    sint4 get_char();

    /** put character back */    
    void put_char(sint4 c);

    /** get+put */
    sint4 peek();

    /** thrown by various functions */
    struct EmptyStackException : public Exception {};
    
    struct ReadErrorException : public Exception {
      ReadErrorException(const std::string &msg_) throw() : Exception(msg_) {}
    };
  
  protected:

    std::stack<Stream*> sstack;
  };
  
}

#endif
