// $Id: VarTable.C 5279 2007-06-23 02:49:08Z mburo $ 

// (c) Jonathan Yip, licensed under the GPL


#include "VarTable.H"

VarTable::VarTable() 
{
  m_func = false;
}

VarTable::~VarTable()
{
}

void VarTable::add_usable_block()
{
  if (m_func) m_func_loc.push(m_func_vars.size());
  else m_loc.push(m_vars.size());
} 

void VarTable::add_func_block()
{
  m_func = true;
  m_func_params.clear();
  m_func_param_objs.clear();
  m_func_vars.clear();
  m_func_objs.clear();
}

void VarTable::del_func_block()
{
  m_func = false;
  m_func_params.clear();
  m_func_param_objs.clear();
  m_func_vars.clear();
  m_func_objs.clear();
}

void VarTable::del_usable_block()
{
  uint4 index;
  
  if (m_func) {
    index = m_func_loc.top();
    m_func_loc.pop();

    while(m_func_vars.size() != index) m_func_vars.pop_back();
  }
  else {
    index = m_loc.top();
    m_loc.pop();
    
    while (m_vars.size() != index) m_vars.pop_back();
  }
}

Mem::mem_types VarTable::find_variable(const string &var_name,
				       sint4& loc,
				       bool& func_var)
{
  // Order of for loops important!
  for (sint4 i = m_func_vars.size()-1; i >= 0; i--)
    if (m_func_vars[i].first == var_name) {
      loc = i;
      func_var = true;
      return m_func_vars[i].second;
    }

  for (sint4 i = m_func_params.size()-1; i >= 0; i--)
    if (m_func_params[i].first == var_name) {
      loc = -(i+1);
      cerr << "WHY " << loc << endl;
      func_var = true;
      return m_func_params[i].second;
    }

  for (sint4 i = m_vars.size()-1; i >= 0; i--)
    if (m_vars[i].first == var_name) {
      loc = i;
      func_var = false;
      return m_vars[i].second;
    }

  if (m_func) {
    func_var = true;
    loc = m_func_vars.size();
  }
  else {
    func_var = false;
    loc = m_vars.size();
  }
  return Mem::UNKNOWN;
}

bool VarTable::exists_var(const string &var_name, uint4 &loc)
{
  if (m_func) {
    if (!m_func_loc.empty()) 
      for (uint4 i = m_func_loc.top(); i < m_func_vars.size(); ++i)
	if (m_func_vars[i].first == var_name) return true; 
    loc = m_func_vars.size();
  }
  else {
    if (!m_loc.empty()) 
      for (uint4 i = m_loc.top(); i < m_vars.size(); ++i)
	if (m_vars[i].first == var_name) return true; 
    loc = m_vars.size();
  }
  
  return false;
}

void VarTable::set_var_type(const string &var_name, 
			    const Mem::mem_types type,
			    bool func_var)
{
  if (func_var) m_func_vars.push_back( VarInfo(var_name, type) );
  else m_vars.push_back( VarInfo(var_name, type) );
}

void VarTable::set_obj_var(const string &var_name,
			   const string &obj_name,
			   bool func_var)
{
  // Order of statements is improtant!
  if (func_var) {
    m_func_objs[m_func_vars.size()] = obj_name;
    m_func_vars.push_back( VarInfo(var_name, Mem::OBJECT) );
  }
  else {
    m_objs[m_vars.size()] = obj_name;
    m_vars.push_back( VarInfo(var_name, Mem::OBJECT) );
  }
}

void VarTable::set_var_param(const string &var_name,
			     const Mem::mem_types type)
{
  m_func_params.push_back( VarInfo(var_name, type) );
}

void VarTable::set_obj_param(const string &var_name,
			     const string &obj_name)
{
  // Order of statements is important!
  m_func_param_objs[m_func_params.size()] = obj_name;
  m_func_params.push_back( VarInfo(var_name, Mem::OBJECT) );
}

string VarTable::get_obj(sint4 loc, bool func_var)
{
  if (loc < 0) {
    loc = -loc - 1;
    return m_func_param_objs[loc];
  }
  if (func_var) return m_func_objs[loc];
  else return m_objs[loc];
}

uint4 VarTable::get_cur_block()
{
  if (m_func) return m_func_loc.size();
  else return m_loc.size();
}

void FuncTable::set_func_type(const string &func_name, Mem::mem_types type)
{
  m_funcs[func_name] = type;
}

Mem::mem_types FuncTable::find_function(const string &func_name)
{
  FIND(itr, m_funcs, func_name);
  if (itr == m_funcs.end())
    return Mem::UNKNOWN;

  return (*itr).second;
}
