#ifndef GMP_HELPER_H
#define GMP_HELPER_H

#include "Global.h"
#include "gmpxx.h"

// exact rational number
typedef mpq_class Exact;

// limited rational number
struct LimRat
{
  sint8 num, den;
};


#if 0
// not needed for gmp 4.2+
//copied from gmp source
extern size_t geo_mpz_size(mpz_srcptr op)
{
  return __GMP_ABS (op->_mp_size);
}


extern mp_limb_t geo_mpz_getlimbn (mpz_srcptr __gmp_z, mp_size_t __gmp_n)
{
  mp_limb_t  __gmp_result = 0;
  if (__GMP_LIKELY (__gmp_n >= 0 && __gmp_n < __GMP_ABS (__gmp_z->_mp_size)))
    __gmp_result = __gmp_z->_mp_d[__gmp_n];
  return __gmp_result;
}
#endif


#if 0 && defined(__i386__)

// only 10% faster, and broken on x86_64?! perhaps insufficient clobber list

// fast 32x32->64 bit signed multiplication on i386
inline sint8 muls32(sint4 a, sint4 b)
{
  // to avoid type-punned warnings
  sint4 parts[2];

  //std::cout << "muls " << u.p4 << " " << u.p4+1 << std::endl;
  
  __asm__ ("imull %0\n\t"
           : "=d" (parts[1]), "=a" (parts[0])
           : "a" (a), "0" (b)
           : "memory"
           );
  return *(sint8*)parts;
}

// fast 32x32->64 bit unsigned multiplication on i386
inline uint8 mulu32(uint4 a, uint4 b)
{
  // to avoid type-punned warnings
  uint4 parts[2];

  //std::cout << "mulu " << u.p4 << " " << u.p4+1 << std::endl;
  
  __asm__ ("mull %0\n\t"
           : "=d" (parts[1]), "=a" (parts[0])
           : "a" (a), "0" (b)
           : "memory"
           );
  return *(uint8*)parts;
}

#else

inline sint8 muls32(sint4 a, sint4 b)
{
  return (sint8)a * (sint8)b;
}

inline sint8 mulu32(uint4 a, uint4 b)
{
  return (uint8)a * (uint8)b;
}

#endif



/** @return index of highest 1 bit, or -1 if none exists */
extern int highestBitSet(sint4 x);
/** @return index of highest 1 bit, or -1 if none exists */
extern int highestBitSet(uint4 x);
/** @return index of highest 1 bit, or -1 if none exists */
extern int highestBitSet(uint8 x);
/** @return index of highest 1 bit, or -1 if none exists */
extern int highestBitSet(unsigned long x);

/** @return true iff x fits in uint8 or sint8 */
extern bool mpz_fits_uint8_p(mpz_srcptr x);
extern bool mpz_fits_sint8_p(mpz_srcptr x);

// assumes mpz_fits_uint8_p is true
extern uint8 mpz_get_uint8(mpz_srcptr x);
// assumes mpz_fits_sint8_p is true  
extern sint8 mpz_get_sint8(mpz_srcptr x);

extern void mpz_set_uint8(mpz_ptr dest, const uint8& val);
extern void mpz_set_sint8(mpz_ptr dest, const sint8& val);

// e1 * e2 without gcd
extern Exact mult(const Exact& e1, const Exact& e2);
// e1 + e2 without gcd
extern Exact add(const Exact& e1, const Exact& e2);
// e1 - e2 without gcd
extern Exact sub(const Exact& e1, const Exact& e2);

extern bool convert(const LimRat& a, Exact& b);
extern bool convert(const Exact& a, LimRat& b);

extern void gmp_helper_test();





/** returns index of highest 1 bit or -1 if there is none */
inline int highestBitSet(sint4 x)
{
  assert(sizeof(x) == 4);  
  int c = 0;
  if (x & 0xffff0000) { x >>= 16; c += 16; }
  if (x & 0xff00)     { x >>= 8;  c += 8;  }
  if (x & 0xf0)       { x >>= 4;  c += 4;  }
  if (x & 0xc)        { x >>= 2;  c += 2;  }
  if (x & 0x2)        { x >>= 1;  c += 1;  }
  if (x & 1) return c;
  return -1; // no 1
}

/** returns index of highest 1 bit or -1 if there is none */
inline int highestBitSet(uint4 x)
{
  assert(sizeof(x) == 4);
  int c = 0;
  if (x & 0xffff0000) { x >>= 16; c += 16; }
  if (x & 0xff00)     { x >>= 8;  c += 8;  }
  if (x & 0xf0)       { x >>= 4;  c += 4;  }
  if (x & 0xc)        { x >>= 2;  c += 2;  }
  if (x & 0x2)        { x >>= 1;  c += 1;  }
  if (x & 1) return c;
  return -1; // no 1
}



/** returns index of highest 1 bit or -1 if there is none */
inline int highestBitSet(sint8 x)
{
  assert(sizeof(x) == 8);  
  int c = 0;
  if (x & 0xffffffff00000000LL) { x >>= 32; c += 32; }
  if (x & 0xffff0000)           { x >>= 16; c += 16; }
  if (x & 0xff00)               { x >>= 8;  c += 8;  }
  if (x & 0xf0)                 { x >>= 4;  c += 4;  }
  if (x & 0xc)                  { x >>= 2;  c += 2;  }
  if (x & 0x2)                  { x >>= 1;  c += 1;  }
  if (x & 1) return c;
  return -1; // no 1
}

/** returns index of highest 1 bit or -1 if there is none */
inline int highestBitSet(uint8 x)
{
  assert(sizeof(x) == 8);  
  int c = 0;
  if (x & 0xffffffff00000000LL) { x >>= 32; c += 32; }
  if (x & 0xffff0000)           { x >>= 16; c += 16; }
  if (x & 0xff00)               { x >>= 8;  c += 8;  }
  if (x & 0xf0)                 { x >>= 4;  c += 4;  }
  if (x & 0xc)                  { x >>= 2;  c += 2;  }
  if (x & 0x2)                  { x >>= 1;  c += 1;  }
  if (x & 1) return c;
  return -1; // no 1
}


/*
inline
extern int highestBitSet(unsigned long x)
{
  if (sizeof(x) == 8) return highestBitSet((uint8)x);
  else                return highestBitSet((uint4)x);
}*/
#endif
