// exact in-circle test 
// modified by Kai

// Uses Boost interval arithmetic to quickly decide whether the
// in-circle determinant is != 0. If that fails, falls back to exact
// rational computation (GMP)

// (c) 2008 Michael Buro, licensed under GPLv3

#include "Geometry.h"
#include <boost/timer.hpp>




using namespace std;

int Geometry::appCount=0;
int Geometry::exactCount=0;
const Geometry::coord_t Geometry::zero;

Exact Geometry::p1xe;
Exact Geometry::p1ye ;
Exact Geometry::p2xe ;
Exact Geometry::p2ye;
Exact Geometry::p3xe ;
Exact Geometry::p3ye ;
Exact Geometry::pxe ;
Exact Geometry::pye ;
//Exact Geometry::varPool[Geometry::poolSize];
//int Geometry::poolIndex=0;

//
//Exact* Geometry::getVar()
//{
//    Geometry::poolIndex++;
//    
//    return &Geometry::varPool[Geometry::poolIndex % (Geometry::poolSize-1)];
//    
//
//}
//
//void Geometry::setVarPool()
//{
//    for(int i =0; i <poolSize;i++)
//	{
//        coord_t x;
//        Exact xe;
//        RatToExact(x,&xe);
//
//        Geometry::varPool[i] =xe;
//		
//		
//	}
//
//}

#if 0

// old version using GCD, 2+ times slower than without in this application!

// returns 0 if all points lie on circle spanned by a b c, +1 if d is
// inside, -1 if outside assuming ccw ordering of a b c
int Geometry::exactInCircleOld(const Exact& p1x, const Exact& p1y, 
                            const Exact& p2x, const Exact& p2y,
                            const Exact& p3x, const Exact& p3y, 
                            const Exact& px, const Exact& py)
{
  const Exact adx = p1x - px;
  const Exact bdx = p2x - px;
  const Exact cdx = p3x - px;
  const Exact ady = p1y - py;
  const Exact bdy = p2y - py;
  const Exact cdy = p3y - py;
  
  Exact bdxcdy = bdx * cdy;
  Exact cdxbdy = cdx * bdy;
  Exact alift = adx * adx + ady * ady;

  Exact cdxady = cdx * ady;
  Exact adxcdy = adx * cdy;
  Exact blift = bdx * bdx + bdy * bdy;

  Exact adxbdy = adx * bdy;
  Exact bdxady = bdx * ady;
  Exact clift = cdx * cdx + cdy * cdy;

  Exact det =
    alift * (bdxcdy - cdxbdy)
    + blift * (cdxady - adxcdy)
    + clift * (adxbdy - bdxady);

  if (det > 0) return +1;
  if (det < 0) return -1;
  return 0;
}
  



/** @return true iff line segments p1-p2 and p3-p4 intersect
*/
bool Geometry::exactIntersection(const IntPoint& p1, const IntPoint& p2,
                                 const IntPoint& p3, const IntPoint& p4)
{
  assert(withinRange(p1, p2, p3, p4));
  
  sint8 dx43 = p4.x-p3.x;
  sint8 dy43 = p4.y-p3.y;
  sint8 dx12 = p1.x-p2.x;
  sint8 dy12 = p1.y-p2.y;
  sint8 dx42 = p4.x-p2.x;
  sint8 dy42 = p4.y-p2.y;    

  sint8 d = dy43*dx12 - dx43*dy12;
  if (d == 0) return false; // parallel
    
  sint8 t = dy43*dx42 - dx43*dy42;

  if (d > 0) {
    if (t < 0 || t > d) return false; // outside [p1,p2]
  } else {
    if (t > 0 || t < d) return false; // outside [p1,p2]
  }

  sint8 s = dy42*dx12 - dx42*dy12;

  if (d > 0) {
    if (s < 0 || s > d) return false; // outside [p3,p4]
  } else {
    if (s > 0 || s < d) return false; // outside [p3,p4]      
  }

  return true;
}



/** @return true iff line segments p1-p2 and p3-p4 intersect
    not tested, not optimized
*/
bool Geometry::exactIntersection(coord_t2& p1x,coord_t2& p1y, 
                                coord_t2& p2x,coord_t2& p2y,
                                coord_t2& p3x,coord_t2& p3y, 
                                coord_t2& p4x,coord_t2& p4y)
{
  coord_t2 dx43 = p4x-p3x; // <= 2 M^2
  coord_t2 dy43 = p4y-p3y;
  coord_t2 dx12 = p1x-p2x;
  coord_t2 dy12 = p1y-p2y;
  coord_t2 dx42 = p4x-p2x;
  coord_t2 dy42 = p4y-p2y;    

  coord_t2 d = dy43*dx12 - dx43*dy12; // <= 8 M^4
  if (d == 0) return false; // parallel
    
  coord_t2 t = dy43*dx42 - dx43*dy42; // <= 8 M^4

  if (d > 0) {
    if (t < 0 || t > d) return false; // outside [p1,p2]
  } else {
    if (t > 0 || t < d) return false; // outside [p1,p2]
  }

  coord_t2 s = dy42*dx12 - dx42*dy12;

  if (d > 0) {
    if (s < 0 || s > d) return false; // outside [p3,p4]
  } else {
    if (s > 0 || s < d) return false; // outside [p3,p4]      
  }
  
  return true;
}

/** @return true iff line segments p1-p2 and p3-p4 intersect
    not tested, not optimized
*/
bool Geometry::exactIntersection(coord_t& p1x,coord_t& p1y, 
                                coord_t& p2x,coord_t& p2y,
                                coord_t& p3x,coord_t& p3y, 
                                coord_t& p4x,coord_t& p4y)
{
  coord_t dx43 = p4x-p3x; // <= M
  coord_t dy43 = p4y-p3y;
  coord_t dx12 = p1x-p2x;
  coord_t dy12 = p1y-p2y;
  coord_t dx42 = p4x-p2x;
  coord_t dy42 = p4y-p2y;    

  coord_t d = dy43*dx12 - dx43*dy12; // <= 2 M^2
  if (d == 0) return false; // parallel
    
  coord_t t = dy43*dx42 - dx43*dy42; // <= 2 M^2

  if (d > 0) {
    if (t < 0 || t > d) return false; // outside [p1,p2]
  } else {
    if (t > 0 || t < d) return false; // outside [p1,p2]
  }

  coord_t s = dy42*dx12 - dx42*dy12;

  if (d > 0) {
    if (s < 0 || s > d) return false; // outside [p3,p4]
  } else {
    if (s > 0 || s < d) return false; // outside [p3,p4]      
  }
  
  return true;
}

/** @return true iff line segments p1-p2 and p3-p4 intersect
    not tested, not optimized
*/
bool Geometry::exactIntersection(const Exact& p1x,const Exact& p1y, 
                                const Exact& p2x,const Exact& p2y,
                                const Exact& p3x,const Exact& p3y, 
                                const Exact& p4x,const Exact& p4y)
{
  
    
  Exact dx43 = p4x-p3x; // <= M
  Exact dy43 = p4y-p3y;
  Exact dx12 = p1x-p2x;
  Exact dy12 = p1y-p2y;
  Exact dx42 = p4x-p2x;
  Exact dy42 = p4y-p2y;    

  Exact d = dy43*dx12 - dx43*dy12; // <= 2 M^2
  if (d == 0) return false; // parallel
    
  Exact t = dy43*dx42 - dx43*dy42; // <= 2 M^2

  if (d > 0) {
    if (t < 0 || t > d) return false; // outside [p1,p2]
  } else {
    if (t > 0 || t < d) return false; // outside [p1,p2]
  }

  Exact s = dy42*dx12 - dx42*dy12;

  if (d > 0) {
    if (s < 0 || s > d) return false; // outside [p3,p4]
  } else {
    if (s > 0 || s < d) return false; // outside [p3,p4]      
  }
  
  return true;
}

#endif




 



