#if 0

#include "rational.h"
#include <math.h>
#include <stdlib.h>

using namespace std;

/**
 * Default Constructor
 * Creates a rational number with value 1
 */ 
template <class T>
Rational<T>::Rational(): _numerator(0), _denominator(1)
{}

/**
 * Creates a rational number equivalent to the integer value of 
 * numerator.  In general, all rational numbers of the form x/1 
 * are equivalent to the corresponding integer x.
 */
template <class T>
Rational<T>::Rational(T numerator):_numerator(numerator), _denominator(1) 
{
	
}

/**
 * Creates a rational number with the given
 * numerator and denominator
 */
template <class T>
Rational<T>::Rational(T numerator, T denominator): _numerator(numerator), _denominator(denominator) 
{
	//assert(_denominator != 0) 
     normalize();
}





/**
 * The normalize function moves the sign to numerator 
 * and removes common divisors from the numerator and 
 * denominator
 */
template <class T>
void Rational<T>::normalize() 
{
	
	if (_denominator < 0) {
		_numerator = -_numerator;
		_denominator = -_denominator;
	}
	// the return of gcd may be negative, so we make sure
	// we send only positive numbers to the function
      T theGCD = gcd( (_numerator>=0 ? _numerator : -_numerator) , (_denominator>=0 ? _denominator : -_denominator) );
	_numerator = _numerator  / theGCD;
	_denominator = _denominator / theGCD;
}


/**
 * Assignment operator 
 */
template <class T>
Rational<T>& Rational<T>::operator=(const Rational<T>& r) 
{

     _numerator = r._numerator;
     _denominator = r._denominator;
     normalize();
     return *this;
}


/**
* Inversion function
*/
template <class T>
Rational<T> Rational<T>::invert( ) const 
{
     
    return Rational<T>( _denominator, _numerator);


}



/**
 * Exponentiation function
 */
/*
Rational Rational::power(T power) const 
{
	int numerator;
	int denominator;
	if (power < 0) {			
		numerator = _denominator;
		denominator = _numerator;
		power *= -1;
	} else {
		numerator = _numerator;
		denominator = _denominator;
	}
	
	// on some platforms, the function "pow" returns a double
	// so we perform a cast to insure no warnings during compilation
	return(Rational((int) pow(numerator,power), (int) pow(denominator,power)));
}
*/

/**
 * Pretty Prints a rational number
 * 
 * We return a reference to the stream so that it's
 * possible syntactically to write statements such as
 * Rational r1;
 * cout << r << "\tFOO\n" << endl;
 * 
 * Rational r2;
 * cout << r2++ << r2++ << r2++ << endl;
 * We would see:
 *
 * (3,0)(2,0)(1,0)
 *
 * and r would have final value (4,0) (because of the post-decrement)
 *
 * There's a lot going on here, but don't worry, we'll go over it
 * later in class.  For now, you should be able to copy this function
 * and mutate it so that it writes the real and imaginary components
 * of a complex number.
 * 
 */
template <class T>
ostream& operator<<(ostream& out, const Rational<T>& r) 
{
	out << "(" << r._numerator << "/" << r._denominator << ")";
	return out ;
}

/**
 * Adding two rational numbers a/b and c/d is defined as
 * (a*d + c*b) / (b*d)
 *
 */
template <class T>
const Rational<T> Rational<T>::operator+(const Rational<T>& r) const 
{
	return  Rational<T>( (_numerator * r._denominator) + (r._numerator * _denominator), _denominator * r._denominator) ;
}


/**
 * Subtracting two rational numbers a/b and c/d is defined as
 * (a*d - c*b) / (b*d)
 * or
 * r1 + -1*r2  (where r1 = a/b and r2 = c/d)
 */
template <class T>
const Rational<T> Rational<T>::operator-(const Rational<T>& r) const 
{
	return (*this) + (Rational<T>(-1) * r);
}

/**
 * Multiplying two rational numbers a/b and c/d is defined as
 * a*c / b*d
 */
template <class T>
const Rational<T> Rational<T>::operator*(const Rational<T>& r) const 
{

	return Rational<T>(_numerator * r._numerator, _denominator * r._denominator) ;
}

/**
 * Dividing Rational numbers a/b and c/d is defined as
 * a*d / b*c
 * or
 * r1 * r2^(-1) (mulitiplication of the inverse)
 * where r1 = a/b and r2 = c/d
 */
template <class T>
const Rational<T> Rational<T>::operator/(const Rational<T>& r) const 	
{
	Rational<T> divisor = r.invert();
	//assert(divisor._denominator != 0);

	return (*this) * divisor ;
}

/**
 * Two rational numbers are equal if their cross product is equal.
 * That is, if the numerator of this number times the denominator
 * of r is equal to the numerator of r times the denominator of this
 * then the two rationals are equal
 */
template <class T>
const bool Rational<T>::operator==(const Rational& r) const 
{
	return (_numerator * r._denominator) == (_denominator * r._numerator);
}

/**
 * Postfix uniary plus operation
 */
template <class T>
const Rational<T> Rational<T>::operator++(int) 
{
	// store the old value
	Rational<T> store = Rational<T>(*this);
	// perform the update
	_numerator += _denominator;
	// normalize it
	normalize();
	// return the stored rational number
	return store;
}
		
/**
 * Prefix uniary plus operation
 */
template <class T>
const Rational<T>& Rational<T>::operator++() { 
	_numerator += _denominator;
	normalize();
	return *this;
}


/**
 * Postfix uniary minux operation
 */
template <class T>
const Rational<T> Rational<T>::operator--(int) {
	// store the old value
	Rational<T> store = Rational<T>(*this);
	// perform the update
	// note that _numerator -= _denominator is shorthand
	// for       _numerator = _numerator - denominator
	_numerator -= _denominator;
	// normalize it
	normalize();
	// return the stored rational number
	return store;
}
	
/**
 * Prefix uniary minux operation
 */
template <class T>
const Rational<T>& Rational<T>::operator--() { 
	_numerator -= _denominator;
	normalize();
	return *this;
}

/**
 * uniary negation operation
 */
template <class T>
const Rational<T>& Rational<T>::operator-()
{
     _numerator = -_numerator;
     normalize();
     return *this;

}

/**
 * The less than operator
 */
template <class T>
const bool Rational<T>::operator<(const Rational<T>& r) const {
	return _numerator * r._denominator < _denominator * r._numerator;
}

/**
 * The less than or equal operator.
 */
template <class T>
const bool Rational<T>::operator<=(const Rational<T>& r) const {
	return (_numerator * r._denominator < _denominator * r._numerator) || (_numerator * r._denominator == _denominator * r._numerator) ;
}

/**
 * The greater than operator (defined in terms of <=)
 */
template <class T>
const bool Rational<T>::operator>(const Rational<T>& r) const {
	return _numerator * r._denominator > _denominator * r._numerator;
}

/**
 * The greater than or equal opreator (defined in terms of <)
 */
template <class T>
const bool Rational<T>::operator>=(const Rational<T>& r) const {
	return (_numerator * r._denominator > _denominator * r._numerator) || (_numerator * r._denominator == _denominator * r._numerator) ;
}
	


/**
 * Returns the greatest common divisor
 * of a and b
 */
template <class T>
T Rational<T>::gcd(T a, T b) 
{
	if (a == 0) 
      {
		return b;
      }
      else if (b == 0) 
      {
          return a;
      }
     
      T r = a % b;
  
      while (r != 0) 
      {
  	    a = b;
  	    b = r;
          r = a % b;
      }
  return b;
}

/**
 * Checks if the rational number is an integer
 * 
 */
template <class T>
bool Rational<T>::isInteger()
{
     return _denominator == 1;
}


/**
 * Convert the rational number to a floating point number
 * 
 */
template <class T>
float Rational<T>::toDouble()
{
     return (float)_numerator/(float)_denominator;

}
#endif
