
/*
Face.cpp

Kai: Definition of Face class
*/

#include "Vertex.h"
#include "Face.h"
#include "Mesh.h"
#include "HalfEdge.h"

Face::Face()
{
  _boundary = false;
  _mark = 0;
}

Face::~Face()
{
}

/**
store the adjacent half edges,
link the adjacent half edges in ccw order,
the function assumes the half edges in ccw order
*/
void Face::setEdges(const std::vector<HalfEdge*>& edges)
{
  _edges = edges;

  for (size_t i = 0 ; i < edges.size()-1; i++) {
    edges[i]->setNext(edges[i+1]);
    edges[i+1]->setPrior(edges[i]);
  }

  edges[0]->setPrior(edges[edges.size()-1]);
  edges[edges.size()-1]->setNext(edges[0]);
}

/**
store the adjacent half edges,
link the adjacent half edges in ccw order 
*/
void Face::setEdges(HalfEdge* e1,HalfEdge* e2, HalfEdge* e3)
{
  V* e1Org = e1->getDestVertex();
  V* e2Org = e2->getDestVertex();
  V* e3Org = e3->getDestVertex();

  C x1,y1,x2,y2,x3,y3;

  x1 = e1Org->getX();
  y1 = e1Org->getY();
  x2 = e2Org->getX();
  y2 = e2Org->getY();
  x3 = e3Org->getX();
  y3 = e3Org->getY();

  if (Geometry::ccw(x1,y1,x2,y2,x3,y3) > 0) { //ccw
    //store adjacent edges in ccw order
    _edges.push_back(e1);
    _edges.push_back(e2);
    _edges.push_back(e3);

		
    //linking the circular ccw edge pointers
    e1->setNext(e2);
    e2->setNext(e3);
    e3->setNext(e1);

    e1->setPrior(e3);
    e3->setPrior(e2);
    e2->setPrior(e1);


  } else {
  
    //store the adjacent half edges in the vector
    _edges.push_back(e1);
    _edges.push_back(e3);
    _edges.push_back(e2);

    //linking the circular ccw edge pointers
    e1->setNext(e3);
    e3->setNext(e2);
    e2->setNext(e1);

    e1->setPrior(e2);
    e2->setPrior(e3);
    e3->setPrior(e1);
  }
}


bool Face::containsVertex(const V* v) const
{
  //boundary face does not contain any vertices
  if (_boundary) {
    return false;
  }

  //loop through the edges and check if v is one of its endpoints
  FORALL (_edges, eit) {
    if ((*eit)->getOrgVertex() == v)
      return true;
  }
 //edge check falls through, v is not adjacent to this face. 
  return false;
}
