#ifndef DCDTPATH_H
#define DCDTPATH_H

// pathfinding based on DCDT
#include <float.h>


#include "PathFace.h"
#include "SearchNode.h"
#include "FunnelDeque.h"
#include "DCDTLite.h"


class DCDTPath
{
public:

  static const int INVALID = -1;
  static const float INFINITY_LEN = FLT_MAX;
  static const int UNABSTRACTED = -1;
  static const float PI;
  

  DCDT& _dcdt;

  typedef DCDT::V V;
  typedef DCDT::H H;
  typedef DCDT::F F;
  typedef PathFace PF;
  
  DCDTPath(DCDT &dcdt) :
    _dcdt(dcdt)
  {

  }

  //generate a random valid x value for testing point location
  float RandomX();
  //generate a random valid y value for testing point location
  float RandomY();

  //return a random value between those given
  float RandomBetween(float min, float max);

  // current channel
  std::vector<H*> currentChannel;

  //the current path
  FPolygon currentPath;
  
  //the current start position
  Point<int> startPoint;
  
  //the current goal position
  Point<int> goalPoint;

  const FPolygon& getPath() const { return currentPath; }

  Polygon<V> getChannelBoundary();

  //abstraction 

  //adds abstraction information to the triangulation
  void abstract();

  //deletes all abstraction information from the triangulation
  void deleteAbstraction();

private:

  //utility functions for TA*
  float pointLineDistance(float x, float y, float x1, float y1, float x2, float y2);
  float orientation(float x1, float y1, float x2, float y2, float x3, float y3);
  bool isAccute(float theta);
  bool inTriangle(F* face, float x, float y);
  float angleBetween(float x1, float y1, float x2, float y2, float x3, float y3);
  bool blocked(H* e);
  void triangleVertices(H* e, float &x1, float &y1, float &x2, float &y2, float &x3, float &y3);
  float length(float x1, float y1, float x2, float y2);
  FPoint closestPointTo(F* face, float x, float y, float r);
  FPoint closestPointOn(float x1, float y1, float x2, float y2, float x, float y, float r);
  float getShortestPath(FPolygon& path, std::vector<H*>& channel, float x1, float y1, float x2, float y2, float r);
  std::vector<H*> constructBaseChannel(SearchNode *goal);

  //utility functions for abstraction
  void calculateWidths(PF* face);
  int degree(PF *face);
  float triangleWidth(HalfEdge *s);
  float searchWidth(float x, float y, HalfEdge *s, float CurrentWidth);
  bool consider(float x, float y, float x1, float y1, float x2, float y2);
  
  bool validEndpoint(F *first, F *second, float x, float y, float r);
  //checks if a unit has to cross the middle of the triangle to enter the path from the point given
  bool validEndpoint(H *s, float x, float y, bool left);
  
  bool validEndpoints(std::vector<H *> & path, float x1, float y1, float x2, float y2, float r);

  
  
  //abstract an acyclic component of the triangulation graph
  void treeAbstract(PF *first, int component);
  //collapse an acyclic portion into the degree-2 root given
  void treeCollapse(PF *root, PF* currentFace, int component);

public:
  //TA*
  bool searchPathBaseFast(float x1, float y1, float x2, float y2, float r);

};



#endif
