//definition of FunnelNode functions 

#include <float.h>
#include <cmath>
#include <iostream>
#include "FunnelDeque.h"

//default constructor - initializes all
//member variables to default values
FunnelNode::FunnelNode()
{
  m_CLeft = 0;
  m_CRight = 0;
  m_CPoint.set(FLT_MIN, FLT_MIN);
}

//constructor - initializes the point member
//variable to the coordinates passed
FunnelNode::FunnelNode(float x, float y)
{
  m_CLeft = 0;
  m_CRight = 0;
  m_CPoint.set(x, y);
}

//constructor - initializes the point member
//variable to the one passed
FunnelNode::FunnelNode(const FPoint& point)
{
  m_CLeft = 0;
  m_CRight = 0;
  m_CPoint = point;
}

//mutator for the left pointer
void FunnelNode::left(FunnelNode *left)
{
  m_CLeft = left;
}

//mutator for the right pointer
void FunnelNode::right(FunnelNode *right)
{
  m_CRight = right;
}

//accessor for the left pointer
FunnelNode *FunnelNode::left()
{
  return m_CLeft;
}

//accessor for the right pointer
FunnelNode *FunnelNode::right()
{
  return m_CRight;
}

//mutator for the point
void FunnelNode::point(const FPoint& point)
{
  m_CPoint = point;
}

//accessor for the point
FPoint FunnelNode::point()
{
  return m_CPoint;
}

//mutator for the point's coordinates
void FunnelNode::point(float x, float y)
{
  m_CPoint.set(x, y);
}

//accessor for the point's X coordinate
float FunnelNode::x()
{
  return m_CPoint.getX();
}

//accessor for the point's Y coordinate
float FunnelNode::y()
{
  return m_CPoint.getY();
}

//definition of FunnelNode functions }

//DJD: definition of FunnelDeque functions {

//constructor - initializes the apex of the funnel
//to the point passed, and the unit radius
FunnelDeque::FunnelDeque(float x, float y, float r)
{
  m_CApex = new FunnelNode(x, y);
  m_CLeft = m_CApex;
  m_CRight = m_CApex;
  m_tApexType = CT_Point;
  m_dRadius = r;
}

//constructor - initializes the apex of the funnel
//to the coordinates passed, and the unit radius
FunnelDeque::FunnelDeque(const FPoint& point, float r)
{
  m_CApex = new FunnelNode(point);
  m_CLeft = m_CApex;
  m_CRight = m_CApex;
  m_tApexType = CT_Point;
  m_dRadius = r;
}

//destructor - deletes any funnel nodes in the deque
FunnelDeque::~FunnelDeque()
{
  FunnelNode *current = m_CLeft;
  while (current != 0) {
    FunnelNode *temp = current;
    current = current->right();
    delete temp;
  }
}

//Adds a new point to the funnel, adds to the existing path if the apex moves
//type = RightTangent => point is being added on left side
//type = LeftTangent => point is being added on right side
//type = Point => point is the end of the channel
void FunnelDeque::add(const FPoint& p, CornerType type, FPolygon& path)
{
  //the new funnel node containing the new point
  FunnelNode *next = new FunnelNode(p);
  //if it's being added on the left side
  //(operations are just reversed for the other side)
  if (type == CT_RightTangent)
    {
      //loop until the point is added to the funnel
      while (true)
        {
          //if the apex is the only point in the funnel,
          //simply add the point to the appropriate side
          if (m_CLeft == m_CRight)
            {
              next->right(m_CApex);
              m_CApex->left(next);
              m_CLeft = next;
              break;
            }
          float x_1, y_1, x_2, y_2;
          CornerType type_1, type_2;
          //if there are no points on the left side of the funnel,
          //get the wedge going to the opposite side of the apex
          if (m_CLeft == m_CApex)
            {
              x_1 = m_CLeft->x();
              y_1 = m_CLeft->y();
              x_2 = m_CLeft->right()->x();
              y_2 = m_CLeft->right()->y();
              type_1 = m_tApexType;
              type_2 = CT_LeftTangent;
            }
          //otherwise get the wedge on this side of the apex
          else
            {
              x_2 = m_CLeft->x();
              y_2 = m_CLeft->y();
              x_1 = m_CLeft->right()->x();
              y_1 = m_CLeft->right()->y();
              type_2 = CT_RightTangent;
              if (m_CLeft->right() == m_CApex)
                {
                  type_1 = m_tApexType;
                }
              else
                {
                  type_1 = CT_RightTangent;
                }
            }
          //calculate the angle associated with this wedge
          float wedge = angle(x_1, y_1, type_1, x_2, y_2, type_2);
          //also calculate the distance between these two points
          float length1 = distance(x_1, y_1, x_2, y_2);
          //and that to the new point (special case for the modofied funnel algorithm)
          float length2 = distance(x_1, y_1, p.getX(), p.getY());
          //now get the wedge leading to the new point
          x_1 = m_CLeft->x();
          y_1 = m_CLeft->y();
          if (m_CApex == m_CLeft)
            {
              type_1 = m_tApexType;
            }
          else
            {
              type_1 = CT_RightTangent;
            }
          x_2 = p.getX();
          y_2 = p.getY();
          type_2 = type;
          //and the angle associated with it
          float toPoint = angle(x_1, y_1, type_1, x_2, y_2, type_2);
          //compare the two angles
          float diff = wedge - toPoint;
          diff += (diff <= -M_PI) ? 2.0f * M_PI : (diff > M_PI) ? -2.0f * M_PI : 0.0f;
          //if the existing wedge is counterclockwise of that to the point
          if (diff < 0.0f)
            {
              //add the new point on this end of the funnel
              m_CLeft->left(next);
              next->right(m_CLeft);
              m_CLeft = next;
              break;
            }
          //if it is clockwise,
          else
            {
              //check if we are popping the apex
              if (m_CLeft == m_CApex)
                {
                  //if the point being added is closer than the one being popped,
                  if (length2 < length1)
                    {
                      //make the new point the apex instead of the one on the opposite side
                      //(special case for the modified funnel algorithm)
                      float ang = angle(m_CApex->x(), m_CApex->y(), m_tApexType, p.getX(), p.getY(), type);
                      addPoint(m_CApex->x(), m_CApex->y(), m_tApexType, ang, path);
                      addPoint(p.getX(), p.getY(), type, ang, path);
                      next->right(m_CApex->right());
                      m_CApex->right()->left(next);
                      delete m_CApex;
                      m_CApex = next;
                      m_CLeft = next;
                      m_tApexType = type;
                      break;
                    }
                  //otherwise,
                  else
                    {
                      //move the apex to the right and add the segment between
                      //the old and new apexes to the path so far
                      //float angle = Angle(m_CApex->X(), m_CApex->Y(), m_tApexType, m_CApex->Right()->X(), m_CApex->Right()->Y(), CornerType::LeftTangent);
                      float ang = angle(m_CApex->x(), m_CApex->y(), m_tApexType, m_CApex->right()->x(), m_CApex->right()->y(), FunnelDeque::CT_LeftTangent);
                      addPoint(m_CApex->x(), m_CApex->y(), m_tApexType, ang, path);
                      //AddPoint(m_CApex->Right()->X(), m_CApex->Right()->Y(), CornerType::LeftTangent, ang, path);
                      addPoint(m_CApex->right()->x(), m_CApex->right()->y(), FunnelDeque::CT_LeftTangent, ang, path);
                      m_CApex = m_CApex->right();
                      m_tApexType = CT_LeftTangent;
                    }
                }
              //pop the leftmost point off of the funnel
              FunnelNode *temp = m_CLeft;
              m_CLeft = m_CLeft->right();
              m_CLeft->left(0);
              delete temp;
            }
        }
    }
  //otherwise, if adding a point on the right side,
  //continue as before, with sides reversed
  //adding a point also ends up here - 
  //we do this to pop off all the necessary points off of
  //the right side of the funnel, so at the end we can just
  //add to the path the points between the apex and the
  //right side of the funnel (done at the end)
  else
    {
      while (true)
        {
          if (m_CLeft == m_CRight)
            {
              next->left(m_CApex);
              m_CApex->right(next);
              m_CRight = next;
              break;
            }
          float x_1, y_1, x_2, y_2;
          CornerType type_1, type_2;
          if (m_CRight == m_CApex)
            {
              x_1 = m_CRight->x();
              y_1 = m_CRight->y();
              x_2 = m_CRight->left()->x();
              y_2 = m_CRight->left()->y();
              type_1 = m_tApexType;
              type_2 = CT_RightTangent;
            }
          else
            {
              x_2 = m_CRight->x();
              y_2 = m_CRight->y();
              x_1 = m_CRight->left()->x();
              y_1 = m_CRight->left()->y();
              type_2 = CT_LeftTangent;
              if (m_CRight->left() == m_CApex)
                {
                  type_1 = m_tApexType;
                }
              else
                {
                  type_1 = CT_LeftTangent;
                }
            }
          float wedge = angle(x_1, y_1, type_1, x_2, y_2, type_2);
          float length1 = distance(x_1, y_1, x_2, y_2);
          float length2 = distance(x_1, y_1, p.getX(), p.getY());
          x_1 = m_CRight->x();
          y_1 = m_CRight->y();
          if (m_CApex == m_CRight)
            {
              type_1 = m_tApexType;
            }
          else
            {
              type_1 = CT_LeftTangent;
            }
          x_2 = p.getX();
          y_2 = p.getY();
          type_2 = type;
          float toPoint = angle(x_1, y_1, type_1, x_2, y_2, type_2);
          float diff = wedge - toPoint;
          diff += (diff <= -M_PI) ? 2.0f * M_PI : (diff > M_PI) ? -2.0f * M_PI : 0.0f;
          if (diff < 0.0f)
            {
              if (m_CRight == m_CApex)
                {
                  if (length2 < length1)
                    {
                      float ang = angle(m_CApex->x(), m_CApex->y(), m_tApexType, p.getX(), p.getY(), type);
                      addPoint(m_CApex->x(), m_CApex->y(), m_tApexType, ang, path);
                      addPoint(p.getX(), p.getY(), type, ang, path);
                      next->left(m_CApex->left());
                      m_CApex->left()->right(next);
                      delete m_CApex;
                      m_CApex = next;
                      m_CRight = next;
                      m_tApexType = type;
                      break;
                    }
                  else
                    {
                      //float angle = Angle(m_CApex->X(), m_CApex->Y(), m_tApexType, m_CApex->Left()->X(), m_CApex->Left()->Y(), CornerType::RightTangent);
                      float ang = angle(m_CApex->x(), m_CApex->y(), m_tApexType, m_CApex->left()->x(), m_CApex->left()->y(), CT_RightTangent);
                      addPoint(m_CApex->x(), m_CApex->y(), m_tApexType, ang, path);
                      //AddPoint(m_CApex->Left()->X(), m_CApex->Left()->Y(), CornerType::RightTangent, ang, path);
                      addPoint(m_CApex->left()->x(), m_CApex->left()->y(), CT_RightTangent, ang, path);
                      m_CApex = m_CApex->left();
                      m_tApexType = CT_RightTangent;
                    }
                }
              FunnelNode *temp = m_CRight;
              m_CRight = m_CRight->left();
              m_CRight->right(0);
              delete temp;
            }
          else
            {
              m_CRight->right(next);
              next->left(m_CRight);
              m_CRight = next;
              break;
            }
        }
    }
  //this is where the points between the apex
  //and the right side o fthe funnel are added to the path
  //when adding the final point to the channel
  if (type == CT_Point) {
    FunnelNode *Current = m_CApex;
    while (Current != m_CRight)
      {
        float x_1, y_1, x_2, y_2, toPoint;
        CornerType type_1, type_2;
        //type_1 = (Current == m_CApex) ? m_tApexType : CornerType::LeftTangent;
        //type_2 = (Current->Right() == m_CRight) ? CornerType::Point : CornerType::LeftTangent;
        type_1 = (Current == m_CApex) ? m_tApexType : CT_LeftTangent;
        type_2 = (Current->right() == m_CRight) ? CT_Point : CT_LeftTangent;
        x_1 = Current->x();
        y_1 = Current->y();
        Current = Current->right();
        x_2 = Current->x();
        y_2 = Current->y();
        toPoint = angle(x_1, y_1, type_1, x_2, y_2, type_2);
        addPoint(x_1, y_1, type_1, toPoint, path);
        addPoint(x_2, y_2, type_2, toPoint, path);
      }
  }
}

//returns the length of a path found by this funnel algorithm with the current radius
float FunnelDeque::length(const FPolygon& path)
{
  //only paths with an even number of points are valid
  if ((path.size() % 2) != 0) {
    return 0.0f;
  }
  float length = 0.0f;
  //go through the straight segments and total their lengths
  for (size_t i = 0; i < path.size(); i += 2) {
    float xDiff = path[i + 1].getX() - path[i].getX();
    float yDiff = path[i + 1].getY() - path[i].getY();
    length += sqrt(xDiff * xDiff + yDiff * yDiff);
  }
  //go through the curved segments and add their length
  //(the difference of the angles of the
  // surrounding segments, times the unit radius)
  for (size_t i = 1; i + 3 < path.size(); i += 2) {
    float angle1 = atan2(path[i + 1].getY() - path[i].getY(),
                         path[i + 1].getX() - path[i].getX());
    float angle2 = atan2(path[i + 3].getY() - path[i + 2].getY(),
                         path[i + 3].getX() - path[i + 2].getX());
    float diff = angle1 - angle2;
    diff += (diff <= -M_PI) ? 2.0f * M_PI : (diff > M_PI) ? -2.0f * M_PI : 0.0f;
    length += m_dRadius * fabs(diff);
  }
  return length;
}

//returns the angle from x_1, y_1 to x_2, y_2, given the types
float FunnelDeque::angle(float x_1, float y_1, CornerType type_1, float x_2, float y_2, CornerType type_2)
{
  //simply calls the appropriate function based
  //on the types of the two points
  if (type_1 == CT_Point) {
    return pointToTangent(x_1, y_1, x_2, y_2, (type_2 == CT_RightTangent));
  } else if (type_2 == CT_Point) {
    float ang = pointToTangent(x_2, y_2, x_1, y_1, (type_1 == CT_LeftTangent));
    return angleRange(ang + M_PI);
  } else if (type_1 != type_2) {
    return toTangentAlt(x_1, y_1, x_2, y_2, (type_1 == CT_LeftTangent));
  } else {
    return toTangent(x_1, y_1, x_2, y_2);
  }
}

//adds a point to the path given a coordinate, its type,
//and the angle of the segment attached
void FunnelDeque::addPoint(float x, float y, CornerType type, float ang, FPolygon& path)
{
  //if it's a point type, just add it unchanged
  if (type == CT_Point) {
    path.push_back(FPoint(x, y));
  } else {
    //otherwise,
    //move the unit radius from the point to one side or the other
    float theta = angleRange(ang + M_PI / ((type == CT_LeftTangent) ? 2.0f : -2.0f));
    path.push_back(FPoint(x + cos(theta) * m_dRadius, y + sin(theta) * m_dRadius));
  }
}

//takes an angle value and restricts it to the range (-PI, PI]
float FunnelDeque::angleRange(float theta)
{
  return ((theta > M_PI) ? (theta - 2.0f * M_PI) :
          (theta <= -M_PI) ? (theta + 2.0f * M_PI) : theta);
}

//gets the distance between the points (x_1, y_1) and (x_2, y_2)
float FunnelDeque::distance(float x_1, float y_1, float x_2, float y_2)
{
  float x = x_2 - x_1;
  float y = y_2 - y_1;
  return sqrt(x * x + y * y);
}

//gets the angle of the line tangent to two points of the same type
float FunnelDeque::toTangent(float x_1, float y_1, float x_2, float y_2)
{
  return atan2(y_2 - y_1, x_2 - x_1);
}

//gets the angle of the line tangent to a point
//of type LeftTangent and one of type RightTangent
float FunnelDeque::toTangentAlt(float x_1, float y_1, float x_2, float y_2, bool LeftRight)
{
  float h = distance(x_1, y_1, x_2, y_2) * 0.5f; // was / 2.0f;
  float l = sqrt(h * h - m_dRadius * m_dRadius);
  float interior = atan(m_dRadius / l);
  float absolute = atan2(y_2 - y_1, x_2 - x_1);
  if (LeftRight) {
    interior = -interior;
  }
  return angleRange(absolute + interior);
}

//gets the angle of the line going through one point
//and running tangent to another
float FunnelDeque::pointToTangent(float x_1, float y_1, float x_2, float y_2, bool Right)
{
  float h = distance(x_1, y_1, x_2, y_2);
  float l = sqrt(h * h - m_dRadius * m_dRadius);
  float interior = atan(m_dRadius / l);
  float absolute = atan2(y_2 - y_1, x_2 - x_1);
  if (Right) {
    interior = -interior;
  }
  return angleRange(absolute + interior);
}

//prints the funnel (used for debugging)
void FunnelDeque::print()
{
  FunnelNode *current = m_CLeft;

  while (current != 0) {
    std::cout << "\t(" << current->x() << ", " << current->y() << ")";
    if (current == m_CLeft) {
      std::cout << " <-- Left";
    }
    if (current == m_CApex) {
      std::cout << " <-- Apex";
    }
    if (current == m_CRight) {
      std::cout << " <-- Right";
    }
    std::cout << std::endl;
    current = current->right();
  }
}

//prints the path and then the funnel (used for debugging)
void FunnelDeque::print(const FPolygon& path)
{
  std::cout << "Path = {";
  for (size_t i = 0; i < path.size(); i++) {
    FPoint p = path[i];
    std::cout << "(" << p.getX() << ", " << p.getY() << ")";
    if (i < path.size()-1) std::cout << ", ";
  }
  std::cout << "}\n";
  print();
}
