//FunnelDeque.h

//DJD: definition of FunnelNode class

#ifndef FUNNELDEQUE_H
#define FUNNELDEQUE_H

#include "Polygon.h"
#include "Point.h"

//define a floating point value for pi
//#define PI 3.1415926535897932384626433832795f

//a node in the funnel deque
class FunnelNode
{
protected:
  //left and right pointers (doubly-connected)
  FunnelNode *m_CLeft;
  FunnelNode *m_CRight;
  //the vertex represented by this funnel node
  Point<float> m_CPoint;
  
public:
  //default constructor
  FunnelNode();
  //constructor - initializes the vertex coordinates
  FunnelNode(float x, float y);
  //constructor - initializes the vertex
  FunnelNode(const FPoint& point);
  //mutator for the left pointer
  void left(FunnelNode *left);
  //mutator for the right pointer
  void right(FunnelNode *right);
  //accessor for the left pointer
  FunnelNode *left();
  //accessor for the right pointer
  FunnelNode *right();
  //mutator for the vertex
  void point(const FPoint& point);
  //accessor for the vertex
  Point<float> point();
  //mutator for the vertex coordinates
  void point(float x, float y);
  //accessor for the vertex x coordinate
  float x();
  //accessor for the vertex y coordinate
  float y();
};

//definition of FunnelNode class }

//DJD: definition of FunnelDeque class {

//the funnel deque used in the funnel algorithm
class FunnelDeque
{
public:

  //different kinds of corners - point, left and right tangent
  //(tangent to a circle around the corner, running along the side specified)
  enum CornerType { CT_Point, CT_LeftTangent, CT_RightTangent };

protected:
  //left end, right end, and apex pointers for the deque
  FunnelNode *m_CLeft;
  FunnelNode *m_CRight;
  FunnelNode *m_CApex;
  //the corner type of the node at the apex
  //(ones on the left side are all right tangent, ones on the right are all left tangent)
  CornerType m_tApexType;
  //radius of the unit for which we are finding the path
  float m_dRadius;
  //angle between the first and second points given their corner types
  float angle(float x_1, float y_1, CornerType type_1, float x_2, float y_2, CornerType type_2);
  //constrains an angfle to the range (-PI, PI]
  float angleRange(float theta);
  //returns the (Euclidean) distance between two points
  float distance(float x_1, float y_1, float x_2, float y_2);
  //returns the angle between the given points
  //(same as the angle to their tangents if they're the same type)
  float toTangent(float x_1, float y_1, float x_2, float y_2);
  //returns the angle between tangents of the given points
  //given whether the first is left tangent type, assuming the second is the opposite
  float toTangentAlt(float x_1, float y_1, float x_2, float y_2, bool LeftRight);
  //returns the angle between the first point and a tangent to the second
  //(the type of the second is specified)
  float pointToTangent(float x_1, float y_1, float x_2, float y_2, bool Right);

  //adds a given point to the path
  void addPoint(float x, float y, CornerType type, float angle, FPolygon& path);
  
public:
  //creates the funnel deque based on a starting point and radius
  FunnelDeque(float x, float y, float r);
  //copy constructor
  FunnelDeque(const FPoint& point, float r);
  //destructor
  ~FunnelDeque();
  //adds a given point of a given type to the funnel deque, adding to the path if necessary
  void add(const Point<float>& p, CornerType type, FPolygon& path);
  //prints the contents of the funnel deque
  void print();
  //prints the contents of the path
  void print(const FPolygon& path);
  //calculates the length of the path for a unit of given radius
  float length(const FPolygon& path);
};

#endif

