#ifndef FORCEFIELDPE_H
#define FORCEFIELDPE_H
// $Id: ForceFieldPE.H 5279 2007-06-23 02:49:08Z mburo $
// This is an ORTS file (c) Michael Buro, licensed under the GPL

///\brief A PathExecutor using force fields

#include "PathExecutor.H"
#include "PathfindTask.H"
#include "LinAlg.H"
#include <math.h>
#include "VecField.H"
#include "TerrainBase.H"
#include "Vector.H"

typedef TerrainBase::Loc     Loc;
typedef TerrainBase::Segment Segment;
typedef TerrainBase::Path    Path;

class ForceFieldPE : public PathExecutor {
public:
  typedef TerrainBase::Loc     Loc;
  typedef TerrainBase::Segment Segment;
  typedef TerrainBase::Path    Path;
  

  struct ObstacleData{
    sint4 team;
    VecField::Emitter *em;
    ObstacleData(): team(-1), em(0){};

    ~ObstacleData() { if (em) delete em; };
  };

  struct FFTaskData : public ExtraData {
    sint4 progress_count;///< counts the number of execute cycles in which no progress was made along the path.
    Loc prev_loc;//fixme: when I tried to use get_center(x, y, px, py) instead of storing the previous location, I got a seg fault which I didn't really understand. So although it shouldn't be necessary to store the previous location, I'm doing it anyway.
    FFTaskData(): progress_count(0), prev_loc(-1, -1){};
    FFTaskData(const FFTaskData &other)
      : ExtraData(), progress_count(other.progress_count),
	prev_loc(other.prev_loc){};
    virtual ExtraData* clone() const {return new FFTaskData(*this);};
    virtual ~FFTaskData(){};
  };
  
  ForceFieldPE();
  virtual ~ForceFieldPE(){};
  
  /// \name PathExecutor Methods
  //@{
  virtual void execute_tasks(std::list<PathfindTask> &tasks, Vector<TerrainBase::MoveCmd> &cmds, Vector<TerrainBase::StatusMsg> &msgs);
  virtual void add_obj(const Object *obj);
  virtual void remove_obj(const Object *obj);
  virtual void update_obj(const Object  *obj);
  virtual void add_segments(const Vector<TerrainBase::Segment>& segment);
  //@}

protected:
 
  void calculate_move(PathfindTask &pt, Vector<TerrainBase::MoveCmd> &cmds);
  /** Test if the object is at the goal of the given task, implementing
      (some) of the various definitions of 'goal'. */
  bool is_at_goal(const Object* obj, const TerrainBase::Task &task);

  /** Calculates the position of the attractor along the path.
      The attractor is positioned a set distance ahead of the point closest to the
      object on the path.
      This method is also responsible for incrementing the current_wp value.
      Returns a Vec2 containing the location expresses as an x, y coordinate
  */
  Vec2<real4> get_attractor(PathfindTask &pt);
  /** Finds the location of point on a line that is closest to the given location*/
  Vec2<real4> find_point_on_line(Vec2<real4> point, Loc line_start, Loc line_end);

  void add_move_command(const Object *obj, Loc target, Vector<TerrainBase::MoveCmd> &cmds);
  void add_stop_command(const Object *obj, Vector<TerrainBase::MoveCmd> &cmds);
  

  VecField::Emitter* create_emitter(const Object *obj, real4 mag);

  /** get the team number of a given object */
  static sint4 get_team(const Object* obj);

  /** get the location of the center of an object */
  static Loc get_center_loc(const Object* obj); 

  inline static sint4 round(real4 real) {
      return (sint4)floor(real + 0.5);
  };

  Vector<Vec2<real4> > attractors; //for drawing purposes;
  Vector<Segment> walls;
  VecField wallForces;
  std::map<const Object*, ObstacleData> obj2obstacle;
  static const bool wp_attract = false;
  static const sint4 TOUCH_THRESHOLD = 10; ///< the center of an object must be at least this close to a location to be considered 'touching' it.
  static const real4 ATTRACT_STRENGTH = -25.0;
  static const real4 PATH_LOOK_AHEAD = 8.0;
  static const real4 REPULSE_STRENGTH = 40.0;
  static const real4 WALL_REPULSE_STRENGTH = 1.0;
  static const sint4 PROGRESS_CUTOFF = 15; ///< The number of execution cycles with no allowed before a Path Movement failure is passed 
  

  friend class AttractorOverlay;
};


#endif
