#ifndef Path_H
#define Path_H

// $Id: Path.H 5477 2007-07-27 00:24:02Z sorsten $
      
// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "Global.H"
#include "GameObj.H"

struct Point2i
{
  sint4 x, y;

  Point2i(sint4 a=0, sint4 b=0) { x = a; y = b;}

  bool operator==(const Point2i &other) {
    return x == other.x && y == other.y;
  }

  bool operator!=(const Point2i &other) {
    return x != other.x || y != other.y;
  }

  double distance(const Point2i &other) {
    real4 dx = x - other.x;
    real4 dy = y - other.y;
    return sqrt(dx*dx + dy*dy);
  }
};


struct Line
{
  Point2i a;
  Point2i b;

  Line() {}
  Line(const Point2i &a_, const Point2i &b_) { a = a_, b = b_;}
};


class UnitPath {
public:
  
  GameObj *unit;
  Vector<Point2i> path;
  Point2i goal;
  int current_target; // index of current way point in path vector
  uint4 wakeup; // if SDL_GetTicks() >= wakeup then this UnitPath is not
                // sleeping ie not to be skipped over.

  int tries; // this is incremented upon unexpected stop and retry

  bool active; // true of the path proccessor has sent the unit towards its
               // first waypoint
  
  bool dead; // path has been tried enough times and will be deleted
  
  Point2i spot; // spot of the last unexpected collition

  UnitPath(GameObj *obj, const Vector<Point2i> &path_);
  
  ~UnitPath() {}
  
  void mark_spot(const Point2i &p) { spot = p; }
  
  void kill() { dead = true; }
  
  void sleep(int time) { wakeup = SDL_GetTicks() + time; }

  bool sleeping() { return wakeup > SDL_GetTicks(); }

  Vector<Line> intersect(const Point2i &, const Point2i &);
  
  const Vector<Point2i> &get_path() const { return path; };
  const GameObj         *get_unit() const { return unit; };
};

REGISTER_TYPEOF(300, Vector<Point2i>::iterator);
REGISTER_TYPEOF(303, Vector<Point2i>::const_iterator);
REGISTER_TYPEOF(304, Vector<UnitPath>::iterator);
REGISTER_TYPEOF(305, Vector<UnitPath>::const_iterator);

#endif
