// $Id: SimplePFAdapter.C 5279 2007-06-23 02:49:08Z mburo $

#include "SimplePFAdapter.H"

typedef TerrainBase::Loc Loc;
typedef TerrainBase::Goal Goal;
typedef TerrainBase::Task Task;
typedef TerrainBase::Path Path;

//======================================================
SimplePFAdapter::SimplePFAdapter()
{

}

SimplePFAdapter::~SimplePFAdapter()
{
}
//======================================================
void SimplePFAdapter::init(sint4 tiles_x_, sint4 tiles_y_, sint4 tile_points_, sint4 client_pID_, sint4 neutral_pID_)
{
  GlobalPathfinder::init(tiles_x_, tiles_y_, tile_points_, client_pID_, neutral_pID_);
  Vector<sint4> levels_of_detail;
  levels_of_detail.push_back(4);// default - use a resolution reduced by a factor of 4.
  levels_of_detail.push_back(1); // also maintain a full resolution view.
  pfEngine = boost::shared_ptr<SimpleTerrain::SimpleGlobalPF>(new SimpleTerrain::SimpleGlobalPF(tiles_x_, tiles_y_, tile_points_, levels_of_detail));
}
//======================================================
void SimplePFAdapter::insert_object(const Object *obj){
  assert(initiated);
  pfEngine->insert_object(obj);
}

//======================================================
void SimplePFAdapter::remove_object(const Object *obj){
  assert(initiated);
  pfEngine->remove_object(obj);
}
//======================================================
void SimplePFAdapter::update_object(const Object *obj){
  assert(initiated);
  pfEngine->update_object(obj);
}
//======================================================
void SimplePFAdapter::insert_boundaries(const Vector<Segment> &s){
  assert(initiated);
  pfEngine->insert_boundaries(s);
}
//======================================================
bool SimplePFAdapter::find_path(const TerrainBase::Task& task, Path& output,  ConsiderObjects consider){
  const Object* obj = *(task.objs.begin());
  bool found = false;
  if (task.goal.target == Goal::OBJ){
    if (task.goal.mode == Goal::VICINITY){
      found = pfEngine->find_path(obj, task.goal.obj, output, task.goal.distance, 0, consider, 0);
    }
    else{
      found = pfEngine->find_path(obj, task.goal.obj, output, 0 , 0, consider, 0);
    }
  }
  else if (task.goal.target == Goal::LOCATION){
    if (task.goal.mode == Goal::VICINITY){
      found = pfEngine->find_path(obj, task.goal.loc, output, task.goal.distance, 0, consider, 0);
    }else{
      found = pfEngine->find_path(obj, task.goal.loc, output, 0 , 0, consider, 0);
    }
  }
  else{
    ERR("Task has Unrecognized Target Mode");
  }

  //hack: reverse the wp order. This pathfinding engine returns the wps in reverse order
  for (uint i = 0; i < (output.locs.size())/2; i++){
    Loc temp;
    temp = output.locs[i];
    output.locs[i] = output.locs[output.locs.size()-1-i];
    output.locs[output.locs.size()-1-i] = temp;
  }
    
  return found;
}
//======================================================
