#ifndef SIMPLEPATHFINDER_H
#define SIMPLEPATHFINDER_H

// $Id: SimplePathfinder.H 5279 2007-06-23 02:49:08Z mburo $
// This is an ORTS file (c) Michael Buro, Nick Wiebe, licensed under the GPL


#include "TerrainBase.H"
#include "PathExecutor.H"
#include "PathfindTask.H"
#include "TerrainBasicImp.H"
#include "boost/shared_ptr.hpp"
#include "GlobalPathfinder.H"
#include "Vector.H"

/// \brief Implementation of the separation of Execution from Planning.
/** 
PathfinderBasicImp is be used to separate Pathfinding from Path Execution
using the TerrainBase interface. It uses PathfindTask, a wrapper for
TerrainBase::Task, to associate a task with a path so that the bookkeeping for
the Path Execution system and Pathfinding system are shared. The bookkeeping is
done by TerrainBasicImp

From the TerrainBase interface, the PathfinderBasicImp implements: 
execute_tasks - calls the PathExecutor to generate the necessary move commands and
status messages.
add_obj, remove_obj, update_obj, add_segments - passes the information on to the PathExecutor

The inheritance is getting a bit absurd at this point, though...
*/
class SimplePathfinder : public TerrainBasicImp<PathfindTask>
{
public:

  SimplePathfinder(GlobalPathfinder &gp_, PathExecutor &pe_);
  SimplePathfinder(const SimplePathfinder &other);

  virtual ~SimplePathfinder() {};

  //---------------------------------------------------------------------
  /// \name Implementation of relevant parts of the TerrainBase interface
  //---------------------------------------------------------------------
  //@{
  virtual void init(sint4 tiles_x_, sint4 tiles_y_, sint4 tile_points_, sint4 client_pID_, sint4 neutral_pID_);

  virtual void add_obj(const Object *obj);
  virtual void update_obj(const Object *obj);
  virtual void remove_obj(const Object *obj);
  virtual void add_segments(const Vector<Segment> &s);

  virtual void cancel_task(const Object *obj);
  virtual void remove_task(TaskId tid);

  virtual void execute_tasks(Vector<MoveCmd> &cmds, Vector<StatusMsg> &msgs);
  virtual bool plan_tasks(uint4 max_time=0);

  virtual real8 find_path(const Loc &l1, const Loc &l2, sint4 radius, Path *path, ConsiderObjects consider = CONSIDER_ALL);
  virtual real8 find_path(const Object *obj, const Goal &goal, Path *path, ConsiderObjects consider = CONSIDER_ALL);
  //@}
  //---------------------------------------------------------------------

protected:

  /** Perform PF for a single pending task, and insert results to data structures.
      Return true iff did some planning. */
  bool plan_pending_task();

  /** Accessory function for planning either a pending or a failed task. */
  void plan_task(TIter tit);

  /** Invalidate existing paths that intersect a given new boundary. */
  void invalidate_paths(const Vector<Segment> &segments);
    
  static real8 path_length(const Path &path);
  static Loc get_center_loc(const Object *obj);
  static bool is_intersect(const Segment &segment, const Path &path);

  PathExecutor      &pe;
  //boost::shared_ptr<GlobalPathfinder> gp;
  GlobalPathfinder &gp;
  bool initiated;

  friend class SimplePathfinderWidget;
  friend class SimplePathfinderOverlay;
};
#endif


