// $Id: bitVector.cpp 2487 2006-12-31 04:09:43Z orts_furtak $

// HOG File

#include <cstdlib>
#include <cstdio>
#include "bitVector.h"

using namespace HOG;

bitVector::bitVector(int _size)
{
  true_size = _size;
  size = (_size>>5)+1;
  storage = new uint32_t[size];
  for (int x = 0; x < size; x++) storage[x] = 0;
}

bitVector::~bitVector()
{
  delete [] storage;
}

void bitVector::clear()
{
  for (int x = 0; x < size; x++) storage[x] = 0;
}

bitVector *bitVector::clone()
{
  bitVector *bv = new bitVector(true_size);
  bv->merge(this);
  return bv;
}

bool bitVector::get(int index) const
{
  if ((index>>5) > size) {
    printf("GET %d OUT OF RANGE\n", index);
    exit(0);
  }
  return (((storage[index>>5])>>(index&0x1F))&0x1);
}

void bitVector::set(int index, bool value)
{
  if ((index>>5) > size) {
    printf("SET %d OUT OF RANGE\n", index);
    exit(0);
  }
  if (value)
    storage[index>>5] = storage[index>>5]|(1<<(index&0x1F));
  else
    storage[index>>5] = storage[index>>5]&(~(1<<(index&0x1F)));
}

void bitVector::merge(bitVector *bv)
{
  if (bv == 0) return;
  if (bv->size != size) {
    printf("Error; can't merge vectors of different sizes (%d/%d)\n", bv->true_size, true_size);
    return;
  }
  for (int x = 0; x < size; x++) storage[x] |= bv->storage[x];
}

bool bitVector::equals(bitVector *bv)
{
  if (bv->size != size) return false;
  for (int x = 0; x < size; x++)
    if (storage[x] != bv->storage[x])
      return false;
  return true;
}

int bitVector::getNumSetBits()
{
  int sum = 0;
  for (int x = 0; x < size; x++) {
    uint32_t iter = storage[x];
    while (iter) {
      if (iter&1) sum++;
      iter = iter>>1;
    }
  }
  return sum;
}
