/*
 * $Id: praStar.h 5279 2007-06-23 02:49:08Z mburo $
 *
 *  Hierarchical Open Graph File
 *
 *  Created by Nathan Sturtevant on 9/28/04.
 *  Copyright 2004 Nathan Sturtevant. All rights reserved.
 *
 */

#ifndef PRASTAR_H
#define PRASTAR_H

#include <iostream>
#include "searchAlgorithm.h"
#include "heap.h"

/**
 * The pra* search algorithm which does partial pathfinding using abstraction.
 */

namespace HOG{

class praStar : public searchAlgorithm {

public:
	praStar();
	virtual ~praStar() {}
	virtual path *getPath(graphAbstraction *aMap, node *from, node *to, reservationProvider *rp = 0);
	virtual const char *getName() { return algName; } 
	void setPartialPathLimit(int limit) { partialLimit = limit; 
		sprintf(algName,"PRA*(%d)",partialLimit); }
	int getPartialPathLimit() { return partialLimit; }
	void setPlanFromMiddle(bool _planFromMiddle) { planFromMiddle = _planFromMiddle; }
	void setExpandSearchRadius(bool _expandSearchRadius) { expandSearchRadius = _expandSearchRadius; }
	void setUseSmoothing(bool _smoothing) { smoothing = _smoothing; }
	void setCache(path **p);
	/** Set a fixed level for abstraction in planning. -1 to return to dynamic level selection */
	void setFixedPlanLevel(int p) { fixedPlanLevel = p; }
protected:

  path *getAbstractPath(graph *g, unsigned int source, unsigned int destParent,
			std::vector<unsigned int> &eligibleNodeParents, int LABEL,
			unsigned int dest);
  
  unsigned int astar(graph *g, unsigned int source, unsigned int destParent,
		     std::vector<unsigned int> &eligibleNodeParents, int LABEL,
		     unsigned int dest);
  
  void relaxEdge(heap *nodeHeap, graph *g, edge *e, int source, int nextNode, int dest,
		 int LABEL);
	path *smoothPath(path *p);
	
	path **cache;
  int partialLimit;
	int fixedPlanLevel;
  char algName[30];
  graphAbstraction *map;
	bool expandSearchRadius;
	bool planFromMiddle;
	bool smoothing;
	reservationProvider *rp;
};

}
#endif
