#ifndef TR_PFENGINE_H
#define TR_PFENGINE_H

#include "GlobalPathfinder.H"
#include "GameObj.H"
#include "Base.h"
#include "SegmentMerge.H"

//DJD: object's position in the triangulation and its influence {

//keeps track of an object and the triangle it currently occupies and its influence
#if 0
struct XTR_Object
{
	//the ID of the object being monitored
	ObjId id;
	//the triangle in which the object lies
	TraFace *face;
	//the influence of the object
	float influence;

	//constructor - initializes member variables
	XTR_Object(ScriptObj *_obj, TraFace *_face = NULL) : id (id = GameObj::get_obj_id(_obj)), face (_face)
	{
		CalculateInfluence(_obj);
	}

	//calculates the object's influence based on its attributes
	void CalculateInfluence(ScriptObj *_obj)
	{
		//get the game object associated with the object
		GameObj *gob = _obj->get_GameObj();
		//get its hit points and weapon
		const sintptr *hp = gob->get_const_int_ptr("hp");
		ScriptObj *weapon = gob->component("weapon");
		//if it has no weapon, it has no influence
		if (weapon == NULL)
		{
			influence = 0.0f;
			return;
		}
		//get the minimum and maximum damage and cooldown for the weapon
		const sintptr *min = weapon->get_const_int_ptr("min_damage");
		const sintptr *max = weapon->get_const_int_ptr("max_damage");
		const sintptr *cooldown = weapon->get_const_int_ptr("cooldown");
		//if any value failed, it has no influence
		if ((hp == NULL) || (min == NULL) || (max == NULL))
		{
			influence = 0.0f;
		}
		else
		{
			//otherwise the influence is calculated as the average damage of its weapon, divided by
			//the number of turns between its attacks, times its hit points
			influence = ((float)(*min) + (float)(*max)) / 2.0f * (float)(*hp) / (float)((*cooldown) + 1);
		}
	}
};
#endif

//DJD: an obstacle's id within ORTS and the triangulation {

//keeps track of the ID of an obstacle within the triangulation so it can be removed
struct TR_Obstacle
{
	//the ID of the obstacle within ORTS
	ObjId ortsId;
	//the ID of the obstacle within the triangulation
	int dcdtId;

	//constructor - initializes the member variables
	TR_Obstacle(ObjId _ortsId, int _dcdtId) : ortsId (_ortsId), dcdtId (_dcdtId) {}

	//returns whether or not the given object is an obstacle
	static bool IsObstacle(GameObj *gob)
	{
		//TODO: should be generalized for use in ORTS outside the tournament
		//should consider resources, buildings, and trees as obstacles
		return ((gob->has_attr("is_mobile")) && (gob->get_int("is_mobile") == 0) &&
			(gob->get_zcat(NULL) == Object::ON_LAND));
	}
};

//an obstacle's id within ORTS and the triangulation }

class TR_PFEngine : public GlobalPathfinder{
public:
  TR_PFEngine(sint4 tiles_x_, sint4 tiles_y_, sint4 tile_points_, sint4 client_pID_, sint4 neutral_pID_ );

  virtual void insert_object(const Object *obj);
  virtual void remove_object(const Object *obj);
  virtual void update_object(const Object *obj);
  virtual void insert_boundaries(const Vector<TerrainBase::Segment> &s);
 
//DJD: function prototypes for updating the terrain for obstacles {
	void AddObstacle(GameObj *gob);
	void RemoveObstacle(GameObj *gob);
	void UpdateAbstraction();
//function prototypes updating the terrain for obstacles }

  /** Perform pathfinding to a location, return true iff path found, and the path itself in 'output'.
        If found then output is guaranteed to include at least one waypoint for the goal. 
        Waypoints are listed in reverse order, goal is in locs[0]. */
  virtual bool find_path(const Object *obj, const Loc &goal, Path& output, ConsiderObjects consider = TerrainBase::CONSIDER_ALL);
  /** Perform pathfinding to a target object, return true iff path found, and the path itself in 'output'
      If found then output is guaranteed to include at least one waypoint for the goal. 
      Waypoints are listed in reverse order, goal is in locs[0]. */
  virtual bool find_path(const Object *obj, const Object *goal, Path& output, ConsiderObjects consider = TerrainBase::CONSIDER_ALL);
  
protected:

  bool find_path(const Loc &l1, const Loc &l2, sint4 radius, Path &output, ConsiderObjects consider);
  
  //the triangulator
  TraDcdt pfDcdt;
  //used to merge segments received
  SegmentMerge sm;
#if 0
  //objects known to the pathfinder
  Vector<XTR_Object> objects;
#endif
  //obstacles known to the pathfinder
  Vector<TR_Obstacle> obstacles;
  sint4 client_pID;
  sint4 neutral_pID;
  
};

#endif
