/** @file MapTool.C
    @see MapTool.H

    $Id: MapMarine.C 6995 2008-05-09 03:48:31Z furtak $
    $Source$
*/
// This is an ORTS file (c) Tim Furtak, licensed under the GPL

#include "Global.H"
#include "MapTool.H"
#include "GameConst.H"
#include "Random.H"
#include "Options.H"

using namespace std;

static bool CREATE_WATER = false;

//===================================================================

/** Generate a random map with CLIFF tiles. */

void MapTool::generate_marine_map(stringstream &world)
{
  opt.get();

  Random rand;
  sint4 seed = opt.seed;
  rand.srand(seed);

  // note: if loading a map, -nplayers values must match
  sint4 num_players = opt.nplayers;
  
  sint4 tiles_x, tiles_y;
  sint4 gtiles_x = opt.width;
  sint4 gtiles_y = opt.height;
  sint4 z_sight = 0;

  bool rotate = opt.rot;
  CREATE_WATER = opt.water;

  tiles_x = gtiles_x;
  tiles_y = gtiles_y;


  world << num_players << endl;
  world << gtiles_x << endl;
  world << gtiles_y << endl;
  world << z_sight << endl;
  if (rotate) world << 1 << endl;
  else        world << 0 << endl;

  sint4 tile_n = tiles_x * tiles_y;

  Map<Tile> map_data;
  map_data.allocate(tiles_x, tiles_y);

  sint4 h_x = tiles_x + 1;
  sint4 h_y = tiles_y + 1;  
  sint4 h_n = h_x * h_y;

  Vector<sint2> height(h_n); // height field (index is not a tile index!)
  Vector<sint2> bad(tile_n); // do not put stuff on tile if true, works on tiles!

  // first heights heights represent layers (will be multiplied by GameConst::HEIGHT_MULT later

  real8 fplat = opt.fplat;
  bool plug_holes = opt.holes;
  sint4 clump = opt.clump;
  bool smooth = opt.smooth;

  if (smooth) plug_holes = false; // smooth > plug_holes

  if (clump < 1) clump = 1;

  if (clump <= 1) {

    FORS (i, sint4(tile_n * fplat)) {
      sint4 r;
      do { r = rand.rand_uint4() % h_n; } while (height[r] == 1);
      height[r] = 1;
    }

  } else {

    FORS (i, sint4(tile_n * fplat)) {
      sint4 r;
      do { r = rand.rand_uint4() % h_n; } while (height[r] >= 3);
    
      sint4 next_to = 0;
      sint4 out_of = 4;
    
      sint4 x = r % h_x;
      sint4 y = r / h_x;
  
      if (x == 0 || x == h_x-1) out_of--;
      if (y == 0 || y == h_y-1) out_of--;
    
      sint4 h = height[r];

      if (x > 0     && height[r-1]   == h+1) next_to++;
      if (x < h_x-1 && height[r+1]   == h+1) next_to++;
      if (y > 0     && height[r-h_x] == h+1) next_to++;
      if (y < h_y-1 && height[r+h_x] == h+1) next_to++;

      const sint4 offset = 4;

      if (out_of == next_to) {
	if (rand.rand_uint4() % 3) height[r] += 1;
	else if (h <= 1) height[r] += 2;
        continue;
      }
      if (next_to * 2 >= out_of) {
        next_to += offset;
        out_of += offset;
      }
      if (next_to > 0) {
        if ((sint4)(rand.rand_uint4() % out_of) < next_to) {
	  height[r] += 1;
	  continue;
	}
      }
      if (rand.rand_uint4() % clump < 1) {
        height[r]++; // = 1;
        continue;
      }
      --i;
    }
  }

  if (plug_holes) {

    FORS (i, h_n) {
      sint4 r = i;

      sint4 next_to = 0;
      sint4 out_of = 4;

      sint4 rx = r % h_x;
      sint4 ry = r / h_x;

      sint4 h = height[r];
    
      if (rx == 0 || rx == h_x-1) out_of--;
      if (ry == 0 || ry == h_y-1) out_of--;

      if (rx > 0     && height[r-1]   > h) next_to++;
      if (rx < h_x-1 && height[r+1]   > h) next_to++;
      if (ry > 0     && height[r-h_x] > h) next_to++;
      if (ry < h_y-1 && height[r+h_x] > h) next_to++;

      if (out_of == next_to) {// && height[r] == 0) {
        //	cout << i << " " << next_to << " " << out_of << " plug" << endl;
        height[r]++;// = 1;
      }
    }
  }

  // the world is now a height field with no ramps
  // flatten the corner and add demo buildings

  if (tiles_x > 40 && tiles_y > 60) {
    FORU (i, 40) {
      FORU (j, 60) {
        height[i + j*h_x] = 0;
      }
    }
  }

  // smooth plateaus:
  //  continue filling tiles iff it has three or four filled HV neighbors

  bool changed = smooth;

  while (changed) {

    changed = false;
    
    sint4 dx[] = { 1,  0, -1, 0 };
    sint4 dy[] = { 0, -1,  0, 1 };

    FORS (i, h_n) {

      sint4 h = height[i];

      sint4 ix = i % h_x;
      sint4 iy = i / h_x;

      sint4 next_to = 0;
      
      FORS (j, 4) {
	sint4 jx = ix + dx[j];
	sint4 jy = iy + dy[j];	  
	if (jx < 0 || jx >= h_x || jy < 0 || jy >= h_y) next_to++;
	else if (height[jy*h_x + jx] > h) next_to++;
      }
      
      if (next_to >= 3) { height[i]++; changed = true; }
    }
  }


  FORS (i, h_n) height[i] *= GameConst::HEIGHT_MULT;

  // from here on: heights are multiples of GameConst::HEIGHT_MULT
  // or at half-heights (ramps)
  
  FORS (i, h_n) {

    sint4 x = i % h_x;
    sint4 y = i / h_x;

    if (x == h_x-1 || y == h_y-1) continue;

    //    cout << height[i] << " ";
    
    sint4 h[4];
    
    h[0] = height[i];
    h[1] = height[i+1];
    h[2] = height[i+h_x+1];
    h[3] = height[i+h_x];

    Tile &tile = map_data.get_tile(x, y);

    tile.set_topo(h[0], h[1], h[2], h[3], Tile::NO_SPLIT);
    tile.set_type(GROUND);

    assert(x >= 0 && x < tiles_x);	
    assert(y >= 0 && y < tiles_y);
    
    // equal heights -> regular ground tile
    
    if (h[0] == h[1] && h[0] == h[2] && h[0] == h[3]) {

      tile.set_topo(h[0], h[1], h[2], h[3], Tile::NO_SPLIT);
      tile.set_type(GROUND);
      continue;
    }

    // 3 equal heights -> split
    //  - make equal height GROUND triangle and the other CLIFF or
    //  - create ridge
    
    {
      if ((h[0] == h[1] && h[1] == h[2]) ||
          (h[0] == h[1] && h[1] == h[3]) ||
          (h[0] == h[2] && h[2] == h[3]) ||
          (h[1] == h[2] && h[2] == h[3])) {

        sint4 i;
        FOR (i, 4) {
          if (h[i] != h[(i+1)&3] && h[i] != h[(i+2)&3]) break;
        }

	if (1 || (rand.rand_uint4() & 8)) {

	  // equal height triangle
	  
	  if (i == 0 || i == 2) {  // |/|
	    tile.set_topo(h[0], h[1], h[2], h[3], Tile::BT_SPLIT);
	  } else { // |\|
	    tile.set_topo(h[0], h[1], h[2], h[3], Tile::TB_SPLIT);
	  }
      
	  if (i == 0 || i == 3) tile.set_type(CLIFF, GROUND); // west CLIFF
	  else                  tile.set_type(GROUND, CLIFF); // east CLIFF

	} else {

	  // ridge

	  if (i == 0 || i == 2) {  // |\|
	    tile.set_topo(h[0], h[1], h[2], h[3], Tile::TB_SPLIT);
	  } else { // |/|
	    tile.set_topo(h[0], h[1], h[2], h[3], Tile::BT_SPLIT);
	  }

	  tile.set_type(CLIFF, CLIFF);
	}
	assert(x >= 0 && x < tiles_x);	
	assert(y >= 0 && y < tiles_y);
	bad[y*tiles_x+x] = 1;
	continue;
      }
    }

    
    // 2 and 2 equal heights -> full CLIFF tile
    
    {
      sint4 i;
      FOR (i, 4) {
	if (h[i] == h[(i+1) & 3] && h[(i+2) & 3] == h[(i+3) & 3]) break;
      }
    
      if (i < 4) {

	tile.set_topo(h[0], h[1], h[2], h[3], Tile::NO_SPLIT);

	// small chance of ramp
	if (rand.rand_uint4() % 100 < 15) tile.set_type(GROUND);
	else                              tile.set_type(CLIFF);

	assert(x >= 0 && x < tiles_x);	
	assert(y >= 0 && y < tiles_y);
	bad[y*tiles_x+x] = 1;
	continue;
      }
    }

    // otherwise: random CLIFF/CLIFF split

    {
      tile.set_type(CLIFF, CLIFF);
   
      if (rand.rand_uint4() & 8) {
	tile.set_topo(h[0], h[1], h[2], h[3], Tile::BT_SPLIT);
      } else {
	tile.set_topo(h[0], h[1], h[2], h[3], Tile::TB_SPLIT);
      }

      assert(x >= 0 && x < tiles_x);	
      assert(y >= 0 && y < tiles_y);
      bad[y*tiles_x+x] = 1;
    }
  }

  if (rotate) {

    FORS (i, tile_n) {
      sint4 x = i % tiles_x;
      sint4 y = i / tiles_x;

      if ((x + y < gtiles_y-1) ||
          (tiles_x - x + tiles_y - y < gtiles_y+1) ||
          (tiles_x - x + y < gtiles_x) ||
          (x + tiles_y - y < gtiles_x)) {
        map_data(x,y).set_type(Tile::UNKNOWN);
      }
    }
  }

  FORS (i, tile_n) {

    sint4 x = i % tiles_x;
    sint4 y = i / tiles_x;

    Tile &t = map_data.get_tile(i);
    Tile::Type tw, te;
    Tile::Split split;
    t.get_type(tw, te);
    sint4 h0, h1, h2, h3;
    t.get_topo(h0, h1, h2, h3, split);

    if (split == Tile::NO_SPLIT && tw != CLIFF && (h0 != h3 || h0 != h1)) {

      // ramp

      if (h0 != h3) {

	// | ramp

	if (x > 0) {  // extend left
	  Tile &t = map_data.get_tile(x-1, y);
	  t.get_type(tw, te);
	  t.get_topo(h0, h1, h2, h3, split);

	  if (rand.rand_uint4() % 100 < 50 && te == CLIFF) {
	    if (split) t.set_type(tw, GROUND);
	    else       t.set_type(GROUND, GROUND);
	  }
	}
	
	if (x < tiles_x-1) {  // extend right
	  Tile &t = map_data.get_tile(x+1, y);
	  t.get_type(tw, te);
	  t.get_topo(h0, h1, h2, h3, split);

	  if (rand.rand_uint4() % 100 < 50 && tw == CLIFF) {
	    if (split) t.set_type(GROUND, te);
	    else       t.set_type(GROUND, GROUND);
	  }
	}

      } else {

	// - ramp

 	if (y > 0) {  // extend up
	  Tile &t = map_data.get_tile(x, y-1);
	  t.get_type(tw, te);
	  t.get_topo(h0, h1, h2, h3, split);

	  if (rand.rand_uint4() % 100 < 50 &&
	      ((!split && tw == CLIFF) ||                    // | |
	       (split == Tile::TB_SPLIT && tw == CLIFF) ||   // |\|
	       (split == Tile::BT_SPLIT && te == CLIFF))) {  // |/|

	    if   (!split)                     t.set_type(GROUND, GROUND);
	    else if (split == Tile::TB_SPLIT) t.set_type(GROUND, te);
	    else                              t.set_type(tw, GROUND);
	  }
	}
	
	if (y < tiles_y-1) {  // extend down
	  Tile &t = map_data.get_tile(x, y+1);
	  t.get_type(tw, te);
	  t.get_topo(h0, h1, h2, h3, split);

	  if (rand.rand_uint4() % 100 < 50 &&
	      ((!split && tw == CLIFF) ||                    // | |
	       (split == Tile::TB_SPLIT && te == CLIFF) ||   // |\|
	       (split == Tile::BT_SPLIT && tw == CLIFF))) {  // |/|

	    if   (!split   )                  t.set_type(GROUND, GROUND);
	    else if (split == Tile::TB_SPLIT) t.set_type(tw, GROUND);
	    else                              t.set_type(GROUND, te);
	  }
	}
      }
    }
  }
  
  world << "<TILES>" << endl;
  map_data.write(world);
  world << "</TILES>" << endl;


  string fname = opt.bp_file;
  fname = sanitize_path(fname);
  parse_map_include(world, fname);

  world << "<OBJECTS>\n";

  Vector<string> types;
  Vector<sint4> counts;

  FORALL (opt.n_units, it) {
    stringstream ss(*it);
    sint4 c;
    ss >> c;
    ss.get();
    string t;
    ss >> t;
    types.push_back(t);
    counts.push_back(c);
  }
  /*
  types.push_back("marine");
  types.push_back("tank");
  counts.push_back(opt.n_marines);
  counts.push_back(opt.n_tanks);
  */

  if (opt.symmetric) {
    FORU (t, types.size()) {
      FORS (i, counts[t]) {
        sint4 r;
        if (opt.split) {
          do { r = rand.rand_uint4() % (tile_n / 2); } while (bad[r]);
        } else {
          do { r = rand.rand_uint4() % tile_n; } while (bad[r]);
        }
        bad[r] = 1;
        sint4 x = r % tiles_x;
        sint4 y = r / tiles_x;
        sint4 y2 = tiles_y - y - 1;
        bad[y2 * tiles_x + x] = 1;
        place_unit(world, types[t], 0, x, y);
        place_unit(world, types[t], 1, x, y2);
      }
    }
  } else {
    FORU (t, types.size()) {
      FORS (i, counts[t]) {
        FORU (p, num_players) {
          sint4 r;
          if (opt.split) {
            do {
              r = rand.rand_uint4() % (tile_n / num_players);
              r += p*(tile_n / num_players);
            } while (bad[r]);
          } else {
            do { r = rand.rand_uint4() % tile_n; } while (bad[r]);
          }
          bad[r] = 1;
          place_unit(world, types[t], p, r % tiles_x, r / tiles_x);
        }
      }
    }
  }

  world << "</OBJECTS>\n";
}

//===================================================================

