/** @file RandomIsland.C
    @see MapTool.H

    $Id: MapRandomIsland.C 12791 2012-11-07 18:19:57Z mburo $
    $Source$
*/


#include "Global.H"
#include "MapTool.H"
#include "GameConst.H"




using namespace std;

#define MIN(x,y) (x < y ? x : y)
#define MAX(x,y) (x > y ? x : y)
#define PI 3.1415

//
//
//struct Point
//{
//  sint4 x;
//  sint4 y;
//};



// x and y are the centre position of the island
//ramps
bool MapTool::n_ramp(sint4 x, sint4 y, Map < Tile > &map_data, sint4 left_edge, sint4 lower_edge, sint4 right_edge, sint4 level_1, sint4 level_2, sint4 width, sint4 height)
{
  sint4 diff = (level_2 - level_1) / 4;
  for(; y < lower_edge; y++) {
    x = left_edge;
    for(; x < right_edge; x++) {
      if(x + 3 > width - 1 || y > height - 1 || y - 4 < 0 || x < 0)
        continue;
      if (map_data(x, y).get_height(0) == level_2 &&
          map_data(x + 1, y).get_height(0) == level_2 &&
          map_data(x + 2, y).get_height(0) == level_2 &&
          map_data(x + 3, y).get_height(0) == level_2 &&
          map_data(x, y - 4).get_height(0) == level_1 &&
          map_data(x + 1, y - 4).get_height(0) == level_1 &&
          map_data(x + 2, y - 4).get_height(0) == level_1 &&
          map_data(x + 3, y - 4).get_height(0) == level_1) {
        FORS(i, 5) {
          FORS(j, 4) {
            map_data(x + j, y - i).set_type(GROUND);
            map_data(x + j, y - i).set_flat(level_2 - diff * i);
          }
        }
        return true;
      }
    }
  }
  return false;
}

bool MapTool::e_ramp(sint4 x, sint4 y, Map < Tile > &map_data, sint4 left_edge, sint4 lower_edge, sint4 right_edge, sint4 level_1, sint4 level_2, sint4 width, sint4 height)
{
  sint4 diff = (level_2 - level_1) / 4;
  for (; y < lower_edge; y++) {
    x = left_edge;
    for (; x < right_edge; x++) {
      if (x + 4 > width - 1 || y + 3 > height - 1 || y < 0 || x < 0)
        continue;
      if(map_data(x, y).get_height(0) == level_2 &&
         map_data(x, y + 1).get_height(0) == level_2 &&
         map_data(x, y + 2).get_height(0) == level_2 &&
         map_data(x, y + 3).get_height(0) == level_2 &&
         map_data(x + 4, y).get_height(0) == level_1 &&
         map_data(x + 4, y + 1).get_height(0) == level_1 &&
         map_data(x + 4, y + 2).get_height(0) == level_1 &&
         map_data(x + 4, y + 3).get_height(0) == level_1) {
        FORS (i, 4) {
          FORS (j, 5) {
            map_data(x + j, y + i).set_type(GROUND);
            map_data(x + j, y + i).set_flat(level_2 - diff * j);
          }
        }
        return true;
      }
    }
  }
  return false;
}

bool MapTool::w_ramp(sint4 x, sint4 y, Map < Tile > &map_data, sint4 left_edge, sint4 lower_edge, sint4 right_edge, sint4 level_1, sint4 level_2, sint4 width, sint4 height)
{
  sint4 diff = (level_2 - level_1) / 4;
  for (; y < lower_edge; y++) {
    x = left_edge;
    for (; x < right_edge; x++) {
      if (x > width - 1 || y + 3 > height - 1 || y < 0 || x - 4 < 0)
        continue;
      if (map_data(x, y).get_height(0) == level_2 &&
          map_data(x, y + 1).get_height(0) == level_2 &&
          map_data(x, y + 2).get_height(0) == level_2 &&
          map_data(x, y + 3).get_height(0) == level_2 &&
          map_data(x - 4, y).get_height(0) == level_1 &&
          map_data(x - 4, y + 1).get_height(0) == level_1 &&
          map_data(x - 4, y + 2).get_height(0) == level_1 &&
          map_data(x - 4, y + 3).get_height(0) == level_1) {
        FORS(i, 4) {
          FORS(j, 5) {
            map_data(x - j, y + i).set_type(GROUND);
            map_data(x - j, y + i).set_flat(level_2 - diff * j);
          }
        }
        return true;
      }
    }
  }
  return false;
}

bool MapTool::s_ramp(sint4 x, sint4 y, Map<Tile> &map_data, sint4 left_edge, sint4 lower_edge, sint4 right_edge, sint4 level_1, sint4 level_2, sint4 width, sint4 height)
{
  sint4 diff = (level_2 - level_1) / 4;
  for (; y < lower_edge; y++) {
    x = left_edge;
    for (; x < right_edge; x++) {
      if (x + 3 > width - 1 || y + 4 > height - 1 || y < 0 || x < 0)
        continue;
      if (map_data(x, y).get_height(0) == level_2 &&
          map_data(x + 1, y).get_height(0) == level_2 &&
          map_data(x + 2, y).get_height(0) == level_2 &&
          map_data(x + 3, y).get_height(0) == level_2 &&
          map_data(x, y + 4).get_height(0) == level_1 &&
          map_data(x + 1, y + 4).get_height(0) == level_1 &&
          map_data(x + 2, y + 4).get_height(0) == level_1 &&
          map_data(x + 3, y + 4).get_height(0) == level_1) {
        FORS(i, 5) {
          FORS(j, 4) {
            map_data(x + j, y + i).set_type(GROUND);
            map_data(x + j, y + i).set_flat(level_2 - diff * i);
          }
        }
        return true;
      }
    }
  }
  return false;
}

bool MapTool::sw_ramp(sint4 x, sint4 y, Map<Tile> &map_data, sint4 left_edge, sint4 lower_edge, sint4 right_edge, sint4 level_1, sint4 level_2, sint4 width, sint4 height)
{
  sint4 diff = (level_2 - level_1) / 4;
  for (; y < lower_edge; y++) {
    x = left_edge;
    for (; x < right_edge; x++) {
      if (x + 2 > width - 1 || y + 6 > height - 1 || y < 0 || x - 4 < 0)
        continue;
      if (map_data(x, y).get_height(0) == level_2 &&
          map_data(x + 1, y + 1).get_height(0) == level_2 &&
          map_data(x + 2, y + 2).get_height(0) == level_2 &&
          map_data(x - 4, y + 4).get_height(0) == level_1 &&
          map_data(x - 3, y + 5).get_height(0) == level_1 &&
          map_data(x - 2, y + 6).get_height(0) == level_1) {
        FORS (i, 5) {
          FORS (j, 4) {
            map_data(x + j, y + j).set_type(GROUND);
            map_data(x + j, y + j).set_flat(level_2 - diff * i);

          }
          if(i % 2 == 0)
            x--;
          else
            y++;
        }
        return true;
      }
    }
  }
  return false;
}

bool MapTool::nw_ramp(sint4 x, sint4 y, Map<Tile> &map_data, sint4 left_edge, sint4 lower_edge, sint4 right_edge, sint4 level_1, sint4 level_2, sint4 width, sint4 height)
{
  sint4 diff = (level_2 - level_1) / 4;
  for (; y < lower_edge; y++) {
    x = left_edge;
    for (; x < right_edge; x++) {
      if (x + 2 > width - 1 || y > height - 1 || y - 6 < 0 || x - 4 < 0)
        continue;
      if (map_data(x, y).get_height(0) == level_2 &&
          map_data(x + 1, y - 1).get_height(0) == level_2 &&
          map_data(x + 2, y - 2).get_height(0) == level_2 &&
          map_data(x - 4, y - 4).get_height(0) == level_1 &&
          map_data(x - 3, y - 5).get_height(0) == level_1 &&
          map_data(x - 2, y - 6).get_height(0) == level_1) {
        
        FORS (i, 5) {
          FORS (j, 4) {
            map_data(x + j, y - j).set_type(GROUND);
            map_data(x + j, y - j).set_flat(level_2 - diff * i);

          }
          if(i % 2 == 0)
            x--;
          else
            y--;
        }
        return true;
      }
    }
  }
  return false;
}

bool MapTool::se_ramp(sint4 x, sint4 y, Map<Tile> &map_data, sint4 left_edge, sint4 lower_edge, sint4 right_edge, sint4 level_1, sint4 level_2, sint4 width, sint4 height)
{
  sint4 diff = (level_2 - level_1) / 4;
  for (; y < lower_edge; y++) {
    x = left_edge;
    for (; x < right_edge; x++) {
      if (x + 4 > width - 1 || y + 6 > height - 1 || y - 2 < 0 || x < 0)
        continue;
      if (map_data(x, y).get_height(0) == level_2 &&
          map_data(x + 1, y - 1).get_height(0) == level_2 &&
          map_data(x + 2, y - 2).get_height(0) == level_2 &&
          map_data(x + 4, y).get_height(0) == level_1 &&
          map_data(x + 3, y + 1).get_height(0) == level_1 &&
          map_data(x + 2, y + 2).get_height(0) == level_1) {
        FORS(i, 5) {
          FORS(j, 4) {
            map_data(x + j, y - j).set_type(GROUND);
            map_data(x + j, y - j).set_flat(level_2 - (diff * i));

          }
          if(i % 2 == 0)
            x++;
          else
            y++;
        }
        return true;
      }
    }
  }
  return false;
}

bool MapTool::ne_ramp(sint4 x, sint4 y, Map < Tile > &map_data, sint4 left_edge, sint4 lower_edge, sint4 right_edge, sint4 level_1, sint4 level_2, sint4 width, sint4 height)
{
  sint4 diff = (level_2 - level_1) / 4;
  for(; y < lower_edge; y++) {
    x = left_edge;
    for(; x < right_edge; x++) {
      if(x + 4 > width - 1 || y + 2 > height - 1 || y - 6 < 0 || x < 0)
        continue;
      if(map_data(x, y).get_height(0) == level_2 &&
         map_data(x + 1, y + 1).get_height(0) == level_2 &&
         map_data(x + 2, y + 2).get_height(0) == level_2 &&
         map_data(x + 4, y - 4).get_height(0) == level_1 &&
         map_data(x + 3, y - 5).get_height(0) == level_1 &&
         map_data(x + 2, y - 6).get_height(0) == level_1) {
        FORS(i, 5) {
          FORS(j, 4) {
            map_data(x + j, y + j).set_type(GROUND);
            map_data(x + j, y + j).set_flat(level_2 - diff * i);

          }
          if(i % 2 == 0)
            x++;
          else
            y--;
        }
        return true;
      }
    }
  }
  return false;
}

// always call this one after cliff_tiles, otherwise cliff_tiles won't work right
// dir 0 east else west
bool MapTool::place_ramp(Map < Tile > &map_data, sint4 x, sint4 y, sint4 box_width, sint4 dir, sint4 level_1, sint4 level_2, sint4 width, sint4 height)
{
  cout << dir << endl;
  cout.flush();
  sint4 lower_edge = y + box_width;
  if(lower_edge >= height)
    lower_edge = height - 1;
  sint4 right_edge = x + box_width;
  if(right_edge >= width)
    right_edge = width - 1;
  sint4 left_edge = x;
  if (dir == 0) {                //NORTH
    return n_ramp(x, y, map_data, left_edge, lower_edge, right_edge, level_1, level_2, width, height);
  } else if (dir == 1) {         //NE
    return ne_ramp(x + width / 2, y, map_data, left_edge, lower_edge, right_edge, level_1, level_2, width, height);
  } else if (dir == 2) {         //E
    return e_ramp(x + width / 2, y, map_data, left_edge, lower_edge, right_edge, level_1, level_2, width, height);
  } else if (dir == 3) {         //SE
    return se_ramp(x, y + width / 4, map_data, left_edge, lower_edge, right_edge, level_1, level_2, width, height);
  } else if (dir == 4) {         //S
    return s_ramp(x, y + width / 4, map_data, left_edge, lower_edge, right_edge, level_1, level_2, width, height);
  } else if (dir == 5) {         //SW
    return sw_ramp(x, y + width / 4, map_data, left_edge, lower_edge, right_edge, level_1, level_2, width, height);
  } else if (dir == 6) {         //W
    return w_ramp(x, y, map_data, left_edge, lower_edge, right_edge, level_1, level_2, width, height);
  } else if (dir == 7) {         //NW
    return nw_ramp(x, y, map_data, left_edge, lower_edge, right_edge, level_1, level_2, width, height);
  } else
    return false;
}



//puts cliff tiles around edge of plateaus/islands
void MapTool::cliff_tiles(Map < Tile > &map_data, sint4 width, sint4 height)
{
  // corner heights
  /*
     0 1
     3 2
   */
  sint4 corner_0 = 0;
  sint4 corner_1 = 0;
  sint4 corner_2 = 0;
  sint4 corner_3 = 0;
  // sint4 count = 0;
  FORS(x, width)
    FORS(y, height) {
    // count = 1;
    Tile & temp = map_data(x, y);
    corner_0 = temp.get_height(0);
    corner_1 = temp.get_height(2);
    corner_2 = temp.get_height(4);
    corner_3 = temp.get_height(6);
    if(!(corner_0 == corner_1 && corner_1 == corner_2 && corner_2 == corner_3)) {
      temp.set_type(CLIFF);
    }
  }
}


//syncs upper_left corner heights of a tile
void MapTool::sync_corner(Map < Tile > &map_data, sint4 width, sint4 height)
{
  /*
     where tiles meet
     3 | 6
     4  |  5
     ______
     1  |  0
     2 | 7
   */

  /*


     2  |  3
     ______
     1  |  0

   */
  // corner heights 
  sint4 corner_0 = 0;
  sint4 corner_1 = 0;
  sint4 corner_2 = 0;
  sint4 corner_3 = 0;
  sint4 height_1;
  sint4 height_2;
  sint4 count = 0;
  for(sint4 x = 1; x < width; x++)
    for(sint4 y = 1; y < height; y++) {
      count = 1;
      Tile & temp = map_data(x, y);
      Tile & left = map_data(x - 1, y);
      Tile & above = map_data(x, y - 1);
      Tile & above_left = map_data(x - 1, y - 1);
      corner_0 = temp.get_height(0);
      corner_1 = left.get_height(1);
      corner_2 = above_left.get_height(3);

      corner_3 = above.get_height(5);
      height_1 = corner_0;
      height_2 = corner_2;
      if(corner_1 == height_1) {
        count++;
      } else
        height_2 = corner_1;
      if(corner_2 == height_1) {
        count++;
      } else
        height_2 = corner_2;
      if(corner_3 == height_1) {
        count++;
      } else
        height_2 = corner_3;
      if (!(corner_0 == corner_1 && corner_1 == corner_2 && corner_2 == corner_3 && corner_3 == corner_0)) {      //any of the corners don't match up
        if(count > 2) {
          //set corner heights to height_1
          temp.set_topo(height_1, temp.get_height(1), temp.get_height(3), temp.get_height(5), Tile::NO_SPLIT);
          left.set_topo(left.get_height(0), height_1, left.get_height(3), left.get_height(5), Tile::NO_SPLIT);
          above.set_topo(above.get_height(0), above.get_height(1), above.get_height(3), height_1, Tile::NO_SPLIT);
          above_left.set_topo(above_left.get_height(0), above_left.get_height(1), height_1, above_left.get_height(5), Tile::NO_SPLIT);


        } else {                //set corner heights to height_2
          temp.set_topo(height_2, temp.get_height(1), temp.get_height(3), temp.get_height(5), Tile::NO_SPLIT);
          left.set_topo(left.get_height(0), height_2, left.get_height(3), left.get_height(5), Tile::NO_SPLIT);
          above.set_topo(above.get_height(0), above.get_height(1), above.get_height(3), height_2, Tile::NO_SPLIT);
          above_left.set_topo(above_left.get_height(0), above_left.get_height(1), height_2, above_left.get_height(5), Tile::NO_SPLIT);
        }
      }
    }
}

//Returns 1 if p inside polygon
bool MapTool::inside_polygon(vector < Point > &polygon, Point p)
{

  sint4 N = polygon.size();
  if(N <= 2) {
    if(N == 0)
      return false;
    if(p.x == polygon[0].x && p.y == polygon[0].y) {
      if(N == 2) {
        if(p.x == polygon[1].x && p.y == polygon[1].y)
          return true;
        else
          return false;
      } else
        return false;
    }
  }
  sint4 counter = 0;
  sint4 i;
  real8 xinters;
  Point p1, p2;

  p1 = polygon[0];
  for(i = 1; i <= N; i++) {
    p2 = polygon[i % N];
    if(p.y > MIN(p1.y, p2.y)) {
      if(p.y <= MAX(p1.y, p2.y)) {
        if(p.x <= MAX(p1.x, p2.x)) {
          if(p1.y != p2.y) {
            xinters = (p.y - p1.y) * (p2.x - p1.x) / (p2.y - p1.y) + p1.x;
            if(p1.x == p2.x || p.x <= xinters)
              counter++;
          }
        }
      }
    }
    p1 = p2;
  }

  if(counter % 2 == 0)
    return false;
  else
    return true;
}

//all is sorted in descending order by x coordinates
// max_angle is in degrees
void MapTool::lower_edge(stack < Point > all, stack < Point > &polygon, real8 min_angle)
{
  real8 angle_1 = 0;
  real8 angle_2 = 0;
  real4 x_1 = 0;
  real4 y_1 = 0;
  real4 x_2 = 0;
  real4 y_2 = 0;
  bool dip = false;
  min_angle = min_angle * PI / 180;
  Point next, first, second;
  if(all.size() <= 2) {
    while(!all.empty()) {
      polygon.push(all.top());
      all.pop();
    }
    return;
  }
  polygon.push(all.top());
  all.pop();
  polygon.push(all.top());
  all.pop();
  while(!all.empty()) {
    next = all.top();
    all.pop();
    second = polygon.top();
    polygon.pop();
    first = polygon.top();
    if(min_angle > 0) {
      x_1 = second.x - first.x;
      x_2 = next.x - first.x;
      y_1 = second.y - first.y;
      y_2 = next.y - first.y;
      if(x_1 == 0) {
        if(y_1 < 0) {
          angle_1 = -PI / 2;
        } else {
          angle_1 = PI / 2;
        }
      } else {
        angle_1 = atan(y_1 / x_1);
      }
      if(x_2 == 0) {
        if(y_2 < 0) {
          angle_2 = -PI / 2;
        } else {
          angle_2 = PI / 2;
        }
      } else {
        angle_2 = atan(y_2 / x_2);
      }
      dip = angle_2 >= angle_1 && !polygon.empty();

    } else
      dip = first.y <= second.y && second.y >= next.y && !polygon.empty();
    while(dip) {
      second = first;
      polygon.pop();
      if(polygon.empty())
        break;
      first = polygon.top();
      if(min_angle > 0) {
        x_1 = second.x - first.x;
        x_2 = next.x - first.x;
        y_1 = second.y - first.y;
        y_2 = next.y - first.y;
        if(x_1 == 0) {
          if(y_1 < 0) {
            angle_1 = -PI / 2;
          } else {
            angle_1 = PI / 2;
          }
        } else {
          angle_1 = atan(y_1 / x_1);
        }
        if(x_2 == 0) {
          if(y_2 < 0) {
            angle_2 = -PI / 2;
          } else {
            angle_2 = PI / 2;
          }
        } else {
          angle_2 = atan(y_2 / x_2);
        }
        dip = angle_2 >= angle_1 && !polygon.empty();
      } else
        dip = first.y <= second.y && second.y >= next.y && !polygon.empty();
    }
    polygon.push(second);
    polygon.push(next);
  }
}

//all is sorted in descending order by x coordinates 
void MapTool::upper_edge(stack < Point > all, stack < Point > &polygon, real8 min_angle)
{
  real8 angle_1 = 0;
  real8 angle_2 = 0;
  real4 x_1 = 0;
  real4 y_1 = 0;
  real4 x_2 = 0;
  real4 y_2 = 0;
  bool dip = false;
  min_angle = min_angle * PI / 180;
  Point next, first, second;
  if(all.size() <= 2) {
    while(!all.empty()) {
      polygon.push(all.top());
      all.pop();
    }
    return;
  }
  polygon.push(all.top());
  all.pop();
  polygon.push(all.top());
  all.pop();
  while(!all.empty()) {
    next = all.top();
    all.pop();
    second = polygon.top();
    polygon.pop();
    first = polygon.top();
    if(min_angle > 0) {
      x_1 = second.x - first.x;
      x_2 = next.x - first.x;
      y_1 = second.y - first.y;
      y_2 = next.y - first.y;
      if(x_1 == 0) {
        if(y_1 < 0) {
          angle_1 = -PI / 2;
        } else {
          angle_1 = PI / 2;
        }
      } else {
        angle_1 = atan(y_1 / x_1);
      }
      if(x_2 == 0) {
        if(y_2 < 0) {
          angle_2 = -PI / 2;
        } else {
          angle_2 = PI / 2;
        }
      } else {
        angle_2 = atan(y_2 / x_2);
      }
      dip = angle_2 <= angle_1 && !polygon.empty();

    } else
      dip = first.y >= second.y && second.y <= next.y && !polygon.empty();
    while(dip) {
      second = first;
      polygon.pop();
      if(polygon.empty())
        break;
      first = polygon.top();
      if(min_angle > 0) {
        x_1 = second.x - first.x;
        x_2 = next.x - first.x;
        y_1 = second.y - first.y;
        y_2 = next.y - first.y;
        if(x_1 == 0) {
          if(y_1 < 0) {
            angle_1 = -PI / 2;
          } else {
            angle_1 = PI / 2;
          }
        } else {
          angle_1 = atan(y_1 / x_1);
        }
        if(x_2 == 0) {
          if(y_2 < 0) {
            angle_2 = -PI / 2;
          } else {
            angle_2 = PI / 2;
          }
        } else {
          angle_2 = atan(y_2 / x_2);
        }
        dip = angle_2 <= angle_1 && !polygon.empty();

      } else
        dip = first.y >= second.y && second.y <= next.y && !polygon.empty();
    }
    polygon.push(second);
    polygon.push(next);
  }
}


//diff is the difference in levels
void MapTool::smooth_edge(sint4 x, sint4 y, Map < Tile > &map_data, sint4 diff)
{
  //  bool cliff = opt.cliff;
  //  bool water = opt.water;
  //  sint4 gtiles_x = opt.width, gtiles_y = opt.height;
  uint4 edge;
  /*if(water)
     edge = WATER; //water
     else */
  edge = GROUND;                //ground
  //assert(x > 0 && y > 0 && x < (gtiles_x - 1) && y < (gtiles_y - 1));
  sint4 height = map_data(x, y).get_height(0);
  //fill concavities
  if(map_data(x, y - 1).get_height(4) > height) {
    if(map_data(x + 1, y).get_height(0) > height) {
      map_data(x, y).set_topo(height + diff, height + diff, height + diff, height, Tile::NO_SPLIT);
      map_data(x, y).set_type(edge, GROUND);
      return;
    } else if(map_data(x - 1, y).get_height(2) > height) {
      map_data(x, y).set_topo(height + diff, height + diff, height, height + diff, Tile::NO_SPLIT);
      map_data(x, y).set_type(GROUND, edge);
      return;
    }
  } else if(map_data(x, y + 1).get_height(0) > height) {
    if(map_data(x + 1, y).get_height(0) > height) {
      map_data(x, y).set_topo(height, height + diff, height + diff, height + diff, Tile::NO_SPLIT);
      map_data(x, y).set_type(edge, GROUND);
      return;
    } else if(map_data(x - 1, y).get_height(2) > height) {
      map_data(x, y).set_topo(height + diff, height, height + diff, height + diff, Tile::NO_SPLIT);
      map_data(x, y).set_type(GROUND, edge);
      return;
    }
  }
  //chop tiles that stick out
  if (map_data(x, y).is_flat()) {
    if(map_data(x, y - 1).get_height(4) < height) {
      if(map_data(x + 1, y).get_height(0) < height) {
        map_data(x, y).set_topo(height, height - diff, height, height, Tile::NO_SPLIT);
        map_data(x, y).set_type(GROUND, edge);
        return;
      } else if(map_data(x - 1, y).get_height(2) < height) {
        map_data(x, y).set_topo(height - diff, height, height, height, Tile::NO_SPLIT);
        map_data(x, y).set_type(edge, GROUND);
        return;
      }
    } else if(map_data(x, y + 1).get_height(0) < height) {
      if(map_data(x + 1, y).get_height(6) < height) {
        map_data(x, y).set_topo(height, height, height - diff, height, Tile::NO_SPLIT);
        map_data(x, y).set_type(GROUND, edge);
        return;
      } else if(map_data(x - 1, y).get_height(4) < height) {
        map_data(x, y).set_topo(height, height, height, height - diff, Tile::NO_SPLIT);
        map_data(x, y).set_type(edge, GROUND);
        return;
      }
    }
  }

}

void MapTool::raise_island(Map < Tile > &map_data, sint4 level_2, sint4 num_islands, sint4 size, sint4 seed)
{
  Random rand;
  rand.srand(seed);
  sint4 tiles_x = opt.width, tiles_y = opt.height;
  sint4 tile_n = tiles_x * tiles_y;
  // sint4 r = rand.rand_uint4();
  //positions of islands
  sint4 *player_x = new sint4[num_islands];
  sint4 *player_y = new sint4[num_islands];
  bool *flags = new bool[tile_n];
  bool *checked = new bool[tile_n];
  if(!player_x || !player_y || !flags || !checked)
    ERR("out of memory");
  memset(player_x, 0, num_islands * sizeof(*player_x));
  memset(player_y, 0, num_islands * sizeof(*player_y));
  memset(flags, false, tile_n * sizeof(*flags));
  memset(checked, 0, tile_n * sizeof(*checked));

  Vector < Point > *edges = new Vector < Point >[num_islands];
  Vector < Point > *islands = new Vector < Point >[num_islands];
  if(!edges || !islands)
    ERR("out of memory");

  sint4 x = 0, y = 0;

  //pick island spots
  FORS (n, num_islands) {
    x = rand.rand_uint4() % tiles_x;
    y = rand.rand_uint4() % tiles_y;

    if (flags[(y * tiles_x) + x] ||
        map_data(x, y).get_height(0) == level_2 ||
        (map_data(x, y).get_height(1) == level_2 && map_data(x, y).get_height(3) == level_2) ||  // () was missing, is this OK?
        map_data(x, y).get_height(5) == level_2) {
      n--;
      continue;
    }
    player_x[n] = x;
    player_y[n] = y;
    flags[(y * tiles_x) + x] = true;
  }

  Point t;
  //  bool water = opt.water;
  //  bool cliff = opt.cliff;
  real8 convex = opt.convex;

  sint4 edge = (sint4)(sqrt((real8)(size)));
  sint4 min = edge;
  FORS(k, num_islands) {
    level_2 = rand.rand_uint4() % 13;
    t.x = player_x[k];
    t.y = player_y[k];
    Tile & tile = map_data.get_tile(x, y);
    tile.set_flat(level_2);
    edges[k].push_back(t);
    flags[(y * tiles_x) + x] = true;
    FORS(i, size) {
      seed = rand.rand_uint4();
      if(seed < 0)
        seed = -seed;
      MapTool::add_tile(seed, edges[k], map_data, flags, tiles_x, tiles_y, level_2);
    }
  }
  //convex hull algo
  FORS(k, num_islands) {
    stack < Point > all;
    stack < Point > u_edge;
    stack < Point > l_edge;
    x = player_x[k] + min - 1;
    if(x < 0)
      x = 0;
    if(x > tiles_x - 1)
      x = tiles_x - 1;
    y = player_y[k] + min - 1;
    if(y < 0)
      y = 0;
    if(y > tiles_y - 1)
      y = tiles_y - 1;
    sint4 left_edge = player_x[k] - min;
    if(left_edge < 0)
      left_edge = 0;
    sint4 top = player_y[k] - min;
    if(top < 0)
      top = 0;
    //get island tiles
    while(x > left_edge) {
      y = player_y[k] + min - 1;
      if(y < 0)
        y = 0;
      if(y > tiles_y - 1)
        y = tiles_y - 1;
      while(y > top) {
        if(map_data(x, y).get_height(0) == level_2) {
          t.x = x;
          t.y = y;
          all.push(t);

        }
        y--;
      }
      x--;
    }
    upper_edge(all, u_edge, convex);
    lower_edge(all, l_edge, convex);
    // set points in convex hull to island ground tiles
    x = player_x[k] + min - 1;
    y = player_y[k] + min - 1;
    Vector < Point > poly;
    while(!u_edge.empty()) {
      poly.push_back(u_edge.top());
      u_edge.pop();
    }
    while(!l_edge.empty()) {
      poly.push_back(l_edge.top());
      l_edge.pop();
    }





    while(x > left_edge) {
      y = player_y[k] + min - 1;
      while(y > top) {
        t.x = x;
        t.y = y;
        if(inside_polygon(poly, t)) {
          map_data(x, y).set_type(GROUND);
          map_data(x, y).set_flat(level_2);
        }
        y--;
      }
      x--;
    }
  }




  y = 1;
  while(y < tiles_y - 1) {
    x = 1;
    while(x < tiles_x - 1) {
      //      smooth_edge(x, y, map_data, level_2 - level_1);
      x++;
    }
    y++;
  }



}



//returns random position of object between
//min and max distance from player position
Point MapTool::place_object(sint4 x, sint4 y, sint4 min_dist, sint4 max_dist, Random r)
{
  Point where;
  if(min_dist >= max_dist) {
    where.x = x + 1;
    where.y = y + 1;
    return where;
  }
  sint4 c = 0;
  while(abs(c) < min_dist)
    c = r.rand_uint4() % (max_dist + 1);
  where.x = r.rand_uint4() % (max_dist + 1);
  if ((where.x % 2) == 0)
    where.x = -where.x;
  where.y = static_cast<sint4>(sqrt(static_cast<real8>(abs((c * c) - (where.x * where.x)))));
  if ((where.y % 2) == 0)
    where.y = -where.y;
  where.x = x + where.x;
  where.y += y + where.y;

  return where;
}


void MapTool::generate_island_map(stringstream & world)
{
  opt.get();
  Random rand;

  sint4 seed = opt.seed;
  rand.srand(seed);

  // note: if loading a map, -nplayers values must match
  sint4 num_players = opt.nplayers;
  sint4 size = opt.island;
  sint4 tiles_x = opt.width;
  sint4 tiles_y = opt.height;
  sint4 z_sight = 0;

  world << num_players << endl;
  world << tiles_x << endl;
  world << tiles_y << endl;
  world << z_sight << endl;
  world << 0 << endl;

  sint4 tile_n = tiles_x * tiles_y;

  Map < Tile > map_data;
  map_data.allocate(tiles_x, tiles_y);
  sint2 *o_safe = new sint2[tile_n];    // valid square if 0
  if(!o_safe)
    ERR("out of memory");
  memset(o_safe, 0, tile_n * sizeof(*o_safe));


  sint4 r;
  r = rand.rand_uint4();
  sint4 rad_i = tiles_x / 8;    //radius of inner disk

  //positions of players
  sint4 *player_x = new sint4[num_players];
  sint4 *player_y = new sint4[num_players];
  bool *flags = new bool[tile_n];
  bool *checked = new bool[tile_n];

  if(!player_x || !player_y || !flags || !checked)
    ERR("out of memory");

  memset(player_x, 0, num_players * sizeof(*player_x));
  memset(player_y, 0, num_players * sizeof(*player_y));
  memset(flags, 0, tile_n * sizeof(*flags));
  memset(checked, 0, tile_n * sizeof(*checked));

  Vector < Point > *edges = new Vector < Point >[num_players];
  Vector < Point > *islands = new Vector < Point >[num_players];
  if(!edges || !islands)
    ERR("out of memory");

  //put players on the disk, doesn't write them to world
  place_players(rad_i, player_x, player_y);

  Point t;
  bool water = opt.water;
  bool cliff = opt.cliff;
  real8 convex = opt.convex;
  FORS(i, tile_n) {
    if(water)
      map_data(i).set_type(WATER);
    else
      map_data(i).set_type(GROUND);
  }

  sint4 level_1 = 0;
  sint4 level_2 = 9;

  //------------------------------------------------
  if(convex == 2) {
    FORS(p, 1) {
      r = -1;
      while(r <= level_1)
        r = rand.rand_uint4() % 13;     //height between 0 and max height
      sint4 i_size = r * 20;
      cout << i_size;
      if(i_size < 100)
        i_size = 100;
      if(i_size > 500)
        i_size = 500;
      raise_island(map_data, r, 2, i_size, rand.rand_uint4());
    }
  }
  // sint4 level_3 = 12;
  sint4 edge = static_cast<sint4>(sqrt((static_cast<real8>(size))) * 0.9);
  sint4 min = edge;
  sint4 x, y;
  
  FORS(k, num_players) {
    x = player_x[k] - min;
    y = player_y[k] - min;
    //make box
    FORS(z, (min + min)) {
      flags[(y * tiles_x) + x + z] = true;
      flags[((y + min + min) * tiles_x) + x + z] = true;
      flags[((y + z) * tiles_x) + x] = true;
      flags[((y + z) * tiles_x) + x + min + min] = true;
    }
    //set box to UNKNOWN tiles
    x = player_x[k] - min;
    y = player_y[k] - min;
    //make box

    //------------------------------------------------
    if(!convex) {
      FORS(z, (min + min)) {
        x = player_x[k] - min;
        FORS(p, (min + min)) {
          map_data.get_tile(x, y).set_type(Tile::UNKNOWN);
          x++;
        }
        y++;
      }
    }
    
    //put 8 X 8 island
    x = player_x[k] - 4;
    y = player_y[k] - 4;
    sint4 left_edge = x;
    sint4 right_edge = x + 8;
    sint4 bottom = y + 8;
    sint4 top = y;
    while(y <= bottom) {
      x = player_x[k] - 4;
      while(x <= right_edge) {
        Tile & tile = map_data.get_tile(x, y);
        tile.set_flat(level_2);
        tile.set_type(GROUND);
        flags[(y * tiles_x) + x] = true;
        if(x == left_edge) {
          t.x = x - 1;
          t.y = y;
          edges[k].push_back(t);
          flags[(tiles_x * t.y) + t.x] = true;
        }
        if(x == right_edge) {
          t.x = x + 1;
          t.y = y;
          edges[k].push_back(t);
          flags[(tiles_x * t.y) + t.x] = true;
        }
        if(y == top) {
          t.y = y - 1;
          t.x = x;
          edges[k].push_back(t);
          flags[(tiles_x * t.y) + t.x] = true;
        }
        if(y == bottom) {
          t.y = y + 1;
          t.x = x;
          edges[k].push_back(t);
          flags[(tiles_x * t.y) + t.x] = true;
        }
        x++;
      }
      y++;
    }

  }

  FORS(i, size - 8 * 8) {
    FORS(j, num_players) {
      seed = rand.rand_uint4();
      if(seed < 0)
        seed = -seed;
      add_tile(seed, edges[j], map_data, flags, tiles_x, tiles_y, level_2);
    }
  }
  FORS(y, tile_n) {
    cout << map_data(y).get_height(0);
    if((y + 1) % tiles_x == 0)
      cout << endl;
  }
  FORS(k, num_players) {
    x = player_x[k] - min;
    y = player_y[k] - min;
    sint4 left_edge = x;
    sint4 right_edge = player_x[k] + min;
    sint4 bottom = player_y[k] + min;
    sint4 top = y;
    //flood
    if(!convex) {
      while(y <= bottom) {
        while(x <= right_edge && map_data.get_tile(x, y).get_height(0) != level_2) {
          if(water)
            map_data(x, y).set_type(WATER);
          else
            map_data(x, y).set_type(GROUND);
          x++;
        }
        x = right_edge - 1;
        while(x >= left_edge && map_data(x, y).get_height(0) != level_2) {
          if(water)
            map_data(x, y).set_type(WATER);
          else
            map_data(x, y).set_type(GROUND);
          x--;
        }
        y++;
        x = left_edge;
      }

      x = left_edge;
      y = top;
      while(x <= right_edge) {
        while(y <= bottom && map_data(x, y).get_height(0) != level_2) {
          if(water)
            map_data(x, y).set_type(WATER);
          else
            map_data(x, y).set_type(GROUND);
          y++;
        }
        y = bottom - 1;
        while(y >= top && map_data(x, y).get_height(0) != level_2) {
          if(water)
            map_data(x, y).set_type(WATER);
          else
            map_data(x, y).set_type(GROUND);
          y--;
        }
        x++;
        y = top;
      }



    }
    // set holes to ground tiles
    y = player_y[k] - min;
    //make box
    if(!convex)
      FORS(z, (min + min)) {
      x = player_x[k] - min;
      FORS(p, (min + min)) {
        if(map_data(x, y).get_typeW() == Tile::UNKNOWN) {
          map_data(x, y).set_type(GROUND);
          map_data(x, y).set_flat(level_2);
        }
        x++;
      }
      y++;
      }

  }
  //convex hull algo
  if(convex)
    FORS(k, num_players) {
    stack < Point > all;
    stack < Point > u_edge;
    stack < Point > l_edge;
    x = player_x[k] + min - 1;
    y = player_y[k] + min - 1;
    sint4 left_edge = player_x[k] - min;
    sint4 top = player_y[k] - min;
    //get island tiles
    while(x > left_edge) {
      y = player_y[k] + min - 1;
      while(y > top) {
        if(map_data(x, y).get_height(0) == level_2) {
          t.x = x;
          t.y = y;
          all.push(t);

        }
        y--;
      }
      x--;
    }
    upper_edge(all, u_edge, convex);
    lower_edge(all, l_edge, convex);

    // set points in convex hull to island ground tiles
    x = player_x[k] + min - 1;
    y = player_y[k] + min - 1;
    Vector < Point > poly;
    while(!u_edge.empty()) {
      poly.push_back(u_edge.top());
      u_edge.pop();
    }
    while(!l_edge.empty()) {
      poly.push_back(l_edge.top());
      l_edge.pop();
    }

    while(x > left_edge) {
      y = player_y[k] + min - 1;
      while(y > top) {
        t.x = x;
        t.y = y;
        if(inside_polygon(poly, t)) {
          map_data(x, y).set_type(GROUND);
          map_data(x, y).set_flat(level_2);
        }
        y--;
      }
      x--;
    }


    }

  y = 1;
  while(y < tiles_y - 1) {
    x = 1;
    while(x < tiles_x - 1) {
      smooth_edge(x, y, map_data, level_2 - level_1);
      x++;
    }
    y++;
  }
  /*

     FORS(y, tile_n){
     cout << map_data(y).get_height(0);
     if((y + 1) % tiles_x == 0) cout << endl;
     }
   */
  sync_corner(map_data, tiles_x, tiles_y);
  if(cliff)
    cliff_tiles(map_data, tiles_x, tiles_y);

  FORS(k, num_players) {
    bool ramp = false;
    x = player_x[k] - min;
    y = player_y[k] - min;
    while(!ramp) {
      ramp = place_ramp(map_data, x, y, (min + min) * 5 / 6, rand.rand_uint4() % 8, level_1, level_2, tiles_x, tiles_y);
    }
  }
  sync_corner(map_data, tiles_x, tiles_y);

  world << "<TILES>" << endl;
  map_data.write(world);
  world << "</TILES>" << endl;

  string fname = opt.bp_file;
  fname = sanitize_path(fname);
  parse_map_include(world, fname);

  world << "<OBJECTS>\n";

  sint4 nobjs = opt.nobjs;
  const sint4 tile_points = GameConst::TILE_POINTS;
  // create random clusters of minerals

#if 1
  Point point;
  FORS(i, num_players) {
    FORS(t, nobjs / num_players / 3) {
      point = place_object(player_x[i], player_y[i], 3, edge / 3, rand);
      while(map_data(point.x, point.y).get_height(0) != level_2)
        point = place_object(player_x[i], player_y[i], 3, edge / 3, rand);
      world << "mineral" << endl;
      world << "<INT_ATTR>" << endl;
      world << "owner " << num_players << endl;
      world << "x " << point.x * tile_points - 6 + rand.rand_uint4() % 13 << endl;
      world << "y " << point.y * tile_points - 6 + rand.rand_uint4() % 13 << endl;
      world << "</INT_ATTR>" << endl;
    }

  }

#endif
  // create random geysers
#if 0
  FORS(i, nobjs / 2) {
    sint4 x0, y0;
    sint4 t = rand.rand_uint4() % tile_n;
    x0 = t % tiles_x;
    y0 = t / tiles_y;
    if(o_safe[t])
      continue;

    FORS(j, 10) {
      sint4 x = x0 - 2 + rand.rand_uint4() % 5;
      sint4 y = y0 - 2 + rand.rand_uint4() % 5;
      if(x < 1 || x >= tiles_x - 1)
        continue;
      if(y < 1 || y >= tiles_y - 1)
        continue;
      bool can_use = true;
      FORU(u, 3) {
        FORU(v, 3) {
          if(o_safe[(x - 1 + u) + tiles_x * (y - 1 + v)])
            can_use = false;
        }
      }
      if(!can_use)
        continue;
      FORU(u, 3) {
        FORU(v, 3) {
          o_safe[(x - 1 + u) + tiles_x * (y - 1 + v)] = true;
        }
      }
      world << "geyser" << endl;
      world << "<INT_ATTR>" << endl;
      world << "owner " << num_players << endl;
      world << "x " << x * tile_points - 6 + rand.rand_uint4() % 13 << endl;
      world << "y " << y * tile_points - 6 + rand.rand_uint4() % 13 << endl;
      world << "</INT_ATTR>" << endl;
    }
  }
#endif

  FORS(p, num_players) {
    o_safe[player_y[p] * tiles_y + player_x[p]] = 1;
    place_unit(world, "worker", p, player_x[p], player_y[p]);

  }
  const bool AIR_UNITS = false;
  const bool LAND_UNITS = false;

  if(AIR_UNITS && LAND_UNITS)
    nobjs /= 2;

  FORS(p, num_players) {

    if(AIR_UNITS) {
      FORS(i, nobjs) {
        sint4 r;
        do {
          r = rand.rand_uint4() % tile_n;
        } while(o_safe[r]);
        o_safe[r] = 1;

        place_unit(world, "destroyer", p, r % tiles_x, r / tiles_x);
      }
    }

    if(LAND_UNITS) {

      FORS(i, nobjs) {
        sint4 r;
        do {
          r = rand.rand_uint4() % tile_n;
        } while(map_data(r).get_typeW() == WATER || map_data(r).get_typeW() == CLIFF || o_safe[r]);
        o_safe[r] = 1;

        place_unit(world, "worker", p, r % tiles_x, r / tiles_x);
      }
    }
  }

  world << "</OBJECTS>\n";
#ifdef GCC
  cout << world;
#else
  cout << world.str();
#endif
  delete[]o_safe;
}


//===================================================================
/**
 * Calculates the largest circle that fits into the given map dimentions
 * and places players on in a random location ouside the radius of the 
 * given inner circle, and inside the radus of the calcualted outer circle.
 * */
void MapTool::place_players(sint4 rad_i, sint4 * player_x, sint4 * player_y)
{
  Random rand;
  sint4 count = 0, space = 0, seed = opt.seed;
  rand.srand(seed);

  if(seed < 0) {
    seed = -seed;
  }

  sint4 gtiles_x = opt.width, gtiles_y = opt.height;
  sint4 x;
  sint4 y;
  sint4 nplayers = opt.nplayers;

  // radius of outer circle squared
  sint4 size = opt.island;
  sint4 min = static_cast<sint4>(sqrt((static_cast<real8>(size))) * 2 + 1);
  sint4 rad_o;

  if(gtiles_x < gtiles_y) {
    rad_o = (gtiles_x - min) / 2;
  } else {
    rad_o = (gtiles_y - min) / 2;
  }

  // x and y aren't in the big circle or are in the little circle
  x = y = 0;                    // safeguard

  sint4 dist = static_cast<sint4>(sqrt(static_cast<real8>(square(x - (gtiles_x / 2)) + square(y - (gtiles_y / 2)))));
  bool flag;
  sint4 t;                      
  sint4 half_y = gtiles_y / 2;
  sint4 half_x = gtiles_x / 2;

  /* FIXME: more efficent to generate a randum radius within range, and
     n random angles that are /min_ganle/ apart and place player there. */

  FORS(i, nplayers) {
    space++;
    t = 0;
    flag = true;
    do {
      count++;
      x = rand.rand_uint4() % gtiles_x;
      y = rand.rand_uint4() % gtiles_y;
      dist = static_cast<sint4>(sqrt(static_cast<real8>(square(x - (half_x)) + square(y - (half_y)))));
      //check min distance retry while not satisfied
    } while((dist > rad_o || dist < rad_i) && count < 9999);

    // check the new location's distance to other players
    while(t < i) {
      if(sqrt(static_cast<double>(square(player_x[t] - x) + square(player_y[t] - y))) <= min) {
        flag = false;
      }
      t++;
    }

    if(flag) {
      player_x[i] = x;
      player_y[i] = y;
    } else
      i--;
    if(count == 9999) {
      i = -1;
    }
    count = 0;
    x = rand.rand_uint4() % gtiles_x;
    y = rand.rand_uint4() % gtiles_y;
    dist = static_cast<sint4>(sqrt(static_cast<double>(square(x - half_x) + square(y - half_y))));
    if(space > 9999) {
      min = static_cast<sint4>(sqrt((static_cast<double>(size))) * 1.2);
      //default placement
      if(nplayers == 2) {
        player_x[0] = min;
        player_y[0] = min;
        player_x[1] = gtiles_x - min;
        player_y[1] = gtiles_y - min;
        return;
      } else if(nplayers == 3) {
        player_x[0] = min;
        player_y[0] = min;
        player_x[1] = gtiles_x - min;
        player_y[1] = gtiles_y - min;
        player_x[2] = min;
        player_y[2] = gtiles_y - min;
        return;
      } else if(nplayers == 4) {
        player_x[0] = min;
        player_y[0] = min;
        player_x[1] = gtiles_x - min;
        player_y[1] = gtiles_y - min;
        player_x[2] = min;
        player_y[2] = gtiles_y - min;
        player_x[3] = gtiles_x - min;
        player_y[3] = min;
        return;
      } else {
        cout << " not enough space, make islands smaller or have fewer players " << endl;
        exit(-1);
      }
    }
  }
}

//===================================================================

void MapTool::add_tile(sint4 seed, Vector < Point > &edges, Map < Tile > &map, bool * flags, sint4 x_tiles, sint4 y_tiles, sint4 level_2)
{

  sint4 size = edges.size();
  if(edges.empty())
    return;
  seed = seed % size;
  Point next = edges.at(seed);
  Tile & tile = map.get_tile(next.x, next.y);

  // flags 1 if tiles has already been added
  flags[(x_tiles * next.y) + next.x] = true;
  tile.set_flat(level_2);
  tile.set_type(GROUND);
  bool none = false;

  // insert_possible_edges
  next.x++;
  Point add;
  edges.erase(edges.begin() + seed);
  if(next.y < y_tiles && next.y >= 0 && next.x >= 0 && next.x < x_tiles && !flags[(x_tiles * next.y) + next.x]) {
    add.x = next.x;
    add.y = next.y;
    edges.push_back(add);
    none = true;
    flags[(x_tiles * next.y) + next.x] = true;
  }

  next.x -= 2;

  if(next.y < y_tiles && next.y >= 0 && next.x >= 0 && next.x < x_tiles && !flags[(x_tiles * next.y) + next.x]) {
    add.x = next.x;
    add.y = next.y;
    edges.push_back(add);
    none = true;
    flags[(x_tiles * next.y) + next.x] = true;
  }

  next.x++;
  next.y++;

  if(next.x >= 0 && next.x < x_tiles && next.y < y_tiles && next.y >= 0 && !flags[(x_tiles * next.y) + next.x]) {
    add.x = next.x;
    add.y = next.y;
    edges.push_back(add);
    none = true;
    flags[(x_tiles * next.y) + next.x] = true;
  }

  next.y -= 2;

  if(next.x >= 0 && next.x < x_tiles && next.y < y_tiles && next.y >= 0 && !flags[(x_tiles * next.y) + next.x]) {
    add.x = next.x;
    add.y = next.y;
    edges.push_back(add);
    none = true;
    flags[(x_tiles * next.y) + next.x] = true;
  }
  if(!none && (edges.begin() + seed + 1) <= edges.end())
    edges.erase(edges.begin() + seed, edges.begin() + seed + 1);
  cout.flush();
}

//=============================================================

void MapTool::raise_island(Map < Tile > &map_data, sint4 level_2, Point pos, sint4 size, sint4 seed)
{
  int num_islands = 1;
  
  Random rand;
  rand.srand(seed);
  sint4 tiles_x = opt.width, tiles_y = opt.height;
  sint4 tile_n = tiles_x * tiles_y;
  // sint4 r = rand.rand_uint4();
  //positions of islands
  sint4 *player_x = new sint4[num_islands];
  sint4 *player_y = new sint4[num_islands];
  bool *flags = new bool[tile_n];
  bool *checked = new bool[tile_n];
  if(!player_x || !player_y || !flags || !checked)
    ERR("out of memory");
  memset(player_x, 0, num_islands * sizeof(*player_x));
  memset(player_y, 0, num_islands * sizeof(*player_y));
  memset(flags, false, tile_n * sizeof(*flags));
  memset(checked, 0, tile_n * sizeof(*checked));

  Vector < Point > *edges = new Vector < Point >[num_islands];
  Vector < Point > *islands = new Vector < Point >[num_islands];
  if(!edges || !islands)
    ERR("out of memory");

  sint4 x = 0, y = 0;

  //pick island spots
  FORS(n, num_islands) {
    x = pos.x;
    y = pos.y;
//    if(flags[(y * tiles_x) + x] || map_data(x, y).get_height(0) == level_2 || map_data(x, y).get_height(1) == level_2 && map_data(x, y).get_height(3) == level_2 || map_data(x, y).get_height(5) == level_2) {
//      n--;
//      continue;
//    }
    player_x[n] = x;
    player_y[n] = y;
    flags[(y * tiles_x) + x] = true;
  }

  Point t;
  //  bool water = opt.water;
  //  bool cliff = opt.cliff;
  real8 convex = opt.convex;

  sint4 edge = (sint4)(sqrt((real8)(size)));
  sint4 min = edge;
  FORS(k, num_islands) {
//    level_2 = rand.rand_uint4() % 13;
//    level_2 = 4;
    t.x = player_x[k];
    t.y = player_y[k];
    Tile & tile = map_data.get_tile(x, y);
    tile.set_flat(level_2);
    edges[k].push_back(t);
    flags[(y * tiles_x) + x] = true;
    FORS(i, size) {
      seed = rand.rand_uint4();
      if(seed < 0)
        seed = -seed;
      MapTool::add_tile(seed, edges[k], map_data, flags, tiles_x, tiles_y, level_2);
    }
  }
  //convex hull algo
  FORS(k, num_islands) {
    stack < Point > all;
    stack < Point > u_edge;
    stack < Point > l_edge;
    x = player_x[k] + min - 1;
    if(x < 0)
      x = 0;
    if(x > tiles_x - 1)
      x = tiles_x - 1;
    y = player_y[k] + min - 1;
    if(y < 0)
      y = 0;
    if(y > tiles_y - 1)
      y = tiles_y - 1;
    sint4 left_edge = player_x[k] - min;
    if(left_edge < 0)
      left_edge = 0;
    sint4 top = player_y[k] - min;
    if(top < 0)
      top = 0;
    //get island tiles
    while(x > left_edge) {
      y = player_y[k] + min - 1;
      if(y < 0)
        y = 0;
      if(y > tiles_y - 1)
        y = tiles_y - 1;
      while(y > top) {
        if(map_data(x, y).get_height(0) == level_2) {
          t.x = x;
          t.y = y;
          all.push(t);

        }
        y--;
      }
      x--;
    }
    upper_edge(all, u_edge, convex);
    lower_edge(all, l_edge, convex);
    // set points in convex hull to island ground tiles
    x = player_x[k] + min - 1;
    y = player_y[k] + min - 1;
    Vector < Point > poly;
    while(!u_edge.empty()) {
      poly.push_back(u_edge.top());
      u_edge.pop();
    }
    while(!l_edge.empty()) {
      poly.push_back(l_edge.top());
      l_edge.pop();
    }





    while(x > left_edge) {
      y = player_y[k] + min - 1;
      while(y > top) {
        t.x = x;
        t.y = y;
        if(inside_polygon(poly, t)) {
          map_data(x, y).set_type(GROUND);
          map_data(x, y).set_flat(level_2);
        }
        y--;
      }
      x--;
    }
  }




  y = 1;
  while(y < tiles_y - 1) {
    x = 1;
    while(x < tiles_x - 1) {
//            smooth_edge(x, y, map_data, level_2 - level_1);
      x++;
    }
    y++;
  }



}
