// $Id: PathfindTask.C 5279 2007-06-23 02:49:08Z mburo $
// This is an ORTS file (c) Michael Buro licensed under the GPL

#include "PathfindTask.H"

PathfindTask::PathfindTask(const TerrainBase::Task& task_)
  : task(task_), path(), current_wp(0), isPending(true),
    noPathFound(false), executor_data(0), pathfinder_data(0)
{

}

//------------------------------------------------------------

PathfindTask::PathfindTask(const PathfindTask &other)
    : task(other.task), path(other.path), current_wp(other.current_wp),
      isPending(other.isPending), noPathFound(other.noPathFound)
{
  if (other.executor_data){
    executor_data = other.executor_data->clone();
  }
  else{
    executor_data = 0;
  }
  
  if (other.pathfinder_data){
    pathfinder_data = other.pathfinder_data->clone();
  }
  else{
    pathfinder_data = 0;
  }


}

//------------------------------------------------------------
PathfindTask::~PathfindTask(){
  clean();
}

//------------------------------------------------------------

PathfindTask& PathfindTask::operator=(const PathfindTask &rhs){
  if (this == &rhs){
    return (*this);
  }
  clean();
  
  task = TerrainBase::Task(rhs.task);
  path = rhs.path;
  current_wp = rhs.current_wp;
  isPending = rhs.noPathFound;
  noPathFound = rhs.noPathFound;
  
  
  if (rhs.executor_data){
    executor_data = rhs.executor_data->clone();
  }
  else{
    executor_data = 0;
  }
  
  if (rhs.pathfinder_data){
    pathfinder_data = rhs.pathfinder_data->clone();
  }
  else{
    pathfinder_data = 0;
  }

  return (*this);
}

//------------------------------------------------------------
void PathfindTask::set_path(TerrainBase::Path &path_){
  path = path_;
  current_wp = 0;
  //depending on the pathfinder, one may want to increment current_wp right
  //away, if the pathfinder returns paths where wp 0 is the starting location of the
  //object.
  isPending = false;
}

//------------------------------------------------------------
void PathfindTask::clean(){
  if (executor_data){
    delete executor_data;
  }
  if (pathfinder_data){
    delete pathfinder_data;
  }
}
//------------------------------------------------------------
