#ifndef PATHFINDTASK_H
#define PATHFINDTASK_H
// $Id: PathfindTask.H 5279 2007-06-23 02:49:08Z mburo $
// This is an ORTS file (c) Michael Buro, Nick Wiebe, licensed under the GPL

#include "TerrainBase.H"


///Empty structure, for inheritance purposes
struct ExtraData{

  virtual ExtraData* clone() const = 0;
  virtual ~ExtraData(){};
}; 

/** Task Wrapper that can be used by both the planning system and the PathExecutor */
struct PathfindTask{
  TerrainBase::Task task;
  
  /** Contains a vector of waypoints.
      Waypoints must be in order from the start to
      the end of the path. That is, path.locs[0] is the starting location
      of the path,and path.locs[path.locs.size()-1] should be the
      end of the path.
  */
  TerrainBase::Path path;
  sint4 current_wp;
  bool isPending; //Set to true if the path has not been planned yet. Allows the Path Executor to skip unplanned paths
  bool noPathFound; //Set to true if no path was found. Allows the Path Executor to send NoPathFound Status Messages
  //There is no isFailed Flag, unlike the SimpleTerrain Task Wrapper. This is because if the path execution returns
  //a movement failure, then whoever created the task is responsible for deciding if it should be replanned or not
  
  ///Hooks to attach whatever extra data the pathfinders and executors need 
  //@{
  ExtraData* executor_data;
  ExtraData* pathfinder_data;
   //@}

  PathfindTask(const TerrainBase::Task& task_);
  PathfindTask(const PathfindTask &other);
  ~PathfindTask();
  PathfindTask& operator=(const PathfindTask &rhs);
  
  void set_path(TerrainBase::Path &path_);
  
private:
  void clean();
};

#endif
