// $Id: SI_Group.cpp 2314 2006-08-10 17:16:02Z nicholas $


#include "SI_Group.h"
#include "SI_Order.h"
#include "SI_Player.h"

#include <vector>
#include <assert.h>
#include <math.h>

namespace Simul{

  Group::Group(uint4 playerID) : MapObject(playerID){

    currentOrder = NULL;
    maxSpeed = 0.0;
    maxAttackRange = 0.0;
    lastDistanceMoved = 0.0;

    radius = 0.0;

  }

  Group::~Group(){
      delete currentOrder;
      currentOrder = NULL;
  }

  Group::Group(const Group& b) : MapObject(b){
    // NOTE: orders are NOT copied by default!  they must be copied manually
    // this is done because in MOST cases, we DON'T need to copy orders

    currentOrder = b.currentOrder->clone();

    maxSpeed = b.maxSpeed;
    maxAttackRange = b.maxAttackRange;
    lastDistanceMoved = b.lastDistanceMoved;

    radius = b.radius;

  }

  Group* Group::clone() const{
    return new Group(*this);
  }

  bool Group::withinRange(Group* otherGroup) {
    return
      (position - otherGroup->getPosition()).length() <=
      (radius + otherGroup->getExtents()+otherGroup->getMaxAttackRange());
  }

/*

  void Group::draw(){
    // here, draw the attack ranges of all the different units in the group...
    uint4 count = 0; // used for graphics

    FORALL(unitTypes,it){
      if((it->second).numUnits > 0){
	drawCircle(position,
		  (it->second).unitPrototype->getAttackRange()+radius, CIRCLE_DETAIL, transparentBlue);

	// this whole thing is really hacky... but there is no easy way
	// to make this work, and I don't want to put that much effort
	// just to make it look pretty

	if(displayObjectInfo){
	std::string unitsString = (it->second).unitPrototype->getName() +": " + convertToString((it->second).numUnits);

	  if(draw3DTextInfo) {
	    draw3DString(position.x-8,
			position.y-8-count*(2+cameraPos.z/-200),
			unitsString,
			color,
			1.0/(-cameraPos.z/4.0));
	  }else{
	    draw2DString(position.x-8,
			position.y-8-count*(2+cameraPos.z/-200),
			unitsString,
			color,
			true);
	  }
	}
      }else{
	ERR2("SOMETHING TERRIBLY WRONG IN GROUP: ", name);
      }
      count++;
    }

    // draw the unit itself...
    drawCircle(position,radius,CIRCLE_DETAIL,color);
  }

  void Group::drawInSelectionMode(){
    glPushName(GROUP_TYPE_ID);

    MapObject::drawInSelectionMode();

    glPopName();
  }
*/

  void Group::setOrder(Order* newOrder){
    delete currentOrder;

    currentOrder = newOrder;
  }

  void Group::executeOrders(real8 timeElapsed){
    if(currentOrder == NULL){
      lastDistanceMoved = 0.0;
      return;
    }

    Vector3D direction = currentOrder->getDestination() - position;

    real8 adjustedMaxSpeed = currentOrder->getSpeed()*timeElapsed;

    if(direction.length() <= adjustedMaxSpeed){
      // if we have reached our current destination order...
      position = currentOrder->getDestination();
      // if we have reached our destination, then we can execute our order action...
      currentOrder->goToNextWayPoint();

        //currentOrder->setCompleted(true);
    }else{
      position+=(direction|adjustedMaxSpeed);

    }
    if(direction.length() == 0.0){
      lastDistanceMoved = 0.0;
    }else{
      lastDistanceMoved = (direction|adjustedMaxSpeed).length();
    }
  }

  void Group::executeOrderAction(SimulationState& state){
    assert(currentOrder!=NULL);

    if(currentOrder->getActionType() == "MOVE"){ // do nothing at the end, since all we were doing is moving...

    }else if(currentOrder->getActionType() == "ATTACK"){
      if(state.enemyInRange(this)){
	return; // if there are still enemies in range, don't end this order...
      }
    }

    cleanupOrder();
  }

  void Group::cleanupOrder(){
    if(currentOrder == NULL){
    ERR("Trying to cleanup an empty order, this should not happen!");
    }else{
      delete currentOrder;
      currentOrder = NULL;
    }

  }

}

