/** @page blueprintcode Blueprint Description

- @ref blueprintoverview
  - @ref blueprintattrib
  - @ref blueprintmods
  - @ref blueprintkeywords
- @ref blueprintactions
  - @ref actionkeywords

@section blueprintoverview Overview

Blueprints are declared within a \<BLUEPRINTS\> \</BLUEPRINTS\> block, using the
\b blueprint keyword, until an \b end is reached.  %Game objects may then be created
as an instantiation of one such declaration.

A restricted subset of blueprints, a \b class, may be declared similarly, replacing
\b blueprint with \b class.
The only distinction is functional: classes can't be instantiated as objects.
They are intended to be used to define common attributes which may then
be inherited by other blueprints/classes.


@subsection blueprintattrib Attribute Declarations

- \b var \em param_name \em value
\par
Declare a typeless attribute (defaults to an integer).

- \b int \em param_name \em value
\par
Declare an integer attribute.

- \b obj \em param_name \em value
\par
Declare a pointer to an object (\em value should currently always be 0).

- \b class \em class_name \em param_name
\par
Create a sub-object of type \em class_name called \em param_name.
This may be a blueprint or a class.



@subsubsection blueprintmods Attribute Modifiers

Inserted between the declaration type and the parameter name.

- \b const
\par
Make the attribute unchanging (for enforcement and transmission brevity).

- \b hidden
\par
Makes the attribute unseen to non-allied units.

- \b internal
\par
Declares a "helper" attribute only seen/used by the server.




@subsection blueprintkeywords Blueprint Keywords

- \b set \em param_name \em  value
\par
Sets the value of an existing (already defined) parameter.

- \b setf \em param_name \em  value
\par
Sets and makes \em const the value of an existing (already defined) parameter.

- \b SET \em param_name \em  value
\par
Emphatically changes the attribute, ignoring previous "const" modifiers (but does
not change the attribute's const-ness).

- \b has/is \em class_name
\par
Include all the attributes and actions of the blueprint/class \em class_name
without creating a sub-object (as if the body text of the included class were copied
here directly).  Both keywords are functionally identitical; use
the one which least butchers English grammar.


@section blueprintactions Action Declarations

\b action [\b hidden] \em name([obj1, obj2, ...]; [int1, int2, ...]; [serv1, ...]) {...}
\par
Semicolons are mandatory within the action declaration, but may be
omitted when calling the action at the point where no more parameters are
forthcoming.  So if there are no parameters being passed in the third set then
the second semicolon is not required.

\par
The first set of parameters is pointers to objects, and the second set is integers.

\par
The third set is also integers, but requires somewhat more explanation.
These parameters can only be specified server-side.  Even if the action is
public, clients cannot specify them when calling -- in this case their
value defaults to 0.  Often the game description will use them to avoid creating
a hidden action that differs only by a counter parameter or some other minor change.

\par
Local parameters are declared using the \b int and \b gob keywords for integers
and object pointers respectively.  All local parameters must be declared at
the start of the action.


@subsection actionkeywords Action Keywords

- \b if (\em condition) {...} [\b else {...}]

- \b this
\par
Pointer to the current (sub)object.

- \b parent
\par
Pointer to the current root object.

- \b %global.foo
\par
Pointer to the global object \em foo.  Each player has different global objects.

- \b %shared.foo
\par
Pointer to the shared object \em foo.  Shared objects are the same for all players.

*/
