// $Id: Quadtree.H 5279 2007-06-23 02:49:08Z mburo $
 
// This is an ORTS file (c) Michael Buro, licensed under the GPL

// Quadtree
// maintains region adjecency graph incrementally

#ifndef Quadtree_H
#define Quadtree_H

#include "Global.H"

class QuadtreeNode {

private:

protected:

public:

  struct Edge {
    QuadtreeNode *node;             // neighbor square
    list<Edge>::iterator list_pos;  // where the back-edge is stored
  };
  
  QuadtreeNode();
  virtual ~QuadtreeNode();

  sint4 xl, yu, w;
  QuadtreeNode *pred;     // predecessor
  QuadtreeNode *succ[4];  // successors  (S_UL,...,S_LR)

  typedef list<Edge> NList;
  
  NList nls[4]; // neighbor lists (N_N,...,N_W)

  enum { S_UL=0, S_UR=1, S_LL=2, S_LR=3 };
  enum { N_N=0, N_E=1, N_S=2, N_W=3 };
  
  bool is_split() const { return succ[0] != 0; }

  // split square and adjust adjecency graph
  void split();

  // return true if inserted
  bool insert(sint4 x, sint4 y);

  // return pointer to QuadtreeNode which covers (x,y)
  const QuadtreeNode *locate(sint4 x, sint4 y) const;

  static void test();
};

REGISTER_TYPEOF(67, QuadtreeNode::NList);

#endif

