// $Id: compr.C 5279 2007-06-23 02:49:08Z mburo $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

// generates output for N moving objects
//   print0  absolute values
//   print1  first difference
//   print2  second difference

#include <iostream>
#include "Global.H"
#include <vector>

using namespace std;

const sint4 N = 200;
const sint4 R = 256*16;
const sint4 F = 32; // motion grid factor
const sint4 S = 10; // max. speed 1..S
const sint4 H = 32; // headings

class Obj {
public:

  sint4 id;
  sint4 mx, my, h, s;
  sint4 x, y; // rounded mx,my
  sint4 px1, py1, ph1;
  sint4 px2, py2, ph2;  

  Obj() {
    x = y = h = s;
    px1 = py1 = ph1 = 0;
    px2 = py2 = ph2 = 0;    
    id = 0;
  }
  
  void randomize() {
    mx = (Rand.rand_uint4() % R) * F;
    my = (Rand.rand_uint4() % R) * F;
    s = (Rand.rand_uint4() % S) + 1;
    h = Rand.rand_uint4() % H;
  }

  bool step() {
    sint4 dx = my_round(S*F*sin((h*2*M_PI)/H));
    sint4 dy = my_round(S*F*cos((h*2*M_PI)/H));
    bool stop = false;
    mx += dx;
    if (mx < 0)    { mx = 0; stop = true; }
    if (mx >= R*F) { mx = R*F-1; stop = true; }    
    my += dy;
    if (my < 0)    { my = 0; stop = true; }
    if (my >= R*F) { my = R*F-1; stop = true; }

    x = my_round(real4(mx)/F);
    y = my_round(real4(my)/F);    
    return stop;
  }

  void store() {
    px2 = px1; px1 = x;
    py2 = py1; py1 = y;
    ph2 = ph1; ph1 = h;    
  }
  
  void action() {

    store();
    
    if (step()) {
      h = Rand.rand_uint4() % H;
    }

  }

  void print0(ostream &os)
  {
    os << x << " "
       << y << " "
       << h;
  }

  void print1(ostream &os)
  {
    os << (x - px1) << " "
       << (y - py1) << " "
       << (h - ph1);
  }

  void print2(ostream &os)
  {
    os << (x - (2*px1-px2)) << " "
       << (y - (2*py1-py2)) << " "
       << (h - (2*ph1-ph2));
  }
  
};


struct comp_Obj {

  bool operator() (const Obj &a, const Obj &b) {
    return a.id < b.id;
  }
  
};


int main()
{
  vector<Obj> objs(N);

  sint4 j = 0;
  FORALL (objs, i) { i->randomize(); i->id = ++j; }

  sort(objs.begin(), objs.end(), comp_Obj());

  // run "simulation" for a while
  FORS (step, 500) {
    FORALL (objs, i) i->action();
  }

  // print current state
  {
    sint4 j = 0;
    FORALL (objs, i) {
      cout << i->id-j << " "; i->print2(cout); // print0 print1 print2
      j = i->id;
      cout << " ";
      // biased random features
      FORS (k, 20) cout << ((Rand.rand_uint4() % 1024) < 128) << " ";
      cout << endl;
    }
  }
}
