// $Id: ortsmap.C 5279 2007-06-23 02:49:08Z mburo $

// This is an ORTS file (c) Michael Chung, licensed under the GPL

#include "Global.H"
#include "MapTool.H"
#include "Options.H"
#include "Tile.H"
#include "ShutDown.H"

// static/global variables
const char *CLIENT_VERSION = "MapTool version 0.21 (2003-Oct-03)";

static Options opt;

static void add_options(Options &opt) {
  // MC - edit options...
  opt.put("-out", string(""), "map file to save");

  // note: not yet supporting arbitrary unit blueprints
  opt.put("-bp", string("data/new_units.bp"), "blueprints file to load");

  opt.put("-x",     sint4(56),   "playing field width");
  opt.put("-y",     sint4(38),   "playing field height");
  opt.put("-rot",   true, "rotate board 45 degrees");

  // todo: document types of maps
  opt.put("-type", string("random"), "type of map: random|ctf|maze|ctftest");

  // options for random maps  
  opt.put("-seed", sint4(1), "rnd number seed (0:time)");
  opt.put("-nplayers", sint4(1), "number of players");
  opt.put("-nobjs", sint4(100), "number of objects per player");
  opt.put("-fplat", real8(0.2), "fraction of terrain that is plateau");
  opt.put("-clump", sint4(800),     "degree to which terrain is clustered (1=none)");
  opt.put("-holes", true, "try harder to plug terrain holes");
  opt.put("-smooth", true, "smooth borders");
}

int main(int argc, char **argv)
{
  signals_shut_down(false);

  // handle command line arguments
  add_options(opt);
  if (opt.process(argc, argv, cerr, CLIENT_VERSION)) exit(20);

  stringstream world;   // description of the world
  
  string map_type;
  opt.get("-type", map_type);
  if (map_type == "random") {
    // generate data for a random map
    generate_random_map(world, opt);
  } else if (map_type == "ctf") {
    // generate data for a capture-the-flag map
    generate_ctf_map(world, opt, false);
  } else if (map_type == "ctftest") {
    // generate ctf map (test mode)
    generate_ctf_map(world, opt, true);
  } else if (map_type == "maze") {
		generate_maze_map(world, opt);
  } else {
    // command-line option error?
    ERR("unsupported map type");
  }

  // output world description to map file or stdout
  write_map_file(world, opt);

  return 0;
}
