/*
    (c) Michael Buro, mic@research.nj.nec.com
    NEC Research Institute
    4 Independence Way
    Princeton, NJ 08540, USA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* $Header$
   $Log$
   Revision 1.1  2004/10/11 09:09:18  orts_mburo
   Initial revision

   Revision 1.1.1.1  2003/06/11 22:04:54  orts_mburo
   import

*/


#include <cstdlib>
#include <cstdio>
#include <zlib.h>
#include "Compress.H"
#include <string>

static voidpf zmalloc(voidpf opaque, uInt items, uInt size)
{
  voidpf r = malloc(items*size);
  if (r) return r;
  return Z_NULL;
}

static void zfree(voidpf opaque, voidpf address)
{
  free(address);
}

Compress::Compress(sint4 level_) {

  out_buffer = new sint1[N];
  if (!out_buffer) ERR("out of memory");

  pzs = new z_stream_s;
  if (!pzs) ERR("out of memory");

  pzs->zalloc = zmalloc;
  pzs->zfree  = zfree;

  init(level_);
}


void Compress::init()
{
  if (deflateInit(pzs, level) != Z_OK) ERR("deflateInit problem");
}


void Compress::init(sint4 level_)
{
  level = level_;
  if (level > 9 || level < 0) ERR("illegal compression level");
  init();
}

Compress::~Compress() {
  if (pzs) {
    deflateEnd(pzs);
    delete pzs;
    pzs = 0;
  }
  if (out_buffer) { delete [] out_buffer; out_buffer = 0; }
}


void Compress::doit(const string &input, string &output) {

  const char *s = input.c_str();

  pzs->next_in = (Bytef*)s;
  pzs->avail_in = input.size();
  pzs->next_out = (Bytef*)out_buffer;
  pzs->avail_out = N;
  
  output = "";
  
  while (pzs->avail_in) {
    
    if (pzs->avail_out == 0) {
      string s((const char*)out_buffer, N);
      output += s;
      pzs->next_out = (Bytef*)out_buffer;
      pzs->avail_out = N;
    }

    deflate(pzs, Z_NO_FLUSH);
  }

  deflate(pzs, Z_SYNC_FLUSH);

  if (pzs->avail_out != N) {
    string s((const char*)out_buffer, N-pzs->avail_out);
    output += s;
  }
}


Decompress::Decompress() {

  out_buffer = new sint1[N];
  if (!out_buffer) ERR("out of memory");
  
  pzs = new z_stream_s;
  if (!pzs) ERR("out of memory");

  pzs->zalloc = zmalloc;
  pzs->zfree  = zfree;

  init();
}

void Decompress::init()
{
  if (inflateInit(pzs) != Z_OK) ERR("inflateInit problem");
}

Decompress::~Decompress() {
  if (pzs) {
    inflateEnd(pzs);
    delete pzs;
    pzs = 0;
  }
  if (out_buffer) { delete [] out_buffer; out_buffer = 0; }
}


bool Decompress::doit(const string &input, string &output) {

  const char *s = input.c_str();
  int zres;
    
  pzs->next_in = (Bytef*)s;
  pzs->avail_in = input.size();
  pzs->next_out = (Bytef*)out_buffer;
  pzs->avail_out = N;

  while (pzs->avail_in) {

    if (pzs->avail_out == 0) {
      string s((const char*)out_buffer, N);
      output += s;
      pzs->next_out = (Bytef*)out_buffer;	
      pzs->avail_out = N;
    }

    zres = inflate(pzs, Z_SYNC_FLUSH);
    if (zres != Z_OK && zres != Z_STREAM_END) return true;
  }

  if (pzs->avail_out != N) {
    string s((const char*)out_buffer, N-pzs->avail_out);
    output += s;
  }

  return false;
}
